﻿#pragma TextEncoding = "UTF-8"
#pragma rtGlobals=3		// Use modern global access method and strict wave access.
#pragma version=9.34

// *** Copyright ***
// © 2023 CNRS Shared by Pierre VINCENT, under the MIT License.
//	Contact: pierre.vincent@cnrs.fr; IGF, 141 rue de la Cardonille, 34094 Montpellier cedex 5, France.

// *** Global things ***
StrConstant ksNomCheminInit="root:z_Initialisation:"	//	path to initialization parameters
StrConstant ksNomObjetInit="f_String;f_Variable;f_Wave;f_WaveStr"
StrConstant ksABC="A;B;C;D;E;F;G;H;I;J"
StrConstant ksZero="_None_"
StrConstant ksNomExport="Export"	//	racine du nom du datafolder qui contient les mesures stats
StrConstant ksPulseParams="Frequency;MaxPeakb;MinPeakb;Ampltd;TimetoPeak;Risetime;TimetoAntiPeak;Decaytime;EventDuration;TAP50R;Decayslope;Riseslope;Average;RMS;FracShort;R0;Smoothing;vMinArea;vMaxArea;Area_change;vLongDmtrDiast;vLongDmtrSyst;vShortDmtrDiast;vShortDmtrSyst;EDV;ESV;EF;SV;CO;LongFS;ShortFS"

// *** Loading ~automatically the correct code ***

//	When starting, the user is offered the option to load regular modules for ratio analysis, fusion or biolum analysis
//	Igor provides a hook function: executed at startup or when a new experiment is created
//	Offers the user with a choice of code to be loaded
Function IgorStartOrNewHook(String igorApplicationNameStr)
	Execute/P/Q "StartUpChoice()"	
End

//	When loading an experiment, the hook function is executed; tries to recognize the type of file, and load the appropriate code
//	IT CAN FAIL if the procedure window contains calls to the code -> compile error when opening the file
Function	AfterFileOpenHook(refNum, fileNameStr, pathNameStr, fileTypeStr, fileCreatorStr, fileKind )
	variable refNum, fileKind
	string fileNameStr, pathNameStr, fileTypeStr, fileCreatorStr
	string sListe

	//	fileKind should be 1, ie packed Igor experiment; don't do anything if the user has opened something other than a experiment
	if (fileKind==1)

		//	Testing if Raplouf type of data
		if ((DataFolderExists("root:Ratio")) || (DataFolderExists("root:Analyse_0")))
			Execute/P/Q/Z "INSERTINCLUDE \"Raplouf\""	//	good thing: if the #include "Raplouf" is already in the procedure, it's not added a 2d time
			Execute/P/Q/Z "COMPILEPROCEDURES "		//	the space at the end is required
		endif

		//	Testing if file is of Fusion type
		sListe=ListeDF("root:", "Group")
		if (StrLen(sListe)>0)
			Execute/P/Q/Z "INSERTINCLUDE \"Fusion\""
			Execute/P/Q/Z "COMPILEPROCEDURES "
			Return(0)
		endif
	endif
End

Window StartUpChoice() : Panel
	PauseUpdate; Silent 1		// building window...
	NewPanel/FLT=2/W=(100,100,300,250) as "What module of Ratioscope to launch ?"
	CheckBox C_Nothing,pos={30.00,20},size={105.00,15.00},proc=CheckStartUp
	CheckBox C_Nothing,title="Nothing special"
	CheckBox C_Nothing,help={"No particular module is activated"},fSize=12
	CheckBox C_Nothing,value=0,mode=1
	CheckBox C_Raplouf,pos={30.00,50},size={80.00,15.00},proc=CheckStartUp
	CheckBox C_Raplouf,title="Raplouf",help={"Load Ratioscope module for image analysis"},fSize=12
	CheckBox C_Raplouf,value=0,mode=1
	CheckBox C_Biolum,pos={30.00,80},size={113.00,15.00},proc=CheckStartUp
	CheckBox C_Biolum,title="Bioluminescence",help={"Load Bioluminescence module"}
	CheckBox C_Biolum,fSize=12,value=0,mode=1
	CheckBox C_Fusion,pos={30.00,110},size={54.00,15.00},proc=CheckStartUp
	CheckBox C_Fusion,title="Fusion",help={"Load Fusion module"},fSize=12
	CheckBox C_Fusion,value=0,mode=1
//	CheckBox C_NeuroMatic,pos={30.00,140},size={54.00,15.00},proc=CheckStartUp
//	CheckBox C_NeuroMatic,title="NeuroMatic",help={"Load NeuroMatic module"},fSize=12
//	CheckBox C_NeuroMatic,value=0,mode=1
EndMacro

Function CheckStartUp(cba) : CheckBoxControl
	STRUCT WMCheckboxAction &cba

	switch( cba.eventCode )
		case 2: // mouse up
			Variable checked = cba.checked
			StrSwitch(cba.ctrlName)
				case "C_Nothing":
				break
				case "C_Raplouf":
					Execute/P/Q/Z "INSERTINCLUDE \"Raplouf\""
				break
				case "C_Biolum":
					Execute/P/Q/Z "INSERTINCLUDE \"Bioluminescence\""
				break
				case "C_Fusion":
					Execute/P/Q/Z "INSERTINCLUDE \"Fusion\""
				break
				case "C_NeuroMatic":
					Execute/P/Q/Z "INSERTINCLUDE \"NeuroMatic\""
				break
			EndSwitch			
			KillWindow/Z StartUpChoice
			Execute/P/Q/Z "COMPILEPROCEDURES "
			break
		case -1: // control being killed
			break
	endswitch

	return 0
End

// *** Datafolder initialization tool ***

//	This function creates, if necessary, variables, strings, waves (of numbers), waves (of strings) in the corresponding datafolder
//	sNomFichierConfig indicates the name of the config file which contains all the object definitions
//	sNomDataFolderSource is the name of source datafolder, containing the object definitions, included in the initialization file.
//	Source datafolders are supposed to start with "f_"
//	f_Global is meant to additionally contain other types of objects supposed to be common to the whole experiment, like color look-up tables...
//	sNomDataFolderCible is the name of the target datafolder, into which all these objects must be defined
//	sNomDataFolderAncien is the (optional) name of an ancient datafolder into which the required objects could be already present.
//	All datafolder names should correspond to an existing datafolder; if these datafolder don't exist, an error is reported and the function is aborted.
//	The function goes through a set of tables defining object names, dimensions (only applies to waves), default values and free comment.
//	For every object, the function checks if the object already exists in the target datafolder. 
//	If it does not, the object is created with the right dimensions (for waves) and initialized to the default value.
//	If sNomDataFolderCible is not "default", the sNomDataFolderCible datafolder is searched for the required objects; if they are found, they are transfered into sNomDataFolderCible
//	def_Dim is a text wave defining the dimension of each corresponding wave: 1 number per dimension. "1;0;0;0" means a vector of 1 point; "10;2;0;0" means a 10x2 wave
//	"delete" in def_Dim leads to suppression of this object in the target datafolder
//	def_Val contains the default value for this object
//	To display and edit these config values, open the sNomFichierConfig and call AfficheInit("") to display 4 tables (one for each object type), then save the config file.
Function InitDataFolder(sNomFichierConfig, sNomDataFolderSource, sNomDataFolderCible, [sNomDataFolderAncien])
	string sNomFichierConfig, sNomDataFolderSource, sNomDataFolderCible, sNomDataFolderAncien
	variable i
	variable vAncien	//	 set to 1 if trying to read ancient values
	string sNomDataFolder, sNom, sListe
	DFREF dfSav=GetDataFolderDFR()

//	Loads the whole content of intialization file $sNomFichierConfig".pxp"
	SetDataFolder root:
	NewPath/O/Q User_Procedures, SpecialDirPath("Igor Pro User Files", 0, 0, 0)+"User Procedures:"	//	makes a path to User Procedures
	LoadData/Q/O=1/P=User_Procedures/R sNomFichierConfig	//	loads config file
	if (V_flag<1)	//	less than 1 object was loaded
		DoAlert 0, "No configuration file loaded."
		return(0)
	endif

//	Datafolder which may contain ancient objects; test if such ancient datafolder is specified as optional parameter
	if (ParamIsDefault(sNomDataFolderAncien)==0)
		vAncien=1	
		DFREF dfAncien=$sNomDataFolderAncien
	endif

//	Source datafolder
	sNomDataFolderSource=ksNomCheminInit+sNomDataFolderSource
	DFREF dfSource=$sNomDataFolderSource

//	Target datafolder
	sNomDataFolderCible="root:"+sNomDataFolderCible
	DFREF dfCible=$sNomDataFolderCible
	if (DataFolderRefStatus(dfCible)==0)	//	dfCible is not valid
		DoAlert/T="Warning" 0, "During initialization, the datafolder " + sNomDataFolderCible + " was not found"
		return(0)
	endif

	//	Copy all waves present in the source datafolder to the target datafolder
	SetDataFolder dfSource
	sListe=WaveList("*", ";", "")
	For (i=0; (i<ItemsInList(sListe)); i+=1)
		sNom=StringFromList(i, sListe)
		Duplicate/O $sNom, dfCible:$sNom
	endfor

	SetDataFolder dfCible

//	initialise les variables
	sNomDataFolder=sNomDataFolderSource+":f_Variable"
	DFREF dfSource=$sNomDataFolder	//	Datafolder source
	Wave/T/SDFR=dfSource wNom=def_Nom	, wDimWave=def_Dim		//	defines the text wawes
	Wave/SDFR=dfSource wVal=def_Val	//	defines the numeric waves
	for (i=0; (i<DimSize(wNom,0)); i+=1)
		//	Checks if the object is indicated to be deleted
		if (CmpStr(wDimWave[i],"delete")==0)
			KillVariables/Z $wNom[i]		//	kill this variable
			Continue	//	skips to the end of the loop
		endif
		NVAR/Z vv=$wNom[i]		//	try to declare the variable with the indicated name; if string is empty, nothing happens, that's lucky
		if (vAncien==1)	//	Looking in the ancient datafolder if this variable may already exist there
			sNom=sNomDataFolderAncien+wNom[i]
			NVAR/Z yy=$sNom		//	try to declare 
			if (NVAR_Exists(yy) && !NVAR_Exists(vv))	//	the ancient variable exists, but NOT the one in the target datafolder
				MoveVariable $sNom, :	//	move the ancient variable into the target datafolder
			endif
		endif
		NVAR/Z vv=$wNom[i]		//	Checks again if the object exists in the current datafolder
		if ((!NVAR_Exists(vv)) && (StrLen(wNom[i])>0))	//	if it doesn't exist,
			Variable/G $wNom[i]=wVal[i]	//	the variable is created and given the default value
		endif
	endfor

//	initialise les strings
	sNomDataFolder=sNomDataFolderSource+":f_String"
	DFREF dfSource=$sNomDataFolder	//	Datafolder source réglages string
	Wave/T/SDFR=dfSource wNom=def_Nom	, wDimWave=def_Dim	//	liste des noms de string et dimensions
	Wave/T/SDFR=dfSource wValString=def_Val	//	valeurs par défaut des strings
	for (i=0; (i<DimSize(wNom,0)); i+=1)
		//	Checks if the object is indicated to be deleted
		if (CmpStr(wDimWave[i],"delete")==0)
			KillStrings/Z $wNom[i]	
			Continue
		endif
		SVAR/Z ss=$wNom[i]	
		if (vAncien==1)	//	an older version of this string may exist
			sNom=sNomDataFolderAncien+wNom[i]
			SVAR/Z ssAncien=$sNom
			if (SVAR_Exists(ssAncien) && !SVAR_Exists(ss))	//	the ancient version exists, but not the newer one
				MoveString $sNom, :	//	move the ancient string into the current df
			endif
		endif

		SVAR/Z ss=$wNom[i]		//	tente à nouveau de déclarer le string
		if ((!SVAR_Exists(ss))	 && (StrLen(wNom[i])>0))	//	si la variable n'existe pas déjà dans le datafolder de destination
			String/G $wNom[i]=wValString[i]	//	crée le string et lui affecte la valeur par défaut
		endif
	endfor

//	initialise les waves
	sNomDataFolder=sNomDataFolderSource+":f_Wave"
	DFREF dfSource=$sNomDataFolder	//	Datafolder source réglages wave
	String defdim, nom
	Variable a,b,c,d, init
	Wave/T/SDFR=dfSource wNomWave=def_Nom
	Wave/T/SDFR=dfSource wDimWave=def_Dim	//	dimensions: string looking like "40;20;0;0" 
	Wave/SDFR=dfSource wInitWave=def_Val

	for (i=0; (i<DimSize(wNomWave,0)); i+=1)
		if  (strlen(wNomWave[i])>0)	//	executed only if the wave name is defined
			//	Checks if the object is indicated to be deleted
			if (CmpStr(wDimWave[i],"delete")==0)
				KillWaves/Z $wNomWave[i]	
				Continue
			endif
			WAVE/Z ww=$wNomWave[i]
			if (vAncien==1)	//	une version plus ancienne de cette wave pourrait exister
				sNom=sNomDataFolderAncien+wNomWave[i]
				Wave/Z wwAncien=$sNom
				if (WaveExists(wwAncien) && !WaveExists(ww))	//	le wave ancien existe mais pas le nouveau
					MoveWave $sNom, :	//	déplace le wave ancien dans le df courant
				endif
			endif
			WAVE/Z ww=$wNomWave[i]
			if ((!WaveExists(ww))	&& (StrLen(wNomWave[i])>0))//	si la wave n'existe pas déjà
				defdim=wDimWave[i]
				nom=wNomWave[i]
				init=wInitWave[i]
				a=Str2Num(StringFromList(0,defdim))
				b=Str2Num(stringFromList(1,defdim))
				c=Str2Num(stringFromList(2,defdim))
				d=Str2Num(stringFromList(3,defdim))
				make/O/N=(a,b,c,d) $nom=init
			endif
		EndIf
	endfor

//	initialise les wavesStr
	string initstr
	sNomDataFolder=sNomDataFolderSource+":f_WaveStr"
	DFREF dfSource=$sNomDataFolder	//	Datafolder source réglages wave
	Wave/T/SDFR=dfSource wNomWave=def_Nom
	Wave/T/SDFR=dfSource wDimWave=def_Dim	//	dimensions: string de la forme "40;20;0;0" 
	Wave/T/SDFR=dfSource wInitWaveStr=def_Val

	for (i=0; (i<DimSize(wNomWave,0)); i+=1)
		if  (strlen(wNomWave[i])>0)	//	n'exécute que si le string définissant le nom de la wave n'est pas nul
			//	Checks if the object is indicated to be deleted
			if (CmpStr(wDimWave[i],"delete")==0)
				KillWaves/Z $wNomWave[i]	
				Continue
			endif
			WAVE/Z ww=$wNomWave[i]
			if (vAncien==1)	//	une version plus ancienne de ce wave pourrait exister
				sNom=sNomDataFolderAncien+wNomWave[i]
				Wave/Z wwAncien=$sNom
				if (WaveExists(wwAncien) && !WaveExists(ww))	//	le wave ancien existe mais pas le nouveau
					MoveWave $sNom, :	//	déplace le wave ancien dans le df courant
				endif
			endif
			WAVE/Z ww=$wNomWave[i]
			if ((!WaveExists(ww))	&& (StrLen(wNomWave[i])>0))//	si la wave n'existe pas déjà
				defdim=wDimWave[i]
				nom=wNomWave[i]
				initStr=wInitWaveStr[i]
				a=Str2Num(StringFromList(0,defdim))
				b=Str2Num(stringFromList(1,defdim))
				c=Str2Num(stringFromList(2,defdim))
				d=Str2Num(stringFromList(3,defdim))
				Make/T/O/N=(a,b,c,d) $nom=initStr
			endif
		EndIf
	endfor
	SetDataFolder dfSav
	return(1)
End

//	Displays the waves which define the objects to be created by InitDataFolder
//	if sNomChoix is empty, the user should choose
Function AfficheInit(sNomChoix)
	String sNomChoix
	Variable i
	string sListe=ListeDF(ksNomCheminInit, "f_"), sNomDFChoix, sNomDF, sNom
	if (StrLen(sNomChoix)==0)
		prompt sNomChoix,"What to display", popup, sListe
		DoPrompt "", sNomChoix	//	choix du type de fusion qu'on veut afficher: profil, condition, groupe... 
		if (V_Flag)
			print "Cancelled by user"; return(0)
		endif
	else
		sNomChoix="f_"+sNomChoix
	endif
	sNomDFChoix=ksNomCheminInit+sNomChoix	//	définit le df d'intérêt contenant les descriptions d'objets
	for (i=0; (i<ItemsInList(ksNomObjetInit)); i+=1)		//	pour chacun des sous-df correspondant à un type de données
		sNom=StringFromList(i,ksNomObjetInit) 
		KillWindow/Z $sNom		//	supprime la fenêtre qui pourrait déjà exister avec ce nom
		sNomDF=sNomDFChoix+":"+sNom	
		SetDataFolder sNomDF
		Wave/T def_Nom, def_Dim, def_Val, def_Commentaire	//	objets qui contiennent les définitions
		//	Remarque: def_Val est une wave Str ou Number, selon que l'objet à définir est str ou number;
		//	Heureusement, edit Wave/T ne balance pas une erreur si la wave sélectionnée est du mauvais type
		//	et Edit n'envoie pas non plus d'erreur
		Edit/k=1/W=(5,44,758,244) def_Nom, def_Dim, def_Val, def_Commentaire as sNomChoix+" "+sNom
		DoWindow/C $sNom	//	change le nom du tableau
		ModifyTable format(Point)=1,alignment=0,width(def_Nom)=132,width(def_Val)=86,width(def_Commentaire)=428
		SetWindow $sNom, hook(AfficheInit)=CrochetAfficheInit
	endfor
	Execute "TileWindows f_String, f_Variable, f_Wave, f_WaveStr"
	SetDataFolder ::
End

//	When one window is closed, all other 3 are also closed; reduces the clutter in the work space
Function CrochetAfficheInit(s)
	STRUCT WMWinHookStruct &s
	variable i
	string sNomTableau
	if (s.eventCode==2)	//	kill event
		for (i=0; (i<ItemsInList(ksNomObjetInit)); i+=1)
			sNomTableau=StringFromList(i,ksNomObjetInit)
			Execute/P/Q/Z "DoWindow/K "+sNomTableau
		endfor
	endif
End

// *** Utilities ***

Function DisplayVersion()
	DoAlert/T="Ratioscope" 0, "Version "+Num2Str(VersionNumber("Ratioscope package globals.ipf"))+"\rDeveloped by Pierre VINCENT, CNRS.\rShared under MIT Public License version 3 or later."
End

//	Provided by Jim at Wavemetrics
//	Returns the version number of procedureWinTitleStr
Function VersionNumber(String procedureWinTitleStr)
	String sEverything = ProcedureText("",Inf, procedureWinTitleStr)
	String sVersions = GrepList(sEverything, "^#pragma version=", 0, "\r")
	String firstVersion = StringFromList(0, sVersions, "\r")
	Variable vVersion
	sscanf firstVersion, "#pragma version=%g", vVersion
	return vVersion
End

//	Returns the list of all datafolder located at sNomDF which name starts with sBase
Function/S ListeDF(sNomDF, sBase)
	String sNomDF, sBase
	sBase+="*"		//	add * for a StringMatch search
	String sListe="", sNomObj
	variable i
	Do
		sNomObj=GetIndexedObjName(sNomDF,4, i)	
		if (strlen(sNomObj) == 0)
			break
		endif
		if (StringMatch(sNomObj,sBase))
			sListe+=sNomObj
			sListe+=";"
		endif
		i+=1
	while(1)
	return(SortList(sListe, ";", 4))
end

//	Remove all instances of nomTrace in the graph nomGraphe
Function EnleveTrace(nomGraphe, nomTrace)
	String nomGraphe, nomTrace
	variable pos
	string temp
	do
		temp=TraceNameList(nomGraphe,";",1)
		pos=WhichListItem(nomTrace, temp)
		DoWindow $nomGraphe
		if (V_Flag)
			RemoveFromGraph/Z/W=$nomGraphe $nomTrace
		endif
	while (pos>=0)
End

//	Remove all instances of traces which name starts with nomTraces
Function EnleveLesTraces(nomGraphe, nomTraces)
	String nomGraphe, nomTraces
	variable i
	string temp, temp2, temp3
	do
		temp=TraceNameList(nomGraphe,";",1)
		temp2=ListMatch(temp, nomTraces)
		i=ItemsInList(temp2)
		temp3=StringFromList(0,temp2)
		DoWindow $nomGraphe
		if (V_Flag)
			RemoveFromGraph/Z/W=$nomGraphe $temp3
		endif
	while (i>1)	//	on s'arrête quand il ne reste plus qu'un graphe à enlever de la liste
	//	Amusant: quant on fait RemoveFromGraph/Z/W=$nomGraphe $temp3 avec temp3="", ça enlève les curseurs
End

//	Dans le graphe NomGraphe, enlève toutes les instances dont le nom n'est pas dans sListeExceptions
//	Accepte les sous-fenêtres, en paramètre optionnel
Function EnleveToutesLesTracesSauf(sNomFenetre, sListeExceptions, [sNomSousfenetre])
	String sNomFenetre, sListeExceptions, sNomSousfenetre
	string sNomGraphe=sNomFenetre, snomException
	variable i, j
	string temp, temp1, temp2
	if (ParamIsDefault(sNomSousfenetre)==0)		//	a value was supplied
		sNomGraphe+=sNomSousfenetre
	endif
	
	//	If the window name doesn't exist, abort.
	if (StrLen(sNomFenetre)<1)
		return(0)
	endif
	
	//	If window doesn't exist, abort.
	DoWindow $sNomFenetre
	if (V_Flag!=1)
		return(0)
	endif
	
	//	need to refresh the list of traces after removing each one
	do
		temp=TraceNameList(snomGraphe,";",1)
		if (ItemsInList(temp)==0)
			Return(0)
		endif
		if (ItemsInList(sListeExceptions)>0)		//	if there are exceptions to be kept
			for (j=0; (j<ItemsInList(sListeExceptions)); j+=1)	//	remove these exceptions from the list of waves to suppress
				sNomException=StringFromList(j,sListeExceptions) 
				temp1=RemoveFromList(sNomException, temp)
				temp=temp1
			endfor
		else
			temp1=temp
		endif
		if (ItemsInList(temp1)>0)
			temp2=StringFromList(0,temp1)
			RemoveFromGraph/Z/W=$sNomGraphe $temp2
		endif
	while (ItemsInList(temp1)>1)
End

//	Fonction qui entremêle les waves d'entrée, et sépare chaque point par un NaN. 
//	Le but est de pouvoir faire un graphe dans lequel chaque point est relié par une ligne au point suivant.
//	Typiquement, des mesures appariées, dans lesquelles on veut matérialiser par un trait qui va d'un point à un autre
//	La fonction reçoit en entrée une wave de wave; la taille de cette wave détermine combien de conditions sont à entremêler
//	La fonction retourne une wave W_MergedWave
Function RangeParGroupe(wEntree)
	Wave/Wave wEntree
	variable i, j	//	Compteurs
	variable vNbClasses=DimSize(wEntree, 0)	//	Nombre de waves définies en entrée
	variable vNbPoints=DimSize(wEntree[0], 0)	//	Nombre de points dans la première wave en entrée
	variable vNbPointsTotal=(vNbClasses+1)*vNbPoints	//	Taille de la wave résultat

	//	Test if input is empty, make a wave of 1 point
	if (numtype(vNbPointsTotal)!=0)
		vNbPointsTotal=1
		Make/O W_MergedWave
		W_MergedWave=NaN
		return(0)
	endif

	//	On crée la wave finale qui contiendra le résultat
	Make/O/N=(vNbPointsTotal) W_MergedWave = NaN
	
	//	Itération: pour chaque wave en entrée, on transfère son contenu dans la wave sortie
	for (i=0; (i<vNbClasses)	; i+=1)
		Wave/Z wEnCours=wEntree[i]
		if (WaveExists(wEnCours))
			for (j=0; (j<vNbPoints); j+=1)	//	pour chaque ligne de wEnCours
				W_MergedWave[j*(vNbClasses+1)+i]=wEnCours[j]
			endfor
		else
		endif
	endfor
End

//	Copy of the static function present in "Wavemetrics Procedures:Waves Average.ipf"
//	To be used widely in raplouf, to check for X-axis consistency
Function MonotonicCheck(wx,smallestXIncrement)
	Wave wx
	Variable &smallestXIncrement	// output

	Variable isMonotonic=0
	
	Duplicate/O/Free wx, diff
	Differentiate/DIM=0/EP=0/METH=1/P diff 
	WaveStats/Q/M=0 diff
	isMonotonic= (V_min > 0) == (V_max > 0)

	diff= abs(diff[p])
	WaveStats/Q/M=0 diff
	smallestXIncrement= V_Min
	
	return isMonotonic && smallestXIncrement != 0 
End

//	Remove Infs and NaNs from input wave
//	replaces NaN by the average of mini and maxi
//	replace -inf by mini; +inf by maxi
ThreadSafe Function EnleveInfsNansW(w, mini, maxi)
	wave w
	variable mini, maxi
	variable infini=inf, moy=(mini+maxi)/2
	MatrixOp/O/FREE W_Temp=replaceNaNs(w, moy); FastOp w=W_Temp
	MatrixOp/O/FREE W_Temp=replace(w, infini, maxi); FastOp w=W_Temp
	MatrixOp/O/FREE W_Temp=replace(w, -infini, mini); FastOp w=W_Temp
end

//	Add the number i to the string A, with j padded 0
//	if j=0, works except when i=0 -> nothing is shown.
//	Does no padding if j=1
ThreadSafe Function/S AjouteNumero(A, i, j)
	String A
	Variable i, j
	String num
	sprintf num, "%.*d", j, i
	A+=num
	Return(A)
End

//	Text trucation in the middle of the text: keeps vLInitiale, then adds the end of the string, replacing the center with "..."
//	Quick fix not thouroughly debugged, use with caution
Function/S RaccourcitTexte(sTexte, vLongInitiale, vLongTotale)
	String sTexte
	Variable vLongInitiale, vLongTotale
	if (StrLen(sTexte)<vLongTotale)
		return(sTexte)
	endif
	Variable vPosFinale=StrLen(sTexte)-vLongTotale+vLongInitiale-2
	String sResultat, sInitial, sFinal

	sInitial=sTexte[0, vLongInitiale-1]
	sFinal=sTexte[vPosFinale, inf]
	sResultat=sInitial+" … "+sFinal
	Return(sResultat)
End

//	Returns the average of columns; forgot why wavestats had to be modified; used in Pulse
Function MyWaveAverage(wData, vColNum)
	Wave wData
	Variable vColNum
	if (vColNum>=DimSize(wData,1))
		return(NaN)
	endif
	WaveStats/M=1/Q/PCST wData
	Wave M_WaveStats
	variable vResult=M_WaveStats[3][vColNum]	//	average is on line 3	
	return(vResult)
End

// *** Conversion between ratio and concentration ***
Function Hill(x, base, maxi, xhalf, rate)
	variable x, base, maxi, xhalf, rate
	variable y
	y=base+(maxi-base)/(1+(xhalf/x)^rate)
	return y
End

Function ReciproqueHill(y, base, maxi, xhalf, rate)
	variable y, base, maxi, xhalf, rate
	variable x
	x=xhalf/(((maxi-base)/(y-base)-1)^(1/rate))
	return x
End

//	Drawing an ellipse
//	Copied from DrawEllipseOnGraph in which the function was unfortunately made static
Static Constant ELLIPSE_NUM_POINTS=500
Function MydrawOvalInAxisCoordinates(semiMajor, semiMinor, centerX, centerY, axisAngle, ellipseName, graphName, drawIt, drawlayer, horizAxis, vertAxis, lineThickness)
	Variable semiMajor, semiMinor, centerX, centerY, axisAngle
	String ellipseName, graphName
	Variable drawIt
	String drawlayer, horizAxis, vertAxis
	Variable lineThickness
	
	String ellipseNameX = ellipseName+"_X"
	String ellipseNameY = ellipseName+"_Y"

	if ( (Exists(ellipseNameX) != 1) || (Exists(ellipseNameY) != 1) )
		Make/N=(ELLIPSE_NUM_POINTS)/O $ellipseNameX
		Make/N=(ELLIPSE_NUM_POINTS)/O $ellipseNameY
	endif
	Wave/Z ellipseX = $ellipseNameX
	Wave/Z ellipseY = $ellipseNameY

	Variable npnts = numpnts(ellipseX)
	Variable i
	Variable XFactor = 2*pi/(npnts-1)

	ellipseX = semiMajor*cos(p*XFactor)
	ellipseY = semiMinor*sin(p*XFactor)

	Variable saveX, saveY
	if (axisAngle != 0)
		Duplicate/O ellipseX, TWTempEllipseX
		Duplicate/O ellipseY, TWTempEllipseY
		ellipseX = TWTempEllipseX*cos(axisAngle) - TWTempEllipseY*sin(axisAngle)
		ellipseY = TWTempEllipseX*sin(axisAngle) + TWTempEllipseY*cos(axisAngle)
		KillWaves/Z TWTempEllipseX, TWTempEllipseY
	endif
	
	ellipseX += centerX
	ellipseY += centerY

	if (drawIt)
		SetDrawLayer/W=$graphName/K $drawlayer
		SetDrawEnv/W=$graphName xcoord = $horizAxis, ycoord = $vertAxis, fillpat = 0, linethick = lineThickNess, linefgc= (65535,65535,65535), dash= 3
		DrawPoly/W=$graphName ellipseX[0], ellipseY[0], 1,1,ellipseX, ellipseY
	endif
end

// *** Managing help buttons ***
//	All help buttons call this function
//	The name of the calling button should be structured like "C_Aide_*". 
//	The "*" should be the name of the item to be called in the help file
//	Since control names can't contain spaces, "_" replaces " ".
Function BoutonAide(ctrlName) : buttoncontrol
	string ctrlName
	string sNomPrefixe=ctrlName[0,6], sNomAide=ctrlName[7,inf]
	sNomAide=ReplaceString("_", sNomAide, " ")		//	replace "_" with " "
	StrSwitch(ctrlName)
		case "C_Help_Button_Something":	//	In case the automatic handling doesn't work, a specific case can be entered here
			//	So far, this has not been needed
		break
		default:
			if (CmpStr(sNomPrefixe, "C_Aide_")==0)
				DisplayHelpTopic/Z sNomAide
				if (V_Flag!=0)	//	This help topic was not found
					DoAlert/T="Help error" 0, "Help topic not found for "+sNomAide
				endif
			else
				DoAlert/T="Help error" 0, "Help topic not found for the button named "+ctrlName
			endif
		break
	endswitch
End

