#pragma TextEncoding = "UTF-8"
#pragma rtGlobals=3		// Use modern global access method and strict wave access.
// *** Copyright ***
// © 2023 CNRS Shared by Pierre VINCENT, under the MIT License.
//	Contact: pierre.vincent@cnrs.fr; IGF, 141 rue de la Cardonille, 34094 Montpellier cedex 5, France.

StrConstant ksROINames="Bck0;Bck1;Bck2;ROI0;ROI1;ROI2"

Menu "Biolum"
	"Start Biolum", OpenBiolumFile(); DisplayBiolum()		//	(re)open raw image file; displays the main window
	"Compress and save", SaveBiolum()
	"Help", DisplayHelpTopic "Biolum overview"
End

Function OpenBiolumFile()
	SetDataFolder root:
	string sNomFich
	SVAR/Z S_fileNameBiolum, S_pathBiolum
	if ( (SVAR_Exists(S_pathBiolum)) && (SVAR_Exists(S_fileNameBiolum)) )	//	the file has already been opened once, and its path and name are known
		sNomFich=S_pathBiolum+S_fileNameBiolum
		ImageLoad/T=tiff/S=0/Q/C=-1/O/N=P_Raw sNomFich
	else
		ImageLoad/T=tiff/S=0/Q/C=-1/O/N=P_Raw
		String/G S_pathBiolum=S_path, S_fileNameBiolum=S_fileName		//	save the path and file name
	endif
	Variable/G V_IMin, V_IMax
	InitAnalysis()
End

Function InitAnalysis()
	variable i
	string sNomROI_X, sNomROI_Y
	NewDatafolder/O/S root:Analysis	//	creates new datafolder if it doesn't already exists, and sets the current datafolder there
	For (i=0; (i<ItemsInList(ksROINames)); i+=1)
		sNomROI_X=StringFromList(i, ksROINames)+"_X"
		sNomROI_Y=StringFromList(i, ksROINames)+"_Y"
		Wave/Z wROI_X=$sNomROI_X, wROI_Y=$sNomROI_Y
		if (!WaveExists(wROI_X))
			Make/N=2/O $sNomROI_X, $sNomROI_Y
			Wave wROI_X=$sNomROI_X, wROI_Y=$sNomROI_Y
			wROI_X=(p+1)*10
			wROI_Y=(p+1)*15
		endif
	EndFor
	SetDataFolder root:
End

Function DisplayBiolum()
	Wave P_Raw
	ImageTransform averageImage P_Raw		//	to generate M_AveImage, which is the average image

	KillWindow/Z P_Biolum
	Execute "P_Biolum()"
	//	Initialize the display with correct values
	Slider C_TimeSlider limits={0,(DimSize(P_Raw, 2)),0}
End

Window P_Biolum() : Graph
	PauseUpdate; Silent 1		// building window...
	String fldrSav0= GetDataFolder(1)
	SetDataFolder root:Analysis:
	Display /W=(460,45,1233,1047)/K=1 /T ROI1_Y vs ROI1_X as "Biolum data"
	AppendImage ::M_AveImage
	ModifyImage M_AveImage ctab= {*,*,Grays,0}
	AppendImage ::M_StdvImage
	ModifyImage M_StdvImage ctab= {*,*,Grays,0}
	AppendImage/T ::P_Raw
	ModifyImage P_Raw ctab= {*,*,Grays,0}
	ModifyImage P_Raw plane= 1622
	SetDataFolder fldrSav0
	ModifyGraph margin(left)=14,margin(bottom)=14,margin(top)=14,margin(right)=14
	ModifyGraph mirror(left)=2,mirror(top)=0,mirror(bottom)=0
	ModifyGraph nticks(left)=4,nticks(top)=4
	ModifyGraph minor(left)=1,minor(top)=1
	ModifyGraph fSize(left)=9,fSize(top)=9
	ModifyGraph standoff(left)=0,standoff(top)=0
	ModifyGraph tkLblRot(left)=90
	ModifyGraph btLen(left)=3,btLen(top)=3
	ModifyGraph tlOffset(left)=-2,tlOffset(top)=-2
	SetAxis/A/R left
	Cursor/P/I A P_Raw 112,29
	ShowInfo
	ControlBar 80
	PopupMenu C_ImageSelector,pos={1.00,6.00},size={119.00,20.00},proc=PopMenuBiolum
	PopupMenu C_ImageSelector,title="Select image"
	PopupMenu C_ImageSelector,mode=1,popvalue="Raw",value=#"\"Raw;Average;StdDev\""
	Slider C_TimeSlider,pos={13.00,31.00},size={382.00,44.00},proc=SliderBiolum
	Slider C_TimeSlider,help={"Select what frame to display"}
	Slider C_TimeSlider,limits={0,100,1},value=0,vert=0
	SetVariable C_IMin,pos={144.00,9.00},size={80.00,14.00},proc=SetVarBiolum
	SetVariable C_IMin,title="I Min",help={"sets the level of black"}
	SetVariable C_IMin,limits={0,inf,100},value=V_IMin
	SetVariable C_IMax,pos={230.00,9.00},size={80.00,14.00},proc=SetVarBiolum
	SetVariable C_IMax,title="I Max",help={"sets the level of white"}
	SetVariable C_IMax,limits={0,inf,100},value=V_IMax
	Button C_Autoscale,pos={317.00,6.00},size={70.00,20.00},proc=ButtonBiolum
	Button C_Autoscale,title="Autoscale",fSize=11
	PopupMenu C_ROIselector,pos={442.00,20.00},size={126.00,20.00},proc=PopMenuBiolum
	PopupMenu C_ROIselector,title="ROI to display"
	PopupMenu C_ROIselector,mode=5,popvalue="ROI1",value=#"ksROINames"
	Button C_Edit,pos={426.00,46.00},size={40.00,20.00},proc=ButtonBiolum
	Button C_Edit,title="Edit",help={"Edit the selected ROI"},fSize=11
	Button C_Validate,pos={478.00,46.00},size={50.00,20.00},proc=ButtonBiolum
	Button C_Validate,title="Validate",help={"Edit the selected ROI"},fSize=11
	Button C_Clear,pos={538.00,45.00},size={50.00,20.00},proc=ButtonBiolum
	Button C_Clear,title="Clear",help={"Clears the selected ROI"},fSize=11
	Button C_Compute,pos={657.00,26.00},size={80.00,20.00},proc=ButtonBiolum
	Button C_Compute,title="Update",help={"Edit the selected ROI"},fSize=11
	Button C_Compute,fColor=(52428,1,1)
	GroupBox C_ROI,pos={414.00,2.00},size={194.00,72.00},title="Regions of Interest"
EndMacro

Function SliderBiolum(ctrlName,sliderValue,event) : SliderControl
	String ctrlName
	Variable sliderValue
	Variable event	// bit field: bit 0: value set, 1: mouse down, 2: mouse up, 3: mouse moved

	if(event & 0x1)	// bit 0, value set
		ModifyImage P_Raw plane=sliderValue
	endif

	return 0
End

Function SetVarBiolum(ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName
	Variable varNum
	String varStr
	String varName
	NVAR V_IMin, V_IMax
	ModifyImage ''#2 ctab= {V_IMin,V_IMax,Grays,0}
End

//	This function is called by any button on the main window. 
//	Depending on what button is pressed (indicated in ctrlName), a different action is performed.
Function ButtonBiolum(ctrlName) : ButtonControl
	String ctrlName
	SetDataFolder root:Analysis
	NVAR vIMin=root:V_IMin, vIMax=root:V_IMax
	variable i, vCounter
	String sNomROI_X, sNomROI_Y, sNomMasque, sNomTrace, sNomTraceCorr

	//	in case the button is somehow related to ROIs, it is necessary to determine the name of ROI objects based on the popup selection
	StrSwitch(ctrlName)	//	in case the function was called by any of these buttons...
		case "C_Edit":
		case "C_Validate":
		case "C_Clear":
			ControlInfo C_ROIselector	//	gets the content of the popup and returns it in S_Value
			sNomROI_X=S_Value+"_X"
			sNomROI_Y=S_Value+"_Y"
			sNomMasque=S_Value+"_Mask"
			sNomTrace=S_Value+"_Trace"
			sNomTraceCorr=S_Value+"_TraceCorr"
			Wave wX=$sNomROI_X, wY=$sNomROI_Y
		break
	EndSwitch
	
	StrSwitch(ctrlName)
		case "C_Autoscale":
			ControlInfo C_ImageSelector
			Switch(V_Value)
				case 1:	//	raw
					Wave pTest=root:P_Raw
					ControlInfo C_TimeSlider		//	to get the image # that we are currently looking at
					ImageStats/P=(V_Value)/M=1 pTest
				break
				case 2:	//	average
					Wave pTest=root:M_AveImage
					ImageStats/M=1 pTest
				break
				case 3:	//	stddev
					Wave pTest=root:M_StdvImage
					ImageStats/M=1 pTest
				break
			EndSwitch
			vIMin=V_min; vIMax=V_max
			ModifyImage ''#2 ctab= {vIMin,vIMax,Grays,0}	//	applies the value to the top image
		break
		case "C_Edit":
			GraphWaveEdit $sNomROI_Y
		break
		case "C_Validate":
			GraphNormal
			SetDataFolder root:Analysis
			SetDrawLayer/K ProgFront		//	selects and clears this layer
			SetDrawEnv xcoord=top, ycoord=left
			DrawPoly/ABS 0,0,1,1, wX, wY
			ImageGenerateROIMask/E=1/I=0 P_Raw	// le masque créé se retrouve dans M_ROIMask 
			Wave M_ROIMask
			Duplicate/O M_ROIMask, $sNomMasque
			SetDrawLayer/K ProgFront		//	selects and clears this layer
			KillWaves/Z M_ROIMask
		break
		case "C_Clear":
			KillWaves/Z $sNomMasque
			Wave/Z wTrace=$sNomTrace, wTraceCorr=$sNomTraceCorr
			if (WaveExists(wTrace))
				wTrace=NaN
			endif
			if (WaveExists(wTraceCorr))
				wTraceCorr=NaN
			endif
			
			EnleveToutesLesTracesSauf("P_Biolum", "")	//	Remove all plots from the window			
		break
		//	****************************************
		//	This is where the analysis is done
		case "C_Compute":
			Wave P_Raw=root:P_Raw

			//	Measures the trace for each ROI, if defined
			for (i=0; (i<ItemsInList(ksROINames)); i+=1)
				sNomMasque=StringFromList(i, ksROINames)+"_Mask"	//	name of the mask
				sNomTrace=StringFromList(i, ksROINames)+"_Trace"	//	name of the trace
				sNomTraceCorr=StringFromList(V_Value-1, ksROINames)+"_TraceCorr"	//	final corrected trace
				Wave/Z wMasque=$sNomMasque
				if (WaveExists(wMasque))		//	if this mask exists, perform the calculation
					ImageStats/BEAM/M=1/R=wMasque P_Raw
					Wave W_ISBeamAvg
					Duplicate/O W_ISBeamAvg, $sNomTrace
				endif
			endfor
			KillWaves/Z W_ISBeamAvg, W_ISBeamMax, W_ISBeamMin
			
			//	Calculate the average background trace
			Make/O/N=(DimSize(P_Raw,2)) BckAvg_Trace
			Fastop BckAvg_Trace=0; vCounter=0
			For (i=0; (i<ItemsInList(ksROINames)); i+=1)
				sNomTrace=StringFromList(i, ksROINames)+"_Trace"	//	name of the trace
				if (CmpStr(sNomTrace[0,2], "Bck")==0)	//	it is a background trace
					Wave/Z wBck=$sNomTrace
					if (WaveExists(wBck))
						BckAvg_Trace+=wBck
						vCounter+=1
					endif
				endif
			EndFor
			BckAvg_Trace/=vCounter	//	divide by the number of valid background zones to get the average background
			
			//	Make the background and consumption correction for each ROI
			For (i=0; (i<ItemsInList(ksROINames)); i+=1)
				sNomTrace=StringFromList(i, ksROINames)+"_Trace"	//	name of the trace
				sNomTraceCorr=StringFromList(i, ksROINames)+"_TraceCorr"	//	name of the corrected trace
				if (CmpStr(sNomTrace[0,2], "ROI")==0)	//	it is a background trace
					Wave/Z wROI=$sNomTrace
					if (WaveExists(wROI))
						wROI=wROI-BckAvg_Trace	//	Subtract the background trace
						Calcium(wROI)
						Wave W_Calcium
						Duplicate/O W_Calcium, $sNomTraceCorr
					endif
				endif
			Endfor
			KillWaves/Z W_Calcium, W_RemainingCounts
		break
	EndSwitch
	SetDataFolder root:
End

Function SaveBiolum()
	SetDataFolder root:
	KillWindow/Z P_Biolum
	KillWaves/Z P_Raw
	SaveExperiment
End

Function PopMenuBiolum(ctrlName,popNum,popStr) : PopupMenuControl
	String ctrlName
	Variable popNum
	String popStr
	
	NVAR vIMin=root:V_IMin, vIMax=root:V_IMax
	
	StrSwitch(ctrlName)
	case "C_ROIselector":		
		EnleveToutesLesTracesSauf("P_Biolum", "")	//	Remove all plots from the window
		
		//	Define waves for this selected bck and ROIs
		SetDataFolder root:Analysis
		string sNomROI_X=StringFromList(popNum-1, ksROINames)+"_X"
		string sNomROI_Y=StringFromList(popNum-1, ksROINames)+"_Y"
		Wave wROI_X=$sNomROI_X, wROI_Y=$sNomROI_Y
		AppendToGraph/T wROI_Y vs wROI_X
		setdatafolder root:
	break
	case "C_ImageSelector":
		Switch(popNum)
			case 1:
				ReorderImages P_Raw,{M_AveImage,M_StdvImage}
			break
			case 2:
				ReorderImages M_AveImage,{P_Raw,M_StdvImage}
			break
			case 3:
				ReorderImages M_StdvImage,{P_Raw,M_AveImage}
			break
		EndSwitch
		ModifyImage ''#2 ctab= {vIMin,vIMax,Grays,0}	//	applies the value to the top image
	break
	EndSwitch
End

////////////////////	Correct for substrate consumption		//////////////////////////
Function Calcium(wCorr)
	Wave wCorr
	Duplicate/O wCorr, W_Calcium, W_RemainingCounts

	variable i
	For (i=0; (i<DimSize(wCorr, 0)); i+=1)
		Wavestats/M=1/Q/R=[i,] wCorr	//	 returns the remaining counts in V_sum
		//	calcium = (biolum at this point) / (all remaining counts)
		W_Calcium[i] = wCorr[i] / V_sum
		W_RemainingCounts[i]=V_sum
	EndFor
end
