#pragma TextEncoding = "UTF-8"
#pragma rtGlobals=3		// Use modern global access method.
#pragma protect=0
#pragma DefaultTab={3,20,4}		// Set default tab width in Igor Pro 9 and later

// *** Copyright ***
// © 2023 CNRS Shared by Pierre VINCENT, under the MIT License.
//	Contact: pierre.vincent@cnrs.fr; IGF, 141 rue de la Cardonille, 34094 Montpellier cedex 5, France.

StrConstant ksTypesAcquisition="---;iVision 2018;NAF split;Poste biolum;tif;tif stack;Biphoton;Optosplit;iVision;iVision 2;"
StrConstant ksNomDataFolders="Analyse;Export;Empty;Ratio;Empty;Pulse;Kymograph"
	
// *** Preferences definition ***
//	The string defines a series of object that are present in dfRatio and should be saved in the preference file
//	Among these, W_WindowPosition stores the XY coordinates of the upper-left corner of windows, in the following order:
//	0: P_Initialisation; 1: P_Controle; 2: F_RM; 3: F_Traces; 4: F_TracesTriees
//	ksPrefs is the list of objects that should be saved in the preference file
//Static StrConstant ksGlobalPrefs="S_Chemin_Calcul;S_Chemin_Analyses;S_Chemin_JPEG;S_Racine_Locale;W_WindowPositions;"
Static StrConstant ksGlobalPrefs="S_Chemin_Calcul;W_WindowPositions;"
Static StrConstant ksDataPrefs="V_ChannelsMerged;V_Split;V_TimepointsMerged;V_NbFichiers;V_NbVoies;V_NbLettresRacine;V_NbLettresInutiles;V_NbChiffresFichier;S_Type_Image;"	//	loaded only if it is a new file; not used right now

//	W_DFREF contains pointers to datafolders. This is only valid at run-time. 
//	When opening an analysis file, the content of this wave can point to arbitrary datafolders.
//	Hence the WS_DFREF, a wave of strings, which contains the name of the datafolders.
//	This function checks that necessary datafolders exist, and fixes the pointers in W_DFREF so that they point to the right datafolder.
//	Called when the analysis is initialized, and when it is just needed to make an export or cursor measurement.
//	When passed "Initialise" as argument, it fixes all datafolder refs, then goes to "Sauve"
//	"Sauve" converts W_DFREF into permanent strings stored in WS_DFREF 
//	W_DFREF[0]: dfROI
//	W_DFREF[1]: dfExport
//	W_DFREF[2]:	
//	W_DFREF[3]:	dfRatio
//	W_DFREF[4]: 
//	W_DFREF[5]: dfPulse
//	W_DFREF[6]: dfKymo
//	Gere_DFREF("Initialise")	to trigger the initialization and update of DFREFs
Function Gere_DFREF(sChoix)
	String sChoix
	Variable vChoix, i
	String sNomDF, sBaseNom, sListe

	SetDataFolder root:
//	If missing, creates W_DFREF and WS_DFREF
	Wave/Z/DF W_DFREF
	if (!WaveExists(W_DFREF))
		Make/DF/N=10 W_DFREF
	endif
	Wave/Z/T WS_DFREF
	if (!WaveExists(WS_DFREF))
		Make/T/N=10 WS_DFREF
	endif	

	StrSwitch(sChoix)
		case "Initialise":
			//	dfRatio: must exist and be unique.
			sNomDF="root:Ratio"
			if (DataFolderExists(sNomDF))
				InitDataFolder("Init raplouf.pxp", "f_Ratio", "Ratio")			//	Initialize ratio objects
			else	//	Ancient versions were storing everything at the root, without datafolders
				NewDataFolder $sNomDF
				//	Transfers these objects from root to dfRatio, and creates the missing ones
				InitDataFolder("Init raplouf.pxp", "f_Ratio", "Ratio", sNomDataFolderAncien="root:")
			endif
			DFREF dfRatio = $sNomDF
			W_DFREF[3]=dfRatio

			//	at least one Analyse_* datafolder must exist, but there can be several to hold different analysis of the same data.
			//	First, test if there is a unique datafolder Analyse_*, in which case it is the good one.
			//	If there is none, must be created
			//	If there are several, ask the user
			DFREF dfROI
			sBaseNom=StringFromList(0, ksNomDataFolders)
			sListe=ListeDF("root:", sBaseNom)		//	Returns the list of datafolders starting with "Analyse"
			if (ItemsInList(sListe)==0)		//	No df with correct name
				//	create it
				sNomDF="root:Analyse_0"
				NewDataFolder $sNomDF
			endif
			sListe=ListeDF("root:", sBaseNom)	//	update the list, now that a new df was possibly created
			for (i=0; (i<ItemsInList(sListe)); i+=1)
				sNomDF=StringFromList(i, sListe)
				//	make sure that all required object are present in dfROI
				InitDataFolder("Init raplouf.pxp", "f_Analyse", sNomDF)	//	initializes content of this df
			endfor
			if (ItemsInList(sListe)>1)	//	There are more than 1 analysis datafolder => ask user
				prompt vChoix, "Select analysis datafolder", popup, sListe
				DoPrompt/HELP="Select what analysis datafolder should be selected" "Raplouf needs to know:", vChoix
				sNomDF=StringFromList(vChoix-1, sListe)
			endif
			dfROI = $sNomDF
			W_DFREF[0]=dfROI
			
		//	No break here is deliberate; after initialization, saving W_DFREF into WS_DFREF
		case "Sauve":
			DFREF dfTest
			For (i=0; (i<DimSize(W_DFREF, 0)); i+=1)
				dfTest=W_DFREF[i]
				WS_DFREF[i]=GetDataFolder(0, dfTest)
			endfor
		break
	EndSwitch
End

//__________________________________________________________________________________
// *** Main initialization function ***
//	This function is started only through the Raplouf menu. Checks datafolders, objects. Creates windows. Creates dependencies.
Function InitialiseRaplouf()

//	Attempted to put a progress bar on top of all the windows being created but...
//	If this progress bar is modal and at top level, all automatic macro commands will be sent to the wrong (top) window
//	NewPanel/K=1/N=F_Attendre /W=(292,292,684,373)
//	DoWindow/T F_Attendre, "Wait for initialisation"
//	ValDisplay valdisp0,pos={18,32},size={342,18},limits={0,100,0},barmisc={0,0}
//	ValDisplay valdisp0,value= _NUM:0
//	ValDisplay valdisp0,mode= 4	// candy stripe

	variable i, vVoie, vCurrentVersion, vOnContinue
	String sNom, sListe
	SetDataFolder root:
	
	//	checks whether the code is usable or outdated
	Test_Utilisation()

//	Handles Ratioscope version number
	NVAR/Z vVersionRatio=root:Ratio:V_VersionRatio
	vCurrentVersion=VersionNumber("Ratioscope package globals.ipf")
	if (NVAR_Exists(vVersionRatio))
		//	Warns the user in case the loaded code is older than the code used to generate the experiment file
		if ((NumType(vVersionRatio)==0) && (vVersionRatio>vCurrentVersion))
			DoAlert/T="Ratioscope version" 1, "This analysis file was processed with a more recent version of Ratioscope; proceed with analysis anyway?"
			if (V_Flag==2)	//	No clicked -> close the file without saving and start a new experiment
				Execute/P/Q/Z "NEWEXPERIMENT "
				return(0)
			endif
		endif
		vVersionRatio=vCurrentVersion
	endif
		
	//	Defines a default GUI that can be compatible with Mac & PC
	DefaultGUIFont all={"Arial", 9, 0}	//	police par défaut, de même taille sur mac et PC
	//	DefaultGUIControls [/W=winName /Mac/Win] [ appearance ] native	//	voir comment gérer la taille des panneaux dans les écrans de taille différente
	
	//	Check if image window already exist; if yes -> warning and stop
	DoWindow F_RM
	i=V_flag
	DoWindow P_Controle
	if ((V_flag) || i)
		DoAlert 0, "The main image window already exists: initialization is probably not required, or something went wrong when saving."
		Return(0)
	endif
	
	//****************	Kill deprecated object related to concentration axis	***************
	//	There used to be a concentration axis at the right of the graph in "F_TracesTriees" but the concept is stupid in the absence
	//	of Rmin and Rmax normalization. Included in Fusion in ?2018?, then removed from Raplouf in 2022.
	//	Need to remove the hidden traces TraceR000#4 to TraceR000#9 from the graph to remove these axes and links to concentration datafolder.
	DoWindow F_TracesTriees
	if (V_Flag)
		for (i=7; (i>=4); i-=1)
			sNom="TraceR0000#"+num2Str(i)
			RemoveFromGraph/W=F_TracesTriees/Z $sNom
		endfor
	endif
	KillDataFolder/Z Concentration
	//****************	Done cleaning	***************
	
	//	Check that W_DFREF contains pointers to the correct datafolders
	//	Initializes the required datafolders: Files, Ratio and at least one Analyse_*
	//	Performs the initialization of Ratio datafolders and all Analyse_* datafolders so that they all contain up-to-date objects
	Gere_DFREF("Initialise")

	//	Now that W_DFREF is verified, df are defined and contain required objects
	Wave/DF W_DFREF=root:W_DFREF
	DFREF dfROI = W_DFREF[0]
	DFREF dfRatio=W_DFREF[3]

	SetDataFolder dfRatio

//	Redimension and comment a few waves
	Wave W_PointsDeVue, W_Temps_min, W_Temps, W_Decalage, W_Exposition
	Redimension/N=(-1, 3) W_PointsDeVue
	Redimension/N=(DimSize(W_Temps,0)), W_Temps_min
// *** \\\ c'est là qu'il faut redimensionner W_Deplacement pour être à 3 dimensions ***
	Redimension/N=(-1, 22) W_Decalage	//	Because optional skewing when registering images, registration wave must hold 22 values.
	setdimlabel 1, 0, A, W_Exposition
	setdimlabel 1, 1, B, W_Exposition

// *** Loading Preferences ***
	string sNomPath=SpecialDirPath("Packages",0,0,0)+"Raplouf"	//	file path to prefs
	NewPath/O/C/Q prefs, sNomPath
	//	reads prefs in dfRatio
	SetDataFolder dfRatio
	LoadData/O/Q/J=ksGlobalPrefs/P=prefs "Raplouf_Prefs.pxp"

	//	Creates the path from what was saved in the pref file: calcul, fichier_analyse and fichier_JPEG
	PopMenuInitChemins("Initialise",0,"")
	
	//	Update S_Type_Image for version 9
	SVAR/SDFR=dfRatio/Z S_Type_Image
	if (WhichListItem(S_Type_Image, ksTypesAcquisition)<0)
		StrSwitch(S_Type_Image)
			case "Générique":
			case "":
				S_Type_Image="---"
			break
			default:
				DoAlert 0, "Check the type of acquisition"
			break
		EndSwitch
	endif
	
	//	Update V_Taille_X0 V_Taille_Y0 that may not be defined in analysis before version 9.31
	NVAR V_Taille_X, V_Taille_Y, V_Taille_X0, V_Taille_Y0, V_Miroir
// *** \\\ if rotation is 90°, warn user ***
	if ((V_Miroir & 4) != 0)
		DoAlert 0, "Rotation of the image: check image size; if necessary, reload first image"
	else
		V_Taille_X0=V_Taille_X; V_Taille_Y0=V_Taille_Y
	endif

	setdatafolder root:
	//	Old raplouf files have all objects, including ROI definitions, traces and objects stored at the root
	//	If they exist, must be moved to the default ROI datafolder
	i=1
	Do
		sNom=AjouteNumero("ROIx", i, 4)
		Wave/Z wTemp=$sNom
		if (WaveExists(wTemp))
			MoveWave wTemp, dfROI
			vOnContinue=1
		else
			vOnContinue=0
		endif
		sNom=AjouteNumero("ROIy", i, 4)
		Wave/Z wTemp=$sNom
		if (WaveExists(wTemp))
			MoveWave wTemp, dfROI
		endif
		sNom=AjouteNumero("ROIy", i, 4)
		Wave/Z wTemp=$sNom
		if (WaveExists(wTemp))
			MoveWave wTemp, dfROI
		endif
		sNom=AjouteNumero("TraceA", i, 4)
		Wave/Z wTemp=$sNom
		if (WaveExists(wTemp))
			MoveWave wTemp, dfROI
		endif
		sNom=AjouteNumero("TraceB", i, 4)
		Wave/Z wTemp=$sNom
		if (WaveExists(wTemp))
			MoveWave wTemp, dfROI
		endif
		sNom=AjouteNumero("TraceR", i, 4)
		Wave/Z wTemp=$sNom
		if (WaveExists(wTemp))
			MoveWave wTemp, dfROI
		endif
		i+=1
	while (vOnContinue)
//	Old file names that may still be at the root; replaced by new name at the right location
	SVAR/Z S_Base_Nom_Fichier
	if (SVAR_Exists(S_Base_Nom_Fichier))
		SVAR/SDFR=dfRatio S_Base_Nom_Fichier_A
		S_Base_Nom_Fichier_A=S_Base_Nom_Fichier
		KillStrings S_Base_Nom_Fichier
	endif
//	Same in more recent dfRatio
	SVAR/Z/SDFR=dfRatio S_Base_Nom_Fichier
	if (SVAR_Exists(S_Base_Nom_Fichier))
		SVAR/SDFR=dfRatio S_Base_Nom_Fichier_A
		S_Base_Nom_Fichier_A=S_Base_Nom_Fichier
		KillStrings S_Base_Nom_Fichier
	endif
	SVAR/Z/SDFR=dfRatio S_Base_Nom_Fichier2=root:S_Base_Nom_Fichier2
	if (SVAR_Exists(S_Base_Nom_Fichier2))
		SVAR/SDFR=dfRatio S_Base_Nom_Fichier_B
		S_Base_Nom_Fichier_B=S_Base_Nom_Fichier2
		KillStrings S_Base_Nom_Fichier2
	endif

//	Check if required trace windows exist; if missing, create them
	DoWindow/HIDE=0 F_Traces
	if (V_Flag==0)
		Execute "F_Traces()"
	endif
	DoWindow/HIDE=0 F_TracesTriees
	if (V_Flag==0)
		Execute "F_TracesTriees()"
	endif
	Init_F_TracesTriees()
	SetVarEspaceEvt("",0,"","")

//	Main window for image display
	Execute "F_RM()"
	DoWindow/T F_RM, "Image display"
	
	NVAR/SDFR=dfRatio V_Taille_X, V_Taille_Y, V_Zoom
	ModifyGraph/W=F_RM height={Aspect,(V_Taille_Y/V_Taille_X)}

//	Create main control panels	
	Init_P_Controle()	

//	Activate the analysis folder : movement for ROIs, dependencies, masks...
	PopMenuSelectAnalyse("",0,"Initialise")
	
//	Create panel for file, path... and image settings
//	must be called after P_Controle is created
	Init_P_Initialisation()

//	Position windows according to prefs
	Wave/SDFR=dfRatio W_WindowPositions
	MoveWindow/W=P_Initialisation W_WindowPositions[0][0], W_WindowPositions[0][1], -1, -1
	MoveWindow/W=P_Controle W_WindowPositions[1][0], W_WindowPositions[1][1], -1, -1
	MoveWindow/W=F_RM W_WindowPositions[2][0], W_WindowPositions[2][1], -1, -1
	MoveWindow/W=F_Traces W_WindowPositions[3][0], W_WindowPositions[3][1], -1, -1
	MoveWindow/W=F_TracesTriees W_WindowPositions[4][0], W_WindowPositions[4][1], -1, -1

	SetVarNbVoies("",0,"","")
	MaJGlissieres()	//	sets the size of the time slider
	
	//	Checks all checkboxes for ratio calculation
	MaJ_CasesACocher("Tous", 0, "", "")	
	
	//	Dependencies
	SetDataFolder dfRatio
	Wave/SDFR=dfRatio W_EvtTempsCorr, W_EvtTempsCorr2
	Redimension/N=(DimSize(W_EvtTempsCorr, 0)) W_EvtTempsCorr2	//	to make sure that W_EvtTempsCorr2 matches the size of W_EvtTempsCorr
	SetFormula W_EvtTempsCorr2, "W_EvtTempsCorr-V_T0/60"
	SetFormula W_Temps_min, "(W_Temps-V_T0)/60"
	SetFormula V_DimCarrePix, "round(V_DimCarre/V_Resolution)"
	SetFormula W_PosX, "W_Temps_min[V_NumCoucheCourante]"	//	plotted as the vertical lane on graphs as time cursor

	//	Zoom of the main image window
	DoUpdate
	SetVarZoom("",NaN,"","")

	NVAR/SDFR=dfRatio Dim
	Dim=FDim()
	
	HideInfo/W=F_TracesTriees
	DoWindow/F P_Initialisation
	
	//	Just for fun
	NVAR/SDFR=dfRatio V_EEgg
	V_EEgg=0//trunc(abs(enoise(2)))
	
	SetDataFolder root:
end	//	End of Intialise()

//	Fixes image size
Function RedimensionImages(ctrlName) : ButtonControl
	String ctrlName

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfRatio=W_DFREF[3]
	SetDataFolder dfRatio
	NVAR V_Taille_X, V_Taille_Y, V_Taille_X0, V_Taille_Y0, V_NbPlans, V_NbVoies, V_MoyUnifA, V_MoyUnifB

	variable numVoie
	string sNomPile, sNomPileCorr

	Wave D_UnifA, D_UnifB, P_A, P_B, P_ACorr, P_BCorr, P_Ratio, P_IMoy, P_RatioxIMoy, P_RM, D_HSLTemp

	for (numVoie=1; (numVoie<=V_NbVoies); numVoie+=1)
		sNomPile="P_"+StringFromList(numVoie-1, ksABC)
		sNomPileCorr="P_"+StringFromList(numVoie-1, ksABC)+"Corr"
		Wave pPile=$sNomPile
		Redimension/S/N=(V_Taille_X,V_Taille_Y,V_NbPlans) $sNomPileCorr, $sNomPile
	endfor

	Redimension/S/N=(V_Taille_X,V_Taille_Y,V_NbPlans) P_Ratio, P_IMoy, P_RatioxIMoy
	Redimension/W/U/N=(V_Taille_X, V_Taille_Y,3,V_NbPlans) P_RM
	Redimension/W/U/N=(V_Taille_X, V_Taille_Y, 3) D_HSLTemp
	Redimension/N=(V_Taille_X0,V_Taille_Y0) D_UnifA, D_UnifB
	
	D_HSLTemp[][][1]=65535		// saturation value is max
	
	SetVarZoom("",1,"","")
End

//________________________________________________________________________________________________________________
//****************************************************************************************************************

Function BoutonSupprimeTemp(ctrlName) : ButtonControl
	String ctrlName
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfRatio=W_DFREF[3]
	NVAR/SDFR=dfRatio V_NumCoucheCourante
	variable j
	string bla, nomfich
	StrSwitch(ctrlName)
		case "C_SupprimeFichierTemp":
			for (j=1; (j<=4); j+=1)
				GereImages(V_NumCoucheCourante, 3, j)
			endfor
		break
		case "C_VideCheminCalcul":
			DoAlert 1, "Do you really want to clear the cache containing temporary calculation files ? (no effect on raw data file)"
			if (V_Flag!=1)
				return(0)
			endif
		case "C_ClearCache_NoConfirmation":
			PathInfo calcul
			if (V_Flag)
				bla=IndexedFile(calcul, -1, ".tmp")
				for (j=0; (j<ItemsInList(bla));j+=1)
					nomfich=StringFromList(j, bla)
					if (StringMatch(nomfich, "im*"))
						DeleteFile/P=calcul nomfich
					endif
				endfor
			endif
		break
	EndSwitch
	//	Check all checkboxes
	MaJ_CasesACocher("Tous", 0, "", "")	
End

Function GereImages(i, t, u)
	variable i,t, u
//i: position courante; c'est en général V_NumCoucheCourante
//t=1: lit; t=2: sauve; t=3: efface
//u=1: P_A (P_B; P_C etc...); u=2: P_ACorr (et autant de voies qu'il en existe); 
//u=3: P_RM; u=4: P_Ratio, P_RatioxIMoy et P_IMoy; u=5: P_Tmp; u=6: P_AProj et P_BProj (Reduit_3D_vers_2D)
//	Résultat: 1 si OK

	DFREF dfSav=GetDataFolderDFR()	//	Save current datafolder

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfRatio=W_DFREF[3]
	Wave/SDFR=dfRatio W_NumTrame
	NVar/SDFR=dfRatio V_NbChiffres, V_NbVoies
	SVAR/SDFR=dfRatio S_Chemin_Calcul, S_Racine_Locale, S_Nom_Manip

	if (i>=DimSize(W_NumTrame, 0))
		return(0)
	endif
	variable j=W_NumTrame[i]	//	tout est indexé sur le n° de fichier original; évite les erreurs si on supprime des images
	Variable resultat	//	retourne 1 si OK, 0 sinon
	Variable vVoie	//	index pour le numéro de voie
	String sIndex, sNomWave, sNomFich
	
	//	On veut que les images lues soient mises dans dfRatio
	SetDataFolder dfRatio
	
	if (NumType(j)==2)		// si le n° de fichier n'est pas défini, on arrête là.
		return(0)
	endif
	sprintf sIndex,"%.*dX%.*f",V_NbChiffres,j,V_NbChiffres,(j-trunc(j))	//	j-trunc(j) est là pour les fichiers cellvizio
	
	Switch(u)
		case 1:	//	P_A, P_B etc...
			For (vVoie=1; (vVoie<=V_NbVoies); vVoie+=1)	
				sNomWave="P_"+StringFromList(vVoie-1, ksABC)
				sNomFich=S_Chemin_Calcul+"im"+StringFromList(vVoie-1, ksABC)+sIndex+".tmp"
				resultat=(LitSauveEfface(sNomFich, sNomWave, t))
				if (resultat==0)
					return(0)
				endif
			endfor
		break
		case 2:	//	P_ACorr, et autant de voies qu'il en existe
			For (vVoie=1; (vVoie<=V_NbVoies); vVoie+=1)	
				sNomWave="P_"+StringFromList(vVoie-1, ksABC)+"Corr"
				sNomFich=S_Chemin_Calcul+"im"+StringFromList(vVoie-1, ksABC)+"Corr"+sIndex+".tmp"
				resultat=(LitSauveEfface(sNomFich, sNomWave, t))
				if (resultat==0)
					return(0)
				endif
			endfor
		break
		case 3:	//	P_RM
			sNomWave="P_RM"
			sNomFich=S_Chemin_Calcul+"imRM"+sIndex+".tmp"
			resultat=(LitSauveEfface(sNomFich, sNomWave, t))
			if (resultat==0)
				return(0)
			endif
		break
		case 4:	//	P_Ratio, P_RatioxIMoy et P_IMoy
			sNomWave="P_Ratio"
			sNomFich=S_Chemin_Calcul+"imRatio"+sIndex+".tmp"
			resultat=(LitSauveEfface(sNomFich, sNomWave, t))
			if (resultat==0)
				return(0)
			endif
			sNomWave="P_RatioxIMoy"
			sNomFich=S_Chemin_Calcul+"imRatioxIMoy"+sIndex+".tmp"
			resultat=(LitSauveEfface(sNomFich, sNomWave, t))
			if (resultat==0)
				return(0)
			endif
			sNomWave="P_IMoy"
			sNomFich=S_Chemin_Calcul+"imIMoy"+sIndex+".tmp"
			resultat=(LitSauveEfface(sNomFich, sNomWave, t))
			if (resultat==0)
				return(0)
			endif
		break
		case 5:	//	images temporaires pour effectuer un calcul; P_TmpA et P_TmpB
			sNomWave="P_TmpA"
			sNomFich=S_Chemin_Calcul+"imTmpA"+sIndex+".tmp"
			resultat=(LitSauveEfface(sNomFich, sNomWave, t))
			sNomWave="P_TmpB"
			sNomFich=S_Chemin_Calcul+"imTmpB"+sIndex+".tmp"
			resultat=(LitSauveEfface(sNomFich, sNomWave, t))
			if (resultat==0)
				return(0)
			endif
		break
		case 6:	//	plan 0 de la projection vertiale
			For (vVoie=1; (vVoie<=V_NbVoies); vVoie+=1)	
				sNomWave="P_"+StringFromList(vVoie-1, ksABC)+"Proj"
				sNomFich=S_Racine_Locale+S_Nom_Manip+":im"+StringFromList(vVoie-1, ksABC)+"Proj"+sIndex+".tmp"
				resultat=(LitSauveEfface(sNomFich, sNomWave, t))
				if (resultat==0)
					print "Erreur en lisant / sauvant la projection"
					return(0)
				endif
			EndFor
		break
	EndSwitch
	SetDataFolder dfSav	//	Restore datafolder
	return(1)
end

Function LitSauveEfface(sNomFich, NomWave, t)
//t=1: lit; t=2: sauve; t=3: efface
	String sNomFich, NomWave
	Variable t 

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfRatio=W_DFREF[3]

	Switch(t)
		case 1:
			GetFileFolderInfo/Q/Z=1 sNomFich
			if (V_Flag==0)	// le fichier existe
				LoadWave/O/H/Q sNomFich
				return(V_Flag)
			else
				return(0)
			endif
		break
		case 2:
			Wave/SDFR=dfRatio w=$NomWave
			save/C/O w as sNomFich
			return(1)
		break
		case 3:
			DeleteFile/Z sNomFich
			return(1)
		break
	EndSwitch	
end

Function tsLitP_AB(i)
	Variable i

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfRatio=W_DFREF[3]
	NVAR/SDFR=dfRatio V_NbChiffres=root:V_NbChiffres
	SVAR/SDFR=dfRatio S_Chemin_Calcul=root:S_Chemin_Calcul

	String nomfichA, nomfichB, temp

	sprintf temp,"%.*dX%.*f",V_NbChiffres,i,V_NbChiffres,(i-trunc(i))
	nomfichA=S_Chemin_Calcul+"imA"+temp+".tmp"
	nomfichB=S_Chemin_Calcul+"imB"+temp+".tmp"
	LoadWave/O/H/Q nomfichA
	LoadWave/O/H/Q nomfichB
	Return(V_Flag)
End

Function tsSauveP_ABCorr(i)
	Variable i
	
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfRatio=W_DFREF[3]
	Wave/SDFR=dfRatio wA=P_ACorr, wB=P_BCorr
	NVAR/SDFR=dfRatio V_NbChiffres=root:V_NbChiffres
	SVAR/SDFR=dfRatio S_Chemin_Calcul=root:S_Chemin_Calcul

	String nomfichA, nomfichB, temp

	sprintf temp,"%.*dX%.*f",V_NbChiffres,i,V_NbChiffres,(i-trunc(i))
	nomfichA=S_Chemin_Calcul+"imACorr"+temp+".tmp"
	nomfichB=S_Chemin_Calcul+"imBCorr"+temp+".tmp"
	Save/C/O wA as NomFichA
	Save/C/O wB as NomFichB
	Return(1)
End

// *** Data storage scheme ***
//	Called by checkboxes in P_Initialisation
//	This is the main switch to analyze the different schemes of image storage
//	Determines the number of files per time-point...
Function CheckDataScheme(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfRatio=W_DFREF[3]
	//	Associated with the checkboxes defining the data scheme
	NVAR/SDFR=dfRatio V_ChannelsMerged		//	1 if all channels are merged into a single file
	NVAR/SDFR=dfRatio V_Split				//	1 if the single image contains both wavelength side by side (image splitter)
	NVAR/SDFR=dfRatio V_TimepointsMerged	//	1 if all time points are merged into a single file
	NVAR/SDFR=dfRatio V_NbFichiers			//	nb of files for user to enter; only controls the number of available lines to enter file definition
	NVAR/SDFR=dfRatio V_NbVoies				//	number of channels containing image data

	StrSwitch(ctrlName)
		case "C_Check_ChannelsMerged_0":	//	One file contains images from a single channel
			V_ChannelsMerged=0
		break
		case "C_Check_ChannelsMerged_1":	//	A single file contains all acquisition channels
			V_ChannelsMerged=1
		break
		case "C_Check_Split_0":	//	Within one file, each image plane corresponds to one acquisition channel
			V_Split=0
		break
		case "C_Check_Split_1":		//	A single image plane for both wavelength: generated by emission splitting devices
			V_Split=1
		break
		case "C_Check_TimepointsMerged_0":
			V_TimepointsMerged=0
		break
		case "C_Check_TimepointsMerged_1":
			V_TimepointsMerged=1
		break
	EndSwitch

	
	//	Based on the value of the variables, sets the other variables / controls' state
	DoWindow/F P_Initialisation
	if (V_ChannelsMerged)
		CheckBox C_Check_Split_0 title="Stacked channels",disable=0
		CheckBox C_Check_Split_1 disable=0
		V_NbFichiers=1
	else
		V_Split=0		//	if separate channels, optosplit is useless
		CheckBox C_Check_Split_0 title="1 channel, 1 file",disable=2
		CheckBox C_Check_Split_1 disable=2
		V_NbFichiers=V_NbVoies
	endif

	if 	(V_TimepointsMerged)
		ValDisplay C_NbFichiers title="Nb of files per experiment"
		SetVariable C_NbChiffresFichier, disable=1
		//	Disable the possibility to add more data points in the analysis on the main control panel
		Button C_BoutonAjoutePoints win=P_Controle,disable=2
		CheckBox C_AjoutePointsAuto win=P_Controle,disable=2
		//	Enable the button to load all images in time stacks
		Button C_BoutonChargePileTemps, disable=0
	else
		ValDisplay C_NbFichiers title="Nb of files per time point"
		SetVariable C_NbChiffresFichier, disable=0
		//	Enable the possibility to add more data points in the analysis on the main control panel
		Button C_BoutonAjoutePoints win=P_Controle,disable=0
		CheckBox C_AjoutePointsAuto win=P_Controle,disable=0
		//	Disable the button to load all images in time stacks
		Button C_BoutonChargePileTemps, disable=1
	endif
	
	SetVarNbFichiers("",V_NbFichiers,"","")		//	Makes available the good number of file name settings

	//	ensure that the radio buttons match the variables
	CheckBox C_Check_ChannelsMerged_0, value=!V_ChannelsMerged
	CheckBox C_Check_ChannelsMerged_1, value=V_ChannelsMerged
	CheckBox C_Check_Split_0, value=!V_Split
	CheckBox C_Check_Split_1, value=V_Split
	CheckBox C_Check_TimepointsMerged_0, value=!V_TimepointsMerged
	CheckBox C_Check_TimepointsMerged_1, value=V_TimepointsMerged
End

//______________________________________________________________________________
// *** Handles NAF files (Hamamatsu) ***

Function BoutonValideNAF(ctrlName) : ButtonControl
	String ctrlName
	KillWindow/Z P_NAFParams
	KillWindow/Z F_NAFParams
End


Function SliderNAF(sa) : SliderControl
	STRUCT WMSliderAction &sa

	switch( sa.eventCode )
		case -3: // Control received keyboard focus
		case -2: // Control lost keyboard focus
		case -1: // Control being killed
			break
		default:
			if( sa.eventCode & 1 ) // value set
				Variable curval = sa.curval
				StrSwitch(sa.ctrlName)
					case "C_TimePoint":
						ModifyImage/W=F_NAFParams P_Image0 plane=curval
					break
					case "C_Intensity_Low":
						ModifyImage/W=F_NAFParams P_Image0 ctab= {curval,,Grays,0}
					break
					case "C_Intensity_High":
						ModifyImage/W=F_NAFParams P_Image0 ctab= {,curval,Grays,0}
					break
				EndSwitch
			endif
			break
	endswitch

	return 0
End

Function SetVarNAF(sva) : SetVariableControl
	STRUCT WMSetVariableAction &sva
	NVAR V_NAF_SizeX, V_NAF_SizeY, V_NAF_dX, V_NAF_dY, V_NAF_TimePoints
	SVAR S_NomPremierFichier, S_Extension
	Wave P_Image0
	switch( sva.eventCode )
		case 1: // mouse up
		case 2: // Enter key
		case 3: // Live update
			Variable dval = sva.dval
			String sval = sva.sval
			ReadNAF("P_Image0", S_NomPremierFichier+S_Extension)
			Redimension/N=(V_NAF_SizeX, V_NAF_SizeY, V_NAF_TimePoints) P_Image0
			Slider C_TimePoint win=P_NAFParams,limits={0,V_NAF_TimePoints,1}
			break
		case -1: // control being killed
			break
	endswitch

	return 0
End

//	Works in the local datafolder
//	Modal window
Function DefineParametersNAF(String sImageName, String sFileName)

	NVAR V_NAF_SizeX, V_NAF_SizeY, V_NAF_dX, V_NAF_dY, V_NAF_TimePoints
	ReadNAF(sImageName, sFileName)
	Wave P_Image0
	NewImage/N=F_NAFParams P_Image0
	DoWindow/T F_NAFParams, "Raw NAF image"

	NewPanel/N=P_NAFParams /K=2 /W=(40,291,538,488) as "Define NAF settings"
	SetDrawLayer UserBack
	DrawText 15,24,"Time-point:"
	DrawText 7,83,"Lowest\rintensity:"
	DrawText 257,83,"Highest\rintensity:"
	Button C_Aide_NAF_Files,pos={404.00,130.00},size={63.00,20.00},proc=BoutonAide
	Button C_Aide_NAF_Files,title="Help",fSize=12,fStyle=1
	Slider C_TimePoint,pos={77.00,10.00},size={406.00,49.00},proc=SliderNAF
	Slider C_TimePoint,help={"to display different time-points in the stack"}
	Slider C_TimePoint,fSize=12,limits={0,250,1},value=50,vert=0
	Slider C_Intensity_Low,pos={56.00,60.00},size={176.00,49.00},proc=SliderNAF
	Slider C_Intensity_Low,help={"set the value for black"},fSize=12
	Slider C_Intensity_Low,limits={-500,500,1},value=0,vert=0
	Slider C_Intensity_High,pos={307.00,60.00},size={176.00,49.00},proc=SliderNAF
	Slider C_Intensity_High,help={"set the value for white"},fSize=12
	Slider C_Intensity_High,limits={100,5000,1},value=0,vert=0
	SetVariable C_SizeX,pos={25.00,134.00},size={130.00,17.00},proc=SetVarNAF
	SetVariable C_SizeX,title="Size - X"
	SetVariable C_SizeX,help={"Size of the image in the X (horizontal) dimension"}
	SetVariable C_SizeX,fSize=12,limits={0,inf,1},value=root:Ratio:V_NAF_SizeX
	SetVariable C_SizeY,pos={25.00,160.00},size={130.00,17.00},proc=SetVarNAF
	SetVariable C_SizeY,title="Size - Y"
	SetVariable C_SizeY,help={"Size of the image in the Y (vertical) dimension"}
	SetVariable C_SizeY,fSize=12,limits={0,inf,1},value=root:Ratio:V_NAF_SizeY
	SetVariable C_dX,pos={175.00,130.00},size={45.00,17.00},disable=1,proc=SetVarNAF
	SetVariable C_dX,title="dX",fSize=12,limits={0,inf,0},value=root:Ratio:V_NAF_dX
	SetVariable C_dY,pos={175.00,160.00},size={45.00,17.00},disable=1,proc=SetVarNAF
	SetVariable C_dY,title="dY",fSize=12,limits={0,inf,0},value=root:Ratio:V_NAF_dY
	SetVariable C_TimePoints,pos={209.00,134.00},size={155.00,17.00},proc=SetVarNAF
	SetVariable C_TimePoints,title="Total nb of time-points"
	SetVariable C_TimePoints,help={"Expected number of frames in the NAF stack"}
	SetVariable C_TimePoints,fSize=12
	SetVariable C_TimePoints,limits={0,inf,0},value=root:Ratio:V_NAF_TimePoints
	Button C_BoutonValide1,pos={406.00,159.00},size={63.00,20.00},proc=BoutonValideNAF
	Button C_BoutonValide1,title="Done",fSize=12,fStyle=1

//	Button C_Aide_NAF_Files,pos={298.00,157.00},size={63.00,20.00},proc=BoutonAide
//	Button C_Aide_NAF_Files,title="Help",fSize=12,fStyle=1
//	Slider C_TimePoint,pos={26.00,10.00},size={440.00,49.00},proc=SliderNAF,fSize=12
//	Slider C_TimePoint,limits={0,500,1},value=0,vert=0
//	Slider C_Intensity_Low,pos={26.00,60.00},size={176.00,49.00},proc=SliderNAF
//	Slider C_Intensity_Low,fSize=12,limits={-500,500,1},value=0,vert=0
//	Slider C_Intensity_High,pos={256.00,60.00},size={176.00,49.00},proc=SliderNAF
//	Slider C_Intensity_High,fSize=12,limits={100,5000,1},value=0,vert=0
//	SetVariable C_SizeX,pos={25.00,130.00},size={130.00,17.00},proc=SetVarNAF
//	SetVariable C_SizeX,title="Size - X",fSize=12
//	SetVariable C_SizeX,limits={0,inf,1},value=root:Ratio:V_NAF_SizeX
//	SetVariable C_SizeY,pos={25.00,160.00},size={130.00,17.00},proc=SetVarNAF
//	SetVariable C_SizeY,title="Size - Y",fSize=12
//	SetVariable C_SizeY,limits={0,inf,1},value=root:Ratio:V_NAF_SizeY
//	SetVariable C_dX,pos={175.00,130.00},size={45.00,17.00},proc=SetVarNAF,disable=1
//	SetVariable C_dX,title="dX",fSize=12,limits={0,inf,0},value=root:Ratio:V_NAF_dX
//	SetVariable C_dY,pos={175.00,160.00},size={45.00,17.00},proc=SetVarNAF,disable=1
//	SetVariable C_dY,title="dY",fSize=12,limits={0,inf,0},value=root:Ratio:V_NAF_dY
//	SetVariable C_TimePoints,pos={244.00,130.00},size={130.00,17.00},proc=SetVarNAF
//	SetVariable C_TimePoints,title="Nb of time points",fSize=12
//	SetVariable C_TimePoints,limits={0,inf,0},value=root:Ratio:V_NAF_TimePoints
//	Button C_BoutonValide1,pos={406.00,159.00},size={63.00,20.00},proc=BoutonValideNAF
//	Button C_BoutonValide1,title="Done",fSize=12,fStyle=1

	AutoPositionWindow/M=1/R=F_NAFParams
	
	Slider C_TimePoint win=P_NAFParams,limits={0,V_NAF_TimePoints,1},value=0,vert=0
	PauseForUser P_NAFParams
	return(1)
End

//	Reads a NAF file and leaves the stack named sImageName in dfRatio
Function ReadNAF(String sImageName, String sFileName)
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfRatio=W_DFREF[3]
	Variable vFileRef
	NVAR/SDFR=dfRatio V_NAF_SizeX, V_NAF_SizeY, V_NAF_dX, V_NAF_dY, V_NAF_TimePoints

	Open/R/Z/P=data vFileRef as sFileName
	if (V_flag!=0)	//	the file was not opened properly
		print "There was an error while opening the NAF file"
		return(0)
	endif
	FStatus vFileRef	//	returns file size in V_logEOF
	if (V_Flag==0)
		beep; 	print "Error while reading the NAF file"; return(0)
	endif
	variable vNbPixels=V_NAF_SizeX*V_NAF_SizeY*V_NAF_TimePoints
	variable vHeader=V_logEOF-vNbPixels*2
	if (vHeader<0)
		beep
		DoAlert/T="Warning" 0, "You indicated more pixels than the file size"
		return(1)
	endif
	Make/O/U/W/N=(vNbPixels) $sImageName	//	destination image created in dfRatio
	Wave wImage=$sImageName
	FSetPos vFileRef, vHeader
	FBinRead vFileRef, wImage
	Redimension/N=(V_NAF_SizeX, V_NAF_SizeY, V_NAF_TimePoints) wImage
	Close vFileRef
	Return(1)
End

//______________________________________________________________________________
// *** Open the first datafile ***
//	-> determines path to image files, handles experiment name, image size, experiment name
//	Determines technical parameters like camera offset, saturation values, optional gain, optional split settings...
//	Defines time, exposure

Function BoutonOuvreFichier(string ctrlName) : ButtonControl
	
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfRatio=W_DFREF[3]
	SetDataFolder dfRatio
	NVAR V_ChannelsMerged, V_Split, V_TimepointsMerged, V_NbFichiers, V_NbVoies
	NVAR V_NbLettresRacine, V_NbLettresInutiles, V_NbChiffresFichier
	NVAR V_Taille_X, V_Taille_Y, V_Taille_X0, V_Taille_Y0, V_NbPlans
	NVAR Dim, kMax, V_OffsetCamera, V_Saturation, V_Split, V_DeltaT, V_CameraPixelSize, V_TimeSelector

	SVAR S_Type_Image, S_Nom_Manip, S_Chemin_Donnees
	SVAR S_Base_Nom_Fichier_A, S_Base_Nom_Fichier_B, S_Base_Nom_Fichier_C, S_Base_Nom_Fichier_D, S_Base_Nom_Fichier_E, S_Base_Nom_Fichier_F
	SVAR S_Suffixe_A, S_Suffixe_B, S_Suffixe_C, S_Suffixe_D, S_Suffixe_E, S_Suffixe_F, S_Extension, S_NomPremierFichier

	Wave W_NumTrame, P_A, W_Exposition
	variable a, b, x, y, z, t, vFileRef, vTest, i, vDeltaT, vExp0, vExp1, vSize, vNbPlans=1
	string sTemp, sIndex
	
//	Open the first file of the experiment
	sTemp="TIF or NAF Image:.tif,.TIF,.naf,.NAF;"	//	allows the user to open tif or naf files
	Open/D/R/F=sTemp/P=data/M="Opening the first image" vFileRef
	if (CmpStr(S_FileName, "")==0)	// if S_FileName returns empty, it means that the user cancelled
		beep
		print "Cancelled by user"
		return(0)
	endif

	//	Store file informations
// *** \\\ debug for Windows ***	
	//	ParseFilePath may require separator to be "\\"
	NewPath/O data, ParseFilePath(1, S_FileName,":",1,0)	//	access files stored in data path...
	PathInfo data; S_Chemin_Donnees=S_Path	//	... and store the path in string S_Chemin_Donnees
	S_NomPremierFichier=ParseFilePath(3, S_FileName,":",0,0)
	S_Extension="."+ParseFilePath(4, S_FileName,":",0,0)	//	define file extension
	
	StrSwitch(S_Extension)
		case ".naf":
			//	This is where the NAF file is read, identified by the file name
			if (DefineParametersNAF("P_Image0", S_FileName))	//	function returns 0 in case of a problem, 1 otherwise
				//	success
			else
				beep; 	print "Error while defining parameters for the NAF file"
				return(0)
			endif
		break
		case ".tif":
			ImageLoad/Q/Z/T=tiff/C=-1/N=P_Image S_FileName
			if (V_Flag==0)
				beep; 	print "Error while reading the tif file"
				return(0)
			endif
		break
	endswitch

	//	The naming of the read image does not work as expected; depending on the situation, the imported image wave
	//	is named either P_Image or P_Image0. Here, need to make sure there's a single imported wave named P_Image0
	Wave/Z P_Image
	if (WaveExists(P_Image))
		killwaves/Z P_Image0
		rename P_Image P_Image0
	endif
	Wave P_Image0
	Redimension/S P_A	//	pourquoi ? pourquoi ici ?

	//	Save image size in global variables
	V_Taille_X=DimSize(P_Image0,0); V_Taille_X0=V_Taille_X
	V_Taille_Y=DimSize(P_Image0,1); V_Taille_Y0=V_Taille_Y
	if (V_TimepointsMerged)
// *** merged time-points -> specific settings ***
		prompt vNbPlans, "Number of planes in Z stack or separate imaging fields"
		DoPrompt "How many planes per time-point in the Z dimension", vNbPlans
		if (V_flag)	//	cancel clicked
			print "canceled by user"; return(0)
		endif
		V_NbPlans=vNbPlans
		//	Test if the loaded stack is indeed a 3D, and not a 4D stack of color images
		if (DimSize(P_Image0, 3)!=0)
			DoAlert 0, "Loaded as a 4D stack; there's a problem in the way the tif file is created"
			KillWaves P_Image0
			Return(0)
		endif
		vSize=DimSize(P_Image0, 2)/V_NbPlans	//	(the number of frames)/(nb of images per time-point) is the nb of data points in the experiment
		W_NumTrame[0, (vSize-1)]=p				//	defines the index value for each data point
	else
		if (V_ChannelsMerged)
			V_NbPlans=1
		else
			V_NbPlans=DimSize(P_Image0,2)
		endif
	endif

// *** Splitted image ***
	//	One image plane contains 2 channels side by side
	//	The user must enters how the channels are positioned
	if (V_Split)
		prompt vTest, "How to split the image", popup, "two halves;user-selected regions"
		DoPrompt "Select imaging zone", vTest
		if (V_flag)	//	cancel clicked
			print "canceled by user"; return(0)
		endif
		Make/O/N=6 W_Zone_A, W_Zone_B	//	Waves that contain the zone definition for A and B channels
		Switch (vTest)
			case 2:	//	User-defined zones
				NewImage/K=1/N=F_OptoSelect P_Image0
				DoWindow/T F_OptoSelect, "To select the A and B zones of the image"
				SelectionneZone("F_OptoSelect", "A channel", W_Zone_A)
				SelectionneZone("F_OptoSelect", "B channel", W_Zone_B)
				KillWindow F_OptoSelect
				//	Ensures that both zones have the same size
				Variable vLargA=W_Zone_A[3]-W_Zone_A[0]
				Variable vLargB=W_Zone_B[3]-W_Zone_B[0]
				if (vLargA<vLargB)
					W_Zone_B[3]=W_Zone_B[0]+vLargA
				else
					W_Zone_A[3]=W_Zone_A[0]+vLargB
				endif
				Variable vHautA=W_Zone_A[4]-W_Zone_A[1]
				Variable vHautB=W_Zone_B[4]-W_Zone_B[1]
				if (vHautA<vHautB)
					W_Zone_B[4]=W_Zone_B[1]+vHautA	
				else
					W_Zone_A[4]=W_Zone_A[1]+vHautB
				endif
			break
			case 1:		//	take half of the image for each channel; need to specify which half
				Variable/G V_ChoixOptosplit
				NewImage/K=1/N=F_OptoSelect P_Image0
				DoWindow/T F_OptoSelect, "Raw split image"

				NewPanel /K=2 /W=(576,44,824,156) as "How image is split"
				DoWindow/C P_ChoixOptosplit
				Button C_BoutonValide,pos={81.00,68.00},size={63.00,20.00},proc=BoutonChoixOptosplit,title="Validate"
				CheckBox C_AsurB,pos={63.00,17.00},size={33.00,15.00},proc=CheckChoixOptosplit,title="A/B"
				CheckBox C_AsurB,value= 0,mode=1
				CheckBox C_BsurA,pos={135.00,17.00},size={33.00,15.00},proc=CheckChoixOptosplit,title="B/A"
				CheckBox C_BsurA,value= 0,mode=1
				CheckBox C_B_A,pos={135.00,43.00},size={37.00,15.00},proc=CheckChoixOptosplit,title="B..A"
				CheckBox C_B_A,value= 0,mode=1
				CheckBox C_A_B,pos={63.00,43.00},size={37.00,15.00},proc=CheckChoixOptosplit,title="A..B"
				CheckBox C_A_B,value= 0,mode=1
				DoWindow/C P_ChoixOptosplit
				AutoPositionWindow/M=0 /R=F_OptoSelect
				PauseForUser P_ChoixOptosplit
				KillWindow F_OptoSelect
			break
		endswitch
		V_Taille_X=W_Zone_A[3]-W_Zone_A[0]; V_Taille_Y=W_Zone_A[4]-W_Zone_A[1]
	endif

	Dim=FDim()								//	updates experiment size
		
// *** Instrument-specific settings ***
	//	Depending on preset types, adjust some instrument-specific settings
	StrSwitch(S_Type_Image)
		case "iVision 2018":
		case "iVision 2":
			//	Depending on the letter indicated in the file name, sets the offset and threshold for saturation
			sTemp=S_Filename[10]	//	letter after the date -> determines which imaging set-up was used for acquisition
			StrSwitch(sTemp)
				case "A":	//	poste cage à Montpellier
					V_CameraPixelSize=3.45	//	caméra sCMOS Spot Imaging
					V_OffsetCamera=0
					V_Saturation=4090
				break
				case "B":	//	poste FRET ancien, avec Liliana
					V_CameraPixelSize=6.44	//	Caméra Orca AG
					V_OffsetCamera=203	
					V_Saturation=4090
				break
				case "E":	//	Poste inversé fura
					V_CameraPixelSize=6.44	//	Caméra Orca AG
					V_OffsetCamera=1
					V_Saturation=65535
				break
			EndSwitch
		break
		case "Biphoton":
			//	Because the stupid way the 2 photon software was designed, have to swap A and B base names
			S_Base_Nom_Fichier_B=S_Base_Nom_Fichier_A
			S_Base_Nom_Fichier_A=S_Base_Nom_Fichier_A[0, (StrLen(S_Base_Nom_Fichier_A)-3)]+"2_"	//	replace last 2 characters
			LitParamsBiphoton("")
			LitTempsBiphoton()
		break
		case "iVision":
		break
		case "NAF split":	//	set some preferred values for NAF split; waiting for this to be saved in prefs when creating new analysis
			V_DeltaT=0.02
			W_Exposition[0][0]=0.02
			W_Exposition[0][1]=0.02
			V_CameraPixelSize=1
			V_OffsetCamera=0
			V_Saturation=65535
			V_TimeSelector=3
			V_DeltaT=0.02
			W_Exposition=0.02
		break
// *** \\\ réglages spécifiques poste biolum ***
		case "Poste biolum":
			DoAlert 0, "à corriger la taille des pixels, l'offset caméra"
			V_CameraPixelSize=1
			V_OffsetCamera=0
			V_Saturation=65535
			S_Base_Nom_Fichier_A=S_NomPremierFichier[0, V_NbLettresRacine-1]+"_w1NLuc_t"
			S_Base_Nom_Fichier_B=S_NomPremierFichier[0, V_NbLettresRacine-1]+"_w2YFP_t"
//		break
		default:
			V_TimeSelector=3	//	suppose the best guess is to assume regularly spaced time-points
			//	Get acquisition period from the user, if not already known
			if ((NumType(V_DeltaT)!=0) && (V_TimeSelector==3))	//	V_DeltaT is still set to default NaN
				vDeltaT=V_DeltaT
				Prompt vDeltaT, "Sampling interval (s):"
				DoPrompt/HELP="If the images are acquired with a regular time interval, enter the period here" "Regular time interval between images", vDeltaT
				V_DeltaT=vDeltaT
			endif
			
			//	Sets exposure
			vExp0=W_Exposition[0][0]
			vExp1=W_Exposition[0][1]
			Prompt vExp0, "Exposure duration for channel A"
			Prompt vExp1, "Exposure duration for channel B"
			DoPrompt/HELP="If the exposure duration is the same for all time points, enter the values for A and B channels" "Exposure time", vExp0, vExp1
			W_Exposition[0,][0]=vExp0; W_Exposition[0,][1]=vExp1
		break
	endswitch

//	Trigger various actions to update the analysis file with the data now defined with this first data file
	PopMenuInitChemins("",0,"")	//	update the titlebox with new path name
	TitleBox C_Extension,win=P_Initialisation,title=S_Extension	//	print the extension on the panel
	MaJGlissieres()							//	Update sliders for number of time points
	SetVarNomManip("",0,"","")	//	updates windows' title to reflect experiment name
	Init_P_Controle()	//	Adjust the sliders to the right number of planes in the Z stack
	SetVarNbVoies("",V_NbVoies,"","")//		Adapts popups and buttons to match the actual number of channels to handle
	RedimensionImages("rien")	//	Make sure that calculation images are of the correct size
	CheckLitUnif("", 1)		//	To read the Unif file if located correctly at the default place
	CheckProc_VraiePileZ("init", 0)	//	to make the radio buttons visible if there is more than 1 Z plane
	KillWaves P_Image0
	SetVarZoom("",1,"","")	//	sets the default zoom
	MaJ_CasesACocher("Tous", 0, "", "")
	DoUpdate
	DoWindow/F F_RM
End

Function CheckChoixOptosplit(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Wave P_Image0, W_Zone_A, W_Zone_B
	StrSwitch(ctrlName)
		case "C_AsurB":
			CheckBox C_AsurB, value=1
			CheckBox C_BsurA, value=0
			CheckBox C_A_B, value=0
			CheckBox C_B_A, value=0
			W_Zone_A[0]=0; W_Zone_A[1]=0; W_Zone_A[3]=DimSize(P_Image0, 0)-1; W_Zone_A[4]=DimSize(P_Image0, 1)/2-1
			W_Zone_B[0]=0; W_Zone_B[1]=DimSize(P_Image0, 1)/2; W_Zone_B[3]=DimSize(P_Image0, 0)-1; W_Zone_B[4]=DimSize(P_Image0, 1)-1
		break
		case "C_BsurA":
			CheckBox C_AsurB, value=0
			CheckBox C_BsurA, value=1
			CheckBox C_A_B, value=0
			CheckBox C_B_A, value=0
			W_Zone_A[0]=0; W_Zone_A[1]=DimSize(P_Image0, 1)/2-1; W_Zone_A[3]=DimSize(P_Image0, 0)-1; W_Zone_A[4]=DimSize(P_Image0, 1)-1
			W_Zone_B[0]=0; W_Zone_B[1]=0; W_Zone_B[3]=DimSize(P_Image0, 0)-1; W_Zone_B[4]=DimSize(P_Image0, 1)/2-1
		break
		case "C_A_B":
			CheckBox C_AsurB, value=0
			CheckBox C_BsurA, value=0
			CheckBox C_A_B, value=1
			CheckBox C_B_A, value=0
			W_Zone_A[0]=0; W_Zone_A[1]=0; W_Zone_A[3]=DimSize(P_Image0, 0)/2-1; W_Zone_A[4]=DimSize(P_Image0, 1)-1
			W_Zone_B[0]=DimSize(P_Image0, 0)/2-1; W_Zone_B[1]=0; W_Zone_B[3]=DimSize(P_Image0, 0)-1; W_Zone_B[4]=DimSize(P_Image0, 1)-1
		break
		case "C_B_A":
			CheckBox C_AsurB, value=0
			CheckBox C_BsurA, value=0
			CheckBox C_A_B, value=0
			CheckBox C_B_A, value=1
			W_Zone_A[0]=DimSize(P_Image0, 0)/2-1; W_Zone_A[1]=0; W_Zone_A[3]=DimSize(P_Image0, 0)-1; W_Zone_A[4]=DimSize(P_Image0, 1)-1
			W_Zone_B[0]=0; W_Zone_B[1]=0; W_Zone_B[3]=DimSize(P_Image0, 0)/2-1; W_Zone_B[4]=DimSize(P_Image0, 1)-1
		break
	EndSwitch
	SetDrawLayer/K/W=F_OptoSelect ProgFront
	SetDrawEnv/W=F_OptoSelect xcoord=top, ycoord=left, linefgc=(1,34817,52428), fillpat=0, linethick=3, textrgb=(1,34817,52428),fsize=20,textyjust=2, save
	DrawRect/W=F_OptoSelect W_Zone_A[0], W_Zone_A[1], W_Zone_A[3], W_Zone_A[4]
	DrawText/W=F_OptoSelect W_Zone_A[0]+50,W_Zone_A[1]+50, "A channel"
	SetDrawEnv/W=F_OptoSelect linefgc= (52428,17472,1), textrgb=(52428,17472,1), save
	DrawRect/W=F_OptoSelect W_Zone_B[0], W_Zone_B[1], W_Zone_B[3], W_Zone_B[4]
	DrawText/W=F_OptoSelect W_Zone_B[0]+50,W_Zone_B[1]+50, "B channel"
End

Function BoutonChoixOptosplit(ctrlName) : ButtonControl
	String ctrlName
	DoWindow/K P_ChoixOptosplit
End

//	To set the radio buttons for the type of 3D stack (Z stack or different fields)
Function CheckProc_VraiePileZ(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	NVAR/SDFR=dfRatio V_NbPlans, V_VraiePileZ
	StrSwitch(ctrlName)
		case "C_Check_VraiePileZ":
			V_VraiePileZ=1
		break
		case "C_Check_VraiePileZFalse":
			V_VraiePileZ=0
		break
	EndSwitch
	CheckBox C_Check_VraiePileZ win=P_Initialisation, value=V_VraiePileZ, disable=(V_NbPlans<2)
	CheckBox C_Check_VraiePileZFalse win=P_Initialisation, value=(!V_VraiePileZ), disable=(V_NbPlans<2)
End


//______________________________________________________________________________
// *** Load all images from time stack ***
//	Available only for merged time-points, ie V_TimepointsMerged=1
//	Called by button on P_Initialisation
//	Called when everything is set; triggers the loading of all files containing time stacks, and kept in memory as P_Image_#
//	Calls ProcessRawData to process these stacks according to technical parameters, which saves these images in cache directory, for further use

Function BoutonChargeToutesImages(string ctrlName) : ButtonControl
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfRatio=W_DFREF[3]
	SetDataFolder dfRatio

	variable i, j, vSize
	string sNomRef, sSuffRef, sNomPile, sNomFichier
	NVAR V_NbFichiers, V_TimepointsMerged, V_NbPlans
	SVAR S_Extension

	if (!V_TimepointsMerged)
		DoAlert/T="Problem" 0, "Error on the definition of Separate / Merged time-points scheme"
		return(0)
	endif

	//	Loads all time stacks
	NewPanel /N=LoadStacks /W=(285,111,739,193) as "Loading file(s) with raw image stacks"
	for (i=0; (i<V_NbFichiers); i+=1)
		sNomRef="S_Base_Nom_Fichier_"+StringFromList(i, ksABC)
		SVAR sNom=$sNomRef
		sSuffRef="S_Suffixe_"+StringFromList(i, ksABC)
		SVAR sSuff=$sSuffRef
		sNomFichier=sNom+sSuff+S_Extension
		sNomPile="P_TStack_"+num2Str(i)
		StrSwitch(S_Extension)
			case ".tif":
				ImageLoad/Q/Z/O/C=-1/T=tiff/P=data/N=$sNomPile sNomFichier
			break
			case ".naf":
				ReadNAF(sNomPile, sNomFichier)
			break
		EndSwitch
	endfor
	KillWindow LoadStacks

	Wave P_TStack_0					//	at least one file has been read
//	vSize=DimSize(P_TStack_0, 2)
	vSize=DimSize(P_TStack_0, 2)/V_NbPlans

	//	Prepare the progress window
	NewPanel /N=ProgressPanel /W=(285,111,739,193) as "Processing raw images"
	ValDisplay valdisp0,pos={18,32},size={342,18},limits={0,100,0},barmisc={0,0},highColor=(0,90*256,160*256)
	ValDisplay valdisp0,value= _NUM:0
	Button bStop,pos={375,32},size={50,20},title="Stop"
	DoUpdate /W=ProgressPanel /E=1	// mark this as our progress window

	//	Calls ProcessRawDatas for each time-point
	for (i=0; (i<vSize); i+=1)	
		ProcessRawData(i)

		//	update progress window
		j=trunc(i/vSize*100)
		ValDisplay valdisp0,value= _NUM:j,win=ProgressPanel
		DoUpdate /W=ProgressPanel
		if( V_Flag == 2 )	// we only have one button and that means stop
			break
		endif
	endfor
	KillWindow ProgressPanel

	//	Dispose of the time stacks
	for (i=0; (i<V_NbFichiers); i+=1)
		sNomPile="P_TStack_"+num2Str(i)
		KillWaves/Z $sNomPile
	endfor
	
	//	Change button appearance to indicate that loading has been done
	Button C_BoutonChargePileTemps win=P_Initialisation,fstyle=2,valueColor=(0,0,0)
	
	SetDataFolder root:
End

//______________________________________________________________________________
// *** Read and process raw images for one time point ***
//	Two different working logics, depending on V_TimepointsMerged:
//	- if 1, time stacks should have been loaded as P_Image_*, with as many stacks as there are files to read; the function is called by BoutonChargeToutesImages
//	- if 0, allows real-time analysis of raw file(s) with an undetermined number of frames to analyze; reads a set of tif files for the current time-point (i); try to read an image file which index is indicated in W_NumTrame[i]
//	At the end of this function, there must be a set of files saved in cache directory containing P_A, P_B, and optional additional channels
//	The function handles the technical informations in the image such as 
//	- reading the time, exposure time etc... in the first pixels of the image (iVision) 
//	- splitting the image when 2 channels merged in 1 image
//	- detect if some pixels reach the saturation level
//	- subtract offset
//	- divide by exposure duration
//	- divide by gain
//	- subtract background
//	- remove NaNs and Infs
//	Returns 0 if there was no such file; 1 if an image was correctly read

Function ProcessRawData(Variable vNumTimePoint)
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfRatio=W_DFREF[3]
	SetDataFolder dfRatio
	WAVE W_Temps, W_Exposition, W_NumTrame, P_A, P_B, D_UnifA, D_UnifB
	NVAR V_NbVoies, V_ChannelsMerged, V_Split, V_TimepointsMerged, V_NbFichiers	//	variables that contain the information about data storage scheme
	NVAR V_OffsetCamera, kMax, V_Taille_X, V_Taille_Y, V_NbChiffresFichier, V_NbPlans, V_Miroir, V_MoyUnifA, V_MoyUnifB
	NVAR V_IMin, V_IMax, V_Saturation, V_Lissage, V_TimeSelector, V_TimeOffset, V_DeltaT
	SVAR S_Base_Nom_Fichier_A, S_Base_Nom_Fichier_B, S_Base_Nom_Fichier_C, S_Base_Nom_Fichier_D, S_Base_Nom_Fichier_E, S_Base_Nom_Fichier_F
	SVAR S_Suffixe_A, S_Suffixe_B, S_Suffixe_C, S_Suffixe_D, S_Suffixe_E, S_Suffixe_F, S_Type_Image, S_Extension

	variable i, j, numVoie, vSatA, vSatB, vAnnule
	String sNomFichier, sNomPile, sNomImage, sNomRef, sSuffRef, sNomTemp	// for making file names

	if (vNumTimePoint>=kMax)
		print "Trying to read more frames than planned; increase kMax constant"
		SignalSonore(1)
		Return(0)
	endif
	if (vNumTimePoint<0)
		print "trying to read a negative frame number; move the slider back to 0."
		SignalSonore(1)
		Return(0)
	endif

	If (V_TimepointsMerged)
		//	The time stacks are in dfRatio with name P_TStack_0, P_TStack_1... with as many stacks as there are files (V_NbFichiers)
		for (i=0; (i<V_NbFichiers); i+=1)
			sNomPile="P_TStack_"+num2Str(i)
			Wave/Z pPile=$sNomPile
			If (!WaveExists(pPile))
				DoAlert 0, "Time stack should be loaded first"
				Return(0)
			endif
// *** \\\ get as many planes as there are in V_NbPlans; repeat ? get a whole chunck ? ***
			sNomImage="P_Raw_"+num2str(i)
			ImageTransform/P=(0) getPlane pPile
			Wave M_ImagePlane
			Duplicate/O M_ImagePlane, $sNomImage
			Wave pImage=$sNomImage
			Redimension/N=(-1, -1, V_NbPlans) pImage
			for (j=0; j<V_NbPlans; j+=1)
				ImageTransform/P=(vNumTimePoint*V_NbPlans+j) getPlane pPile		//	gets the image plane corresponding to the time point
				ImageTransform/D=M_ImagePlane/P=(j) setPlane pImage
			endfor
		endfor
		KillWaves M_ImagePlane
	else
		// Analysis of separate files for different time-points; need to make the file name, read the file, and return P_Raw_*
		for (i=0; (i<V_NbFichiers); i+=1)
			sNomRef="S_Base_Nom_Fichier_"+StringFromList(i, ksABC)
			SVAR sNom=$sNomRef
			sSuffRef="S_Suffixe_"+StringFromList(i, ksABC)
			SVAR sSuff=$sSuffRef
			sNomFichier=AjouteNumero(sNom,W_NumTrame[vNumTimePoint], V_NbChiffresFichier)+sSuff+S_Extension

			//	Gives the right name to the extracted image
			sNomPile="P_Raw_"+num2str(i)			

			//	Loads the image into an image with that name
			StrSwitch(S_Extension)
				case ".tif":
					ImageLoad/Q/Z/O/C=-1/T=tiff/P=data/N=$sNomPile sNomFichier
					if (V_Flag==0)
						print "No file", sNomFichier, "was found. End of data."
						return(0)	//	stops if no file was read and returns error to calling function
					endif
				break
				case ".naf":
					if (ReadNAF(sNomPile, sNomFichier))	//	file was read successfully
					else
						print sNomFichier, " not found."
						return(0)
					endif
				break
			EndSwitch
		endfor
	endif
	
	//	Handles files generated with a splitting device like optosplit or Gemini
	if (V_ChannelsMerged)	//	only one wave to contain all channels
		Wave P_Raw_0
		if (V_Split)	//	a single image with two channels defined by W_Zone_*
			Wave W_Zone_A, W_Zone_B
			P_A=P_Raw_0[p+W_Zone_A[0]][q+W_Zone_A[1]]
			P_B=P_Raw_0[p+W_Zone_B[0]][q+W_Zone_B[1]]
		else				//	2 or more channels, each one corresponding to one image plane of the stack
			for (i=0; (i<V_NbVoies); i+=1)
				ImageTransform/P=(i) getplane P_Raw_0
				sNomPile="P_"+StringFromList(i, ksABC)
				Wave M_ImagePlane
				Duplicate/O M_ImagePlane, $sNomPile
			endfor
			KillWaves M_ImagePlane
		endif
	else				//	Each channel in a separate P_Raw_* stack
		for (i=0; (i<V_NbVoies); i+=1)
			sNomPile="P_Raw_"+num2str(i)
			Wave wRaw=$sNomPile
			sNomPile="P_"+StringFromList(i, ksABC)
			Duplicate/O wRaw, $sNomPile
		endfor
	endif

	//	Kill stacks of raw data
	for (i=0; (i<V_NbFichiers); i+=1)
		sNomPile="P_Raw_"+num2str(i)
		Wave wRaw=$sNomPile
		KillWaves wRaw
	endfor
	//	At this stage, there are only P_A, P_B..., as many as acquisition channels

	//	Reads exposure duration
	StrSwitch(S_Type_Image)
		case "iVision 2018":
		case "iVision 2":
			W_Exposition[vNumTimePoint][0]=P_A[0][1][0]/1000
			W_Exposition[vNumTimePoint][1]=P_B[0][1][0]/1000
		break
		case "iVision":
			W_Exposition[vNumTimePoint][0]=P_A[0][5][0]/1000
			W_Exposition[vNumTimePoint][1]=P_B[0][5][0]/1000
		break
	endswitch

// *** This is where time for the image is determined ***
	StrSwitch(S_Type_Image)
		case "iVision 2018":
		case "iVision 2":
		case "iVision":
			Switch(V_TimeSelector)
				case 1:	//	For all iVision schemes, experiment time is saved in the first pixel of each image
					if (Numtype(W_Temps[vNumTimePoint])!=0)		//	read time only if there is no pre-existing value
						W_Temps[vNumTimePoint]=P_A[0][0][0]
					endif
				break
				case 2:	//	read time from image file
					W_Temps[vNumTimePoint]=LitTempsFichier(sNomFichier)-V_TimeOffset
				break
				case 3:
					W_Temps[vNumTimePoint]=vNumTimePoint
				break
			EndSwitch
			//	Clears the first 6 pixels, which are used for storing acquisition parameters
			P_A[0][0,5][0]=V_OffsetCamera
			P_B[0][0,5][0]=V_OffsetCamera
		break
		default:	//	other situation than iVision
			Switch(V_TimeSelector)
				case 1:	//	In this default configuration, the time is not determined
					W_Temps[vNumTimePoint]=vNumTimePoint
				break
				case 2:	//	read time from image file
					W_Temps[vNumTimePoint]=LitTempsFichier(sNomFichier)-V_TimeOffset
				break
				case 3:
					W_Temps[vNumTimePoint]=vNumTimePoint*V_DeltaT
				break
			EndSwitch
		break
	EndSwitch

	//	Checks if there are saturated pixels in the image, and if yes, issues a warning
	if (NumType(V_Saturation)==0)
		ImageStats/M=1 P_A
		vSatA=(V_Max>=V_Saturation) ? 1 : 0
		ImageStats/M=1 P_B
		vSatB=(V_Max>=V_Saturation) ? 1 : 0
		if (vSatA || vSatB)
			vSatA+=1; vSatB+=1
			Prompt vSatA, "Channel A", popup, "OK;Saturated;"
			Prompt vSatB, "Voie B", popup, "OK;Saturated;"
			Prompt vAnnule, "Cancel automatic detection of saturation", popup, "No;Yes;"
			DoPrompt/HELP="Raw data value exceed V_Saturation, indicating a possible saturation of the raw image." "Caution: saturated data", vSatA, vSatB, vAnnule
			if (vAnnule==2)
				V_Saturation=inf
			endif
		endif
	endif
	
	//	if an analysis has already been done that included a unif image, and if the current unif image is 0, it must be initialized
	if ( (V_MoyUnifA!=1) && ( (D_UnifA[0][0])==0) )
		CheckLitUnif("", 1)
	endif

	//	Corrections applied to all channels
	for (i=0; (i<2); i+=1)
		sNomPile="P_"+StringFromList(i, ksABC)
		Wave pPile=$sNomPile

	//	Makes wave Real, for calculation
	//	The rest of the code works on 3D, with at least 1 layer (can't be a 2D (matrix) wave), so need to make sure the 3d dimension exists
		Redimension/S/N=(-1,-1,V_NbPlans) pPile
		
		//	Point at the correct variable values
		sNomTemp="V_Gain"+StringFromList(i, ksABC)
		NVAR vGain=$sNomTemp
		sNomTemp="D_Unif"+StringFromList(i, ksABC)
		Wave dUnif=$sNomTemp
		sNomTemp="P_Fond"+StringFromList(i, ksABC)
		Wave/Z dBackGround=$sNomTemp
		
		//	Perform instrumental corrections : offset, mirror, exposure, gain, unif
		//	NB: V_MoyUnifA is there only to detect whether unif has already been processed; doesn't matter whether it's A or B
		if (WaveExists(dBackground))
			ProcessRawDataCalculations(pPile, V_OffsetCamera, V_Miroir, W_Exposition[vNumTimePoint][i], vGain, V_MoyUnifA, dUnif, dFond=dBackGround)
		else
			ProcessRawDataCalculations(pPile, V_OffsetCamera, V_Miroir, W_Exposition[vNumTimePoint][i], vGain, V_MoyUnifA, dUnif)
		endif
	endfor

	//	Clears the waves of infs and nans
	EnleveInfsNansW(P_A, V_IMin, V_IMax)
	EnleveInfsNansW(P_B, V_IMin, V_IMax)

	//	Save in cache directory
	GereImages(vNumTimePoint,2, 1)
	Return(1)
End

//	Function which performs the corrections which are applied to all images
//	Called by the previous function, and also when reading an image stack
Function ProcessRawDataCalculations(pPile, vOffsetCamera, vMiroir, vExposition, vGain, vMoyUnif, dUnif, [dFond])
	Wave pPile, dUnif, dFond
	Variable vOffsetCamera, vMiroir, vExposition, vGain, vMoyUnif
	
	//	Subtract background image, if any
	if (WaveExists(dFond))
		if (DimSize(pPile,2)>1)
			MatrixOp/O/FREE P_Temp=pPile-dFond
			FastOp pPile=P_Temp	
		else
			FastOp pPile=pPile-dFond
		endif
	endif	

	//	Subtract camera offset
	Fastop pPile=pPile-(vOffsetCamera)
	
	//	Exposure duration
	if ((vExposition!=1))	//	Divide by exposure duration, to normalize as counts / pix / s
		Fastop pPile=(1/vExposition)*pPile
	endif

	//	Divide by gain value, if any
	if (vGain!=1)
		FastOp pPile=(1/vGain)*pPile
	endif

// *** Shading (Unif) correction ***
	//	Divide by Shading (unif) image, if any
	if (vMoyUnif!=1)
		if (DimSize(pPile,2)>1)
			MatrixOp/O/FREE P_Temp=pPile/dUnif
			FastOp pPile=P_Temp	
		else
			FastOp pPile=pPile/dUnif
		endif
	endif	

	//	Mirror and rotation
	if ((vMiroir & 1) != 0)
		ImageRotate/V/O pPile
	endif
	if ((vMiroir & 2) != 0)
		ImageRotate/H/O pPile
	endif
	if ((vMiroir & 4) != 0)
		ImageRotate/A=90/C/O pPile
	endif		
End

//	Manages radio buttons to select how time is read or calculated
Function CheckTimeSelector(ctrlName, checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfRatio=W_DFREF[3]
	NVAR/SDFR=dfRatio V_TimeSelector
	StrSwitch(ctrlName)
		case "C_Check_TimeDefault":
			V_TimeSelector=1
		break
		case "C_Check_TimeFromFile":
			V_TimeSelector=2
		break
		case "C_Check_TimeRegular":
			V_TimeSelector=3
		break
	EndSwitch
	//	Update checkboxes to reflect variable value
	CheckBox C_Check_TimeDefault win=P_Initialisation,value=V_TimeSelector==1
	CheckBox C_Check_TimeFromFile win=P_Initialisation,value=V_TimeSelector==2
	CheckBox C_Check_TimeRegular win=P_Initialisation,value=V_TimeSelector==3
	return 0
End

//	Reads the creation date of the file in the "data" file path
//	Time is expressed in seconds since midnight
function LitTempsFichier(nomfich)
	string nomfich

	GetFileFolderInfo/Q/P=data nomfich
	if (V_Flag !=0)	//erreur de lecture du ficher
		return(NaN)
	endif
	string puce=secs2date(V_creationDate, -2, ";")
	variable annee, mois, jour
	annee=str2num(StringFromList(0, puce, ";"))
	mois=str2num(StringFromList(1, puce, ";"))
	jour=str2num(StringFromList(2, puce, ";"))
	return (V_creationDate-date2secs(annee,mois,jour))
end

Function SetVarIntervalleTemps(ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName
	Variable varNum
	String varStr
	String varName

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	Wave/SDFR=dfRatio W_NumTrame, W_Temps
	NVAR/SDFR=dfRatio V_DeltaT, V_TimeSelector, Dim
	
	//	Sets time selector to 3 and update radio buttons
	V_TimeSelector=3
	CheckTimeSelector("",0)
	
	variable i
	DoWindow/F T_Time
	W_Temps=NaN
	W_Temps[0, Dim-1]=p*V_DeltaT
//	for (i=0; (i<DimSize(W_NumTrame,0)); i+=1)
//		if (NumType(W_NumTrame[i])==2)
//			W_Temps[i]=NaN
//		endif
//	endfor
End

//	Charge des images de fond à soustraire de chaque image de donnée
Function CheckLitFond(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfRatio=W_DFREF[3]
	SetDataFolder dfRatio
	killwaves/Z P_FondA, P_FondB
	Switch(checked)
		case 0:
		break
		case 1:
			DoAlert 0, "Load background for A channel"
			ImageLoad/T=tiff/O/C=1/N=P_FondA/P=data
			if (V_Flag==0)
				Print "No background loaded for A channel"
				CheckBox C_CheckFond value=0, win=P_Initialisation
				return(0)
			endif
			DoAlert 0, "Load background for B channel"
			ImageLoad/T=tiff/O/C=1/N=P_FondB/P=data
			if (V_Flag==0)
				Print "No background loaded for B channel"
				CheckBox C_CheckFond value=0, win=P_Initialisation
				return(0)
			endif
			Wave P_FondA, P_FondB
			Redimension/S P_FondA, P_FondB
		break
	EndSwitch
End

//	Loading shading file; could be stored in a tif file with 2 planes, one for each channel
Function CheckLitUnif(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfRatio=W_DFREF[3]
	SetDataFolder dfRatio
	Wave D_UnifA, D_UnifB
	NVAR V_Taille_X0, V_Taille_Y0, V_MoyUnifA, V_MoyUnifB, V_R0, V_Miroir
	SVAR S_Type_Image

	variable MoyUnifAB
	string sNomFich, sListeNomsFich

	if (checked)	//	Try to load shading images, assuming it is a single file with two planes: plane 0 -> shading for channel A; plane 1 -> B
		sListeNomsFich=IndexedFile(data, -1, ".tif")
		sNomFich=StringFromList(0, ListMatch(sListeNomsFich, "*unif*"))	//	look for any file containing the text "unif"
		if (StrLen(sNomFich)==0)
			Beep; print "No suitable *unif* file found in the data folder"
			CheckBox C_checkUnif value=0, win=P_Initialisation
			Fastop D_UnifA=1; Fastop D_UnifB=1; V_MoyUnifA=1; V_MoyUnifB=1
			return(0)
		endif
		//	trying to load each layers of this two-layered image
		ImageLoad/T=tiff/O/Q/C=1/N=Unif0/P=data sNomFich
		ImageLoad/T=tiff/O/Q/C=2/N=Unif1/P=data sNomFich						
		if (V_Flag==0)
			Beep; print "The *unif* file did not load properly: check that it contains 2 layers"
			CheckBox C_checkUnif value=0, win=P_Initialisation
			Fastop D_UnifA=1; Fastop D_UnifB=1; V_MoyUnifA=1; V_MoyUnifB=1
			return(0)
		endif

		Wave Unif0, Unif1
		if ((DimSize(Unif0, 0)!= V_Taille_X0) | (DimSize(Unif0, 1)!= V_Taille_Y0))	//	check if unif size is different from image size
			if ((DimSize(Unif0, 0) == (V_Taille_X0*2)) && (DimSize(Unif0, 1)== (V_Taille_Y0*2)))	//	Unif is 2x larger than image -> probably 2x2 binning
				print "Unif file loaded and processed assuming 2x2 binning"
				Resample/DOWN=2/DIM=0 Unif0
				Resample/DOWN=2/DIM=1 Unif0
				Resample/DOWN=2/DIM=0 Unif1
				Resample/DOWN=2/DIM=1 Unif1
			else
				beep
				print "The size of Unif images don't match that of fluorescence images"
				CheckBox C_checkUnif value=0, win=P_Initialisation
				Fastop D_UnifA=1; Fastop D_UnifB=1; V_MoyUnifA=1; V_MoyUnifB=1
				return(0)
			endif
		endif

		ImageStats/M=1 Unif0
		V_MoyUnifA=V_avg
		ImageStats/M=1 Unif1
		V_MoyUnifB=V_avg
		MoyUnifAB=(V_MoyUnifA+V_MoyUnifB)/2

		//	Unif images usually have values that are huge (20000 and more)
		//	To make it easier to handle, normalize the amplitude of these images to the average of both channels -> values around 1
		//	Since it is divided by the same value, the shading correction is still valid.
		FastOp D_UnifA=(1/MoyUnifAB)*Unif0
		FastOp D_UnifB=(1/MoyUnifAB)*Unif1
		KillWaves Unif0, Unif1

		//	Remove infs and nans
		EnleveInfsNansW(D_UnifA, V_MoyUnifA, V_MoyUnifA)
		EnleveInfsNansW(D_UnifB, V_MoyUnifB, V_MoyUnifB)

	else			//	clears shading images and sets average to 0 -> cancels unif correction
		Fastop D_UnifA=1; Fastop D_UnifB=1; V_MoyUnifA=1; V_MoyUnifB=1
	endif
	CheckBox C_checkUnif win=P_Initialisation, value=checked	//	Updates control to match the situation
end

//	Controls the number of active SetVariable boxes to enter file names
Function SetVarNbFichiers(ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName
	Variable varNum
	String varStr
	String varName
	variable i, vNum
	for (i=1; (i<=6); i+=1)
		vNum = i<=varNum ? 0 : 1
		string sNomString="C_Nom_Fich_"+StringFromList(i-1, ksABC) 
//		SetVariable $sNomString win=P_Initialisation, disable=2*vNum	//	just disable the box
		SetVariable $sNomString win=P_Initialisation, disable=1*vNum	//	hide the box
		sNomString="C_Suffixe_"+StringFromList(i-1, ksABC) 
		SetVariable $sNomString win=P_Initialisation, disable=1*vNum
	endfor
End

//	Number of imaging channels
Function SetVarNbVoies(ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName
	Variable varNum		//	number of acquired channels = V_NbVoies; at least 2
	String varStr
	String varName

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfRatio=W_DFREF[3]
	Wave/SDFR=dfRatio P_A, W_Exposition
	NVAR/SDFR=dfRatio V_NbVoies

	string sNomString, sNomStringCorr
	variable i, num
	
	//	A and B channels are mandatory
	//	Creates as many additional image stacks as acquisition channels
	for (i=2; (i<6); i+=1)
		sNomString="P_"+StringFromList(i, ksABC)
		sNomStringCorr="P_"+StringFromList(i, ksABC)+"Corr"
		if (i<varnum)
			Duplicate/O P_A, dfRatio:$sNomString, dfRatio:$sNomStringCorr	//	Duplicates P_A with the required name
			Wave wTemp=dfRatio:$sNomString; FastOp wTemp=0
			Wave wTemp=dfRatio:$sNomStringCorr; FastOp wTemp=0
		endif
		if (i>=varnum)		//	going from higher to lower number, remove useless waves
			killwaves/Z $sNomString, $sNomStringCorr
		endif		
	endfor

	Redimension/N=(-1,V_NbVoies) W_Exposition

	//	updates the popup which selects the displayed channel with the right list of channels
	String quote = "\""
	String sList=quote+"RM;"+ksABC[0,(V_NbVoies-1)*2]+quote	//	pas clair pourquoi il faut ce fourbi de #, mais ça marche
	PopupMenu/Z C_Affichage win=P_Controle, value=#sList	//	pour l'affichage d'une voie ou d'une autre
	PopupMenu/Z C_Affichage win=F_RM, value=#sList	//	pour l'affichage d'une voie ou d'une autre
	CheckDataScheme("",0)
End

Function PopMenuObjectif(ctrlName,popNum,popStr) : PopupMenuControl
	String ctrlName
	Variable popNum
	String popStr

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfRatio=W_DFREF[3]
	NVAR/SDFR=dfRatio V_Resolution, V_DimCarre, V_CameraPixelSize
	SVAR/SDFR=dfRatio S_Nom_Manip, S_Type_Image

	StrSwitch(S_Type_Image)
		case "iVision":
		case "iVision 2":
		case "iVision 2018":
//			V_Resolution=0.644
			StrSwitch(popStr)
				case "10":
					V_Resolution=V_CameraPixelSize/10
				break
				case "20":
					V_Resolution=V_CameraPixelSize/20
				break
				case "40":
					V_Resolution=V_CameraPixelSize/40
				break
				case "60":
					V_Resolution=V_CameraPixelSize/60
					V_DimCarre=10
				break
			endswitch
		break
	EndSwitch
End

// *** \\\--- New versison of BoutonSauveVersImage ***
//	Saves the current object as an image
//	Called by buttons on the main control panel with:
//	C_SauveVignetteChamp	save the field of view to database folder
//	C_SauveVignetteTraces	save the sorted trace to database folder
//	C_Sauve1Image			save the currently displayed image
//	C_SauveSerie			save a series of time-points
//	C_SauveGraphesVersPict	save the result graphs : F_Traces, F_TracesTriees, P_Kymograph, P_Pulse (if exist)
//	End-up with saving SavePict/O/E=-vType/P=$sChemin as sNomFichier
//	Export formats:
//	-9	svg
//	-8	Igor pdf
//	-7	tiff
//	-6	jpeg
//	-5	png
//	-2	macos: quartz pdf; windows: High-resolution Enhanced Metafile (EMF)
	
Function BoutonSauveVersImage(ctrlName) : ButtonControl
	String ctrlName

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfRatio=W_DFREF[3]
	SetDataFolder dfRatio
	Wave W_NumTrame
	NVAR V_NumCoucheCourante, V_NbChiffresFichier, Dim
	SVAR S_Nom_Manip, S_Chemin_JPEG, S_Chemin_Export
	String sQuote = "\"", sTemp, sListeType, sListeDestination, sTitreDestination, sChemin, sNomFichier
	variable vTemp, vType, vResolution

	//	Image types that are offered, depending on the platform
	if (CmpStr(IgorInfo(2), "Macintosh")==0)
		sTemp="Quartz pdf;"
	else
		sTemp="Enhanced Windows Metafile;"
	EndIf
	sListeType=sQuote+sTemp+"png;tif;svg;"+sQuote

	//	Prepare for a modal window
	NewDataFolder/O root:tmp_SauveVersImage	//	to store the result of the action
	Variable/G root:tmp_SauveVersImage:Annule	//	will store the result of "do it" or "cancel"
	NewPanel /K=1 /W=(100,100,450,400) as "Exporting as image"	//	create the panel
	DoWindow/C F_SauveVersImage	//	give it a name
	SetDrawEnv fsize=9,fstyle=3; DrawText 25, 15, "Hint: press \"tab\" to jump to the \"Do it\" button"
	//	create the controls on this window
	PopupMenu C_PopupDestinationFichier,pos={15,30},fsize=12,size={170,20},help={"Select to export the data"}
	PopupMenu C_PopupTypeFichier,pos={15,90},fsize=12,size={170,20},help={"Select the format for export"}
	PopupMenu C_PopupTypeFichier,title="image type",noproc,value=#sListeType,mode=1
	Button C_BoutonValide,pos={15,120},fsize=12,fStyle=1,size={70,20},proc=BoutonExecuteSauveVersImage,title="Do it"

	Button C_BoutonAnnule,pos={100,120},fsize=12,fStyle=2,size={70,20},proc=BoutonExecuteSauveVersImage,title="Cancel"
	Button C_BoutonTest,pos={200,120},fsize=12,size={70,20},proc=BoutonTest,title="Test",disable=1

	//	Depending on the button that called this procedure, sets-up the control properties
	StrSwitch(ctrlName)
		case "C_SauveVignetteChamp":
			PopupMenu C_PopupDestinationFichier,title="to:",disable=2,mode=1,value=#(sQuote+"database folder"+sQuote)
			SetDrawEnv fsize=10,fstyle=2; DrawText 15, 60, RaccourcitTexte(S_Chemin_JPEG, 25, 60)
			PopupMenu C_PopupTypeFichier,disable=2,mode=2
//			vType=
		break
		case "C_SauveVignetteTraces":
//			PopupMenu C_PopupDestinationFichier,title="Export graph to database folder."
		break
		case "C_Sauve1Image":
			sListeDestination=sQuote+"_PictGallery_;Export folder;"+sQuote
			PopupMenu C_PopupDestinationFichier,title="Export a single image to:",mode=1,value=#sListeDestination
		break
		case "C_SauveSerie":
			sTitreDestination="Export a series of images to export folder."
		break
		case "C_SauveGraphesVersPict":
			sTitreDestination="Export graphs to:"
		break
		default:
			sTitreDestination="Rien du tout"
		break
	EndSwitch

//	PathInfo export	//	Detects if the export path exists
//	if (V_Flag==0)	//	if not, offers to create it
//		NewPath/M="Path to export files:" export
//		if (V_flag)
//			return(0)	//	do something in case it was cancelled
//		endif
//	endif


	//	Stops interaction with any other window until done with this one
	PauseForUser F_SauveVersImage	//	making the window modal until BoutonExecuteSauveVersImage kills this window

	NVAR vAnnule=root:tmp_SauveVersImage:Annule
	if (vAnnule)	//	check if global variable reports that "Cancel" was pressed
		DoAlert 0, "Cancelled by user"
	else	//	proceed with export
//		SavePict/O/E=(vType)/B=72/P=$sChemin/N=F_RM as sNomExport
	endif
	//	Clean-up after use
	KillDataFolder root:tmp_SauveVersImage
End

//	Function called by a button on F_SauveVersImage
Function BoutonTest(ctrlName) : ButtonControl
	String ctrlName
	DoAlert 0, "Ca marche !"
End

//	Function to close the modal window and proceed with saving or cancelling
//	If user cancelled, the information is saved into root:tmp_SauveVersImage:Annule
Function BoutonExecuteSauveVersImage(ctrlName) : ButtonControl
	String ctrlName
	NVAR vAnnule=root:tmp_SauveVersImage:Annule
	StrSwitch(ctrlName)
		case "C_BoutonValide":
			vAnnule=0
		break
		case "C_BoutonAnnule":
			vAnnule=1
		break
	EndSwitch
	DoWindow/K F_SauveVersImage
End

// *** \\\--- Old versison of BoutonSauveImage ***
//	une image qui représente F_RM, ou bien un layout complet incluant F_RM et des graphes, ou bien toutes les images brutes:
//	export de soit une image unique qui va vers la galerie de peinture ou vers le chemin calcul
//	soit export d'une série d'images; indiquer le premier et dernier n° à exporter; export uniquement vers le chemin calcul
//	nom: si export vers le chemin calcul, tous les noms sont autorisés
//	si export vers la galerie de peinture, uniquement un nom défini par l'utilisateur

Function BoutonSauveImage(ctrlName) : ButtonControl
	String ctrlName

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfRatio=W_DFREF[3]
	Wave/SDFR=dfRatio W_NumTrame
	NVAR/SDFR=dfRatio V_NumCoucheCourante, V_NbChiffresFichier, Dim
	SVAR/SDFR=dfRatio S_Nom_Manip

	Variable vOu	//	lieu de l'export; 1=calcul; 2=galerie de peinture
	Variable vType	//	1=pdf; 2=Tif; 3=png
	Variable vType2	//	code pour export;
	String sPrefixeNom=""
	Variable vTypeImage
	variable debut=0, fin=Dim-1
	String sChemin, sNomExport, sSuffixe, sType, sNomSerie=""
	
//	on laisse le choix à l'utilisateur de où les ranger, par défaut dans la galerie de peinture
	prompt vOu, "Where to export the image:", popup, "_PictGallery_;export folder"
	prompt vType, "What format:", popup, "pdf;tif;png"
	prompt sPrefixeNom, "Add a prefix to the image name: "
	DoPrompt "Export current image", vOu, vType, sPrefixeNom
	if (V_Flag)
		print "Cancelled by user"
		return(0)
	endif

	//	sélection du type de ficher exporté
	Switch (vType)
		case 1:
			sSuffixe=".pdf"
			vType2=-8
		break
		case 2:
			sSuffixe=".tif"
			vType2=-7
		break
		case 3:
			sSuffixe=".png"
			vType2=-5
		break
	EndSwitch
	vTypeImage=QuelAffichage()
	if (vTypeImage == 0)	//	affichage de l'image ratio
		sType="RM"
	else
		sType=StringFromList(vTypeImage-1, ksABC)
	endif
	//	On ajoute le préfixe
	if (StrLen(sPrefixeNom)>0)
		sType+="_"+sPrefixeNom
	endif
	
	Strswitch(ctrlName)
		case "C_Sauve1Image":
			//	sélection du chemin et num de fichier de l'export
			Switch(vOu)
				case 1:	// galerie de peinture; nom court
					sChemin="_PictGallery_"
		//			sNomExport=AjouteNumero(sType+"_",((W_NumTrame[V_NumCoucheCourante])), V_NbChiffresFichier)
					sNomExport=AjouteNumero(sType+"_",V_NumCoucheCourante, V_NbChiffresFichier)
				break
				case 2:	//	chemin des données; nom comportant le n° de la manip
					sChemin="export"
		//			sNomExport=AjouteNumero(S_Nom_Manip+"_"+sType,(W_NumTrame[V_NumCoucheCourante]), V_NbChiffresFichier)+sSuffixe
					sNomExport=AjouteNumero(S_Nom_Manip+"_"+sType,V_NumCoucheCourante, V_NbChiffresFichier)+sSuffixe
				break
			EndSwitch
			SavePict/O/E=(vType2)/B=72/P=$sChemin/N=F_RM as sNomExport
		break
		case "C_SauveSerie":
			Prompt debut, "First image to put in the movie"
			Prompt fin, "Last image"
			DoPrompt "Export active window as movie", debut, fin
			if (V_Flag)	//	annulé
				return(0)
			endif
		
			for (V_NumCoucheCourante=debut; (V_NumCoucheCourante<=fin); V_NumCoucheCourante+=1)
				if (V_NumCoucheCourante==debut)
					CheckAfficheROI("C_AfficheROI_TousPlans",1)
				else
					CheckAfficheROI("C_AfficheROI_TousPlans",0)
				endif
				MajImages(0)	//	met à jour la fenêtre de premier plan pour chaque point de la série
				sNomExport=AjouteNumero(S_Nom_Manip+"_"+sType,W_NumTrame[V_NumCoucheCourante], V_NbChiffresFichier)
				SavePict/O/E=(vType2)/B=72/P=export/N=F_RM as sNomExport
			endfor
		break
	endswitch
End

//	Exporte F_TracesTriees en image, soit dans le répertoire calcul, soit dans la galerie de peinture
Function BoutonSauveGrapheImage(ctrlName) : ButtonControl
	string ctrlName

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfRatio=W_DFREF[3]
	SVAR/SDFR=dfRatio S_Nom_Manip

	variable a, b
	string chemin, nomFichier
	DoWindow/F F_TracesTriees
	prompt a, "What format", popup, "pdf;png;svg"
	prompt b, "Where", popup, "pict gallery;export folder;"
	DoPrompt "Exported graph:", a, b
	if (V_Flag)
		print "annulé par l'utilisateur"
		return(0)
	endif

	Switch(b)	//	chemin de l'export
		case 1:	// galerie de peinture
			chemin="_PictGallery_"
			DoWindow/F F_TracesTriees
			nomFichier="Sorted_traces"
			SavePict/O/E=-8/P=$chemin as nomFichier
			DoWindow/F F_Traces
			nomFichier="All_traces"
			SavePict/O/E=-8/P=$chemin as nomFichier
		break
		case 2:	//	chemin des données
			chemin="export"
			nomFichier=S_Nom_Manip+" Traces"
			PathInfo export	//	Déclenche une erreur et stoppe la boucle si le chemin n'est pas accessible
			if (V_Flag==0)	//	le chemin "data" ne correspond à rien
				NewPath/M="Path to export files:" export
			endif
			Switch(a)
				case 1:	//	pdf
					NomFichier+=".pdf"
					SavePict/O/E=-8/P=$chemin as nomFichier
				break
				case 2:	//	png
					NomFichier+=".png"
					SavePICT/O/P=$chemin/E=-5/RES=150 as nomFichier
				break
				case 3:	//	svg
					NomFichier+=".svg"
					SavePICT/O/P=$chemin/E=-9 as nomFichier
				break
			endswitch
		break
	EndSwitch
end

//	To save an image stamp as a "memory" of the experiment : one for the field of view, one for the traces
//	Will be exported in the path "fichier_JPEG" which should be where images are stored for access with a database manager
//	Historically in JPEG, traces in pdf, now PNG is preferred
Function BoutonSauveVignette(ctrlName) : ButtonControl
	string ctrlName

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfRatio=W_DFREF[3]
	NVAR/SDFR=dfRatio V_Zoom
	SVAR/SDFR=dfRatio S_Nom_Manip

	StrSwitch(ctrlName)
		case "C_SauveVignetteChamp":
			variable vZoomSav=V_Zoom
			V_Zoom=0.4
			SetVarZoom("",NaN,"","")
			SavePict/O/E=-6/N=F_RM/P=fichier_JPEG as S_Nom_Manip+".jpg"
			V_Zoom=vZoomSav
			SetVarZoom("",NaN,"","")
		break
		case "C_SauveVignetteTraces":
			CheckAffichePosition("", 0)
			SavePict/O/B=(2*72)/E=-5/N=F_TracesTriees/P=fichier_JPEG as S_Nom_Manip+"_Traces"+".png"
		break
	EndSwitch
end

Function SauvePrefs()
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfRatio=W_DFREF[3]
	DFREF dfSav=GetDataFolderDFR()
	SetDataFolder dfRatio
	Wave W_WindowPositions
	//	Get the position of the windows which position should be remembered, and saves it in the table
	GetWindow/Z P_Initialisation wsizeRM
	W_WindowPositions[0][0]=V_Left; W_WindowPositions[0][1]=V_Top
	GetWindow P_Controle wsizeRM
	W_WindowPositions[1][0]=V_Left; W_WindowPositions[1][1]=V_Top
	GetWindow F_RM wsizeRM
	W_WindowPositions[2][0]=V_Left; W_WindowPositions[2][1]=V_Top
	GetWindow F_Traces wsizeRM
	W_WindowPositions[3][0]=V_Left; W_WindowPositions[3][1]=V_Top
	GetWindow F_TracesTriees wsizeRM
	W_WindowPositions[4][0]=V_Left; W_WindowPositions[4][1]=V_Top
	SaveData/O/Q/J=ksGlobalPrefs/P=prefs "Raplouf_Prefs.pxp"
	Print "preferences saved"
	SetDataFolder dfSav
End

// *** Exporting movies ***
Function ExporteFilm()

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfRatio=W_DFREF[3]
	Wave/SDFR=dfRatio W_Numtrame
	NVAR/SDFR=dfRatio V_NumCoucheCourante, Dim, V_NbChiffresFichier
	SVAR/SDFR=dfRatio S_Nom_Manip
	
	variable vFilmOuExport=0		//	0 pour film; 1 pour export
	string sExtension
	
	variable debut=0, fin=Dim-1, frq=4, res=100
	variable i
	string nomFichier
	Prompt debut, "First image to put in the movie"
	Prompt fin, "Last image"
	Prompt frq, "nb of images per second"
	Prompt res, "Resolution"
	Switch(vFilmOuExport)
		case 0:	//	film
			DoPrompt "Export active window as movie", debut, fin, frq, res
			sExtension=""
		break
		case 1: 	//	export
			DoPrompt "Export active window as series of TIF images", debut, fin
			sExtension=".TIF"
		break
	endswitch
	if (V_Flag)	//	annulé
		return(0)
	endif

	for (V_NumCoucheCourante=debut; (V_NumCoucheCourante<=fin); V_NumCoucheCourante+=1)
		if (V_NumCoucheCourante==debut)
			CheckAfficheROI("C_AfficheROI_TousPlans",1)
		else
			CheckAfficheROI("C_AfficheROI_TousPlans",0)
		endif
		MajImages(0)	//	met à jour la fenêtre de premier plan pour chaque point de la série
//		nomFichier=AjouteNumero("Film",W_NumTrame[V_NumCoucheCourante], V_NbChiffresFichier)+sExtension
		if (vFilmOuExport==0)
//			SavePICT/O/E=-5/RES=(res)/P=_PictGallery_ as nomFichier
			if (V_NumCoucheCourante==debut)	//	à exécuter seulement pour la première trame
				NewMovie/O/CF=0/P=calcul/F=(frq) as S_Nom_Manip+"_FilmT"
			endif
			AddMovieFrame
		else
			SavePICT/O/E=-7/RES=(res)/P=calcul as nomFichier			
		endif
	endfor
	if (vFilmOuExport==0)
		CloseMovie
	endif
	SignalSonore(5)	
End

//	fait défiler toutes les valeurs de V_NumCoucheCourante et transfère l'image couleur vers une pile créée pour l'occasion. 
//	Cette pile sera exportée sous forme d'un film
//Function ExporteFilm()
//
//	Wave/DF W_DFREF=root:W_DFREF; DFREF dfRatio=W_DFREF[3]
//	Wave/SDFR=dfRatio W_Numtrame
//	NVAR/SDFR=dfRatio V_NumCoucheCourante, Dim, V_NbChiffresFichier
//	SVAR/SDFR=dfRatio S_Nom_Manip
//	
//	variable vFilmOuExport=0		//	0 pour film; 1 pour export
//	string sExtension
//	
//	variable debut=0, fin=Dim-1, frq=4, res=100
//	variable i
//	string nomFichier
//	Prompt debut, "First image to put in the movie"
//	Prompt fin, "Last image"
//	Prompt frq, "nb of images per second"
//	Prompt res, "Resolution"
//	Switch(vFilmOuExport)
//		case 0:	//	film
//			DoPrompt "Export active window as movie", debut, fin, frq, res
//			sExtension=""
//		break
//		case 1: 	//	export
//			DoPrompt "Export active window as series of TIF images", debut, fin
//			sExtension=".TIF"
//		break
//	endswitch
//	if (V_Flag)	//	annulé
//		return(0)
//	endif
//
//	for (V_NumCoucheCourante=debut; (V_NumCoucheCourante<=fin); V_NumCoucheCourante+=1)
//		if (V_NumCoucheCourante==debut)
//			CheckAfficheROI("C_AfficheROI_TousPlans",1)
//		else
//			CheckAfficheROI("C_AfficheROI_TousPlans",0)
//		endif
//		MajImages(0)	//	met à jour la fenêtre de premier plan pour chaque point de la série
//		nomFichier=AjouteNumero("Film",W_NumTrame[V_NumCoucheCourante], V_NbChiffresFichier)+sExtension
//		if (vFilmOuExport==0)
//			SavePICT/O/E=-5/RES=(res)/P=_PictGallery_ as nomFichier
//			if (V_NumCoucheCourante==debut)	//	à exécuter seulement pour la première trame
//				NewMovie/O/P=calcul/CF=0/PICT=$nomFichier/F=(frq) as S_Nom_Manip+"_FilmT"+".mov"
//			endif
//			AddMovieFrame/PICT=$nomFichier
//			KillPICTs $nomFichier
//		else
//			SavePICT/O/E=-7/RES=(res)/P=calcul as nomFichier			
//		endif
//	endfor
//	if (vFilmOuExport==0)
//		CloseMovie
//	endif
//	SignalSonore(5)	
//End

//	fait défiler toutes les valeurs de V_NumPlan et transfère l'image de F_RM vers une pile créée pour l'occasion. 
//	Cette pile sera exportée sous forme d'un film
Function ExporteFilmZ()
	
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfRatio=W_DFREF[3]
	NVAR/SDFR=dfRatio V_NumPlan, Dim, V_NbChiffresFichier, V_NbPlans
	SVAR/SDFR=dfRatio S_Nom_Manip
	
	variable debut=0, fin=V_NbPlans-1, frq=4, res=300, k
	variable i
	string nomFichier

	Prompt debut, "Premier plan de la pile"
	Prompt fin, "Dernier plan"
	Prompt frq, "nombre d'image par seconde"
	Prompt res, "Résolution"
	DoPrompt "Exporte fenêtre F_RM", debut, fin,  frq, res
	if (V_Flag)	//	annulé
		return(0)
	endif
	for (V_NumPlan=debut; (V_NumPlan<=fin); V_NumPlan+=1)
		GlissierePlan("",V_NumPlan,0)
		DoWindow/F F_RM
//		ModifyImage/W=F_RM ''#0 plane= V_NumPlan
		nomFichier=AjouteNumero("FilmZ",V_NumPlan, V_NbChiffresFichier)
		SavePICT/O/E=-5/RES=(res)/P=_PictGallery_ as nomFichier
		if (V_NumPlan==debut)	//	à exécuter seulement pour la première trame
			NewMovie/O/P=calcul/CTYP="mp4v"/PICT=$nomFichier/F=(frq) as S_Nom_Manip+"_FilmZ"+".mov"
		endif
		AddMovieFrame/PICT=$nomFichier
		KillPICTs $nomFichier
	endfor
	CloseMovie
End


// *** Image mirror ***
//	Status saved in root:ratio:V_Miroir bitwise variable: bit 0: horizontal mirror; bit 1: vertical mirror; bit 2: rotation +90°
//	The function sets the checkboxes depending on the variable value; -> called by CheckMiroir("init",0) during panel initialization
Function CheckMiroir(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfRatio=W_DFREF[3]
	NVAR/SDFR=dfRatio V_Miroir, V_Taille_X, V_Taille_Y
	variable vTemp

	if (checked)
		StrSwitch(ctrlName)
			case "C_MiroirHorizontal":
				V_Miroir = V_Miroir | (1)
			break
			case "C_MiroirVertical":
				V_Miroir = V_Miroir | (2)
			break
			case "C_Rotation":
				V_Miroir = V_Miroir | (4)
				//	Exchange X and Y dimensions
				vTemp=V_Taille_X; V_Taille_X=V_Taille_Y; V_Taille_Y=vTemp
			break
		EndSwitch
	else
		StrSwitch(ctrlName)
			case "C_MiroirHorizontal":
				V_Miroir = V_Miroir & ~ (1)
			break
			case "C_MiroirVertical":
				V_Miroir = V_Miroir & ~ (2)
			break
			case "C_Rotation":
				V_Miroir = V_Miroir & ~ (4)
				//	Exchange X and Y dimensions
				vTemp=V_Taille_X; V_Taille_X=V_Taille_Y; V_Taille_Y=vTemp
			break
		EndSwitch
	endif
	//	Checks the boxes according to the value
	CheckBox C_MiroirHorizontal win=P_Initialisation, value=((V_Miroir & 1) != 0)
	CheckBox C_MiroirVertical win=P_Initialisation, value=((V_Miroir & 2) != 0)
	CheckBox C_Rotation win=P_Initialisation, value=((V_Miroir & 4) != 0)
	//	After the mirror/rotation has changed, the calculation images must be redimensioned
	RedimensionImages("rien")
	//	Checks all checkboxes for ratio calculation
	MaJ_CasesACocher("Tous", 0, "", "")
	//	Clear cache folder
	BoutonSupprimeTemp("C_ClearCache_NoConfirmation")
End

//	Multi-purpose function for button calls on P_Initialisation

//	Called when the experiment name is changed, or nb of characters in file name
//	-> suggest a name for the experiment, file index
//	update the title of data windows
Function SetVarNomManip(ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName
	Variable varNum
	String varStr
	String varName

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfRatio=W_DFREF[3]
	Wave/SDFR=dfRatio W_NumTrame
	NVAR/SDFR=dfRatio V_NbLettresRacine, V_NbLettresInutiles, V_NbChiffresFichier, V_TimepointsMerged, V_NbFichiers
	SVAR/SDFR=dfRatio S_Nom_Manip, S_NomPremierFichier, S_Base_Nom_Fichier_A, S_Suffixe_A, S_Base_Nom_Fichier_B

	String sTemp
	Variable i

	StrSwitch(ctrlName)
		case "C_Nom_Manip":	//	the user edited the experiment name => change nothing to file names, strings etc...
		break
		Default:	//	applies to all situations where the experiment name should be guessed
			//	Managing file names, playing with strings
			S_Nom_Manip=ReplaceString("/", S_NomPremierFichier[0,V_NbLettresRacine-1], "-")
			S_Base_Nom_Fichier_A=S_NomPremierFichier[0, V_NbLettresRacine+V_NbLettresInutiles-1]
			S_Suffixe_A=S_NomPremierFichier[V_NbLettresRacine+V_NbLettresInutiles+V_NbChiffresFichier, inf]
		
			//	Trying to read the file index in the name of the first file
			if (V_TimepointsMerged==0)
				sTemp=S_NomPremierFichier[V_NbLettresRacine+V_NbLettresInutiles, V_NbLettresRacine+V_NbLettresInutiles+V_NbChiffresFichier-1]
				W_NumTrame[0]=str2num(sTemp)		//	if str2num fails to read a number, it puts a NaN in W_NumTrame
				if (numtype(W_NumTrame[0])!=0)
					W_NumTrame[0]=0
				endif
			endif

			//	Knowing the file name for channel A, try to build the name for channels B, C...
			if (CmpStr(S_Suffixe_A, "A")==0)	//	check if the suffix is just the letter A
				for (i=1; (i<V_NbFichiers); i+=1) //	If the suffix in the file name is "A", automatically puts "B", "C"... as suffix for other channels
					sTemp="S_Base_Nom_Fichier_"+StringFromList(i, ksABC)
					SVAR/SDFR=dfRatio sString=$sTemp
					sString=S_Base_Nom_Fichier_A
					sTemp="S_Suffixe_"+StringFromList(i, ksABC)
					SVAR/SDFR=dfRatio sString=$sTemp
					sString=StringFromList(i, ksABC)
				endfor
			endif
		break
	EndSwitch
	
//	//	Update C_Index on control panel
//	sTemp=""
//	For (i=0; (i<V_NbChiffresFichier); i+=1)
//		sTemp+="X"
//	endfor
//	if (V_NbChiffresFichier==1)
//		sTemp="◼︎"
//	endif
//	TitleBox/Z C_Index win=P_Initialisation,title=sTemp

	//	Update window name based on experiment name
	DoWindow/T F_RM, S_Nom_Manip+": ratio image"
	DoWindow/T F_Traces, S_Nom_Manip+": all measurements"
	DoWindow/T F_TracesTriees, S_Nom_Manip+": sorted traces"
End

//	______________ Initialisation of paths to system folders ________________
//	Called by the popup to enter preset values, but also by buttons via the relay function BoutonInitialisation
//	updates the strings and display on P_Initialisation
Function PopMenuInitChemins(ctrlName,popNum,popStr) : PopupMenuControl
	String ctrlName
	Variable popNum
	String popStr

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfRatio=W_DFREF[3]
	SVAR/SDFR=dfRatio S_Chemin_Calcul, S_Chemin_Donnees, S_Chemin_Analyses, S_Chemin_JPEG, S_Nom_Manip, S_Racine_Locale, S_Chemin_Local, S_Chemin_Export
	string sNom, sSep, sNomServeur
	
	//	Define the syntax to use depending on Mac or Win
	StrSwitch(IgorInfo(2))
		case "Macintosh":
			sSep=":"
			sNomServeur="perroy:commun:Common lab data:Microscopy:"
		break
		case "Windows":
//			sSep="\\"
			sSep=":"
//			sNomServeur="\\\\ocean.igf.internal\\perroy\\commun\\Common lab data\\Ratio\\"
			sNomServeur="Z:commun:Common lab data:Microscopy:"
		break
	EndSwitch

	StrSwitch(ctrlName)
		case "Initialise":	//	called at initialisation, after loading prefs, to create the path according to the saved pref
			NewPath/O/Q/Z data, S_Chemin_Donnees
			NewPath/Q/O/Z calcul, S_Chemin_Calcul
			NewPath/O/Q/Z fichier_analyse, S_Chemin_Analyses
			NewPath/O/Q/Z fichier_JPEG, S_Chemin_JPEG
		break
		case "C_ChoixServeurOuLocal":
			Switch(popNum)
				case 1:
					return(0)
				break
				case 2:	//	Server Ocean (current in Montpellier)
					//	Mac: /perroy/commun/Common lab data/Ratio/Images/2020/2020-11-17/2020-11-17B001/2020-11-17B001_001A.tif
					//	PC: \\ocean.igf.internal\perroy\commun\...
					S_Chemin_Donnees=sNomServeur+"Images"+sSep+S_Nom_Manip[0,3]+sSep+S_Nom_Manip[0,9]+sSep+S_Nom_Manip[0,13]+sSep
					NewPath/O/Q/Z data, S_Chemin_Donnees
					//	/perroy/commun/Common lab data/Ratio/Analyses/2020/2020-11-17B001.pxp
					S_Chemin_Analyses=sNomServeur+"Raplouf"+sSep+S_Nom_Manip[0,3]+sSep
					NewPath/Z/O/Q fichier_analyse, S_Chemin_Analyses
					//	/perroy/commun/Common lab data/Ratio/Linked files/2020/2020-11-17B001.jpg
					S_Chemin_JPEG=sNomServeur+"Linked files"+sSep+S_Nom_Manip[0,3]+sSep
					NewPath/Z/O/Q fichier_JPEG, S_Chemin_JPEG
				break
				case 3:	//	Server DISTT (ancient location)
					//	typically /Partage DISTT/Données/Ratio/Images/2020/2020-11-17/2020-11-17B001/2020-11-17B001_001A.tif
					S_Chemin_Donnees="Partage DISTT:Données:Ratio:Images:"+S_Nom_Manip[0,3]+":"+S_Nom_Manip[0,9]+":"+S_Nom_Manip[0,13]+":"
					NewPath/O/Q/Z data, S_Chemin_Donnees
					//	/Partage DISTT/Données/Ratio/Analyses/2020/2020-11-17B001.pxp
					S_Chemin_Analyses="Partage DISTT:Données:Ratio:Analyses:"+S_Nom_Manip[0,3]+":"
					NewPath/Z/O/Q fichier_analyse, S_Chemin_Analyses
					//	/Partage DISTT/Données/Ratio/Fichiers liés/2020-11-17B001.jpg
					S_Chemin_JPEG="Partage DISTT:Données:Ratio:Fichiers liés:"+S_Nom_Manip[0,3]+":"
					NewPath/Z/O/Q fichier_JPEG, S_Chemin_JPEG
				break
				case 4:	//	local
					S_Chemin_Analyses=SpecialDirPath("Documents", 0, 0, 0)
					NewPath/Z/O/Q fichier_analyse, S_Chemin_Analyses
					S_Chemin_JPEG=SpecialDirPath("Documents", 0, 0, 0)
					NewPath/Z/O/Q fichier_JPEG, S_Chemin_JPEG
				break
				case 5:	//	old serveur
				break
			EndSwitch

			//	cache for temp files; user and machine-specific
			//	on se met un petit plaisir: la détection automatique de ma session pour sélectionner le bon dossier temp sur le disque dur local
			if (CmpStr(IgorInfo(7), "pierre")==0)
				PathInfo Igor	//	to get the name of the hard disk
				sNom=StringFromList(0, S_Path, ":")
				StrSwitch(sNom)
					case "Montagne maison":
						NewPath/Q/O/Z calcul, "Stockage:Données courantes labo:Temp"
					break
					case "Portable Pierre":
						NewPath/Q/O/Z calcul, "Portable Pierre:Users:Shared:Temp Raplouf"
					break
					default:
						NewPath/Q/O/Z calcul, "Disque SSD:Temp:"
					break
				EndSwitch
			else
				S_Chemin_Calcul=SpecialDirPath("Documents", 0, 0, 0)+"Wavemetrics:Temp:"
			endif
		break
		case "C_ChoixCheminDonnees":
			NewPath/O/Q/M="Where raw data are located" data
		break
		case "C_ChoixCheminAnalyses":
			NewPath/O/Q/M="Where the analysis file will be saved" fichier_analyse
		break
		case "C_ChoixCheminJPEG":
			NewPath/O/Q/M="Where an image and trace will be exported (to the server)" fichier_JPEG
		break
		case "C_ChoixCheminCalcul":
			NewPath/Q/O/M="Cache for calculation images" calcul
		break
		case "C_ChoixCheminExport2":
			NewPath/Q/M="Folder for exporting final figures" export
		break
	EndSwitch

	//	If NewPath failed, or if for some reason, the default ended up being the application's folder, defaults to "Documents" 
	PathInfo data
	if ((V_Flag==0) || GrepString(S_Path, "Applications"+sSep+"Igor Pro"))
		NewPath/Z/O/Q data, SpecialDirPath("Documents", 0, 0, 0)
	endif
	PathInfo fichier_analyse
	if ((V_Flag==0) || GrepString(S_Path, "Applications"+sSep+"Igor Pro"))
		NewPath/Z/O/Q fichier_analyse, SpecialDirPath("Documents", 0, 0, 0)
	endif	
	PathInfo fichier_JPEG
	if ((V_Flag==0) || GrepString(S_Path, "Applications"+sSep+"Igor Pro"))
		NewPath/Z/O/Q fichier_JPEG, SpecialDirPath("Documents", 0, 0, 0)
	endif
	PathInfo calcul
	if ((V_Flag==0) || GrepString(S_Path, "Applications"+sSep+"Igor Pro"))
		NewPath/Z/O/Q calcul, SpecialDirPath("Documents", 0, 0, 0)
	endif
	PathInfo local
	if ((V_Flag==0) || GrepString(S_Path, "Applications"+sSep+"Igor Pro"))
		NewPath/Z/O/Q local, SpecialDirPath("Documents", 0, 0, 0)
	endif

//	Updates the string definition with the actual path
	Pathinfo data; S_Chemin_Donnees=S_Path
	Pathinfo fichier_analyse; S_Chemin_Analyses=S_Path
	PathInfo fichier_JPEG; S_Chemin_JPEG=S_Path
	PathInfo calcul; S_Chemin_Calcul=S_Path
	PathInfo local; S_Racine_Locale=S_Path
	PathInfo export; S_Chemin_Export=S_Path

	//	Writes the path string on the Initialisation window; if needed, the string is shortened in the center
	//	This must be done here, as there is no ValueDisplay that could show the shortened text
	DoWindow P_Initialisation
	if (V_Flag==1)	//	do it only if the window exists
		TitleBox C_NomCheminDonnees win=P_Initialisation,title=RaccourcitTexte(S_Chemin_Donnees, 25, 60)
		TitleBox C_NomCheminAnalyses win=P_Initialisation,title=RaccourcitTexte(S_Chemin_Analyses, 25, 60)
		TitleBox C_NomCheminJPEG win=P_Initialisation,title=RaccourcitTexte(S_Chemin_JPEG, 25, 60)
		TitleBox C_NomCheminCalcul win=P_Initialisation,title=RaccourcitTexte(S_Chemin_Calcul, 25, 60)
		TitleBox C_NomCheminExport win=P_Initialisation,title=RaccourcitTexte(S_Chemin_Export, 25, 60)
	endif
End

// *** Load a set of predefined values ***
//	Presets parameters for handling image saving scheme and file names
//	Can be passed a string through popStr to trigger the correct action
Function PopMenuPreReglage(string ctrlName,variable popNum, string popStr) : PopupMenuControl
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfRatio=W_DFREF[3]
	NVAR/SDFR=dfRatio V_ChannelsMerged		//	1 if all channels are merged into a single file
	NVAR/SDFR=dfRatio V_Split				//	1 if the single image contains both wavelength side by side (image splitter)
	NVAR/SDFR=dfRatio V_TimepointsMerged	//	1 if all time points are merged into a single file
	NVAR/SDFR=dfRatio V_NbFichiers			//	nb of files for user to enter; only controls the number of available lines to enter file definition
	NVAR/SDFR=dfRatio V_NbVoies				//	number of channels containing image data
	NVAR/SDFR=dfRatio V_NbLettresRacine		//	nb of characters in the file name that define the experiment name 
	NVAR/SDFR=dfRatio V_NbLettresInutiles	//	nb of meaningless characters before the file index
	NVAR/SDFR=dfRatio V_NbChiffresFichier	//	nb of digits in the file index
	NVAR/SDFR=dfRatio V_TimeSelector		//	how time for each time point is obtained
	SVAR/SDFR=dfRatio S_Type_Image			//	to keep the user-entered type of data
	
	S_Type_Image=popStr	
	StrSwitch(S_Type_Image)
		case "iVision 2018":
			V_ChannelsMerged=0		//	1 if all channels are merged into a single file
			V_Split=0				//	1 if the single image contains both wavelength side by side (image splitter)
			V_TimepointsMerged=0	//	1 if all time points are merged into a single file
			V_NbFichiers=2			//	nb of files for user to enter; only controls the number of available lines to enter file definition
			V_NbVoies=2				//	number of channels containing image data
			V_NbLettresRacine=14	//	nb of characters in the file name that define the experiment name 
			V_NbLettresInutiles=1	//	nb of meaningless characters before the file index
			V_NbChiffresFichier=3	//	nb of digits in the file index
			V_TimeSelector=1		//	time is read in the first pixel of the image
		break
		case "NAF split":
			V_ChannelsMerged=1		//	1 if all channels are merged into a single file
			V_Split=1				//	1 if the single image contains both wavelength side by side (image splitter)
			V_TimepointsMerged=1	//	1 if all time points are merged into a single file
			V_NbFichiers=1			//	nb of files for user to enter; only controls the number of available lines to enter file definition
			V_NbVoies=2				//	number of channels containing image data
			V_NbLettresRacine=14	//	nb of characters in the file name that define the experiment name 
			V_NbLettresInutiles=0	//	nb of meaningless characters before the file index
			V_NbChiffresFichier=3	//	nb of digits in the file index
			V_TimeSelector=3		//	regular sampling
		break
		case "Poste biolum":
			V_ChannelsMerged=0		//	1 if all channels are merged into a single file
			V_Split=0				//	1 if the single image contains both wavelength side by side (image splitter)
			V_TimepointsMerged=0	//	1 if all time points are merged into a single file
			V_NbFichiers=2			//	nb of files for user to enter; only controls the number of available lines to enter file definition
			V_NbVoies=2				//	number of channels containing image data
			V_NbLettresRacine=14	//	nb of characters in the file name that define the experiment name 
			V_NbLettresInutiles=9	//	nb of meaningless characters before the file index
			V_NbChiffresFichier=1	//	nb of digits in the file index
			V_TimeSelector=3		//	time is read in the first pixel of the image
		break
		case "Biphoton":	
			V_ChannelsMerged=0		//	1 if all channels are merged into a single file
			V_Split=0				//	1 if the single image contains both wavelength side by side (image splitter)
			V_TimepointsMerged=0	//	1 if all time points are merged into a single file
			V_NbFichiers=2			//	nb of files for user to enter; only controls the number of available lines to enter file definition
			V_NbVoies=2				//	number of channels containing image data
			V_NbLettresRacine=10	//	nb of characters in the file name that define the experiment name 
			V_NbLettresInutiles=2	//	nb of meaningless characters before the file index
			V_NbChiffresFichier=3	//	nb of digits in the file index
			V_TimeSelector=1		//	time is read with the specific 2 photon code
		break
		case "Optosplit":
			V_ChannelsMerged=1		//	1 if all channels are merged into a single file
			V_Split=1				//	1 if the single image contains both wavelength side by side (image splitter)
			V_TimepointsMerged=0	//	1 if all time points are merged into a single file
			V_NbFichiers=1			//	nb of files for user to enter; only controls the number of available lines to enter file definition
			V_NbVoies=2				//	number of channels containing image data
			V_NbLettresRacine=14	//	nb of characters in the file name that define the experiment name 
			V_NbLettresInutiles=0	//	nb of meaningless characters before the file index
			V_NbChiffresFichier=3	//	nb of digits in the file index
			V_TimeSelector=1		//	check
		break
		case "tif":
			V_ChannelsMerged=0		//	1 if all channels are merged into a single file
			V_Split=0				//	1 if the single image contains both wavelength side by side (image splitter)
			V_TimepointsMerged=0	//	1 if all time points are merged into a single file
			V_NbFichiers=2			//	nb of files for user to enter; only controls the number of available lines to enter file definition
			V_NbVoies=2				//	number of channels containing image data
			V_TimeSelector=2		//	time is read from the image file time stamp
		break
		case "tif stack":
			V_ChannelsMerged=0		//	1 if all channels are merged into a single file
			V_Split=0				//	1 if the single image contains both wavelength side by side (image splitter)
			V_TimepointsMerged=1	//	1 if all time points are merged into a single file
			V_NbFichiers=2			//	nb of files for user to enter; only controls the number of available lines to enter file definition
			V_NbVoies=2				//	number of channels containing image data
			V_TimeSelector=3		//	regular sampling
		break
		case "iVision 2":
			V_ChannelsMerged=0		//	1 if all channels are merged into a single file
			V_Split=0				//	1 if the single image contains both wavelength side by side (image splitter)
			V_TimepointsMerged=0	//	1 if all time points are merged into a single file
			V_NbFichiers=2			//	nb of files for user to enter; only controls the number of available lines to enter file definition
			V_NbVoies=2				//	number of channels containing image data
			V_NbLettresRacine=11	//	nb of characters in the file name that define the experiment name 
			V_NbLettresInutiles=0	//	nb of meaningless characters before the file index
			V_NbChiffresFichier=3	//	nb of digits in the file index
			V_TimeSelector=1		//	time is read in the first pixel of the image
		break
		case "iVision":	//	old iVision scheme
			V_ChannelsMerged=1		//	1 if all channels are merged into a single file
			V_Split=0				//	1 if the single image contains both wavelength side by side (image splitter)
			V_TimepointsMerged=0	//	1 if all time points are merged into a single file
			V_NbFichiers=1			//	nb of files for user to enter; only controls the number of available lines to enter file definition
			V_NbVoies=2				//	number of channels containing image data
			V_NbLettresRacine=11	//	nb of characters in the file name that define the experiment name 
			V_NbLettresInutiles=0	//	nb of meaningless characters before the file index
			V_NbChiffresFichier=4	//	nb of digits in the file index
			V_TimeSelector=1		//	time is read in the first pixel of the image
		break
		case "---":	//	nothing
		break
		default:
		break
	EndSwitch
	
	CheckTimeSelector("",0)						//	Update radio button appearance on P_Initialisation to match V_TimeSelector value
	SetVarNbFichiers("",V_NbFichiers,"","")		//	Updates enable status of file names setvars
	SetVarNbVoies("",V_NbVoies,"","")			//	Adapts popups and buttons to match the actual number of channels to handle
End
 
//________________________________________________________________________________________________________________________________________________________
//	******************************************		compress the experiment file before saving		******************************************
Function CompresseManip()
	variable i, j
	SetDataFolder root:
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfExport=W_DFREF[1]; DFREF dfRatio=W_DFREF[3]
	string sNomWave, sNomFenetre, sNomFenetresExistant, sNomUneFenetre, sListeWaves, sNomvariable, sListeVariables, sNom, sListeDF, sListe
	SVAR/SDFR=dfRatio S_Nom_Manip

	//	Many windows can be closed when compressing. sListeNomFenetre lists the root of window names that can be closed.
	//	For example, all windows starting with "F_Panneau" (many) are closed
	String sListeNomFenetre="F_Pile;F_Panneau;F_Initialisation;F_RM;F_AB;F_Pile_RM;F_GrapheDecalage;F_GrapheDeplacement;P_TableauCarreCalib;P_MoveROI;"
	sListeNomFenetre+="F_Evenements;F_ROI;F_MesuresRatio;F_PdV;F_Ref;F_Test;F_Concentration;F_CurseursMesure;F_Export;F_PanneauDeplaceROI;P_DeplaceROI;P_TableauROI;P_ROI;"
	sListeNomFenetre+="P_Evts;P_Export;P_Deplacement;P_Decalage;P_Derive_Base;P_Initialisation;P_Controle;P_PdV;P_Masquage;T_DFREF;P_Recadrage_Image;P_Seuillage;"
	sListeNomFenetre+="T_Pulse;P_Pulse;P_Temps;"//P_Kymograph;"
	sListeNomFenetre+="F_InterpProfileGraphe;F_ProfileGraphe;T_PointsProfil;"
	//	Not yet implemented: if the analysis file is less than something, then also remove old outdated F_Traces and F_TracesTriees
	if (0)
		sListeNomFenetre+="F_Traces;F_TracesTriees;"
	endif
	
	//	Lists all the waves that are not necessary to keep
	sListeWaves="P_A;P_Image;P_Image0;P_Image1;D_ACorr;D_BCorr;D_Ratio;D_IMoy;D_RatioxIMoy;D_IMoyCorr;D_RatioCorr;D_Saturation;D_RM;D_RMbyte;"
	sListeWaves+="D_Temp;D_HSLTemp;M_HSL2RGB;M_ROIMask;P_Masque1;D_Image0;D_Image1;M_OffsetImage;P_MasqueDep;P_MasqueDec;"
	sListeWaves+="M_RegMaskOut;M_RegOut;M_InterpolatedImage;M_Masque0;M_Masque1;D_Carrei;root:D_CarreR;D_Ref;Unif0;Unif1;"
	sListeWaves+="P_TempA0;P_TempB0;D_Temp0;P_Optosplit;P_Ref;M_zProjection;P_RefDep0;P_A0;P_B0;P_B1;"
	sListeWaves+="P_Image1;P_IMoy;P_temp0;P_temp1;M_AveImage;M_ImagePlane;M_StdvImage;P_Ratio;P_RatioTemp;P_RatioxIMoy;P_Ref;"
	sListeWaves+="P_A;P_ACorr;P_B;P_BCorr;P_Image0;P_Image1;P_IMoy;P_IMoyCorr;P_RatioCorr;P_Temp;P_RM;P_Masque0;pRef;pTest;P_Tmp;P_AProj;P_BProj;"
	sListeWaves+="P_Corr_Ref;fit_Ch1Tot;fit_Ch2Tot;P_ACorr_Zone;P_BCorr_Zone;P_IMoy_Zone;P_RatioxIMoy_Zone;"
	sListeWaves+="W_Temp;M_Convolution;M_MatrixFilter;P_RatioRef;P_Pile;W_RegParams;P_PilePrecedente;"
	sListeWaves+="D_UnifA;D_UnifB;"	//	Critical decision: should we keep the unif file within the analysis or not ?
	
	//	List of all old objects that can be disposed of, but only if they are at the root
	String sListeOldRootWaves="P_RefDep1;W_NumTrame2;W_TypeImage;W_Curseurs;W_coef;W_CouleurTagROI;W_CouleurTagTraces;W_DRsurR;W_Fit;W_Ratio;W_RatioTag;W_sigma;W_TempsCorr;D_CarreR;W_CouleurROI;W_CouleurTrace;W_EpaisseurTrace;W_Fond;W_PlanROI;W_Tag;"
	sListeOldRootWaves+="W_AfficheROI;W_AfficheTrace;W_EpaisseurROI;W_Intensite;W_Reponse;W_Tag;"
	String sListeOldRootVariables="V_NbChiffresParPile;V_NbTrame;V_NumChamp;V_NumDernierFichierCVZ;V_NumFichCourant;V_PlanDec;V_Reanalyse;V_SeuilCanal;V_SeuilROI;V_SeuilValeur;V_3D_2D;V_Position0Recalage;V_Position0DeplaceROI;V_NbFonds;"
	sListeOldRootVariables+="V_Taille_X2;V_Taille_Y2;V_TempsDernierFichierCVZ;V_ModeCurseurs;V_numImages;V_VoieRefDeplacement;V_DecalageX;V_DecalageY;V_DeplacementX;V_DeplacementY;V_DeplacementZ;V_ModeAnalyse;V_FondA;V_FondB;V_NumROI;V_Tag;"
	String sListeOldRootStrings="S_ABC;S_Nom_Fenetre_Mesure;S_fileName;S_info;S_path;S_waveNames;S_Affichage;S_NomImageCorr;S_NomImageRef;S_SuffixeA;S_SuffixeB;S_Fonds;"

	//	Useless root variables generated during normal operation
	sListeVariables="V_bottom;V_left;V_right;V_top;V_marquee;V_Flag"

	NVAR/SDFR=dfROI V_NbROI
	For (i=1; (i<=V_NbROI); i+=1)
		DependencesROI(0, i)	//	Suppresses dependencies in mobile ROIs
	endfor

	//	Kills windows first, to later allow killing objects that may be represented in these windows
	sNomFenetresExistant=WinList("*",";","")
	For (i=0; (i<ItemsInList(sNomFenetresExistant)); i+=1)
		sNomUneFenetre=StringFromList(i, sNomFenetresExistant)
		For (j=0; (j<ItemsinList(sListeNomFenetre)); j+=1)
			sNomFenetre=StringFromList(j, sListeNomFenetre)
			if (StringMatch(sNomUneFenetre, sNomFenetre+"*")==1)
				killwindow/Z $sNomUneFenetre
			endif
		EndFor
	EndFor
	
	//	Suppresses the datafolder that contains initialization values; up-to-date init values will be loaded next time
	KillDataFolder/Z root:Initialisation
	KillDataFolder/Z root:z_Initialisation
	
	//	Kills waves in the list
	For (i=0; (i<ItemsInList(sListeWaves)); i+=1)
		sNomWave=StringFromList(i, sListeWaves)
		KillWaves/Z root:$sNomWave
		KillWaves/Z dfRatio:$sNomWave
	endfor

	//	More waves to kill, which depend on a more specific selection:
	//	Image stacks
	for (j=0; j<7; j+=1)
		sNomWave="P_"+StringFromList(j, ksABC)
		KillWaves/Z dfRatio:$sNomWave
		sNomWave+="Corr"
		KillWaves/Z dfRatio:$sNomWave
	endfor
	
	sListeWaves=WaveList("*_Stats", ";", "")
	for (i=0; (i<ItemsInList(sListeWaves)); i+=1)
		sNomWave=StringFromList(i, sListeWaves)
		Wave wPile=$sNomWave
		KillWaves wPile
	endfor					

	SetDataFolder dfRatio
	sListeWaves=WaveList("P_RefDep*", ";", "")
	sListeWaves+=WaveList("P_CorrDep*", ";", "")
	for (i=0; (i<ItemsInList(sListeWaves)); i+=1)
		sNomWave=StringFromList(i, sListeWaves)
		KillWaves/Z dfRatio:$sNomWave
	endfor

	//	Kill all masks ("Masque*") and P_Ref in Analyse_* datafolders
	SetDataFolder root:
	sListeDF=ListeDF("root:", "Analyse_")
	for (i=0; (i<ItemsInList(sListeDF)); i+=1)
		sNom=StringFromList(i, sListeDF)
		SetDataFolder root:$sNom
		sListe=WaveList("Masque*", ";", "")
		for (j=0; (j<ItemsInList(sListe)); j+=1)
			sNom=StringFromList(j, sListe)
			KillWaves/Z $sNom
		endfor
		KillWaves/Z D_Masque, P_Ref
	endfor
	SetDataFolder root:
	
	//	Kill useless variables
	For (i=0; (i<ItemsInList(sListeVariables)); i+=1)
		sNomvariable=StringFromList(i, sListeVariables)
		KillVariables/Z root:$sNomvariable
		KillVariables/Z dfRatio:$sNomvariable
		KillStrings/Z root:gWMSetNextTextFilesTextEncoding
	endfor
	
	//	Kill 3D ROI masks
	GetFileFolderInfo/Q/P=data/Z "ROI_3D"
	if (V_Flag==0)
		DeleteFolder/M="Supprimer le dossier temporaire des masques 3D?"/P=data "ROI_3D"
	endif
	
	DoWindow F_TracesTriees
	if (V_Flag)
		HideInfo/W=F_TracesTriees
	endif
	
	//	Final root cleanup
	SetDataFolder root:
	KillDataFolder/Z root:Concentration		//	concentration feature removed from Raplouf
	//	Kill useless root objects
	sListeOldRootWaves+=WaveList("fit_TraceMoy*", ";", "")
	sListeOldRootWaves+=WaveList("TraceMoy*", ";", "")
	sListeOldRootWaves+=WaveList("TraceRCorr*", ";", "")
	sListeOldRootWaves+=WaveList(S_Nom_Manip+"_*", ";", "")
	For (i=0; (i<ItemsInList(sListeOldRootWaves)); i+=1)
		sNom=StringFromList(i, sListeOldRootWaves)
		KillWaves/Z root:$sNom
	endfor
	For (i=0; (i<ItemsInList(sListeOldRootVariables)); i+=1)
		sNom=StringFromList(i, sListeOldRootVariables)
		KillVariables/Z root:$sNom
	endfor
	For (i=0; (i<ItemsInList(sListeOldRootStrings)); i+=1)
		sNom=StringFromList(i, sListeOldRootStrings)
		KillStrings/Z root:$sNom
	endfor
	
end
//	******************************************		end of compress		******************************************


// *** Instrument-specific pieces of code ***
//________________________________________________________________________________________________________________________________________________________
//	This part contains a few pieces of code that read specific files associated with specific instruments:
//	Two-photon microscope, Image splitter, iVision events...

//	_________________		Taille des pixels pour le biphoton	_____________________________
//	valeurs utilisées par défaut jusqu'au 2016-10-25
//	objectif 60x; 512 pixels -> 130 µm
//	zoom 1	1 pixel = 0,254
//	zoom 1,3	=0,195
//	zoom 1,4	=0,181
//	zoom 1,5	=0,169	
//	A partir du 2016-10-25u (ou peut-être déjà avant cette date, à vérifier), on a une nouvelle taille de pixel = 0,366 µm
	Constant kTaillePixelBiphoton=0.366

//	Lit les fichiers contenant les indications d'événements de la manip
Function BoutonLitEvt(ctrlName) : ButtonControl
	String ctrlName

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	Wave/SDFR=dfRatio W_EvtTemps, W_NumTrame, W_Evt
	Wave/T/SDFR=dfRatio W_EvtStr
	SVAR/SDFR=dfRatio S_Type_Image, S_Nom_Manip, S_Base_Nom_Fichier_A

	variable i, j, vResultat=0
	String sListeNomsFich	//	liste des noms de fichiers
	String sListeNomsFichEvt
	String sNomFich	//	le nom du fichier evt

	SetDataFolder dfRatio
	
	StrSwitch(S_Type_Image)
		case "iVision 2018":
		case "iVision 2":
			sListeNomsFich=IndexedFile(data, -1, ".csv")
			//	Recherche dans la liste le nom qui contient "evt"
			sListeNomsFichEvt=ListMatch(sListeNomsFich, "*evt*")	//	retourne la liste des noms de fichiers contenant "evt"
			if (ItemsInList(sListeNomsFichEvt)==1)	//	 un unique fichier csv, dont on suppose que c'est le fichier evt
				sNomFich=StringFromList(0, sListeNomsFichEvt)
				LoadWave/A/ENCG=2/Q/O/J/L={0,0,0,0,2}/B="F=-2,N=W_EvtStr;F=0,N=W_EvtTemps;"/P=data sNomFich
				W_EvtTemps/=60	// pour mettre le temps en minutes
				for (i=0; (i<DimSize(W_EvtTemps,0));i+=1)	// on remplace les temps 0 par des NaN pour éviter un graphe moche
					if (W_EvtTemps[i]!=0 )
						W_Evt[i]=1
						W_EvtStr[i]=ReplaceString("\"", W_EvtStr[i], "")		//	enlève les guillemets
					else
						W_EvtTemps[i]=NaN
						W_Evt[i]=0
						W_EvtStr[i]=""
					endif
				endfor
				vResultat=1
			else
				vResultat=0
			endif
		break
		case "iVision":
			sListeNomsFich=IndexedFile(data, -1, ".txt")
			sNomFich="Evt.txt"
			if (FindListItem(sNomFich, sListeNomsFich)>=0)
				LoadWave/A/Q/O/J/L={0,4,0,0,2}/B="F=-2,N=W_EvtStr;F=0,N=W_EvtTemps;"/P=data sNomFich
				W_EvtTemps/=60	// pour mettre le temps en minutes
				for (i=0; (i<DimSize(W_EvtTemps,0));i+=1)	// on remplace les temps 0 par des NaN pour éviter un graphe moche
					if (W_EvtTemps[i]!=0 )
						W_Evt[i]=1
					else
						W_EvtTemps[i]=NaN
						W_Evt[i]=0
					endif
				endfor
				vResultat=1
			else
				vResultat=0
			endif
		break
		case "Biphoton":	//	fichiers générés par le biphoton; le temps correspond à un point d'acquisition d'une image
			sListeNomsFich=IndexedFile(data, -1, ".txt")
			sNomFich="Evt_2ph.txt"
			if (FindListItem(sNomFich, sListeNomsFich)>=0)
				LoadWave/A/Q/O/J/L={0,1,0,0,3}/B="F=0,N=W_Test;F=0,N=W_poube;F=-2,N=W_EvtStrTemp;"/V={"\t"," $",0,0}/P=data sNomFich
				Wave W_Test, W_Temps_min
				Wave/T W_EvtStrTemp
				W_Evt=NaN
				W_EvtStr=""
				for (i=0; (i<DimSize(W_Test,0)); i+=1)	//	pour chaque evt noté...
					for (j=0; (j<DimSize(W_NumTrame,0)); j+=1)	// pour chaque n° de fichier
						if (W_Test[i]==W_NumTrame[j])	//	teste si égalité, c'àd si l'evt correspond à un n° de fichier existant
							W_EvtTemps[i]=W_Temps_min[j]
							W_Evt[i]=1
							W_EvtStr[i]=W_EvtStrTemp[i]
						endif
					endfor
				endfor
				KillWaves/Z W_Poube, W_Test, W_EvtStrTemp
				vResultat=1
			else
				vResultat=0
			endif
		break
		default:
			vResultat=0
		break
	EndSwitch
	SetDataFolder root:	
	return(vResultat)
End

//	Fonction qui lit le temps dans les fichiers config Leica 
Function Lit1ParamLeica(i)
	variable i	//	 n° du point pour lequel on lit le temps

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	Wave/SDFR=dfRatio W_NumTrame, W_Temps, W_Exposition
	NVAR/SDFR=dfRatio V_NbChiffresFichier
	SVAR/SDFR=dfRatio S_Base_Nom_Fichier_A

	String Nom_Fichier, ligne, temp
	Variable refnum, position, hh, mm, ss, t
	
//	Nom_Fichier=AjouteNumero(S_Base_Nom_Fichier_A,W_NumTrame[i], V_NbChiffresFichier)+"_Properties.xml"
	Nom_Fichier="Series024_Properties.xml"
	open/Z/R/P=data refnum as Nom_Fichier
	if (V_Flag!=0)
//		print "fichier de description non trouvé"
		return(0)
	endif
	//	Lecture exposition voie A
	do
		FReadLine refnum, ligne
		position=StrSearch(ligne, "ChannelDescription2",0)	//	détecte la fin de ligne
	while ((position==-1) && (strlen(ligne)>0))
	position=StrSearch(ligne, "Exposure",0)
	W_Exposition[i][0]=str2num(ligne[position+10, position+13])/1000
	//	Lecture exposition voie B
	do
		FReadLine refnum, ligne
		position=StrSearch(ligne, "ChannelDescription2",0)	//	détecte la fin de ligne
	while ((position==-1) && (strlen(ligne)>0))
	position=StrSearch(ligne, "Exposure",0)
	W_Exposition[i][1]=str2num(ligne[position+10, position+13])/1000
	do
		FReadLine refnum, ligne
		position=StrSearch(ligne, "</StartTime>",0)	//	détecte la fin de ligne
	while ((position==-1) && (strlen(ligne)>0))
	ligne=ligne[position-13, position-1]
	temp=StringFromList(0, ligne, ":")
	hh=str2num(temp)
	temp=StringFromList(1, ligne, ":")
	mm=str2num(temp)
	temp=StringFromList(2, ligne, ":")
	ss=str2num(temp)
	t=3600*hh+60*mm+ss
	if (t==0)
		t=NaN
	endif
	W_Temps[i]=t
	close(refnum)
	return(1)
end

Function LitParamsLeica()
	variable i
	
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	NVAR/SDFR=dfRatio Dim
	for (i=0; (i<Dim); i+=1)
		Lit1ParamLeica(i)
	endfor
end

//	Fonction qui lit les paramètres de manip biphoton; infos stockées dans un fichier texte; zoom, gain PMT
Function LitParamsBiphoton(ctrlName)	: ButtonControl	//	retourne 1 si tout a fonctionné
	String ctrlName
	
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	Wave/SDFR=dfRatio W_NumTrame
	NVAR/SDFR=dfRatio V_PMTA, V_PMTB, V_GainA, V_GainB
	SVAR/SDFR=dfRatio S_Base_Nom_Fichier_A
	DFREF dfInit=root:z_Initialisation:f_Global:
	Wave/SDFR=dfInit fit_Ch1Tot, fit_Ch2Tot

	Variable refnum, position
	String ligne, temp

	//	sélection du fichier contenant les infos biphoton
	//	nom du type 100615_001.txt; nom de fichier basé sur le premier fichier enregistré de la manip
	String nomFich=S_Base_Nom_Fichier_A[0,5]+"_"
	nomFich=AjouteNumero(nomFich, W_NumTrame[0], 3)
	nomFich+=".txt"
	open/Z/R/P=data refnum as nomFich
	if (V_Flag!=0)
		open/D=2/R/P=data refnum		//	Displays a dialog to select the file
		if (StrLen(S_fileName)>0)		//	a file is selected
			Open/R/P=data refnum	as S_fileName	//	Open the file and update refnum to point to that file
		else
			Print "No description file loaded"
			return(0)
		endif
	endif

	do
		FReadLine refnum, ligne
		position=StrSearch(ligne, "GainPMT1",0)
	while ((position==-1) && (strlen(ligne)>0))
	V_PMTB=str2num(ligne[8,30])

	FSetPos refNum, 0	//	on doit se repositionner au début du fichier pour être sûr de tout lire
	do
		FReadLine refnum, ligne
		position=StrSearch(ligne, "GainPMT2",0)
	while ((position==-1) && (strlen(ligne)>0))
	V_PMTA=str2num(ligne[8,30])
	
	//	fait calculer la valeur du gain V_Gain à partir de la tension du PMT V_PMT
	V_GainA=fit_Ch1Tot(V_PMTA)
	V_GainB=fit_Ch2Tot(V_PMTB)
	
	//	gère le facteur de zoom du biphoton
	FSetPos refNum, 0	//	on doit se repositionner au début du fichier pour être sûr de tout lire
	do
		FReadLine refnum, ligne
		position=StrSearch(ligne, "Zoom:",0)
	while ((position==-1) && (strlen(ligne)>0))
	position=StrSearch(ligne, ",",0)
	variable zoom=Str2Num("1."+ligne[position+1,50])
	NVAR/SDFR=dfRatio V_Resolution
	V_Resolution=kTaillePixelBiphoton/zoom
	close(refnum)
	return(1)
end

Function LitTempsBiphoton()
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	Wave/SDFR=dfRatio W_Temps, W_Temps_min
	NVAR/SDFR=dfRatio V_NbLettresRacine, V_NbLettresInutiles
	SVAR/SDFR=dfRatio S_Base_Nom_Fichier_A

	variable i
	string nomfich		//	on veut au final   "100113uRatio.txt"

	nomfich=S_Base_Nom_Fichier_A[0, (V_NbLettresRacine-1)]+"Ratio.txt"
	//	Skip the first line; only read the first column; use space as delimiter
	LoadWave/A/Q/O/J/L={0,1,0,0,1}/B="F=0,N=W_Temps;"/V={" ","$",0,0}/P=data nomfich
	i=DimSize(W_Temps, 0)							//	nb of valid time values
	Redimension/N=(DimSize(W_Temps_min, 0)) W_Temps	//	So the two waves match in size, because W_Temps_min is dependent on W_Temps
	W_Temps[i, inf]=NaN								//	Clears the 0 that have been added in the previous line
end
//	End of code that read specific files associated with specific instruments:
//________________________________________________________________________________________________________________________________________________________

Function BoutonEvt(ctrlName) : ButtonControl
	String ctrlName
	
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	Wave/SDFR=dfRatio/T W_EvtStrCorr
	String sListeTraces=TraceNameList("F_TracesTriees", ";", 1)
	StrSwitch(ctrlName)
		case "C_AjouteBarres":
			if (WhichListItem("W_EvtCorr", sListeTraces)==-1)	//	 la trace n'est pas déjà présente
				AppendToGraph/R/L=right/W=F_TracesTriees dfRatio:W_EvtCorr vs dfRatio:W_EvtTempsCorr2
				ModifyGraph/W=F_TracesTriees lsize(W_EvtCorr)=2,rgb(W_EvtCorr)=(0,0,0),mode(W_EvtCorr)=4,textMarker(W_EvtCorr)={W_EvtStrCorr,"default",0,0,0,0.00,1.00}
				ModifyGraph/W=F_TracesTriees noLabel(right)=2,axThick(right)=0,standoff=0
			endif
		break
		case "C_EnleveEvt":
			RemoveFromGraph/Z/W=F_TracesTriees W_Evt#1,W_Evt, W_EvtCorr#1
		break
	EndSwitch
End

Function SetVarEvt(ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName
	Variable varNum
	String varStr
	String varName

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	NVAR/SDFR=dfRatio V_Evt_Epaiss, V_Evt_Justif, V_Evt_OffX, V_Evt_OffY, V_Evt_Size
	
	ModifyGraph lsize(W_EvtCorr)=V_Evt_Epaiss,msize(W_EvtCorr)=V_Evt_Size,textMarker(W_EvtCorr)={root:Ratio:W_EvtStrCorr,"default",0,0,V_Evt_Justif,V_Evt_OffX,V_Evt_OffY}
End

// *** Saving the experiment ***
//	Executed only the first time the experiment is saved.
//	Gives a name to the experiment, based on the unique ID S_Nom_Manip
Function SauveManip()
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfRatio=W_DFREF[3]
	SVAR/SDFR=dfRatio S_Nom_Manip, S_Chemin_Analyses
	
	CtrlNamedBackground AjoutePointsAuto, stop

// *** Saving preferences ***
	SauvePrefs()
	
	DoAlert 1, "Compress the experiment and save it in the folder "+S_Chemin_Analyses+" as "+S_Nom_Manip+".pxp"+" ?"
	if (V_Flag!=1)
		return(0)
	endif
	GetFileFolderInfo/Q/Z=1/P=fichier_analyse S_Nom_Manip+".pxp"
	if (V_Flag==0)	// le fichier existe
		DoAlert 1, "This file name already exists; OK to overwrite ?"
		if (V_Flag!=1)
			return(0)
		endif
	endif
	Gere_DFREF("Sauve")
	CompresseManip()
	SaveExperiment/P=fichier_analyse as S_Nom_Manip+".pxp"
	ExperimentModified 0
	BoutonSupprimeTemp("C_ClearCache_NoConfirmation")
End

//	Hook function: executed before saving: checks that the file does not contain useless things
Function BeforeExperimentSaveHook(refNum, fileNameStr, pathNameStr, fileTypeStr, fileCreatorStr, fileKind)
	Variable refNum,fileKind
	String fileNameStr, pathNameStr, fileTypeStr, fileCreatorStr
	if (fileKind==1)	//	Do something only if it is a packed experiment

		//	Test if it is a Ratioscope analysis file
		if (DataFolderExists("root:Ratio"))	//	Ratioscope type of data
			//	Stop the background function, if running
			CtrlNamedBackground AjoutePointsAuto, stop
			//	make sure the text wave saves the current selection of DFREFs
			Gere_DFREF("Sauve")

// *** Saving version number ***
			NVAR vVersion=root:Ratio:V_VersionRatio
			vVersion=VersionNumber("Ratioscope package globals.ipf")

			DoWindow F_RM
			if  (DataFolderExists("root:z_Initialisation"))
				if (V_Flag)
					SauvePrefs()				// *** Saving preferences ***
				endif
				//	Proposes to compress before saving
				//	Would be nice to offer "Cancel" as an option, to prevent saving; but the value returned by BeforeExperimentSaveHook is ignored, saving can't be aborted.
				DoAlert 1, "The file should probably be compressed before saving. Yes: clear cache, compress and then save the experiment; No: keep cache and save experiment as it is."
				Switch(V_Flag)
					case 1:	//	yes, compress
						CompresseManip()		//	removes large useless objects 
						BoutonSupprimeTemp("C_ClearCache_NoConfirmation")	//	clears the cache
						return(0)	//	will proceed with saving
					break
					case 2:	//	no
						return(0)	//	do nothing : save the way it is.
					break
				EndSwitch
			endif
		endif

	endif
	Return(0)
End


Function BoutonInitialisation(String ctrlName) : ButtonControl
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfRatio=W_DFREF[3]
	Wave/SDFR=dfRatio W_NumTrame,W_Temps,W_Exposition
	NVAR/SDFR=dfRatio Dim
	StrSwitch(ctrlName)
		case "C_EditTable":
			Edit/K=1 W_NumTrame,W_Temps,W_Exposition
		break
		case "C_ChoixCheminDonnees":
		case "C_ChoixCheminAnalyses":
		case "C_ChoixCheminJPEG":
		case "C_ChoixCheminCalcul":
		case "C_ChoixCheminExport2":
			PopMenuInitChemins(ctrlName,0,"")
		break
	EndSwitch
End
