#pragma TextEncoding = "UTF-8"
#pragma rtGlobals=3		// Use modern global access method and strict wave access.

// *** Copyright ***
// © 2023 CNRS Shared by Pierre VINCENT, under the MIT License.
//	Contact: pierre.vincent@cnrs.fr; IGF, 141 rue de la Cardonille, 34094 Montpellier cedex 5, France.

// *** \\\	gradient image should be updated when the kymo line # is changed ***

//	Called by CheckActivePanneau when the checkbox is checked
Function Init_P_Kymograph()
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
//	NVAR/SDFR=dfROI V_NbROI
//	Wave/SDFR=dfROI W_Tag
//	Wave/SDFR=dfRatio/T W_NombreTagStr

	variable i
	string sNom=StringFromList(6, ksNomDataFolders)
	SetDataFolder root:
	if (!DataFolderExists(sNom))
		NewDataFolder $sNom
	endif
	DFREF dfKymo=$sNom
	W_DFREF[6]=dfKymo
	InitDataFolder("Init raplouf.pxp", "f_Kymograph", "Kymograph")
	SetDataFolder dfKymo
	
	KillWindow/Z P_Kymograph
	Execute "P_Kymograph()"
	//	Set default settings for checkboxes and other things on P_Kymograph
	DoWindow/F P_Kymograph
	CheckBox C_AfficheProfil value=1

	SetFormula W_PosKymoX, "root:Ratio:V_NumCoucheCourante"
	SetVarEdges("",0,"","")	//	to place the thresholds of edges on the kymograph
	UpdateProfilesGraph()	//	to display the profile lines on F_RM

End

Function Kymograph(wX, wY, vWidth)
	Wave wX, wY	//	waves defining X and Y coordinates of the profile
	Variable vWidth		//	profile is measured across this much width

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]; DFREF dfKymo=W_DFREF[6]
	NVAR/SDFR=dfRatio Dim
	
	variable i
	string sNomFenetre

	SetDataFolder dfKymo		//	To return the resulting kymographs in the Pulse datafolder
	Wave D_Kymograph_Intensity, D_Kymograph_Ratio, D_Kymograph_RM, D_Kymograph_Filtered

	//	Need to run the profile once in order to know its size
	ImageLineProfile xWave=wX, yWave=wY, srcwave=root:Ratio:P_IMoy, width=vWidth
	Wave W_ImageLineProfile
	
	//	Redimension kymograph waves, using the number of time points in X and size of profile in Y dimensions
	Redimension/N=(Dim, DimSize(W_ImageLineProfile,0), 1) D_Kymograph_Intensity, D_Kymograph_Ratio
	Redimension/N=(Dim, DimSize(W_ImageLineProfile,0), 3, 1)/U/W D_Kymograph_RM
	
	//	Calculating kymographs for intensity and ratio
	for (i=0; (i<Dim);i+=1)	//	Cycle through all time points
		GereImages(i, 1, 4)	//	gets the P_Ratio, P_RatioxIMoy and P_IMoy images from cache
		ImageLineProfile xWave=wX, yWave=wY, srcwave=root:Ratio:P_IMoy, width=vWidth
		D_Kymograph_Intensity[i][]=W_ImageLineProfile[q]
		ImageLineProfile xWave=wX, yWave=wY, srcwave=root:Ratio:P_Ratio, width=vWidth
		D_Kymograph_Ratio[i][]=W_ImageLineProfile[q]
	endfor
	KillWaves W_ImageLineProfile	//	Done with this one, can be killed

// derivative of the intensity
	Duplicate/O D_Kymograph_Intensity, D_Kymograph_Filtered
	MatrixFilter gradN D_Kymograph_Filtered	
	
	//	Generate the pseudocolored kymograph
	IntensityModulatedDisplay(D_Kymograph_Ratio, D_Kymograph_Intensity, D_Kymograph_RM)
End

//	Calculate the position of the region edges on the kymograph.
//	Edges are calculated on the derivative of the intensity image
Function Edges()
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfRatio=W_DFREF[3]; DFREF dfKymo=W_DFREF[6]
	variable i

	SetDataFolder dfKymo

	Wave D_Kymograph_Intensity, D_Kymograph_Ratio, D_Kymograph_RM, D_Kymograph_Filtered
	Wave W_Edge_1, W_Edge_2, W_Distance
	NVAR V_Edge_1_bottom,V_Edge_1_top, V_Edge_2_bottom,V_Edge_2_top, V_NumKymograph
	NVAR/SDFR=dfRatio V_Resolution, V_NbChiffres
		
	//	Calculate the distance between the two edges
	Make/O/FREE/N=(DimSize(D_Kymograph_Filtered, 1)) wLine
	Redimension/N=(DimSize(D_Kymograph_Filtered, 0)) W_Edge_1, W_Edge_2, W_Distance
	for (i=0; (i<DimSize(D_Kymograph_Filtered, 0)); i+=1)
		wLine=D_Kymograph_Filtered[i][p]	//	wLine is the vertical line extracted from filtered kymograph
		WaveStats/Q/M=1/R=[V_Edge_1_bottom,V_Edge_1_top] wLine
		W_Edge_1[i]=V_maxloc
		WaveStats/Q/M=1/R=[V_Edge_2_bottom,V_Edge_2_top] wLine
		W_Edge_2[i]=V_minloc
		W_Distance[i]=abs(W_Edge_1[i]-W_Edge_2[i])*V_Resolution
	endfor

	//	Duplicate the final result, W_Distance, into a wave with the correct name
	string sWaveName=AjouteNumero("Kymo", V_NumKymograph, V_NbChiffres)
	Duplicate/O W_Distance, $sWaveName	
End

//	Respond to any button pressed on the Pulse window
//	Handles profile drawing for making a kymograph
Function BoutonKymograph(ctrlName) : ButtonControl
	String ctrlName

	Variable vXMin, vXMax, vYMin, vYMax	//	to determine the min and max positions of the profile
	String sNameProfile_X, sNameProfile_Y
	String sList, sNom
	
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]; DFREF dfKymo=W_DFREF[6]
	SetDataFolder dfKymo
	NVAR V_ProfileThickness, V_NumKymograph
	Wave/SDFR=dfKymo D_Kymograph_Intensity, D_Kymograph_Ratio, D_Kymograph_RM, D_Kymograph_Filtered
	DoWindow/F F_RM	
	sNameProfile_X="W_Profile_X_"+Num2Str(V_NumKymograph)
	sNameProfile_Y="W_Profile_Y_"+Num2Str(V_NumKymograph)
	Wave/Z wProfile_X=$sNameProfile_X, wProfile_Y=$sNameProfile_Y
	if (!WaveExists(wProfile_X))	//	In case the profiles have not been created yet
		//	Makes a wave to define the profile, with convenient starting values
		GetAxis/W=F_RM/Q top
		vXMin=trunc(V_Min+0.2*(V_max-V_min))
		vXMax=trunc(V_Min+0.8*(V_max-V_min))
		GetAxis/W=F_RM/Q left
		vYMin=trunc(V_Min+0.2*(V_max-V_min))
		vYMax=trunc(V_Min+0.8*(V_max-V_min))
		Make/N=2 $sNameProfile_X, $sNameProfile_Y
		Wave wProfile_X=$sNameProfile_X, wProfile_Y=$sNameProfile_Y
		wProfile_X[0]=vXMin; wProfile_X[1]=vXMax
		wProfile_Y[0]=vYMin; wProfile_Y[1]=vYMax
	endif

	UpdateProfilesGraph()

	StrSwitch(ctrlName)
		case "C_BoutonEditKymo":		//	edit the waves defining the profile
			GraphWaveEdit $sNameProfile_Y
		break
		case "C_BoutonValidateKymo":
			GraphNormal	//	quits interactive wave editing mode
			//	calls kymo calculation
			Kymograph(wProfile_X, wProfile_Y, V_ProfileThickness)
			//	Duplicates kymo images by adding kymo #
			sNom="D_Kymograph_Intensity_"+Num2Str(V_NumKymograph)
			Duplicate/O D_Kymograph_Intensity, $sNom
			sNom="D_Kymograph_Ratio_"+Num2Str(V_NumKymograph)
			Duplicate/O D_Kymograph_Ratio, $sNom
			sNom="D_Kymograph_RM_"+Num2Str(V_NumKymograph)
			Duplicate/O D_Kymograph_RM, $sNom
			sNom="D_Kymograph_Filtered_"+Num2Str(V_NumKymograph)
			Duplicate/O D_Kymograph_Filtered, $sNom
			Edges()
		break
	EndSwitch
End

//	Automatic update from the window except:
//	- remove draw commands at the end of the code
Window P_Kymograph() : Graph
	PauseUpdate; Silent 1		// building window...
	String fldrSav0= GetDataFolder(1)
	SetDataFolder root:Kymograph:
	Display /W=(628,240,1501,879)/K=2 /T W_Edge_1,W_Edge_2 as "Kymograph"
	AppendToGraph/L=Distance/T W_Distance
	AppendToGraph/R/T W_PosKymoY vs W_PosKymoX
	AppendImage/T D_Kymograph_RM
	ModifyImage D_Kymograph_RM ctab= {*,*,Grays,0}
	AppendImage/T D_Kymograph_Intensity
	ModifyImage D_Kymograph_Intensity ctab= {*,*,Grays,0}
	AppendImage/T D_Kymograph_Filtered
	ModifyImage D_Kymograph_Filtered ctab= {*,*,Grays,0}
	SetDataFolder fldrSav0
	ModifyGraph margin(left)=56,margin(bottom)=14,margin(top)=14,margin(right)=14
	ModifyGraph mode(W_PosKymoY)=1
	ModifyGraph rgb(W_Edge_2)=(1,52428,26586),rgb(W_Distance)=(52428,1,41942),rgb(W_PosKymoY)=(65535,32764,16385)
	ModifyGraph mirror(left)=0,mirror(top)=0,mirror(right)=0
	ModifyGraph nticks(left)=3,nticks(top)=3
	ModifyGraph minor(left)=1,minor(top)=1
	ModifyGraph fSize(left)=9,fSize(top)=9
	ModifyGraph standoff(left)=0,standoff(top)=0,standoff(right)=0
	ModifyGraph axThick(right)=0
	ModifyGraph lblPos(left)=14
	ModifyGraph tkLblRot(left)=90
	ModifyGraph btLen(left)=3,btLen(top)=3
	ModifyGraph tlOffset(left)=-2,tlOffset(top)=-2
	ModifyGraph freePos(Distance)=0
	ModifyGraph axisEnab(left)={0.17,1}
	ModifyGraph axisEnab(Distance)={0,0.15}
	SetAxis/A/E=1 Distance
	SetAxis right 0,1
	ControlBar 25
	ControlBar/L 120
	Button C_BoutonEditKymo,pos={12.00,44.00},size={90.00,20.00},proc=BoutonKymograph
	Button C_BoutonEditKymo,title="Edit profile"
	Button C_BoutonEditKymo,help={"Graphically edit the wave that define the profile"}
	Button C_BoutonEditKymo,fSize=11
	Button C_BoutonValidateKymo,pos={12.00,71.00},size={90.00,20.00},proc=BoutonKymograph
	Button C_BoutonValidateKymo,title="Validate profile"
	Button C_BoutonValidateKymo,help={"Ends the interactive edition"},fSize=11
	GroupBox C_KymographBox,pos={4.00,27.00},size={108.00,96.00},title="Kymograph"
	SetVariable C_Thickness,pos={14.00,99.00},size={90.00,16.00},proc=SetVarKymograph
	SetVariable C_Thickness,title="thickness",help={"how thick is the profile"}
	SetVariable C_Thickness,fSize=11
	SetVariable C_Thickness,limits={1,10,0.5},value=root:Kymograph:V_ProfileThickness
	SetVariable C_NumKymograph,pos={8.00,4.00},size={110.00,16.00},proc=SetVarKymograph
	SetVariable C_NumKymograph,title="Kymograph #",fSize=11
	SetVariable C_NumKymograph,limits={0,20,1},value=root:Kymograph:V_NumKymograph
	PopupMenu C_Display,pos={128.00,4.00},size={111.00,20.00},proc=PopMenuKymograph
	PopupMenu C_Display,title="Display"
	PopupMenu C_Display,mode=3,popvalue="Filtered",value=#"\"Intensity;Ratio;Filtered\""
	GroupBox C_Edge_1,pos={4.00,235.00},size={108.00,72.00}
	GroupBox C_Edge_1,title="Edge 1 (high gradient)"
	SetVariable C_Edge_1_bottom,pos={14.00,255.00},size={70.00,16.00},proc=SetVarEdges
	SetVariable C_Edge_1_bottom,title="top"
	SetVariable C_Edge_1_bottom,help={"position the bottom boundary for edge 1"}
	SetVariable C_Edge_1_bottom,fSize=11
	SetVariable C_Edge_1_bottom,limits={1,100,1},value=root:Kymograph:V_Edge_1_bottom
	SetVariable C_Edge_1_top,pos={14.00,280.00},size={70.00,16.00},proc=SetVarEdges
	SetVariable C_Edge_1_top,title="bott."
	SetVariable C_Edge_1_top,help={"position the top boundary for edge 1"},fSize=11
	SetVariable C_Edge_1_top,limits={1,100,1},value=root:Kymograph:V_Edge_1_top
	SetVariable C_Edge_2_bottom,pos={14.00,163.00},size={70.00,16.00},proc=SetVarEdges
	SetVariable C_Edge_2_bottom,title="top"
	SetVariable C_Edge_2_bottom,help={"position the bottom boundary for edge 2"}
	SetVariable C_Edge_2_bottom,fSize=11
	SetVariable C_Edge_2_bottom,limits={1,inf,1},value=root:Kymograph:V_Edge_2_bottom
	SetVariable C_Edge_2_top,pos={14.00,189.00},size={70.00,16.00},proc=SetVarEdges
	SetVariable C_Edge_2_top,title="bott."
	SetVariable C_Edge_2_top,help={"position the top boundary for edge 2"},fSize=11
	SetVariable C_Edge_2_top,limits={1,inf,1},value=root:Kymograph:V_Edge_2_top
	GroupBox C_Edge_2,pos={4.00,143.00},size={108.00,72.00}
	GroupBox C_Edge_2,title="Edge 2 (low gradient)"
	SetVariable C_Edge_2_top_1,pos={91.00,189.00},size={12.00,16.00},proc=SetVarEdges
	SetVariable C_Edge_2_top_1,title=" ",fSize=11
	SetVariable C_Edge_2_top_1,limits={1,inf,10},value=root:Kymograph:V_Edge_2_top
	SetVariable C_Edge_2_bottom_1,pos={91.00,163.00},size={12.00,16.00},proc=SetVarEdges
	SetVariable C_Edge_2_bottom_1,title=" "
	SetVariable C_Edge_2_bottom_1,help={"position the bottom boundary for edge 2"}
	SetVariable C_Edge_2_bottom_1,fSize=11
	SetVariable C_Edge_2_bottom_1,limits={1,inf,10},value=root:Kymograph:V_Edge_2_bottom
	SetVariable C_Edge_1_top_1,pos={92.00,280.00},size={12.00,16.00},proc=SetVarEdges
	SetVariable C_Edge_1_top_1,title=" ",fSize=11
	SetVariable C_Edge_1_top_1,limits={1,100,10},value=root:Kymograph:V_Edge_1_top
	SetVariable C_Edge_1_bottom_1,pos={92.00,255.00},size={12.00,16.00},proc=SetVarEdges
	SetVariable C_Edge_1_bottom_1,title=" "
	SetVariable C_Edge_1_bottom_1,help={"position the bottom boundary for edge 1"}
	SetVariable C_Edge_1_bottom_1,fSize=11
	SetVariable C_Edge_1_bottom_1,limits={1,100,10},value=root:Kymograph:V_Edge_1_bottom
	Button C_Aide_Kymograph,pos={31.00,393.00},size={50.00,20.00},proc=BoutonAide
	Button C_Aide_Kymograph,title="Help",fSize=12,fStyle=1,fColor=(52428,1,1)
	CheckBox C_AfficheProfil,pos={16.00,327.00},size={94.00,16.00},proc=CheckAfficheProfils
	CheckBox C_AfficheProfil,title="Display profile lines"
	CheckBox C_AfficheProfil,help={"Displays the profile line over the image"}
	CheckBox C_AfficheProfil,value=1
EndMacro

Function SetVarKymograph(ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName
	Variable varNum
	String varStr
	String varName
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfKymo=W_DFREF[6]
	Wave/SDFR=dfKymo D_Kymograph_Intensity, D_Kymograph_Ratio, D_Kymograph_RM, D_Kymograph_Filtered
	NVAR/SDFR=dfKymo V_ProfileThickness
	
	String sNom, sListe
	variable i

	SetDataFolder dfKymo

	StrSwitch(ctrlName)
		case "C_NumKymograph":	//	changing kymo #
			//	If existing, copy corresponding kymo into current (displayed) images
			//	If not existing, put NaN in it
			sNom="D_Kymograph_Intensity_"+Num2Str(varNum)
			Wave/Z wKymo=$sNom
			if (WaveExists(wKymo))
				Duplicate/O wKymo, D_Kymograph_Intensity
			else
				D_Kymograph_Intensity=NaN
			endif
			sNom="D_Kymograph_Ratio_"+Num2Str(varNum)
			Wave/Z wKymo=$sNom
			if (WaveExists(wKymo))
				Duplicate/O wKymo, D_Kymograph_Ratio
			else
				D_Kymograph_Ratio=NaN
			endif
			sNom="D_Kymograph_RM_"+Num2Str(varNum)
			Wave/Z wKymo=$sNom
			if (WaveExists(wKymo))
				Duplicate/O wKymo, D_Kymograph_RM
			else
				D_Kymograph_RM=NaN
			endif
			sNom="D_Kymograph_Filtered_"+Num2Str(varNum)
			Wave/Z wKymo=$sNom
			if (WaveExists(wKymo))
				Duplicate/O wKymo, D_Kymograph_Filtered
			else
				D_Kymograph_Filtered=NaN
			endif
		break
	endswitch
	UpdateProfilesGraph()		//	to display the correct profile trace on F_RM
	SetVarEdges("",0,"","")	//	to update the edge line corresponding to the current profile
End

Function UpdateProfilesGraph()

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfKymo=W_DFREF[6]
	NVAR/SDFR=dfKymo V_NumKymograph, V_ProfileThickness
	string sListOfProfiles, sNom, sListOfTraces
	variable i
	
	setdatafolder dfKymo
	sListOfProfiles=WaveList("W_Profile_Y*", ";", "")
	for (i=0; (i<ItemsInList(sListOfProfiles)); i+=1)
		sNom=StringFromList(i, sListOfProfiles)
		//	Check that the wave is correctly displayed, and if not, appends it to the graph
		sListOfTraces=TraceNameList("F_RM", ";",1)
		if (WhichListItem(sNom, sListOfTraces)<0)	//	The profile is not displayed on F_RM
			Wave wProfile_Y=$sNom
			sNom=ReplaceString("Y", sNom, "X")
			Wave wProfile_X=$sNom
			AppendToGraph/W=F_RM/L=left/T=top wProfile_Y vs wProfile_X
		endif
	endfor

	ControlInfo/W=P_Kymograph C_AfficheProfil	//	returns the checkbox status in V_Value
	//	Make all kymo traces thin dotted line, and visible or hidden
//	sListOfTraces=TraceNameList("F_RM", ";", 1)			//	get list of all plots
//	sListOfTraces=ListMatch(sListOfTraces, "W_Profile_Y*")	//	select profile plots
	for (i=0; (i<ItemsInList(sListOfProfiles)); i+=1)
		sNom=StringFromList(i, sListOfProfiles)
		ModifyGraph/W=F_RM lstyle($sNom)=2,lsize($sNom)=0.5,rgb($sNom)=(36873,14755,58982),mode($sNom)=0,hideTrace($sNom)=(!V_Value)
	endfor

	//	make the selected profile NOT dotted
	sNom="W_Profile_Y_"+Num2Str(V_NumKymograph)
	ModifyGraph/W=F_RM/Z lstyle($sNom)=0,lsize($sNom)=V_ProfileThickness
//	ModifyGraph/W=F_RM/Z marker($sNom[0])=19, msize($sNom[0])=5
end

Function PopMenuKymograph(ctrlName,popNum,popStr) : PopupMenuControl
	String ctrlName
	Variable popNum
	String popStr
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfKymo=W_DFREF[6]
	Wave/SDFR=dfKymo D_Kymograph_Intensity, D_Kymograph_Ratio, D_Kymograph_RM
	
	Switch(popNum)
		case 1:
			ReorderImages/W=P_Kymograph D_Kymograph_Intensity, {D_Kymograph_RM, D_Kymograph_Filtered}
		break
		case 2:
			ReorderImages/W=P_Kymograph D_Kymograph_RM, {D_Kymograph_Intensity, D_Kymograph_Filtered}
		break
		case 3:
			ReorderImages/W=P_Kymograph D_Kymograph_Filtered, {D_Kymograph_RM, D_Kymograph_Intensity}
		break
	endswitch
End

Function SetVarEdges(ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName
	Variable varNum
	String varStr
	String varName

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfKymo=W_DFREF[6]
	NVAR/SDFR=dfKymo V_Edge_1_bottom, V_Edge_1_top, V_Edge_2_bottom, V_Edge_2_top 
	
	SetDrawLayer/W=P_Kymograph/K ProgFront
	SetDrawEnv xcoord= prel,ycoord= left,linefgc= (52428,1,1), dash= 1
	DrawLine 0,V_Edge_1_bottom,1,V_Edge_1_bottom
	SetDrawEnv xcoord= prel,ycoord= left,linefgc= (52428,1,1), dash= 1
	DrawLine 0,V_Edge_1_top,1,V_Edge_1_top
	
	SetDrawEnv xcoord= prel,ycoord= left,linefgc= (1,52428,26586), dash= 1
	DrawLine 0,V_Edge_2_bottom,1,V_Edge_2_bottom
	SetDrawEnv xcoord= prel,ycoord= left,linefgc= (1,52428,26586), dash= 1
	DrawLine 0,V_Edge_2_top,1,V_Edge_2_top
	
	Edges()
End

Function CheckAfficheProfils(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	
	UpdateProfilesGraph()		//	updates the representation of profile lines on the image
End