#pragma TextEncoding = "UTF-8"
#pragma rtGlobals=3		// Use modern global access method and strict wave access.

// *** Copyright ***
// © 2023 CNRS Shared by Pierre VINCENT, under the MIT License.
//	Contact: pierre.vincent@cnrs.fr; IGF, 141 rue de la Cardonille, 34094 Montpellier cedex 5, France.

//Kill first datapoints: DeletePoints 0,10, root:Analyse_0:TraceR0001

//	Possible mistakes to check in the previous version
//	slope : was calculated as the total amplitude divided by 10-90 rise-time

//	Fractional area change : (largest area - smallest area) / largest area

//	This function analyzes an input wave, wData, for periodic properties, and returns a wave that contains all the results
FUNCTION/WAVE Pulse(wData, vSmoothing)
	Wave wData
	variable vSmoothing

	SetDataFolder root:Pulse
	Wave W_Crossings_Decay, W_100_Decay, W_90_Decay, W_50_Decay, W_10_Decay, W_0_Decay, W_90_10, W_100_0
	Wave W_Crossings_Rise, W_100_Rise, W_90_Rise, W_50_Rise, W_10_Rise, W_0_Rise, W_10_90, W_0_100
	W_Crossings_Decay=NaN; W_100_Decay=NaN; W_90_Decay=NaN; W_50_Decay=NaN; W_10_Decay=NaN; W_0_Decay=NaN; W_90_10=NaN; W_100_0=NaN
	W_Crossings_Rise=NaN; W_100_Rise=NaN; W_90_Rise=NaN; W_50_Rise=NaN; W_10_Rise=NaN; W_0_Rise=NaN; W_10_90=NaN; W_0_100=NaN
	
	//	Output a free wave that contains all these results:
	Make/N=15/FREE wPulseResult; wPulseResult=NaN

	//	Duplicating the original measurement wave into the Pulse datafolder, 
	//	called W_Trace, that can be modified, plotted..., without affecting the original wave
	Duplicate/O wData, W_Trace
	
	//	Smoothing the local wave
	ControlInfo/W=P_Pulse C_Smooting
	if (V_Value)
		Smooth/S=2 vSmoothing, W_Trace
	endif
	
	Variable vAvg=Mean(W_Trace), vNbPulsesDecay, vNbPulsesRise, i
	Variable vMin, vMax, vTemp

// *** Rising phase of the wave ***

	//	Detecting where the wave crosses the average, going towards bottom peak -> contains a whole rising phase of the wave
	//	FindLevels to detect where the wave crosses the average, in the downward direction (EDGE=2)
	//	para buscar los puntos en x que cruzan la onda en y (y=promedio) con direccion hacia abajo //	I think you mean "arriba"
	FindLevels/Q/EDGE=2 W_Trace, vAvg
	if (V_LevelsFound==0)		//	no signal detected on the source wave
		return wPulseResult	//	returns wave with NaN
	endif

	//	stores the number of decaying 1/2 waves
	vNbPulsesRise=V_LevelsFound-1		//	-1 because there is 1 more crossing than the total number of complete rising phases

	//	Now that the number of rising phases is known, objects to hold the results can be dimensioned and initialized
	Redimension/N=(vNbPulsesRise+1,2) W_Crossings_Rise	//	+1 because there is 1 more crossing than total number of phases
	Redimension/N=(vNbPulsesRise,2) W_100_Rise, W_90_Rise, W_50_Rise, W_10_Rise, W_0_Rise, W_10_90, W_0_100
	W_Crossings_Rise=NaN; W_100_Rise=NaN; W_90_Rise=NaN; W_50_Rise=NaN; W_10_Rise=NaN; W_0_Rise=NaN; W_10_90=NaN; W_0_100=NaN

	//	To store the position of where W_Trace crossed its average
	Wave W_FindLevels	//	contains the result of findlevels
	W_Crossings_Rise[][0]=W_FindLevels[p]	//	X position
	W_Crossings_Rise[][1]=vAvg					//	Y position
	
	//	Analyzing each Rise phase
	for (i=0; (i < vNbPulsesRise); i=i+1)

		//	Looking for top peak between two consecutive crossings
		FindPeak/Q/R=(W_Crossings_Rise[i][0],W_Crossings_Rise[i+1][0]) W_Trace
		W_100_Rise[i][0]=V_PeakLoc; W_100_Rise[i][1]=V_PeakVal

		//	Looking for bottom peak (FindPeak/N)
		FindPeak/Q/N/R=(W_Crossings_Rise[i][0],W_Crossings_Rise[i+1][0]) W_Trace
		W_0_Rise[i][0]=V_PeakLoc;	W_0_Rise[i][1]=V_PeakVal

		//	Analyze the Rise phase, from bottom towards top
		vMin=W_0_Rise[i][0]; vMax=W_100_Rise[i][0]
		if (vMax>vMin+deltax(W_Trace))		//	executed only if the space between points is > sampling
			EdgeStats/Q/R=(vMin, vMax) W_Trace
			W_10_Rise[i][0]=V_EdgeLoc1 // x del punto 10% en Rise
			W_10_Rise[i][1]=V_EdgeLvl1 // y del punto 10% en Rise
			W_50_Rise[i][0]=V_EdgeLoc2 // x del punto 50% en Rise
			W_50_Rise[i][1]=V_EdgeLvl2 // y del punto 50% en Rise
			W_90_Rise[i][0]=V_EdgeLoc3 // x del punto 90% en Rise
			W_90_Rise[i][1]=V_EdgeLvl3 // y del punto 90% en Rise
			W_10_90[i][0]=V_EdgeDLoc3_1	//(Rise time 10-90%)
			W_10_90[i][1]=V_EdgeLvl3-V_EdgeLvl1	//amplitud 10-90% Rise
			W_0_100[i][0]=NaN				//	peak to peak duration is not returned by EdgeStats
			W_0_100[i][1]=V_EdgeAmp4_0 	//	Amplitude
		endif
	endfor

// *** Decaying phase of the wave ***

	//	Detecting where the wave crosses the average, going towards top peak -> contains a whole decaying phase of the wave
	//	FindLevels to detect where the wave crosses the average, in the upward direction (EDGE=1)
	//	para buscar los puntos en x que cruzan la onda en y (y=promedio) con direccion hacia arriba //	I think you mean "abajo"
	FindLevels/Q/EDGE=1 W_Trace, vAvg
	//	stores the number of decaying 1/2 waves
	vNbPulsesDecay=V_LevelsFound-1		//	-1 because there is 1 more crossing than the total number of complete decay phases

	//	Now that the number of decaying phases is known, objects to hold the results can be dimensioned and initialized
	Redimension/N=(vNbPulsesDecay+1,2) W_Crossings_Decay	//	There is 1 more crossing than total number of phases
	Redimension/N=(vNbPulsesDecay,2) W_100_Decay, W_90_Decay, W_50_Decay, W_10_Decay, W_0_Decay, W_90_10, W_100_0
	W_Crossings_Decay=NaN; W_100_Decay=NaN; W_90_Decay=NaN; W_50_Decay=NaN; W_10_Decay=NaN; W_0_Decay=NaN; W_90_10=NaN; W_100_0=NaN

	//	To store the position of where W_Trace crossed its average
	Wave W_FindLevels	//	contains the result of findlevels
	W_Crossings_Decay[][0]=W_FindLevels[p]
	W_Crossings_Decay[][1]=vAvg
	
	//	Analyzing each Decay phase
	for (i=0; (i < vNbPulsesDecay); i=i+1)

		//	Looking for top peak between two consecutive crossings
		FindPeak/Q/R=(W_Crossings_Decay[i][0],W_Crossings_Decay[i+1][0]) W_Trace
		W_100_Decay[i][0]=V_PeakLoc; W_100_Decay[i][1]=V_PeakVal

		//	Looking for bottom peak (FindPeak/N)
		FindPeak/Q/N/R=(W_Crossings_Decay[i][0],W_Crossings_Decay[i+1][0]) W_Trace
		W_0_Decay[i][0]=V_PeakLoc;	W_0_Decay[i][1]=V_PeakVal

		//	Analyze the decay phase, from top towards bottom
		vMin=W_100_Decay[i][0]; vMax=W_0_Decay[i][0]
		if (vMax>vMin+deltax(W_Trace))		//	executed only if the space between points is > sampling
			EdgeStats/Q/R=(vMin, vMax) W_Trace
			W_90_Decay[i][0]=V_EdgeLoc1 // x del punto 10% en decay
			W_90_Decay[i][1]=V_EdgeLvl1 // y del punto 10% en decay
			W_50_Decay[i][0]=V_EdgeLoc2 // x del punto 50% en decay
			W_50_Decay[i][1]=V_EdgeLvl2 // y del punto 50% en decay
			W_10_Decay[i][0]=V_EdgeLoc3 // x del punto 90% en decay
			W_10_Decay[i][1]=V_EdgeLvl3 // y del punto 90% en decay
			W_90_10[i][0]=V_EdgeDLoc3_1 //(Decay time 10-90%)
			W_90_10[i][1]=V_EdgeLvl1-V_EdgeLvl3	//amplitud 10-90% decay
			W_100_0[i][0]=NaN		//	peak to peak duration is not returned by EdgeStats
			W_100_0[i][1]=V_EdgeAmp4_0		//	amplitude
		endif
	endfor

	//	Delete waves that are no longer needed
	KillWaves/Z W_FindLevels, M_WaveStats

	//	Calculating onset slope from Rise data
	Make/FREE/N=(vNbPulsesRise) W_RiseSlope
	W_RiseSlope=W_10_90[p][1]/W_10_90[p][0]
	
	//	Calculating decay slope from Decay data
	Make/FREE/N=(vNbPulsesDecay) W_DecaySlope
	W_DecaySlope=W_90_10[p][1]/W_90_10[p][0]

	//	Calculating periods and frequency from Rise data
	Make/FREE/N=(vNbPulsesRise-1) W_Period_Top, W_Period_Bottom, W_Frequency	//	vNbPulsesRise-1 because iteration covers i to i+1
	for (i=0; (i<vNbPulsesRise-1); i+=1)
		W_Period_Top[i]=W_100_Rise[i+1][0]-W_100_Rise[i][0]
		W_Period_Bottom[i]=W_0_Rise[i+1][0]-W_0_Rise[i][0]
	endfor
	W_Frequency=60/W_Period_Top

	//	Calculating bottom peak jitter RMS
	Make/FREE/N=(vNbPulsesRise-1) W_RMS
	for (i=0; (i<vNbPulsesRise-2); i+=1)
		W_RMS[i]= ((W_Period_Bottom[i]*1000) - (W_Period_Bottom[i+1]*1000))^2
	endfor
	
	//time after peak to 50% relaxation calculated from Decay data
	//	Translation from what was proframmed previously; not sure it is OK anymore.
	Make/FREE/n=(vNbPulsesDecay) W_TAP50R
	W_TAP50R=W_50_Decay[p][0]-W_90_Decay[p][0]		//	W_EdgeLocB[i] - W_EdgeLocA[i]
	//	Suggested instead : W_TAP50R=W_50_Decay[p][0]-W_100_Decay[p][0]
	
	//	Fractional shortening
	Make/FREE/N=(vNbPulsesRise) W_FracShortening
	W_FracShortening=(W_100_Rise[p][1]-W_0_Rise[p][1])/W_100_Rise[p][1]
	
	//	Time from bottom to peak
	Make/FREE/N=(vNbPulsesRise) W_Time_0_100
	W_Time_0_100=W_100_Rise[p][0]-W_0_Rise[p][0]

	//	Time from peak to bottom
	Make/FREE/N=(vNbPulsesDecay) W_Time_100_0
	W_Time_100_0=W_0_Decay[p][0]-W_100_Decay[p][0]
	
	wPulseResult=NaN	//	clears the wave
	//	0 Frequency; 1 MaxPeakb; 2 MinPeakb; 3 Ampltd; 4 TimetoPeak; 5 Risetime; 6 TimetoAntiPeak; 7 Decaytime; 8 EventDuration;
	//	9 TAP50R; 10 Decayslope; 11 Riseslope; 12 Average; 13 RMS; 14 FracShort; 15 R0; 16 Smoothing;
	wPulseResult[0]=60/mean(	W_Period_Top	)				//	frequency in beats per min
	wPulseResult[1]=MyWaveAverage(W_100_Rise, 1)		//	average amplitude of the top peak
	wPulseResult[2]=MyWaveAverage(W_0_Rise, 1)			//	average amplitude of the bottom peak
	wPulseResult[3]=MyWaveAverage(W_0_100, 1)			//	Amplitude
	wPulseResult[4]=mean(W_Time_0_100)					//	Time from 0 to 100%
	wPulseResult[5]=MyWaveAverage(W_10_90,0)			//	10-90% rise time
	wPulseResult[6]=mean(W_Time_100_0)					//	Time from 100 to 0%
	wPulseResult[7]=MyWaveAverage(W_90_10, 0)			//	90-10% decay time
	wPulseResult[8]=mean(W_Period_Bottom)				//	Event duration
	wPulseResult[9]=mean(W_TAP50R)							//	Time to 50% relaxation
	wPulseResult[10]=mean(W_DecaySlope)					//	Decay slope
	wPulseResult[11]=mean(W_RiseSlope)					//	Rise slope
	wPulseResult[12]=vAvg										//	amplitude
	wPulseResult[13]=sqrt(mean(W_RMS))
	wPulseResult[14]=mean(W_FracShortening)				//	Fractional shortening
	
	return wPulseResult
End

//	This function does the job of selecting the right wave to analyze, displaying it, sending it to Pulse analysis and properly handling the results
Function SetVarPulse(ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName
	Variable varNum
	String varStr
	String varName

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]; DFREF dfPulse=W_DFREF[5]; DFREF dfKymo=W_DFREF[6]
	NVAR/SDFR=dfRatio Dim, V_NbChiffres, V_R0, V_DeltaT

	//	Pulse analysis, results, all pulse objects etc... should end up in this datafolder
	SetDataFolder dfPulse
	NVAR V_Smoothing
	
	Variable vLineNumber, vNumberOfLines, vAreaMin, vAreaMax, vLongDmtrDiast, vLongDmtrSyst, vShortRadDiast, vShortRadSyst, vShortDmtrDiast, vShortDmtrSyst, vEDV, vESV, vEF, vSV, vCO, LongFS, ShortFS //CHANGE JUSSEP

	//	The pulse result table has the size of pulse analysis plus a variable number of columns, to add more info
	//	Currently, add 5 columns for V_R0, V_Smoothing, min ROI area, max ROI area, fractional area change
	Variable vNbDataExtra=16///////CHANGE JUSSEP
	String sInputWaveName, sOutputWaveName, sTemp
	variable vTemp
	
	//	depending on the popup, looks for data wave in kymo or ratio datafolder;
	//	determines the name of the result wave and the line number in which to put the result
	ControlInfo C_SourceSelector
	StrSwitch(S_Value)
		case "Kymo":
			NVAR/SDFR=dfKymo V_NumKymograph
			NVAR V_Smoothing_Kymo
			vLineNumber=V_NumKymograph
			V_Smoothing=V_Smoothing_Kymo
// *** \\\little fix needed here ***
			vNumberOfLines=5	//	should be the total number of kymo analysis; there should be a counter in kymo datafolder
			
			sOutputWaveName="Pulse_Kymo"
			sInputWaveName=AjouteNumero("Kymo", V_NumKymograph, 4)
			Wave/Z/SDFR=dfKymo wData=$sInputWaveName
			if (!WaveExists(wData))
				print sInputWaveName, " not found"
				Wave W_Trace; W_Trace=NaN
				return(0)
			endif
		break
		case "Ratio":
			NVAR/SDFR=dfROI V_NumROI, V_NbROI
			NVAR V_Smoothing_Ratio
			vLineNumber=V_NumROI		//	currently analyzed ROI
			vNumberOfLines=V_NbROI+1	//	total number of ROIs in this analysis
			V_Smoothing=V_Smoothing_Ratio
			sOutputWaveName="Pulse_Ratio"
			sInputWaveName=AjouteNumero("TraceR", V_NumROI, V_NbChiffres)
			Wave/SDFR=dfROI wData=$sInputWaveName

			//	Redimensioning the measurement wave with correct size: remove the trailing NaN
			Redimension/N=(Dim) wData

			//	to highlight the currently selected ROI
			DoWindow P_Controle	// to check that the control panel is open
			if (V_Flag)
				SetVarNumRoi("C_SetVar_Nb",0,"","")
			endif
		break
	EndSwitch

	//Set data scale on X axis:
	if (NumType(V_DeltaT)!=0)	//	V_DeltaT is not defined => need to give it a value
		vTemp=0.02
		Prompt vTemp, "Sampling interval"
		DoPrompt "Need to define this variable", vTemp
		V_DeltaT=vTemp
	endif
	SetScale/P x 0,V_DeltaT,"s", wData

	//	This is where the Pulse analysis is called
	Wave wPulseResult = Pulse(wData, V_Smoothing)
	
	//	Make sure the table which will contain pulse results exists and has the right size
	Wave/Z wAllResults=$sOutputWaveName
	if (WaveExists(wAllResults))	//	the wave has already been created
		//	make sure it has the right size to accomodate the result of the analysis, plus extra data; new data may be added in the future
		Redimension/N=(vNumberOfLines, DimSize(wPulseResult, 0) + vNbDataExtra) wAllResults
	else
		//	This is where the table is created to hold the global result
		Make/N=(vNumberOfLines, DimSize(wPulseResult, 0) + vNbDataExtra) wAllResults
		wAllResults=NaN
		Rename wAllResults, $sOutputWaveName
	endif
	//	Replace 0 values with NaN
	MatrixOp/O/FREE W_Temp=replace(wAllResults, 0, NaN)
	FastOp wAllResults=W_Temp
	
	//	copy the pulse analysis in the result table
	wAllResults[vLineNumber][0, (DimSize(wPulseResult, 0)-1)]=wPulseResult[q]
	
// *** additional data added to the table	 ***

	//	R0 value
	wAllResults[vLineNumber][DimSize(wPulseResult, 0)]=V_R0
	//	Smoothing : must test whether smoothing was on or off
	ControlInfo/W=P_Pulse C_Smooting
	if (V_Value)	
		wAllResults[vLineNumber][DimSize(wPulseResult, 0)+1]=V_Smoothing
	else
		wAllResults[vLineNumber][DimSize(wPulseResult, 0)+1]=NaN
	endif
	//	Fractional area change
	vAreaMin=wAllResults[vLineNumber][DimSize(wPulseResult, 0)+2]
	vAreaMax=wAllResults[vLineNumber][DimSize(wPulseResult, 0)+3]
	wAllResults[vLineNumber][DimSize(wPulseResult, 0)+4]=(vAreaMax-vAreaMin)/vAreaMax
	
// *** Haemodynamics ***
// *** \\\	Need to fix according to the new measurement in ROI now gives the ROI diameter instead of the radius; does that matter here ? ***

	//Long ventricular diameter 
	vLongDmtrDiast=wAllResults[vLineNumber][DimSize(wPulseResult, 0)+5]
	vLongDmtrSyst=wAllResults[vLineNumber][DimSize(wPulseResult, 0)+6]
	
	//Short radius
	vShortRadDiast=(vAreaMax/(Pi*(vLongDmtrDiast/2)))
	vShortRadSyst=(vAreaMin/(Pi*(vLongDmtrSyst/2)))
		
	//Short diastolic diameter 
	vShortDmtrDiast=vShortRadDiast*2
	wAllResults[vLineNumber][DimSize(wPulseResult, 0)+7]=vShortDmtrDiast

	//Short systolic diameter
	vShortDmtrSyst=vShortRadSyst*2
	wAllResults[vLineNumber][DimSize(wPulseResult, 0)+8]=vShortDmtrSyst
	
	//End-diastolic volume
	vEDV=(4/3 * vShortRadDiast * vAreaMax)/1000
	wAllResults[vLineNumber][DimSize(wPulseResult, 0)+9]=vEDV
	
	//End-systolic volume
	vESV=(4/3 * vShortRadSyst * vAreaMin)/1000
	wAllResults[vLineNumber][DimSize(wPulseResult, 0)+10]=vESV
	
	//Ejection fraction
	vEF=(vEDV-vESV)/vEDV
	wAllResults[vLineNumber][DimSize(wPulseResult, 0)+11]=vEF
	
	//Stroke volume
	vSV=(vEDV-vESV)
	wAllResults[vLineNumber][DimSize(wPulseResult, 0)+12]=vSV
	
	//Cardiac output
	vCO=(vSV*(wPulseResult[0][0]))/1000
	wAllResults[vLineNumber][DimSize(wPulseResult, 0)+13]=vCO
	 
	//Long fractional shortening
	LongFS=(vLongDmtrDiast-vLongDmtrSyst)/vLongDmtrDiast
	wAllResults[vLineNumber][DimSize(wPulseResult, 0)+14]=LongFS
	 
	//Short fractional shortening
	ShortFS=(vShortDmtrDiast-vShortDmtrSyst)/vShortDmtrDiast
	wAllResults[vLineNumber][DimSize(wPulseResult, 0)+15]=ShortFS
	
	//	End of haemodynamic parameters
	
// *** Atrio-ventricular delay ***

	
	//	If not already created, make a table to show the result
	DoWindow T_Pulse
	if (V_Flag!=1)
		//	Update the dimension labels so it matches the structure of pulse analysis
		for (vTemp=0; (vTemp<DimSize(wAllResults, 1)); vTemp+=1)
			sTemp=StringFromList(vTemp, ksPulseParams)
			SetDimLabel 1, vTemp, $sTemp, wAllResults
		endfor
		Edit/K=1/W=(0,45,1110,279)/N=T_Pulse wAllResults.ld
		ModifyTable format(Point)=1,size=12,width=50
		AutoPositionWindow/R=P_Pulse/M=1
	endif
	KillWaves/Z M_WaveStats, WS_TagName
End

Function PopMenuSourceSelector(ctrlName,popNum,popStr) : PopupMenuControl
	String ctrlName
	Variable popNum
	String popStr
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]; DFREF dfPulse=W_DFREF[5]; DFREF dfKymo=W_DFREF[6]
	
	//	If changing the data source, kill the result window that will be eventually recreated to display the correct result wave
	KillWindow/Z T_Pulse

	StrSwitch(popStr)
		case "Ratio":
			NVAR/SDFR=dfPulse V_Smoothing_Ratio
			NVAR/SDFR=dfROI V_NumROI, V_NbROI
			SetVariable C_SetVar_Nb win=P_Pulse,value=V_NumROI,limits={1,V_NbROI,1}
			SetVariable C_SmoothingFactor win=P_Pulse,value=V_Smoothing_Ratio
			DoWindow P_Controle	// to check that the control panel is open
			if (V_Flag)
				SetVarNumRoi("C_SetVar_Nb",0,"","")
			endif
		break
		case "Kymo":
			//	Tests if kymo datafolder exists
			if (DataFolderExists("root:kymograph"))
				NVAR/SDFR=dfPulse V_Smoothing_Kymo
				NVAR/SDFR=dfKymo V_NumKymograph
				SetVariable C_SetVar_Nb win=P_Pulse,value=V_NumKymograph,limits={0,10,1}		//	there should be a variable indicating the total nb of kymographs
				SetVariable C_SmoothingFactor win=P_Pulse,value=V_Smoothing_Kymo
			else
				DoAlert 0, "There is no kymograph analysis"
				PopupMenu C_SourceSelector mode=1		//	resets popup to 1st item
				return(0)
			endif
		break
	EndSwitch
	SetVarPulse("",NaN,"","")	//	Update Pulse analysis
End

//	Relay function triggered by the C_Smooting checkbox
Function CheckPulse(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked	
	SetVarPulse("",NaN,"","")	//	Update Pulse analysis
End

//	Automatic update of that window definition is not OK.
//	In SetVariable, remove references to variables, like V_NbROI

Window P_Pulse() : Graph
	PauseUpdate; Silent 1		// building window...
	String fldrSav0= GetDataFolder(1)
	SetDataFolder root:Pulse:
	Display /W=(13,443,762,787)/K=1  W_Trace as "Pulse"
	AppendToGraph W_Crossings_Rise[*][1] vs W_Crossings_Rise[*][0]
	AppendToGraph W_0_Rise[*][1] vs W_0_Rise[*][0]
	AppendToGraph W_10_Rise[*][1] vs W_10_Rise[*][0]
	AppendToGraph W_50_Rise[*][1] vs W_50_Rise[*][0]
	AppendToGraph W_90_Rise[*][1] vs W_90_Rise[*][0]
	AppendToGraph W_100_Rise[*][1] vs W_100_Rise[*][0]
	AppendToGraph W_Crossings_Decay[*][1] vs W_Crossings_Decay[*][0]
	AppendToGraph W_0_Decay[*][1] vs W_0_Decay[*][0]
	AppendToGraph W_10_Decay[*][1] vs W_10_Decay[*][0]
	AppendToGraph W_50_Decay[*][1] vs W_50_Decay[*][0]
	AppendToGraph W_90_Decay[*][1] vs W_90_Decay[*][0]
	AppendToGraph W_100_Decay[*][1] vs W_100_Decay[*][0]
	SetDataFolder fldrSav0
	ModifyGraph mode(W_Crossings_Rise)=3,mode(W_0_Rise)=4,mode(W_10_Rise)=3,mode(W_50_Rise)=3
	ModifyGraph mode(W_90_Rise)=3,mode(W_100_Rise)=4,mode(W_Crossings_Decay)=3,mode(W_0_Decay)=3
	ModifyGraph mode(W_10_Decay)=3,mode(W_50_Decay)=3,mode(W_90_Decay)=3,mode(W_100_Decay)=3
	ModifyGraph marker(W_0_Rise)=41,marker(W_10_Rise)=17,marker(W_50_Rise)=26,marker(W_90_Rise)=19
	ModifyGraph marker(W_100_Rise)=41,marker(W_Crossings_Decay)=1,marker(W_0_Decay)=13
	ModifyGraph marker(W_10_Decay)=17,marker(W_50_Decay)=26,marker(W_90_Decay)=19,marker(W_100_Decay)=13
	ModifyGraph rgb(W_Crossings_Rise)=(0,0,0),rgb(W_0_Rise)=(1,34817,52428),rgb(W_10_Rise)=(30583,30583,30583)
	ModifyGraph rgb(W_50_Rise)=(30583,30583,30583),rgb(W_90_Rise)=(30583,30583,30583)
	ModifyGraph rgb(W_100_Rise)=(19729,1,39321),rgb(W_Crossings_Decay)=(0,0,0),rgb(W_0_Decay)=(0,0,0)
	ModifyGraph rgb(W_10_Decay)=(0,0,0),rgb(W_50_Decay)=(0,0,0),rgb(W_90_Decay)=(0,0,0)
	ModifyGraph rgb(W_100_Decay)=(0,0,0)
	ModifyGraph msize(W_Crossings_Rise)=3,msize(W_0_Rise)=6,msize(W_50_Rise)=3,msize(W_100_Rise)=6
	ModifyGraph msize(W_Crossings_Decay)=3,msize(W_50_Decay)=3
	ControlBar/L 120
	SetVariable C_SetVar_Nb,pos={30.00,70.00},size={50.00,16.00},proc=SetVarPulse,title=" #"
	SetVariable C_SetVar_Nb,fSize=11,limits={1,2,1},value= root:Analyse_0:V_NumROI
	SetVariable C_SmoothingFactor,pos={5.00,191.00},size={106.00,16.00},proc=SetVarPulse,title="sm. factor"
	SetVariable C_SmoothingFactor,help={"defines the smoothing factor"},fSize=11
	SetVariable C_SmoothingFactor,limits={5,50,2},value= root:Pulse:V_Smoothing_Ratio
	CheckBox C_Smooting,pos={10.00,169.00},size={70.00,16.00},title="Smoothing"
	CheckBox C_Smooting,fSize=11,value= 1,side= 1,proc=CheckPulse
	PopupMenu C_SourceSelector,pos={20.00,40.00},size={56.00,23.00},proc=PopMenuSourceSelector
	PopupMenu C_SourceSelector,help={"select the source of data"}
	PopupMenu C_SourceSelector,mode=1,popvalue="---",value= #"\"Ratio;Kymo\""
	GroupBox C_DataSource,pos={10.00,20.00},size={100.00,80.00},title="Data source"
	Button C_Aide_Pulse_analysis,pos={30.00,260.00},size={50.00,20.00},proc=BoutonAide,title="Help"
	Button C_Aide_Pulse_analysis,fStyle=1
EndMacro

//	Creates a datafolder with name "Pulse" and containing all the required objects
//	Initializes the P_Pulse window
Function Init_P_Pulse()
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]; DFREF dfPulse=W_DFREF[5]
	NVAR/SDFR=dfROI V_NbROI, V_NumROI
	Wave/SDFR=dfROI W_Tag
	Wave/SDFR=dfRatio/T W_NombreTagStr
	
	variable i
	string sNom="root:Pulse"
	SetDataFolder root:
	if (!DataFolderExists("root:Pulse"))
		NewDataFolder $sNom
		DFREF dfPulse=$sNom
		W_DFREF[5]=dfPulse
	endif
	//	looks in the z_Initialisation datafolder for the description of waves, variables and strings
	//	 If these objects did not already exist, they are created with default value
	InitDataFolder("Init raplouf.pxp", "f_Pulse", "Pulse")
	SetDataFolder dfPulse
	KillWindow/Z P_Pulse
	KillWIndow/Z T_Pulse
	Execute "P_Pulse()"
End

//	Probably no longer needed
Function ClearAndRunPulse()
	SetDataFolder root:
	Gere_DFREF("Initialise")								//	make sure DFREF wave is updated
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]; DFREF dfPulse=W_DFREF[5]
	variable vSmoothing_Ratio, vSmoothing_Kymo
	//	Save smoothing value
	NVAR/Z/SDFR=dfPulse V_Smoothing_Ratio, V_Smoothing_Kymo
	if (NVAR_Exists(V_Smoothing_Ratio))
		vSmoothing_Ratio=V_Smoothing_Ratio
	else
		vSmoothing_Ratio=NaN
	endif
	if (NVAR_Exists(V_Smoothing_Kymo))
		vSmoothing_Kymo=V_Smoothing_Kymo
	else
		vSmoothing_Kymo=NaN
	endif
	//	Kill all pulse stuff
	KillDataFolder/Z Export_Pulse	//	old variant no longer needed
	KillWindow/Z T_Pulse
	KillWindow/Z P_Pulse
	KillDataFolder/Z Pulse
	if (V_Flag==0)	//	KillDataFolder worked properly
		NewDataFolder/S Pulse
		//	Write in the empty Pulse datafolder the previous values of smoothing
		if (NumType(vSmoothing_Ratio)==0)
			Variable/G V_Smoothing_Ratio=vSmoothing_Ratio
		endif
		if (NumType(vSmoothing_Kymo)==0)
			Variable/G V_Smoothing_Kymo=vSmoothing_Kymo
		endif
		CheckActivePanneau("C_Active_P_Pulse",1)		//	opens the Pulse window
		DoWindow/F P_Pulse
		PopMenuSourceSelector("C_SourceSelector",1,"Ratio")		//	selects "Ratio" as the analysis source
		NVAR/SDFR=dfROI V_NumROI
		V_NumROI=1
		SetVarPulse("C_SetVar_Nb",1,"1","V_NumROI") //	Calculates Pulse for ROI #1
		V_NumROI=2
		SetVarPulse("C_SetVar_Nb",2,"2","V_NumROI") //	Calculates Pulse for ROI #2		
	else
		DoAlert 0, "Pulse datafolder could not be killed; check what objects are still in use, kill them and run again."
	endif
end
