#pragma TextEncoding = "UTF-8"
#pragma rtGlobals=3		// Use modern global access method.
#pragma IgorVersion=7.00	
#pragma protect=0

#Include "Initialise lit et sauve"
#Include "Panneaux et graphes"
#Include "Mesures"
#Include "Stats"
#Include "Kymograph"
#Include "Pulse"

// *** Copyright ***
// © 2023 CNRS Shared by Pierre VINCENT, under the MIT License.
//	Contact: pierre.vincent@cnrs.fr; IGF, 141 rue de la Cardonille, 34094 Montpellier cedex 5, France.

//	To improve:
//	name and function organization of export buttons: should be a single function for all that

//	Wave definition:
//		Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
//		DFREF dfExport=W_DFREF[1]; DFREF dfPulse=W_DFREF[5]; DFREF dfKymo=W_DFREF[6]

//_______________________________________________________________________________________
// *** Main Ratioscope menu ***

Menu "Ratioscope"
	"Start Ratioscope",  InitialiseRaplouf()
	"-------------"
	"Save for the first time", /Q, SauveManip()
	"-------------"
	"Make movie", ExporteFilm()
	"Make movie with Z dimension", ExporteFilmZ()
	"Cursor statistics", 	Gere_DFREF("Initialise"); CheckActivePanneau("C_Active_P_Export",1)
	"Pulse", KillWindow/Z P_Pulse; KillWindow/Z T_Pulse; Gere_DFREF("Initialise"); CheckActivePanneau("C_Active_P_Pulse",1)
//	"Pulse: clear previous and run again", ClearAndRunPulse()
	"Repair DFREFs", Gere_DFREF("Initialise")
	"Events", KillWindow/Z P_Evts; Gere_DFREF("Initialise"); CheckActivePanneau("C_Active_P_Evts",1)
	"Export graph", Gere_DFREF("Initialise"); BoutonSauveGrapheImage("")
	"Export traces to database folder", Gere_DFREF("Initialise"); BoutonSauveVignette("C_SauveVignetteTraces")
	"-------------"
	"Compress file (now useless)", 	Gere_DFREF("Initialise"); CompresseManip()
	"-------------"
	"Help", DisplayHelpTopic "Ratioscope description"; AfficheInit("")
	"Ratioscope informations", DisplayVersion()
End

//_________________________________________________________________________________________
//*****************************************************************************************
//*********     Controlling the calculation of ratio and pseudocolor images   *************
//	New architecture since version 7

//	Main function that does the analysis
Function BoutonAnalyse(ctrlName) : ButtonControl
	String ctrlName
	Variable j
	Variable debut, fin, vIndexPdV, vOnContinue=1, vImageLue, vOptimiseDep
	Variable timerRefNum, vDuree	//	Pour suivre le temps d'exécution

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	SetDataFolder dfRatio
	NVAR i=V_NumCoucheCourante, Dim
	SVAR S_Duree_Analyse
	Wave/SDFR=dfROI W_Tag
	NVAR/SDFR=dfROI V_NbROI
	Wave/SDFR=dfRatio W_Deplacement

	//	Actionné par les boutons:
	// C_BoutonCePoint; C_BoutonAjoutePoints; C_BoutonTousPoints; C_TraiteCettePosition
	//	Actionné par la fonction BoutonDecalage
	//	Actionné par la fonction ProcAjoutePointsAuto qui travaille en tâche de fond si le bouton est coché
	//	Comportement déterminé par les checkboxes: C_CorrigeAB; C_CalculerRatio; C_Mesurer; C_Calculer_P_RM
	timerRefNum=StartMSTimer

	//	La case C_CorrigeAB est désactivée lorsqu'on a réduit de 3D à 2D
	//	La case ne doit plus être mise dans l'état coché si elle est désactivée.
	//	Si c'est le cas (bouton C_BoutonTousCalculs pressé), il faut qu'elle soit décochée
	
	StrSwitch(ctrlName)
		case "C_BoutonCePoint":
		case "C_TraiteCettePosition":
			debut=i; fin=i; vOptimiseDep=0
		break		
		case "C_OptimiseCettePosition":	//	on vise à optimiser le déplacement à partir de valeurs déjà indiquées
			debut=i; fin=i; vOptimiseDep=1
			if (NumType(W_Deplacement[i][0])!=0)
				W_Deplacement[i][]=0
			endif
			MaJ_CasesACocher("AB", 0, "", "")
		break
		case "C_BoutonAjoutePoints":
		case "AjoutePointsAuto":
			Dim=FDim()
			debut=Dim		//	nombre de points de temps, démarre à 1
			MaJ_CasesACocher("Tous", 0, "", "")
		break
		case "C_BoutonPointsSuivants":
			debut=i+1; fin=Dim-1
		break
		case "C_BoutonPointsPrecedents":
			debut=0; fin=i-1
		break	
		case "C_BoutonTousPoints":
			debut=0; fin=Dim-1
		break
		case "C_LitPdV":
			Wave W_PointsDeVue
			vIndexPdV=0
			debut=W_PointsDeVue[vIndexPdV]
		break
	EndSwitch
	i=debut
	Do
		ControlInfo/W=P_Controle C_CorrigeAB
		if (V_Value)
			vImageLue=LitCettePosition()	//	Récupère P_A, P_B (et autres)
			if (vImageLue==0)
				vOnContinue=0
				break
			endif
			CorrigeAB(vOptimiseDep)
		else
			GereImages(i, 1, 2)
		endif
		ControlInfo/W=P_Controle C_CalculerRatio
		if (V_Value)
			CalculerRatio()
		else
			GereImages(i, 1, 4)
		endif
		ControlInfo/W=P_Controle C_Mesurer
		if (V_Value)
			for (j=1; (j<=V_NbROI); j+=1)
				if (W_Tag[j]>-1)	//	mesure seulement les traces qui ne sont pas le fond
					MesureNouveau(j)
				endif
			endfor
		endif
		ControlInfo/W=P_Controle C_Calculer_P_RM
		if (V_Value)
			Calculer_P_RM()
		else
			GereImages(i, 1, 3)
		endif
		Legendes()
		MarqueurEvt()
		DoUpdate
		//	On teste si c'est fini ou s'il faut continuer
		StrSwitch(ctrlName)
			case "C_LitPdV":
				vIndexPdV+=1
				if (NumType(W_PointsDeVue[vIndexPdV])==0)
					i=W_PointsDeVue[vIndexPdV]
				else
					vOnContinue=0
				endif
			break
			case "C_BoutonAjoutePoints":
			case "AjoutePointsAuto":
					i+=1
			break
			default:
				if (i>=fin)
					vOnContinue=0
				else
					i+=1
				endif
			break
		EndSwitch
	While (vOnContinue)

	//	Mesure du temps d'exécution
	vDuree = stopMSTimer(timerRefNum)/1e6
	if (vDuree<60)
		sprintf S_Duree_Analyse, "Duration: %g s", vDuree
	else
		vDuree/=60
		sprintf S_Duree_Analyse, "Duration: %g min", vDuree
		if (vDuree/120)		//	si le traitement prend plus de 2 min
			SignalSonore(5)		//	met un signal sonore
		endif
	endif

	//	Désactive tous les boutons
	MaJ_CasesACocher("Desactive", 0, "", "")
	
	//	on met à jour la trace moyenne, si elle est activée
	ControlInfo/W=P_Controle C_AfficheMoy
	if (V_Value)
		CheckAfficheMoy("", 1)
	endif
	SetDataFolder root:
End

//____________________________________________________________________
// *** Setting checkboxes ***
//	Activate all checkboxes: 	MaJ_CasesACocher("Tous", 0, "", "")	
// *** translate ***
//	Fonctions activées par les différents contrôles P_Controle
//	Met à jour les cases à cocher en fonction de la commande qui a déclenché l'action
//	Si ctrlName ne fait pas partie de la liste, les variables restent NaN et in fine, pas de changement des cases à cocher
Function MaJ_CasesACocher(ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName
	Variable varNum
	String varStr
	String varName
	variable AB=NaN, Ratio=NaN, Mesure=NaN, Couleur=NaN
	
	//	Check that the destination window exists
	DoWindow P_Controle
	if (V_Flag==0)
		return(0)
	endif
	
	StrSwitch(ctrlName)
		case "Desactive":
			AB=0;Ratio=0;Mesure=0;Couleur=0
		break
		case "ToutSaufAB":
			AB=0;Ratio=1;Mesure=1;Couleur=1
		break
		case "Mesure":
			Mesure=1
		break
		case "Couleur":
		case "C_SetResolution":
		case "C_IMin":
		case "C_IMax":
		case "C_RMin":
		case "C_RMax":
		case "C_CorrGamma":
		case "C_CorrRatio":
		case "C_CorrRatio2":
			Couleur=1
		break		
		case "C_R0":
		case "C_SeuilRatio":
		case "C_CheckIMoyAB":
		case "C_CheckIMoyA":
		case "C_CheckIMoyB":		
		case "C_CheckRatioModule":
		case "C_CheckRatioDesMoyennes":
			Ratio=1;Mesure=1;Couleur=1
		break
		case "Tous":
		case "C_NbPlansLissage":
		case "C_PMoy":
		case "C_PMI":
		case "CheckActivePanneauP_Masquage":
		case "C_Active_P_SelectionPlans":
		case "C_CheckBoxDeplacement":
		case "C_CheckBoxDecalage":
			AB=1;Ratio=1;Mesure=1;Couleur=1
		break
		case "AB":
			AB=1
		break
	EndSwitch
	//	Coche les cases correspondantes selon la valeur 0 ou 1
	if (NumType(AB)==0)
		CheckBox C_CorrigeAB win=P_Controle, value=AB
	endif
	if (NumType(Ratio)==0)
		CheckBox C_CalculerRatio win=P_Controle, value=Ratio
	endif
	if (NumType(Mesure)==0)
		CheckBox C_Mesurer win=P_Controle, value=Mesure
	endif
	if (NumType(Couleur)==0)
		CheckBox C_Calculer_P_RM win=P_Controle, value=Couleur
	endif
end

//	Fonctions relais qui transmettent à MaJ_CasesACocher le nom du contrôle; 
//	nécessaire car les CheckBoxControl ne peuvent pas activer une fonction: SetVariableControl
Function CheckProc(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	MaJ_CasesACocher(ctrlName, 0, "", "")
End

//	Pour cocher toutes les cases de calcul
Function BoutonTousCalculs(ctrlName) : ButtonControl
	String ctrlName
	CheckProc("Tous", 1)
End

//	Radio buttons to determine how the ratio is calculated
//	This function is also called when P_Control is initialized, 
//	to set the radio buttons to match the value of the related variable
Function CheckModeCalcul(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	NVAR/SDFR=dfROI V_ModeCalcul
	NVAR/SDFR=dfRatio V_ModeIMoy, V_SensRatio

	StrSwitch(ctrlName)
//	Handling ratio quantif group
		case "C_CheckRatioDesMoyennes":
			V_ModeCalcul=0
		break
		case "C_CheckRatioModule":
			V_ModeCalcul=1
		break
		case "C_CheckRatioSimple":
			V_ModeCalcul=2
		break
//	Handling IMoy group
		case "C_CheckIMoyAB":
			V_ModeIMoy=1
		break
		case "C_CheckIMoyA":
			V_ModeIMoy=2
		break
		case "C_CheckIMoyB":
			V_ModeIMoy=3
		break
//	Handling calculation mode group		
		case "C_CheckBA":
			V_SensRatio=0
		break
		case "C_CheckAB":
			V_SensRatio=1
		break
		case "C_CheckB1":
			V_SensRatio=2
		break
		case "C_CheckA1":
			V_SensRatio=3
		break
	EndSwitch

//	Sets radio buttons to correspond to the value of the associated variable
//	Also executed when the panel is initialized
	CheckBox C_CheckRatioDesMoyennes, value=V_ModeCalcul==0
	CheckBox C_CheckRatioModule, value=V_ModeCalcul==1
	CheckBox C_CheckRatioSimple, value=V_ModeCalcul==2

	CheckBox C_CheckIMoyAB, value=V_ModeIMoy==1
	CheckBox C_CheckIMoyA, value=V_ModeIMoy==2
	CheckBox C_CheckIMoyB, value=V_ModeIMoy==3

	CheckBox C_CheckBA, value=V_SensRatio==0
	CheckBox C_CheckAB, value=V_SensRatio==1
	CheckBox C_CheckB1, value=V_SensRatio==2
	CheckBox C_CheckA1, value=V_SensRatio==3

	//	Check the calculation of all steps
	MaJ_CasesACocher("ToutSaufAB", 0, "", "")
End

//	For the current data point, tries to read a file for each P_A, P_B (P_C etc...) from the cache directory
//	If the expected file does not exist, call ProcessRawData to read the raw file

//	Si le fichier n'existe pas pour ce n° de fichier, incrémente le n° de fichier de 1, tente à nouveau
//	Répète la tentative de lecture 5 fois, puis arrête.
Function LitCettePosition()
	variable k

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	Wave/SDFR=dfRatio W_Deplacement, W_Decalage, W_NumTrame, W_Temps
	NVAR/SDFR=dfRatio V_NumCoucheCourante, Dim, V_NumPlan, V_NbVoies

	if (V_NumCoucheCourante<0)
		V_NumCoucheCourante=0
	endif

	Variable compteur=1, resultat=0
	
	// First looks for P_A et P_B... in the cache directory
	if (GereImages(V_NumCoucheCourante,1,1))
		Return(1)	//	P_A et P_B were read properly
		
	//	no image in the cache => (re)read the image from the raw file
	else
		if  (NumType(W_NumTrame[V_NumCoucheCourante]) == 2)	//	the index of the raw file is not defined (NaN)
			do	//	try to read the file # that follows the last file number in the table W_NumTrame
				W_NumTrame[V_NumCoucheCourante]=W_NumTrame[V_NumCoucheCourante-1]+compteur
				Resultat=ProcessRawData(V_NumCoucheCourante)
				if (resultat!=1)	//	failed
					compteur+=1
				endif
			//	repeat the loop until a file is read with a maximum of 5 attempts
			while ((resultat!=1) && (compteur<5))	
			if (resultat==1)
				//	A raw file was successfully read => update total number of datatpoints in the experiment
				Dim=FDim()
				MaJGlissieres()
			else
				//	no new data point was read : reached the end of the experiment
				//	This case happens regularly when analyzing an experiment in real time, 
				//	when new data files keep being added on disk
				W_NumTrame[V_NumCoucheCourante]=NaN	//	the current time point failed
				if (V_NumCoucheCourante>0)
					V_NumCoucheCourante-=1
				endif
				BoutonLitEvt("")	//	update events
				W_Temps[Dim,]=NaN	//	clears time values that may have been created at startup and now known to be useless
				return(0)
			endif
		
		else // W_NumTrame is defined, -> a corresponding raw file should exist
			Resultat = ProcessRawData(V_NumCoucheCourante)
			if (Resultat!=1)
				//	failed to read the expected raw file
				if (V_NumCoucheCourante>0)
					V_NumCoucheCourante-=1
				endif
				return(0)
			endif
		endif	//	a file was read; closure of: if  ((NumType(W_NumTrame[V_NumCoucheCourante]) == 2) && (!V_Value))
	endif	// closure of: if (!GereImages(V_NumCoucheCourante,1))	
	return(1)	//	Success
end

//____________________________________________________________________________________
// *** A & B corrections ***
//	Performs the user-selected corrections in the following order:
//	movement
//	registration
//	various things on image stacks such as maximum intensity projection, masking, plane selection...
//	background subtraction

Function CorrigeAB(vOptimiseDep)
	variable vOptimiseDep		//	if 1, movement optimization is based on the previous movement value
	Variable FondA, FondB, compteur
	Variable dx, dy,dz, res, i, j
	Variable dx0, dy0, dz0
	Variable numVoie, vCorrZ
	String sNomWave, sNomWaveCorr, sNomPileRef, sNomPileCorr
	Variable x0, y0, z0, x1, y1, z1, vNbZones	//	zones for movement and registration
	Variable vSX, vSY, vSZ
	variable timer=0, timerRefNum, temps1, temps2
	variable vTempX, vTempY

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	Wave/SDFR=dfROI W_Tag, W_Fond
	NVAR/SDFR=dfROI V_NbROI

	SetDataFolder dfRatio
	NVAR/SDFR=dfRatio V_NumCoucheCourante, V_RMin, V_RMax, V_IMin, V_IMax, V_NumPlan, V_R0, V_NbVoies
	NVAR/SDFR=dfRatio V_Res, V_Position0Deplacement, V_MethodeDeplacement, V_GrossiereteDep, V_GrossiereteDepOK, V_GrossiereteDec, V_GrossiereteDecOK
	NVAR/SDFR=dfRatio V_Lissage, V_NbPlans, V_Taille_X, V_Taille_Y, V_NbPlans, V_NbChiffres
	SVAR/SDFR=dfRatio S_VoieRefDeplacement, S_VoieRefDecalage
	Wave/SDFR=dfRatio P_A, P_B, D_UnifA, D_UnifB, P_ACorr, P_BCorr, W_Deplacement, W_Decalage, W_ZoneRefDeplacement, W_ZoneRefDecalage

//	------------   Movement correction   --------------------

	Switch(V_MethodeDeplacement)
		case 1:		//	méthode en calculant la modification par-rapport à l'image précédente
			Controlinfo/W=P_Controle C_CheckBoxDeplacement	// case déplacement automatique cochée? Réponse dans V_Value
			if ((V_NumCoucheCourante>0) && V_Value && (NumType(W_Deplacement[V_NumCoucheCourante][0])!=0))
				dx=W_Deplacement[V_NumCoucheCourante-1][0]; dy=W_Deplacement[V_NumCoucheCourante-1][1]; dz=W_Deplacement[V_NumCoucheCourante-1][2]
				if ((NumType(dx) != 0) || (NumType(dy) != 0) || (NumType(dz) != 0))	//	si le déplacement au point précédent n'est pas connu, termine
					DoAlert 0, "Déplacement au point précédent n'est pas connu"
					return(0)
				endif
				GereImages((V_NumCoucheCourante-1), 1, 1)		//	Lit P_A et P_B pour le point de temps précédent
				Duplicate/O P_B, P_PilePrecedente
				tsDeplacePile(P_PilePrecedente, dx, dy, dz, 0)		//	Applique à P_PilePrecedente le déplacement calculé pour le point précédent
				GereImages(V_NumCoucheCourante, 1, 1)		//	Lit P_A et P_B pour ce point de temps
				tsDeplacePile(P_B, dx, dy, dz, 0)				//	Déplace P_B du mouvement déjà connu
				Controlinfo/W=P_Deplacement C_CheckOptimiseZ	//	Check if optimization includes Z
				vCorrZ=V_Value
				//	Compare la pile du point courant avec le point précédent
				ImageRegistration/Q/CSNR=0/BVAL=0/CONV=1/FLVL=2/PRDL=6/TRNS={1,1,vCorrZ}/ROT={0,0,0}/REFM=0/TSTM=0 testwave=P_B, refwave=P_PilePrecedente
				Wave W_RegParams
				vSX=round(W_RegParams[0]);vSY=round(W_RegParams[1]);vSZ=round(W_RegParams[2])	//	différence entre point précédent et point courant
	
				W_Deplacement[V_NumCoucheCourante][]=W_Deplacement[V_NumCoucheCourante-1][q]	//	copie le déplacement du point précédent dans le point courant
				W_Deplacement[V_NumCoucheCourante][0]+=vSX; W_Deplacement[V_NumCoucheCourante][1]+=vSY; W_Deplacement[V_NumCoucheCourante][2]+=vSZ		//	Ajoute la différence
	
				print "Déplacement au point ", V_NumCoucheCourante," de dx= ", vSX, " dy= ", vSY, "dz=", vSZ
			endif
	
			// Relit toutes les images P_?Corr à partir de P_?
			GereImages(V_NumCoucheCourante, 1, 1)		//	Relit P_A et P_B pour ce point de temps
			for (numVoie=1; (numVoie<=V_NbVoies); numVoie+=1)
				sNomWave="P_"+StringFromList(numVoie-1, ksABC)
				sNomWaveCorr=sNomWave+"Corr"
				Wave w=$sNomWave, wCorr=$sNomWaveCorr
				FastOp wCorr=w
			endfor
			dx=W_Deplacement[V_NumCoucheCourante][0]; dy=W_Deplacement[V_NumCoucheCourante][1]; dz=W_Deplacement[V_NumCoucheCourante][2]
			DeplaceLesPiles(dx, dy, dz, 0)		// on applique au point courant le déplacement déjà connu
		break
	
		case 0:		//	méthode en prenant comme référence des zones sur une image initiale
	
		//	On passe l'image brute à l'image corrigée:
		//	on définit le nom des waves P_?Corr=P_? pour toutes les voies de données, et on passe la valeur brute vers la valeur Corr
			for (numVoie=1; (numVoie<=V_NbVoies); numVoie+=1)
				sNomWave="P_"+StringFromList(numVoie-1, ksABC)
				sNomWaveCorr=sNomWave+"Corr"
				Wave w=$sNomWave, wCorr=$sNomWaveCorr
				FastOp wCorr=w
			endfor
	
			dx=W_Deplacement[V_NumCoucheCourante][0]
			dy=W_Deplacement[V_NumCoucheCourante][1]
			dz=W_Deplacement[V_NumCoucheCourante][2]
	
			//	Si une valeur de déplacement est déjà connue, on l'applique, et c'est tout.
			if ((NumType(dx) == 0) && (NumType(dy) == 0) && (NumType(dz) == 0))
				DeplaceLesPiles(dx, dy, dz, 0)		// on applique le déplacement déjà connu; peut être un déplacement manuel à optimiser encore
			endif
			
			Controlinfo/W=P_Controle C_CheckBoxDeplacement	// case déplacement automatique cochée? Réponse dans V_Value
			//	Cette condition s'effectue dans deux cas: 1/ l'optimisation est requise explicitement; 2/ pas de valeur de déplacement connue (dx=NaN)
			if (((vOptimiseDep==1) || (NumType(dx) != 0)) && (V_Value==1) && (V_NumCoucheCourante!=V_Position0Deplacement))
				
				//	On part sur l'optimisation
				
				//	Vérifie si les zones de référence sont définies; arrête la fonction si zones déplacement n'existent pas et fait ce qu'il faut pour les créer
				DefinitZonesDeplacement()	//	Arrête l'exécution si les zones ref ne sont pas initialisées

				vNbZones=DimSize(W_ZoneRefDeplacement,1)	//	Get the number of zones from the size of this table
				
				//	Met à jour l'indicateur de pertinence de la grossièreté d'ajustement
				//	Cherche la taille la plus petite parmi les zones de correction de déplacement
				vTempX=V_Taille_X; vTempY=V_Taille_Y
				For (i=0; (i<vNbZones); i+=1)
					vTempX=min(vTempX, (W_ZoneRefDeplacement[3][i]-W_ZoneRefDeplacement[0][i]))
					vTempY=min(vTempY, (W_ZoneRefDeplacement[4][i]-W_ZoneRefDeplacement[1][i]))					
				endfor
				vTempX=min(vTempX, vTempY)	//	On garde le plus petit de X et Y
				V_GrossiereteDepOK=vTempX/2^(V_GrossiereteDep-1)		//	H/2^(depth -1) ≈ 30 selon référence de ImageRegistration
				
				ControlInfo/W=P_Deplacement C_CheckOptimiseFromPrevious	//	indique si on se base sur le point précédent
				if ((V_NumCoucheCourante>0) && V_Value && (vOptimiseDep==0))	//	on se base sur le déplacement de l'image précédente
					dx0=W_Deplacement[V_NumCoucheCourante-1][0]; dy0=W_Deplacement[V_NumCoucheCourante-1][1]; dz0=W_Deplacement[V_NumCoucheCourante-1][2]
					if ((NumType(dx0) == 0) && (NumType(dy0) == 0) && (NumType(dz0) == 0))
						DeplaceLesPiles(dx0, dy0, dz0, 0)
						W_Deplacement[V_NumCoucheCourante][0]=dx0; W_Deplacement[V_NumCoucheCourante][1]=dy0; W_Deplacement[V_NumCoucheCourante][2]=dz0
					else
						W_Deplacement[V_NumCoucheCourante][0]=0; W_Deplacement[V_NumCoucheCourante][1]=0; W_Deplacement[V_NumCoucheCourante][2]=0
					endif
				else	
					if (vOptimiseDep==0)	//	no starting value was given
						W_Deplacement[V_NumCoucheCourante][]=0	//	on initialise le déplacement à 0
						dx0=0; dy0=0; dz0=0
					else
						//	on garde juste dx, dy, dz, valeurs suggérées manuellement
					endif
				endif
		
				//	At this stage, the stack has been moved by any known value
				//	Go for optimization
				sNomWaveCorr="P_"+S_VoieRefDeplacement+"Corr"	//	Name of the wave containing the channel to be used for this calculation
				Wave/SDFR=dfRatio pCorr=$sNomWaveCorr
				Controlinfo/W=P_Deplacement C_CheckOptimiseZ	//	Check if optimization includes Z
				vCorrZ=V_Value
				vSX=0; vSY=0; vSZ=0			//	Initialize counter for average calculation
				Make/N=6/I/U/FREE wZone		//	To hold the zone definition
				For (i=0; (i<vNbZones); i+=1) //	loop for each defined zone
					//	Get the image zone to do the registration
					wZone=W_ZoneRefDeplacement[p][i]
					Wave pTest=ExtraitZone(wZone, pCorr)
					//	Reference zone
					sNomPileRef="P_RefDep"+num2str(i)
					Wave pRef=$sNomPileRef
		
		//	Testé et éprouvé dans la douleur!
		//	en résumé: il faut s'assurer qu'il n'y ait pas de NaN et inf dans les images qui vont être traitées, sinon ça part en sucette
		//	le refmask est appliqué, mais pas le testmask, sinon les données M_RegOut sont coupées par le masque
		//	si on utilise conv=0, la précision est vraiment pourrie, de l'ordre de la dizaine de pixels; conv=1 marche bien et jusqu'à des dizaines de pixels
		//			WaveStats/M=1 pCorr	//	Eventuellement un test pour vérifier l'absence de NaN
		//			if (V_numNans>0)
		//				print "attention: un NaN dans l'image ", V_NumCoucheCourante, ": calcul de déplacement impossible"
		//			endif
					if (V_NbPlans>1)
						ImageRegistration/Q/CSNR=0/BVAL=0/CONV=1/FLVL=2/PRDL=(V_GrossiereteDep)/TRNS={1,1,vCorrZ}/ROT={0,0,0}/REFM=0/TSTM=0 testwave=pTest, refwave=pRef
					else	//	pas encore testé si TRNS={1,1,1} s'applique bien aux données à 1! plan
						ImageRegistration/Q/CSNR=0/BVAL=0/CONV=1/FLVL=1/PRDL=(V_GrossiereteDep)/TRNS={1,1,0}/ROT={0,0,0}/REFM=0/TSTM=0 testwave=pTest, refwave=pRef
					endif
					Wave W_RegParams
					vSX+=W_RegParams[0];vSY+=W_RegParams[1];vSZ+=W_RegParams[2]
				EndFor	//	fin boucle pour chaque zone
//				Killwaves pTest		//	supprime après avoir utilisé la pile test
		
				vSX=round(vSX/vNbZones);vSY=round(vSY/vNbZones);vSZ=round(vSZ/vNbZones)	//	calcule la moyenne du déplacement
				print "Déplacement au point ", V_NumCoucheCourante," de dx= ", vSX, " dy= ", vSY, "dz=", vSZ
				W_Deplacement[V_NumCoucheCourante][0]+=vSX; W_Deplacement[V_NumCoucheCourante][1]+=vSY; W_Deplacement[V_NumCoucheCourante][2]+=vSZ
				DeplaceLesPiles(vSX, vSY, vSZ, 0)
			endif
		break	//	fin de la méthode référence à une image initiale
	EndSwitch

//	--------------- Correction du décalage  -------------

	//	on détermine les noms des voies de référence et corrigée
	sNomWave="P_"+S_VoieRefDecalage+"Corr"
	StrSwitch(S_VoieRefDecalage)
		case "A":	//	voie de référence = A; on décale B
			sNomWaveCorr="P_BCorr"
		break
		case "B":	//	voie de référence = B; on décale A
			sNomWaveCorr="P_ACorr"
		break
	endswitch
	Wave wPRef=$sNomWave, wPCorr=$sNomWaveCorr

	Controlinfo/W=P_Controle C_CheckBoxDecalage
//	Si la case est cochée et le décalage est indéterminé (NaN), alors on calcule automatiquement la valeur de décalage
	if ((V_Value) && ((NumType(W_Decalage[V_NumCoucheCourante][0])==2) || (NumType(W_Decalage[V_NumCoucheCourante][1])==2)))	// le décalage n'est pas défini et la case est cochée
		ControlInfo/W=P_Decalage C_Skew
		if (V_Value==1)		//	la case "Allow skewing" est cochée -> correction impliquant l'étirement de l'image
		//	Effectue le calcul sur la totalité de l'image: ne nécessite pas un masque
			ImageRegistration/ASTP=4/BVAL=0/CONV=1/CSNR=1/FLVL=1/INTR=2/PRDL=4/Q/REFM=0/ROT={0,0,0}/SKEW={1,1,0}/TSTM=0/TRNS={1,1,0} testwave=wPCorr, refwave=wPRef
			Wave M_RegOut, W_RegParams
			FastOp wPCorr=M_RegOut
			W_Decalage[V_NumCoucheCourante][]=W_RegParams[q]	//	A la fin, le résultat du calcul du décalage est remis dans le tableau
		else
			//	Une série de zones sont utilisées pour calculer le décalage local. A la fin, la translation mesurée sur chacune de ces zones
			//	est moyennée pour donner la translation totale. Enfin, le décalage moyen est appliqué à toute l'image.
			vNbZones=DimSize(W_ZoneRefDecalage,1)
			vSX=0; vSY=0; vSZ=0
			for (i=0; (i<vNbZones); i+=1)
				x0=W_ZoneRefDecalage[0][i]; y0=W_ZoneRefDecalage[1][i]; z0=W_ZoneRefDecalage[2][i]
				x1=W_ZoneRefDecalage[3][i]; y1=W_ZoneRefDecalage[4][i]; z1=W_ZoneRefDecalage[5][i]
				if ((numtype(x0)==2) || (numtype(y0)==2) || (numtype(x1)==2) || (numtype(y1)==2))	//	la zone de calcul n'est pas définie
					DoAlert 0, "The zone #"+num2char(i)+" is not defined; calculation aborted."
					Return(0)
				endif
				Make/O/FREE/N=(x1-x0,y1-y0,z1-z0) P_Ref, P_Corr
				P_Ref=wPRef[x0+p][y0+q][z0+r]
				P_Corr=wPCorr[x0+p][y0+q][z0+r]
				ImageRegistration/CONV=1/CSNR=0/FLVL=1/PRDL=5/INTR=2/Q/TRNS={1,1,0}/ROT={0,0,0}/ZMAN testwave=P_Corr, refwave=P_Ref
				Wave W_RegParams
				vSX+=W_RegParams[0];vSY+=W_RegParams[1];vSZ=W_RegParams[2]
			endfor
			vSX/=vNbZones; vSY/=vNbZones; vSZ/=vNbZones	//	calcule la moyenne du décalage
			W_Decalage[V_NumCoucheCourante][]=nan
			W_Decalage[V_NumCoucheCourante][0]=round(vSX*V_Res)/V_Res	//	réduit le décalage à la résolution fixée
			W_Decalage[V_NumCoucheCourante][1]=round(vSY*V_Res)/V_Res
			DeplacePlanSubPix(sNomWaveCorr, W_Decalage[V_NumCoucheCourante][0], W_Decalage[V_NumCoucheCourante][1], V_Res)
		endif
	else
		if ((NumType(W_Decalage[V_NumCoucheCourante][0])==0) && (NumType(W_Decalage[V_NumCoucheCourante][1])==0))	// décalage déjà calculé
			if (NumType(W_Decalage[V_NumCoucheCourante][3]==0))
				W_RegParams=W_Decalage[V_NumCoucheCourante][p]
				ImageRegistration/Q/ROT={0,0,0}/SKEW={1,1,0}/TSTM=0/TRNS={1,1,0}/USER=W_RegParams testwave=wPCorr, refwave=wPRef
				Wave M_RegOut
				FastOp wPCorr=M_RegOut
			else	//	décalage translation simple
				DeplacePlanSubPix(sNomWaveCorr, W_Decalage[V_NumCoucheCourante][0], W_Decalage[V_NumCoucheCourante][1], V_Res)
			endif
		endif
	endif
//	--------------  fin du décalage -------------- 

//	Une série de corrections et modifications de piles d'images

//	On efface éventuellement certains plans non souhaités dans l'image finale
	ControlInfo/W=P_Controle C_Active_P_SelectionPlans
	if (V_Value)
		Wave W_SelectionPlans
		for (i=0;(i<V_NbPlans);i+=1)
			if (W_SelectionPlans[i]==0)
				P_ACorr[][][i]=0
				P_BCorr[][][i]=0
			endif
		endfor
	endif
	
//	On efface des régions de l'image à l'aide d'un masque 3D
	ControlInfo/W=P_Controle C_Active_P_Masquage
	if (V_Value)
		Wave P_Masque
		FastOp P_ACorr=P_ACorr*P_Masque
		FastOp P_BCorr=P_BCorr*P_Masque
	endif
		
//	Pour les piles, éventuellement lissage en calculant la moyenne de plusieurs plans
	if ((V_Lissage>1) && (V_NbPlans>2))
		Smooth/DIM=2/B/F V_Lissage, P_ACorr
		Smooth/DIM=2/B/F V_Lissage, P_BCorr
	endif

//	projection du maximum de fluorescence
	ControlInfo/W=P_Controle C_PMI

	if (V_Value)
		for (j=0; j<V_NbVoies; j+=1)
			sNomWave="P_"+StringFromList(j, ksABC)+"Corr"
			Wave wPile=$sNomWave
			ImageTransform/METH=1 zProjection, wPile
			Wave M_zProjection
			ImageTransform/D=M_zProjection/P=0 setplane wPile
		endfor
	endif
//	projection de la moyenne de fluorescence
	ControlInfo/W=P_Controle C_PMoy
	if (V_Value)
		ImageTransform/METH=2 zProjection, P_ACorr
		Wave M_zProjection
		ImageTransform/D=M_zProjection/P=0 setplane P_ACorr
		ImageTransform/METH=2 zProjection, P_BCorr
		ImageTransform/D=M_zProjection/P=0 setplane P_BCorr
	endif
//	ImageTransform/METH=1 zProjection, P_ACorr
//	P_ACorr et P_BCorr contiennent les images corrigées pour le déplacement et le décalage, éventuellement moyennées

//	Measure background intensity
	FondA=0; FondB=0; compteur=0
	for (j=1; (j<=V_NbROI); j+=1)	//	test each ROI
		if(W_Tag[j]==-1)	//	if W_Tag[j] = -1, this ROI is background and its intensity must be measured
			String NomTraceA, NomTraceB
			NomTraceA=AjouteNumero("TraceA",j, V_NbChiffres)
			NomTraceB=AjouteNumero("TraceB",j, V_NbChiffres)
			Wave/SDFR=dfROI  TraceA=$NomTraceA, TraceB=$NomTraceB
			MesureNouveau(j)	// Calls the measurement function which measures ROI intensity, then returns because tagged -1
			FondA+=TraceA[V_NumCoucheCourante]; FondB+=TraceB[V_NumCoucheCourante]; compteur+=1
		EndIf
	endfor
	if (compteur>=1)
		FondA/=compteur; FondB/=compteur	// calculates the average
		W_Fond[V_NumCoucheCourante][0]=FondA; W_Fond[V_NumCoucheCourante][1]=FondB	// updates background value in the waves
		DoUpdate
	endif

//	Subtracts background
	if (W_Fond[V_NumCoucheCourante][0]!=0)
		FastOp P_ACorr=P_ACorr-(W_Fond[V_NumCoucheCourante][0])
	endif
	if (W_Fond[V_NumCoucheCourante][1]!=0)
		FastOp P_BCorr=P_BCorr-(W_Fond[V_NumCoucheCourante][1])
	endif
	
//	P_ACorr & P_BCorr are now corrected, -> saved
	GereImages(V_NumCoucheCourante, 2, 2)
	SetDataFolder root:
end

//	Correction de déplacement: actionné par onglet de sélection de méthode
Function OngletDeplacement(ctrlName,tabNum) : TabControl
	String ctrlName
	Variable tabNum
	
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	NVAR/SDFR=dfRatio V_MethodeDeplacement

	//	Liste des contrôles qui doivent être modifiés lorsque l'onglet est changé
	String sListeOnglet0="C_Position0;C_DessineRegDeplacement;C_EffaceRegDeplacement;C_CheckOptimiseFromPrevious"
	String sListeOnglet1=""

	V_MethodeDeplacement=tabnum

	Switch(tabnum)
		case 0:
			ModifyControlList sListeOnglet0, win=P_Deplacement, disable=0
			ModifyControlList sListeOnglet1, win=P_Deplacement, disable=1
			SetVariable C_Position0 win=P_Deplacement, disable=2	//	sinon apparaît modifiable, ce qui ne doit pas être permis
		break
		case 1:
			ModifyControlList sListeOnglet0, win=P_Deplacement, disable=1
			ModifyControlList sListeOnglet1, win=P_Deplacement, disable=0
		break
	endswitch	
	return 0
End

//	Gets the movement wave and draws an avatar of it for manual smooting of datapoints
//	After it's done, copy these smoothed values back in W_Deplacement
Function LisseDeplacement(ctrlName)
	String ctrlName
	variable i
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	
	SetDataFolder dfRatio
	NVAR DIm
	Wave W_Deplacement

	StrSwitch(ctrlName)
		case "init":
			Make/O/N=(Dim) W_Dep_X, W_Dep_Y, W_Dep_Z
			W_Dep_X=W_Deplacement[p][0]
			W_Dep_Y=W_Deplacement[p][1]
			W_Dep_Z=W_Deplacement[p][2]
			Display W_Dep_Y vs W_Dep_X
			GraphWaveEdit/NI W_Dep_Y
		break
		default:
			graphnormal
			Wave W_Dep_X, W_Dep_Y, W_Dep_Z
			For (i=0; i<DimSize(W_Dep_X, 0); i+=1)
				W_Deplacement[i][0]=W_Dep_X[i]
				W_Deplacement[i][1]=W_Dep_Y[i]
				W_Deplacement[i][2]=W_Dep_Z[i]
			endfor
		break
	endswitch
End

//________________________________________________________________________
// *** Ratio calculation *****************************************

Function CalculerRatio()
	variable mini, maxi

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]	
	SetDataFolder dfRatio
	NVAR V_ModeIMoy, V_R0, V_RMin, V_RMax, V_SensRatio, V_NumCoucheCourante, V_SeuilRatio
	Wave P_ACorr, P_BCorr, P_Ratio, P_IMoy, P_RatioxIMoy
	variable temp

	Switch(V_ModeIMoy)
		case 1:
			FastOp P_IMoy=0.5*P_ACorr+0.5*P_BCorr
		break
		case 2:
			FastOp P_IMoy=P_ACorr
		break
		case 3:
			FastOp P_IMoy=P_BCorr
		break
	EndSwitch
	Switch(V_SensRatio)
		case 0:	//	B / A
			FastOp P_Ratio=(V_R0)*P_BCorr/P_ACorr
		break
		case 1:	//	A / B
			temp=1/V_R0
			FastOp P_Ratio=(temp)*P_ACorr/P_BCorr
		break
		case 2:	//	B / 1
			FastOp P_Ratio=(V_R0)*P_BCorr
		break
		case 3:	//	A / 1
			temp=1/V_R0
			FastOp P_Ratio=(temp)*P_ACorr
		break
	EndSwitch

	//	clean ratio image of inf, nan, too big and too small values
	mini=V_RMin/V_SeuilRatio; maxi=V_RMax*V_SeuilRatio
	EnleveInfsNansW(P_Ratio, mini, maxi)	//	supprime les infs et nans
	MatrixOp/O W_Temp=clip(P_Ratio, mini, maxi)
	FastOp P_Ratio=W_Temp	//	seuille et élimine les ratios extrêmes
	
	FastOp P_RatioxIMoy=P_Ratio * P_IMoy	

	//	Save calculated images
	GereImages(V_NumCoucheCourante, 2, 4)
	Return(1)
end

//_____________________________________________________________________________________________
// *** Build the pseudocolor image ***
Function Calculer_P_RM()
	variable compteur, vCouleur
	
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	SetDataFolder dfRatio
	NVAR i=V_NumCoucheCourante, V_NbPlans, V_Taille_X, V_Taille_Y, V_Trait
	Wave P_IMoy, P_Ratio, P_RatioxIMoy, P_RM, D_HSLTemp, W_Exposition, W_CarreCalib
	Wave/T W_EvtStr
	NVAR V_RMin, V_RMax, V_IMin, V_IMax, V_CorrGamma, V_CorrRatio, V_CorrRatio2, V_DimCarrePix, V_PosCarreXPix, V_PosCarreYPix

// Make the calibration square
	if (W_CarreCalib[i]>0)
		Make/O/FREE/N=(V_DimCarrePix, V_DimCarrePix) D_Carrei, D_CarreR	//	carrés gradient N/B qui forment l'échelle de couleurs
		D_Carrei=V_IMin+p*(V_IMax-V_IMin)/V_DimCarrePix
		D_CarreR=V_RMin+(V_DimCarrePix-q)*(V_RMax-V_RMin)/V_DimCarrePix
		for (compteur=0; (compteur<V_NbPlans);compteur+=1)
			imagetransform/INSI=D_Carrei/INSX=(V_PosCarreXPix)/INSY=(V_PosCarreYPix)/P=(compteur) insertimage P_IMoy
			imagetransform/INSI=D_CarreR/INSX=(V_PosCarreXPix)/INSY=(V_PosCarreYPix)/P=(compteur) insertimage P_Ratio
		endfor
	endif		

	//	calls the function that calculates a pseudocolor image
	IntensityModulatedDisplay(P_Ratio, P_IMoy, P_RM)

//	Draw a white square around the rainbow calibration square; thickness is 2*V_trait+1 pixels
	if (W_CarreCalib[i]>0)
		P_RM[V_PosCarreXPix-V_trait, V_PosCarreXPix+V_DimCarrePix+V_trait][V_PosCarreYPix-V_trait, V_PosCarreYPix][]=65535
		P_RM[V_PosCarreXPix-V_trait, V_PosCarreXPix+V_DimCarrePix+V_trait][V_PosCarreYPix+V_DimCarrePix, V_PosCarreYPix+V_DimCarrePix+V_trait][]=65535
		P_RM[V_PosCarreXPix-V_trait, V_PosCarreXPix+V_trait][V_PosCarreYPix-V_trait, V_PosCarreYPix+V_DimCarrePix+V_trait][]=65535
		P_RM[V_PosCarreXPix-V_trait+V_DimCarrePix, V_PosCarreXPix+V_DimCarrePix+V_trait][V_PosCarreYPix-V_trait, V_PosCarreYPix+V_DimCarrePix+V_trait][]=65535
	endif
	GereImages(i, 2, 3)	//	Save pseudocolor image
end

// *** Make a pseudocolor image ***
//	used by Calculer_P_RM and also to make an intensity-modulated display of the kymograph
Function IntensityModulatedDisplay(P_Ratio, P_IMoy, P_RM)
	Wave P_Ratio, P_IMoy, P_RM

	DFREF dfSav=GetDataFolderDFR()
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	SetDataFolder dfRatio
	NVAR V_RMin, V_RMax, V_IMin, V_IMax, V_CorrGamma, V_CorrRatio, V_CorrRatio2
	Variable i, vNbPlans=DimSize(P_Ratio,2)

	//	Intensity
	variable vInt=65535/(V_IMax-V_IMin)/2	// Maximal intensity correspond to 32000
	//	spread intensity linearly and clips to the min and max values
	MatrixOp/O/FREE pIMoyCorr=clip((vInt)*P_IMoy-(vInt*V_IMin),0,65535/2)
	if (V_CorrGamma !=1)	// gamma correction of intensity
		MatrixOp/FREE pIMoyCorr=clip(32768*(powR((pIMoyCorr/32768),(1/V_CorrGamma))), 0, 65535/2)
		//	attention, bien galéré sur ce coup: ^ n'est pas interprété comme "power" dans matrixop mais comme  ^h ou ^t; il manque alors
		//	un opérande. C'est une erreur qui est signalée en ligne de commande mais pas dans l'exécution du programme.
	endif
	
	//	Ratio
	variable couleurmax=45300	// keeping only red to blue; we skip the violet, above 45300
	if (V_CorrRatio==1)
		//	spread ratio linearly and clip to min and max values
		variable vRatio=couleurmax/(V_RMax-V_RMin)
		MatrixOp/O/FREE pRatioCorr=clip((-vRatio)*P_Ratio+(vRatio*V_RMax),0,couleurmax)
	else	//	Additional correction with x^(1/3) function. But this doesn't work for negative values. Therefore use y=(abs(x)^^0,3)*sgn(x)
		variable vRdemi=(V_RMax+V_RMin)/2, vCouleurDemi=couleurmax/2
		variable vConst=-V_CorrRatio2*couleurmax/((V_RMax-V_RMin)^V_CorrRatio)
		MatrixOp/O/FREE pRatioCorr= clip((vConst * sgn(P_Ratio-vRdemi) * PowR((abs(P_Ratio-vRdemi)), V_CorrRatio) + vCouleurDemi),0,couleurmax)
	endif

//	This is a weird behavior of Igor, differring between Macos (no error) and windows (reports a wave dimension error)
//	If V_NbPlans=1, pRatioCorr and pIMoyCorr are generated as 2D waves on both systems. 
//	Then, MatrixOp/FREE hh=pRatioCorr[][][i] generates an error on Windows, since pRatioCorr is only 2D, but no error on MacOS
//	Therefore add here a check on dimension to correct it if needed
	if (dimsize(pRatioCorr,2)==0)
		Redimension/N=(-1, -1, DimSize(P_Ratio,2)) pRatioCorr, pIMoyCorr
	endif

//	recombining the different layers
	Make/N=(DimSize(P_Ratio,0),DimSize(P_Ratio,1),3)/O/U/W/FREE pHSL
	FastOp pHSL=65535
	For (i=0; i<vNbPlans; i+=1)
		MatrixOp/FREE hh=pRatioCorr[][][i]
		MatrixOp/FREE vv=pIMoyCorr[][][i]
		ImageTransform/P=(0)/D=hh setplane pHSL
		ImageTransform/P=(2)/D=vv setplane pHSL
		ImageTransform hsl2rgb pHSL
		Wave M_HSL2RGB
		ImageTransform/CHIX=(i) /D=M_HSL2RGB setChunk P_RM
	endfor
	KillWaves M_HSL2RGB
	SetDataFolder dfSav
End

////	Suggestion de AG
//	Testé avec Igor 7, sur grosse pile d'image: la concaténation est plus lente que imagetransform setplane
//Function testAG(hStackWave,sStackWave,lStackWave)
//	Wave hStackWave,sStackWave,lStackWave
//	Variable rows=DimSize(hStackWave,0)
//	Variable cols=DimSize(hStackWave,1)
//	Variable layers=DimSize(hStackWave,2)
//
//	Make/O/N=(rows,cols,3,layers) rgb4DWave
//	Variable i,j
//	for(i=0;i<layers;i+=1)
//		MatrixOP/Free hh=hStackWave[][][i]
//		MatrixOP/Free ss=sStackWave[][][i]
//		MatrixOP/Free ll=lStackWave[][][i]
//		Redimension/N=(-1,-1,1) hh, ss, ll
//		Concatenate/O/NP=2 {hh,ss,ll}, tmpHSL
//		Wave tmpHSL
//		ImageTransform hsl2rgb tmpHSL
//		Wave M_HSL2RGB
//		ImageTransform/CHIX=(i) /D=M_HSL2RGB setChunk rgb4DWave
//	endfor
//End
//


//_____________________________________________________________________________________
// *** Legend and labels on calibration square ***
Function Legendes()
	Variable test, TaillePolice, vChiffres, vCouleur
	String sNom, sFormat

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	SetDataFolder dfRatio
	NVAR i=V_NumCoucheCourante
	Wave W_CarreCalib, W_Temps_min
	Wave/T W_EvtStr
	NVAR V_RMin, V_RMax, V_iMin, V_iMax, V_CorrGamma, V_R0, V_DimCarrePix, V_PosCarreXPix, V_PosCarreYPix,V_DimCarre,V_R0, V_Trait
	SVAR/SDFR=dfRatio S_Nom_Manip

//	Clear what may remain on this layer
	SetDrawLayer/W=F_RM/K ProgFront
	
// Add labels around calibration	square
	if ((W_CarreCalib[i]>0) && (QuelAffichage()==0))
		Switch(W_CarreCalib[i])
			case 1:
				vCouleur=65535
			break
			case 2:
				vCouleur=0
			break
		endswitch
		ControlInfo/W=P_Controle C_SetTaillePolice
		TaillePolice=V_Value
		ControlInfo/W=P_Controle C_SetChiffres
		vChiffres=V_Value
		SetDrawEnv/W=F_RM xcoord= top,ycoord= left,linefgc= (vCouleur,vCouleur,vCouleur),fillpat=0
		SetDrawEnv/W=F_RM textrgb= (vCouleur,vCouleur,vCouleur), fname="Helvetica",fsize= TaillePolice, save
//		SetDrawEnv linethick= 2
//		DrawRect V_PosCarreX-1, V_PosCarreY-1,V_PosCarreX+V_DimCarre-0.5, V_PosCarreY+V_DimCarre-0.5
		SetDrawEnv/W=F_RM textxjust= 1,textyjust= 2, fsize=TaillePolice
		DrawText/W=F_RM V_PosCarreXPix,V_PosCarreYPix+V_DimCarrePix+V_Trait+4, Num2Str(V_iMin)
		SetDrawEnv/W=F_RM textxjust= 1,textyjust= 2, fsize=TaillePolice
		DrawText/W=F_RM V_PosCarreXPix+V_DimCarrePix,V_PosCarreYPix+V_DimCarrePix+V_Trait+4, Num2Str(V_iMax)
		SetDrawEnv/W=F_RM textxjust= 0,textyjust= 2, fsize=TaillePolice
		sFormat="%*.*f"
		sprintf sNom, sFormat, 2, vChiffres, V_RMax
		DrawText/W=F_RM V_PosCarreXPix+V_DimCarrePix+V_Trait+5,V_PosCarreYPix, sNom
		SetDrawEnv/W=F_RM textxjust= 0,textyjust= 2, fsize=TaillePolice
//		DrawText/W=F_Pile_RM V_PosCarreXPix+V_DimCarrePix+2,V_PosCarreYPix+V_DimCarrePix/2-20, "Ro="+Num2Str(V_R0)
		SetDrawEnv/W=F_RM textxjust= 0,textyjust= 0, fsize=TaillePolice
		sprintf sNom, sFormat, 2, vChiffres, V_RMin		
		DrawText/W=F_RM V_PosCarreXPix+V_DimCarrePix+V_Trait+5,V_PosCarreYPix+V_DimCarrePix, sNom
		SetDrawEnv/W=F_RM textxjust= 1,textyjust= 0, fsize=TaillePolice
		DrawText/W=F_RM V_PosCarreXPix+V_DimCarrePix/2,V_PosCarreYPix-4,num2str(V_DimCarre)+" µm"				
		SetDrawEnv/W=F_RM textxjust= 1,textyjust= 0, fsize=TaillePolice
//		DrawText/W=F_Pile_RM V_PosCarreXPix+V_DimCarrePix/2,V_PosCarreYPix-35,S_Nom_Manip				
	endif
end

Function SetVarLegende(ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName
	Variable varNum
	String varStr
	String varName
	Legendes()
End

//	Responds to buttons on the P_TableauCarreCalib window
Function BoutonCarreCalib(ctrlName) : ButtonControl
	String ctrlName
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfRatio=W_DFREF[3]
	Wave/SDFR=dfRatio W_Carrecalib
	StrSwitch(ctrlName)
		case "C_Clear":
			W_CarreCalib=0
		break
		case "C_SetTo1":
			W_CarreCalib=1
		break
	EndSwitch
	Maj_CasesACocher("Couleur",0,"","")
End

//_____________________________________________________________________________________
// *** Display events on P_RM ***
//	Requires that
//	- W_EvtTempsCorr is monotonous
//	- W_EvtCorr is >0 et not NaN for an event
Function MarqueurEvt()
	variable i, j, TaillePolice, PosX, PosY, vPixParPix
	String NomEvt

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	NVAR/SDFR=dfRatio V_NumCoucheCourante, V_PosCarreXPix, V_PosCarreYPix, V_PixParPix
	Wave/SDFR=dfRatio W_Temps_Min, W_EvtTempsCorr2, W_EvtCorr, W_EvtTempsCorr
	Wave/T/SDFR=dfRatio W_EvtStrCorr

	//	terminate if checkbox checked
	Controlinfo/W=P_Controle C_AfficheEvt
	if (V_Value==0)
		return(0)
	endif
	
	SetDrawLayer/W=F_RM/K UserFront	//	clear draw layer
	for (i=0; i<min(DimSize(W_EvtTempsCorr,0), DimSize(W_EvtTempsCorr2,0)); i+=1)
		if ( (W_Temps_Min[V_NumCoucheCourante]>=W_EvtTempsCorr2[i]) &&  (W_Temps_Min[V_NumCoucheCourante]<=W_EvtTempsCorr2[i+1]) && (W_EvtCorr[i]>0) && (W_EvtCorr[i+1]>0) )
			NomEvt=W_EvtStrCorr[i]
			ControlInfo/W=P_Controle C_SetTaillePolice
			TaillePolice=V_Value
			ControlInfo/W=P_Controle C_PosXEvt
			PosX=V_Value
			ControlInfo/W=P_Controle C_PosYEvt
			PosY=V_Value+j*TaillePolice/V_PixParPix
			SetDrawEnv/W=F_RM xcoord= top,ycoord= left,linefgc= (65535,65535,65535),fillpat=0
			SetDrawEnv/W=F_RM textrgb= (65535,65535,65535), fname="Helvetica",fsize= TaillePolice, save
			SetDrawEnv/W=F_RM textxjust= 0,textyjust= 2, fsize=TaillePolice
			DrawText/W=F_RM PosX, PosY, NomEvt
			j+=1
		endif
	EndFor
End

// *** Removing data points ***
Function BoutonSupprime(ctrlName) : ButtonControl
	String ctrlName
	String NomTrace
	variable intensite, numsequence, vDimTemp

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	NVAR/SDFR=dfROI V_NbROI
	NVAR/SDFR=dfRatio V_NumCoucheCourante, Dim, V_NbChiffres
	Wave/SDFR=dfRatio W_NumTrame
	
	vDimTemp=DimSize(W_NumTrame, 0)
	Supprime(V_NumCoucheCourante,0)
	MaJGlissieres()
	MaJImages(0)
End

//	Suppress or add a data point in the experiment
//	Clears all cached files corresponding to this time point and redimension various waves
Function Supprime(i,j)	//	i: # of datapoint to modify; j=0: suppress; j=1: add
	variable i, j
	variable n

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	SetDataFolder dfRatio
	NVAR kMax, V_NbChiffres, Dim
	NVAR/SDFR=dfROI V_NbROI
	Wave W_Temps, W_Temps_min, W_NumTrame, W_CarreCalib, W_Exposition, W_Decalage, W_Deplacement
	Wave/SDFR=dfROI W_Fond

	switch(j)
		case 0:
			BoutonSupprimeTemp("C_SupprimeFichierTemp")	// d'abord on supprime le fichier temporaire
			DeletePoints i, 1, W_Temps, W_NumTrame, W_CarreCalib, W_Exposition, W_Fond, W_Decalage, W_Deplacement
			for (n=1; (n<=V_NbROI); n+=1)		//	on supprime les points sur les graphes de mesure
				Wave traceA=dfROI:$AjouteNumero("TraceA", n, V_NbChiffres)
				Wave traceB=dfROI:$AjouteNumero("TraceB", n, V_NbChiffres)
				Wave traceR=dfROI:$AjouteNumero("TraceR", n, V_NbChiffres)
				DeletePoints i, 1, traceA, traceB, traceR
				Redimension/N=(kMax) traceA, traceB, traceR
				// le point créé en bout de wave est initialisé à NaN; sinon -> pbs d'autoscaling
				traceA[kMax-1]=NaN; traceB[kMax-1]=NaN; traceR[kMax-1]=NaN	
			endfor
		break
//	Not tested the following
//		case 1:
//			InsertPoints i, 1, W_Temps, W_NumTrame, W_CarreCalib, W_Exposition, W_Fond, W_Decalage, W_Deplacement
//			for (n=1; (n<=V_NbROI); n+=1)		//	on ajoute les points sur les graphes de mesure
//				Wave traceA=dfROI:$AjouteNumero("TraceA", n, V_NbChiffres)
//				Wave traceB=dfROI:$AjouteNumero("TraceB", n, V_NbChiffres)
//				Wave traceR=dfROI:$AjouteNumero("TraceR", n, V_NbChiffres)
//				InsertPoints i, 1, traceA, traceB, traceR
//				Redimension/N=(kMax) traceA, traceB, traceR
//				traceA[kMax-1]=NaN; traceB[kMax-1]=NaN; traceR[kMax-1]=NaN	// le point créé en bout de wave est initialisé à NaN; sinon -> pbs d'autoscaling
//			endfor		
//		break
	EndSwitch
	// ici, il faut redimensionner les waves en question à kMax pour éviter un bug de taille incohérente entre les différentes waves
	Redimension/N=(kMax) W_Temps, W_NumTrame, W_CarreCalib
	Redimension/N=(kMax,-1) W_Exposition, W_Fond, W_Decalage, W_Deplacement
	W_Temps[(dim), (kmax-1)]=nan
	Dim=FDim()
	MaJGlissieres()
end

//______________________________________________________________________________________
//___________________________   moyennes d'images dans le temps   ___________________________________
//	fait la moyenne des fichiers P_ACorr et P_BCorr, sur un nb de points autour du point courant; 
//	sauve cette moyenne dans le path calcul, sous forme de fichier P_Tmp
//	une fois fini le calcul des moyennes, remplace P_ACorr et P_BCorr par ces moyennes
//	Activé par C_BoutonMoyenne1 et C_BoutonMoyenneAll

Function BoutonMoyenneTemporelle(ctrlName) : ButtonControl
	String ctrlName

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	NVAR/SDFR=dfRatio V_NbMoyTemporelle, V_NumCoucheCourante, Dim
	Wave/SDFR=dfRatio P_ACorr, P_BCorr

	Variable vDebut, vFin, i, j, k, coeff
	
	SetDataFolder dfRatio
	
	Duplicate/O P_ACorr, P_TmpA, P_TmpB
	StrSwitch(ctrlName)
		case "C_BoutonMoyenne1":
			vDebut=V_NumCoucheCourante-V_NbMoyTemporelle
			if (vDebut<0)
				DoAlert/T="Can't do it" 0, "Current position can't be averaged with enough preceding points"
				Return(0)
			endif
			vFin=V_NumCoucheCourante+V_NbMoyTemporelle
			if (vFin>Dim-1)
				DoAlert/T="Can't do it" 0, "Current position can't be averaged with enough following points"
				Return(0)
			endif
			vDebut=V_NumCoucheCourante
			vFin=V_NumCoucheCourante
		break
		case "C_BoutonMoyenneAll":
			vDebut=V_NbMoyTemporelle; vFin=Dim-1-V_NbMoyTemporelle
		break
	EndSwitch
	//	Prépare la barre de progression
	NewPanel /N=ProgressPanel /W=(285,111,739,193) as "Averaging corrected images"
	ValDisplay valdisp0,pos={18,32},size={342,18},limits={0,100,0},barmisc={0,0},highColor=(0,90*256,160*256)
	ValDisplay valdisp0,value= _NUM:0
	Button bStop,pos={375,32},size={50,20},title="Stop"
	DoUpdate /W=ProgressPanel /E=1	// mark this as our progress window

	//	Lit les P_?Corr et calcule les moyennes
	for (i=vDebut; (i<=vFin); i+=1)
		Fastop P_TmpA=0
		Fastop P_TmpB=0
		for (j=-V_NbMoyTemporelle; (j<=V_NbMoyTemporelle); j+=1)
			GereImages(i+j, 1, 2)		//	Lit P_?Corr
			Fastop P_TmpA=P_TmpA+P_ACorr
			Fastop P_TmpB=P_TmpB+P_BCorr
		endfor
		//	Calcule la moyenne
		coeff=1/(2*V_NbMoyTemporelle+1)
		Fastop P_TmpA=(coeff)*P_TmpA
		Fastop P_TmpB=(coeff)*P_TmpB
		//	sauve les fichiers P_TmpA et P_TmpB qui contiennent la moyenne
		GereImages(i, 2, 5)

		//	mise à jour barre de progression
		k=trunc(i/dim*100)
		ValDisplay valdisp0,value= _NUM:k,win=ProgressPanel
		DoUpdate /W=ProgressPanel
	endfor
	//	Supprime la barre de progression
	killwindow ProgressPanel

	//	On refait la même boucle pour transférer les moyennes stockées dans P_Tmp? vers P_?Corr
	//	Prépare la barre de progression
	NewPanel /N=ProgressPanel /W=(285,111,739,193) as "Restoring corrected images from averages"
	ValDisplay valdisp0,pos={18,32},size={342,18},limits={0,100,0},barmisc={0,0},highColor=(0,90*256,160*256)
	ValDisplay valdisp0,value= _NUM:0
	Button bStop,pos={375,32},size={50,20},title="Stop"
	DoUpdate /W=ProgressPanel /E=1	// mark this as our progress window

	for (i=vDebut; (i<=vFin); i+=1)
		GereImages(i, 1, 5)	//	lit les fichiers P_Tmp
		Fastop P_ACorr=P_TmpA
		Fastop P_BCorr=P_TmpB
		GereImages(i, 2, 2)	//	sauve les fichiers P_ACorr et P_BCorr
		GereImages(i, 3, 5)	//	efface les fichiers P_Tmp?
		//	mise à jour barre de progression
		k=trunc(i/dim*100)
		ValDisplay valdisp0,value= _NUM:k,win=ProgressPanel
		DoUpdate /W=ProgressPanel
	EndFor
	//	Supprime la barre de progression
	killwindow ProgressPanel
	
	//	Nettoyage final	
	KillWaves/Z P_TmpA, P_TmpB
	MaJ_CasesACocher("ToutSaufAB",0,"","")	

	SetDataFolder root:
End


//___________________________________________________________________________
//*************	réglage des glissières		***************************************

// Actionné par la glissière contrôlant le n° du fichier affiché dans la pile RM
Function GlissiereAffichage(ctrlName,sliderValue,event) : SliderControl
	String ctrlName
	Variable sliderValue
	Variable event	// bit field: bit 0: value set, 1: mouse down, 2: mouse up, 3: mouse moved
	
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	NVAR/SDFR=dfRatio V_EEgg
	variable eegg
	
	if (event==9 && V_EEgg)
		if (abs(enoise(10))<1)
			eegg=1
		else
			eegg=0
		endif
	endif
	eegg=0
	MajImages(eegg)
end

// Appelée par les contrôles de numéro de fichier
Function SetNum(ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName
	Variable varNum
	String varStr
	String varName
	MajImages(0)
End

 //	pour lire toutes les images disponibles au-dessus de la dernière image lue; utile au démarrage, et si de nouvelles images sont générée en cours de manip
Function BoutonDebutFin(ctrlname) : ButtonControl
	string ctrlname

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	Wave/SDFR=dfRatio W_PointsDeVue
	NVAR/SDFR=dfRatio Dim, V_NumCoucheCourante

	variable i, j, k

	StrSwitch (ctrlname)
		case "C_Debut":
			//	trouver la position de V_NumCoucheCourante par-rapport aux points de vue
			//	i=-1: soit pas de point de vue, soit V_NumCoucheCourante en-dessous du premier point
			//	i=0: V_NumCoucheCourante dans  ]point 0, point 1]
			//	i=1: V_NumCoucheCourante dans  ]point 1, point 2]  etc...
			i=-1
			do
				i+=1
			while (V_NumCoucheCourante>W_PointsDeVue[i])
			i-=1
			if (i==-1)
				V_NumCoucheCourante=0
			else
				V_NumCoucheCourante=W_PointsDeVue[i]
			endif
			break
		case "C_Fin":
			//	trouver la position de V_NumCoucheCourante par-rapport aux points de vue
			//	j=-1: soit pas de point de vue, soit V_NumCoucheCourante en-dessous du premier point
			//	j=0: V_NumCoucheCourante dans  [point 0, point 1[
			//	j=1: V_NumCoucheCourante dans  [point 1, point 2[  etc...
			j=-1
			do
				j+=1
			while (V_NumCoucheCourante>=W_PointsDeVue[j])
			j-=1
			k=W_PointsDeVue[j+1]
			if (NumType(k)==2)	//	point de vue = NaN
				V_NumCoucheCourante=Dim-1
			else
				V_NumCoucheCourante=k
			endif
			break
	endswitch
	MajImages(0)
end

//	_______________________	Points de vue	_________________________________________________________________
//	Affiche une sélection de points de tracé

//	Petite fonction pour afficher le temps corrigé dans la bonne colonne.
Function MaJPdV()
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	Wave/SDFR=dfRatio W_PointsDeVue, W_Temps_min
	variable i, num

	for (i=0; (i<DimSize(W_PointsDeVue,0));i+=1)
		num=W_PointsDeVue[i][0]
		if (NumType(num)==0)
			W_PointsDeVue[i][1]=W_Temps_min[num]
		endif
	endfor
End

//	Function BoutonAjouteMarqueursTemps(ctrlName) : ButtonControl
Function BoutonPdV(ctrlName) : ButtonControl
	String ctrlName
	
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	Wave/SDFR=dfRatio W_PointsDeVue, W_PointsDeVueStr
	NVAR/SDFR=dfRatio V_NumCoucheCourante, V_NbChiffresFichier

	variable i, j
	string sNom, sNomExport, sPrefixeNom

	MaJPdV()
	StrSwitch(ctrlName)
		case "C_LitPdV":
			BoutonAnalyse("C_LitPdV")
		break
		case "C_ExportePdV":
			prompt sPrefixeNom, "Add a prefix to the image name: "
			DoPrompt "Export series of images to pict gallery", sPrefixeNom
			if (V_Flag)
				print "Cancelled by user"
				return(0)
			endif
			if (StrLen(sPrefixeNom)>0)
				sNom="RM_"+sPrefixeNom+"_"
			else
				sNom="RM_"
			endif
			for (i=0; (NumType(W_PointsDeVue[i])==0); i+=1)
				print "Exporting image", W_PointsDeVue[i], " to pict gallery"
				V_NumCoucheCourante=W_PointsDeVue[i]
				MajImages(0)
				sNomExport=AjouteNumero(sNom,V_NumCoucheCourante, V_NbChiffresFichier)
				SavePict/O/E=-8/P=_PictGallery_/N=F_RM as sNomExport
			endfor
		break
		case "C_AjouteMarqueurs":
			sNom=TraceNameList("F_TracesTriees", ";", 1)
			if (WhichListItem("W_PointsDeVue", sNom) == -1)
				AppendToGraph/R/L=right/W=F_TracesTriees W_PointsDeVue[][2] vs W_PointsDeVue[][1]
				ModifyGraph/W=F_TracesTriees lsize(W_PointsDeVue)=2,rgb(W_PointsDeVue)=(0,0,0),mode(W_PointsDeVue)=3,textMarker(W_PointsDeVue)={W_PointsDeVueStr,"default",0,0,1,0.00,3.00}
				AppendToGraph/R/L=right/W=F_TracesTriees W_PointsDeVue[][2] vs W_PointsDeVue[][1]
				ModifyGraph/W=F_TracesTriees lsize(W_PointsDeVue#1)=2,rgb(W_PointsDeVue#1)=(0,0,0),mode(W_PointsDeVue#1)=3,marker(W_PointsDeVue#1)=10
			else
				DoAlert 0, "Markers are already displayed"
			endif
		break
		case "C_EnleveMarqueurs":
			RemoveFromGraph/Z/W=F_TracesTriees W_PointsDeVue#1,W_PointsDeVue
		break
		case "C_HauteurMarqueurs":
			W_PointsDeVue[][2]=W_PointsDeVue[0][2]
		break
	EndSwitch
End

Function MajImages(eegg)
	variable eegg

	//	If changing the displayed image, the automatic search for new datapoints must be stopped
	CheckAjoutePointsAuto("",0)	//	inactivates background task of automatic searching new datapoints
	
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	DFREF dfSav=GetDataFolderDFR()
	SetDataFolder dfRatio
	NVAR V_NumCoucheCourante, V_NbPlans, V_NumPlan, V_NbVoies, V_Taille_X, V_Taille_Y
	Wave P_ACorr, P_BCorr, P_RM
	
	Variable vVoie, a, b, c, vX, vY, i
	String sNomWave, sNomImg

	Switch(QuelAffichage())
		case 0:	//	display P_RM
			if (!(GereImages(V_NumCoucheCourante, 1, 3)))	//	Reads the pseudocolor image
				FastOp P_RM=0	//	in case nothing is read, put black in the image
			else
				if (eegg)
					c=trunc(abs(enoise(8)))
					sNomImg="im"+num2str(c)
					Wave wImg=root:img:$sNomImg
					a=DimSize(wImg,0); b=DimSize(wImg,1)
					vX=abs(enoise(V_Taille_X-a))
					vY=abs(enoise(V_Taille_Y-b))
					for (i=0; (i<3); i+=1)
						//ImageTransform/P=(i+V_NumPlan)/INSX=(vX)/INSY=(vY)/INSI=wImg InsertImage P_RM
						P_RM[vX, vX+a-1][vY, vY+b-1][i][V_NumPlan]=wImg[p-vX][q-vY][i]
					endfor
				else
					Legendes()
				endif
			endif
		break
		default:		//	display B/W image for channels A, B, C...
			if (!(GereImages(V_NumCoucheCourante, 1, 2)))	//	GereImages -> reads all B/W images
				For (vVoie=1; (vVoie<=V_NbVoies); vVoie+=1)	//	in case nothing is read, put black in the image
					sNomWave="P_"+StringFromList(vVoie-1, ksABC)+"Corr"
					Wave wPile=$sNomWave
					FastOp wPile=0
				endfor
			endif
		break
	EndSwitch
	MarqueurEvt()
	
	//	Position the cursor on movement and registration graphs
	DoWindow P_Decalage
	if (V_Flag)
		cursor/A=1/h=0/S=0/W=P_Decalage #F_Decalage A W_Decalage V_NumCoucheCourante
	endif
	DoWindow P_Deplacement
	if (V_Flag)
		cursor/A=1/h=0/S=0/W=P_Deplacement# F_Deplacement A W_Deplacement V_NumCoucheCourante
		cursor/A=1/h=0/S=0/W=P_Deplacement# F_DeplacementZ A W_Deplacement V_NumCoucheCourante
	endif
	DoUpdate
	SetDataFolder dfSav
End 

//	Permet de changer l'affichage entre RM,  A, B etc...
// P_RM et P_?Corr sont toutes deux présentes dans la fenêtre F_RM. Si on sélectionne "RM", c'est cette image qui est mise au-dessus, et donc visible.
//	Si on sélectionne une image Corr, selon la lettre dans popStr, qui sera A, B, C... ou autre, on remplacera la wave actuelle par la P_?Corr correspondante.
Function PopMenuAffichage(ctrlName,popNum,popStr) : PopupMenuControl
	String ctrlName
	Variable popNum
	String popStr
	
	DFREF dfSav=GetDataFolderDFR()
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	SetDataFolder dfRatio
	
	String sImages, sNomWaveAncien, sNomWaveNouveau	//	indique le nom de la pile P_ABCDCorr actuellement dans l'affichage, et la nouvelle qui va remplacer
	
	SetDrawLayer/W=F_RM/K ProgFront	//	Efface les dessins précédents sur l'image RM et définit les préférences de dessin
	//	détermine le nom de la pile affichée sur le dessus; pourrait être simplifié en utilisant QuelAffichage()
	//	Could be simplified if using novel syntax for ReorderImages _front_ {}, but that doesn't seem to work according to the manual
	sImages=imagenamelist("F_RM",";")
	sNomWaveAncien=StringFromList(0, ListMatch(sImages, "P_*Corr"))//	trop con: ListMatch retourne un string qui comporte encore le listSepStr; => utiliser en plus StringFromList
	StrSwitch(popStr)
		case "RM":
			DoWindow/T F_RM,"Intensity-modulated ratio"
			ReorderImages/W=F_RM P_RM, {$sNomWaveAncien}
		break
		default:
			//	calcule le nom de la pile qu'il faut afficher sur le dessus
			sNomWaveNouveau="P_"+popStr+"Corr"
			ReplaceWave/W=F_RM image=$sNomWaveAncien, $sNomWaveNouveau
			ReorderImages/W=F_RM $sNomWaveNouveau, {P_RM}
			DoWindow/T F_RM,"intensity "+popStr
			SetIntensiteAB("",0,"","")	//	pour forcer la lecture des valeurs d'intensité A ou B
		break
	endSwitch
	//	on met à jour les deux popups, car si on en bouge un, il faut mettre à jour l'autre.
	PopupMenu C_Affichage win=P_Controle, mode=QuelAffichage()+1
	PopupMenu C_Affichage win=F_RM, mode=QuelAffichage()+1
	MajImages(0)	//	pour afficher la bonne image, correspondant au point courant
	SetDataFolder dfSav
End

//	teste quelle image est affichée dans P_RM et retourne: 0 pour P_RM; 1 pour P_ACorr; 2 pour P_BCorr; 3 pour P_C etc...
Function QuelAffichage()
	String sImages, sTopImage
	Variable vNum, vNbImages
	sImages=imagenamelist("F_RM",";")
	vNbImages=ItemsInList(sImages)
	if (vNbImages>2)	//	détecte le cas où plusieurs images seraient mises dans F_RM
		print "Erreur: plusieurs images dans la fenêtre F_RM"
		return(-1)
	endif
	sTopImage=StringFromList(1, sImages)
	StrSwitch(sTopImage)
		case "P_RM":
			return(0)
		break
		default:
			vNum=char2num(sTopImage[2])	// convertir la lettre dans "P_?Corr" en nombre
			return(vNum-64)
		break
	endswitch
end

//	To set the size of the time slider to match the actual number of data points
Function MaJGlissieres()
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	NVAR/SDFR=dfRatio Dim=root:Ratio:Dim
	Dim=FDim()
	Slider C_GlissierePositionCourante,limits={0,Dim-1,1},win=P_Controle
	SetVariable C_NumTrame,limits={0,Dim-1,1},win=P_Controle
	Slider C_GlissierePositionCourante,limits={0,Dim-1,1},win=F_RM
	SetVariable C_NumTrame,limits={0,Dim-1,1},win=F_RM
end

Function GlissierePlan(ctrlName,sliderValue,event) : SliderControl
	String ctrlName
	Variable sliderValue
	Variable event	// bit field: bit 0: value set, 1: mouse down, 2: mouse up, 3: mouse moved

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	Wave/SDFR=dfROI W_PlanROI, W_AfficheROI
	NVAR/SDFR=dfRatio V_NumPlan, V_Lissage
	NVAR/SDFR=dfROI V_NbROI, V_NumROI

	variable i, k, a, b
	string sImage

	//	trouver toutes les images présentes dans cette fenêtre, dont le nom est "P_???"
	String sNomsImages=ImageNameList("F_RM",";")
	for (i=0; (i<=ItemsInList(sNomsImages)); i+=1)
		sImage=StringFromList(i,sNomsImages)
		ModifyImage/W=F_RM $sImage plane= V_NumPlan
	endfor 
	
	ControlInfo/W=P_Controle C_Active_P_Masquage
	if (V_Value)
		sNomsImages=ImageNameList("P_Masquage",";")
		ModifyImage/W=P_Masquage#G0 P_Masque plane= V_NumPlan
	endif
		
	//	voir quelque-chose avec ImageAnalyzeParticles, imagesnake, imageseedfill

	//	Détermine s'il faut ou non afficher les ROI sur le plan correspondant
	ControlInfo/W=P_Controle C_AfficheROI	//	case "affiche les ROI" est cochée -> V_Value=1
	a=V_Value
	ControlInfo/W=P_Controle C_AfficheROI_TousPlans
	b=V_Value
	if (a && (!b))
		//	Teste pour chaque ROI s'il faut l'afficher dans cette image
		for (k=1; (k<=V_NbROI); k+=1)
			if ((W_PlanROI[k]>=V_NumPlan-V_Lissage/2) && (W_PlanROI[k]<=V_NumPlan+V_Lissage/2))	//	la ROi fait partie de la tranche
				W_AfficheROI[k]=1
			else
				W_AfficheROI[k]=0
			endif
		endfor
	endif
	MaJApparenceToutesROI(4)		//	Displays or hides the ROI depending on W_AfficheROI, group selection...
End

Function SetIntensiteAB(ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName
	Variable varNum
	String varStr
	String varName
	variable mini, maxi
	string sImages, sNomWave

	//	calcule le nom de la pile affichée sur le dessus, actuellement
	sImages=imagenamelist("F_RM",";")
	sNomWave=StringFromList(0, ListMatch(sImages, "P_*Corr"))//	trop con: ListMatch retourne un string qui comporte encore le listSepStr; => utiliser en plus StringFromList

	Switch(QuelAffichage())
		case 0:	//	RM; ne rien faire
		break
		case 1:	//	A
			ControlInfo/W=P_Controle C_IMinA; mini=V_Value
			ControlInfo/W=P_Controle C_IMaxA; maxi=V_Value
			ModifyImage/W=F_RM $sNomWave ctab= {mini,maxi,Grays,0}
		break
		case 2:	//	B
			ControlInfo/W=P_Controle C_IMinB; mini=V_Value
			ControlInfo/W=P_Controle C_IMaxB; maxi=V_Value
			ModifyImage/W=F_RM $sNomWave ctab= {mini,maxi,Grays,0}
		break
		default:	//	pour les éventuelles piles C, D etc...
			ModifyImage/W=F_RM $sNomWave ctab= {*,*,Grays,0}	//	échelle de gris automatique
		break
	EndSwitch
End

Function CheckIntensiteAuto(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Switch(checked)
		case 0:
			SetIntensiteAB("",0, "","")
		break
		
		case 1:
			ModifyImage/W=F_RM ''#0 ctab= {0,*,Grays,0}, ctabAutoscale=3
		break
	EndSwitch
End

//________________________________________________________________________________________________________________
//*************************** correction du déplacement et décalage  **********************************************
Function BoutonInitDepDec(ctrlName) : ButtonControl
	String ctrlName

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	SetDataFolder dfRatio
	Wave W_Deplacement, W_Decalage, W_Zone, W_ZoneRefDecalage, W_ZoneRefDeplacement
	NVAR V_Position0Deplacement, V_NumcoucheCourante, V_NbVoies, V_NbPlans, V_NumPlan, V_VraiePileZ
	SVAR S_VoieRefDeplacement, S_VoieRefDecalage

	variable numVoie, vNumZone, vQuelleVoie, vPlanMin=0, vPlanMax=V_NbPlans-1
	string sNomPile, sNomPileCorr, sNomPileRef=S_VoieRefDeplacement, sListe

	//	Bloque la mise à jour automatique
	CheckAjoutePointsAuto("",0)
	
	//	une version plus ancienne créait W_Zone avec seulement 4 lignes
	redimension/N=6 W_Zone
	
	StrSwitch(ctrlName)
		case "C_DessineRegDeplacement":	
			//	Use V_VraiePileZ to change the behavior: if V_VraiePileZ=1 -> a single correction wave; if V_VraiePileZ=0, as many corrections as "planes" (acquisition zones)
			prompt sNomPileRef, "What channel to use as reference: ", popup, "A;B;C"//	should be programmed using ksABC V_NbVoies
			DoPrompt/HELP="Choose the data channel to be considered as reference for movement correction." "Reference channel:", sNomPileRef
			if (V_Flag==1)
				return(0)
			endif
			S_VoieRefDeplacement=sNomPileRef

			GereImages(V_NumcoucheCourante, 1, 1)		//	lit les fichiers P_A, P_B, P_C etc... correspondant à la position courante
			W_Deplacement[V_NumcoucheCourante][]=0		//	marque nul le déplacement pour ce point
			PopMenuAffichage("rien", NaN, S_VoieRefDeplacement)	//	Affiche la voie de référence
			sNomPile="P_"+S_VoieRefDeplacement
			Wave pPile=$sNomPile
			sNomPileCorr=sNomPile+"Corr"
			Wave pPileCorr=$sNomPileCorr
			FastOp pPileCorr=pPile	//	La pile affichée est la même que la pile brute

			vNumZone=DimSize(W_ZoneRefDeplacement,1)
			redimension/N=(-1, vNumZone+1) W_ZoneRefDeplacement
			SelectionneZone("F_RM", "zone #"+Num2Str(vNumZone)+" for automatic movement correction", W_Zone)
			//	Get min and max position in the stack
			if (V_NbPlans>1)
				Prompt vPlanMin, "lower Z limit for this zone"
				Prompt vPlanMax, "higher Z limit for this zone"
				DoPrompt "Define the vertical limits of the zone", vPlanMin, vPlanMax
				if (V_Flag==1)
					return(0)
				endif
			else
				vPlanMin=0; vPlanMax=1
			endif
			W_Zone[2]=vPlanMin; W_Zone[5]=vPlanMax
			W_ZoneRefDeplacement[][vNumZone]=W_Zone[p]
			//	Extrait la pile de référence
			Wave pRef=ExtraitZone(W_Zone, pPileCorr)
			sNomPileRef="P_RefDep"+num2str(vNumZone)
			Killwaves/Z $sNomPileRef
			Rename pRef, $sNomPileRef
			V_Position0Deplacement=V_NumcoucheCourante
			CheckBox C_CheckBoxDeplacement disable=0, win=P_Controle
		break

		//	Définit les régions sur lesquelles le décalage sera calculé.
		case "C_DessineRegDecalage":
			//	Si intéterminé, on choisit quelle voie va servir de référence
			prompt sNomPileRef, "What channel to use as reference: ", popup, "A;B;"
			DoPrompt/HELP="Choose the data channel to be considered as reference for the correction of mis-registration." "Registration channel:", sNomPileRef
			if (V_Flag==1)
				return(0)
			endif
			S_VoieRefDecalage=sNomPileRef

			//	Affiche la voie de référence
			PopMenuAffichage("rien",NaN, S_VoieRefDecalage)
			//	W_ZoneRefDecalage est créé avec 0 colonne; la taille de la dimension 1 est donc égal à n° de la zone à dessiner ici
			vNumZone=DimSize(W_ZoneRefDecalage,1)
			redimension/N=(-1, vNumZone+1) W_ZoneRefDecalage
			SelectionneZone("F_RM", "zone #"+Num2Str(vNumZone)+" for registration", W_Zone)
			W_Zone[2]=V_NumPlan; W_Zone[5]=V_NumPlan
			W_ZoneRefDecalage[][vNumZone]=W_Zone[p]
			CheckBox C_CheckBoxDecalage disable=0, win=P_Controle
		break

		//	Efface les paramètres de déplacement ou décalage
		case "C_EffaceRegDeplacement":
			DoAlert 1, "Clear selected regions ?"
			switch(V_Flag)
				case 1:
					redimension/N=(-1, 0) W_ZoneRefDeplacement
					W_ZoneRefDeplacement=NaN
					V_Position0Deplacement=NaN
					//	Supprime les waves extraites
					sListe=WaveList("P_RefDep*", ";", "")
					for (vNumZone=0; (vNumZone<ItemsInList(sListe)); vNumZone+=1)
						sNomPile=StringFromList(vNumZone, sListe)
						Wave wPile=$sNomPile
						KillWaves wPile
					endfor					
				break
				default:
					print "Cancelled by user"
			EndSwitch		
		break
		case "C_EffaceRegDecalage":
			DoAlert 1, "Clear selected regions ?"
			if(V_Flag==1)
				redimension/N=(-1, 0) W_ZoneRefDecalage
				W_ZoneRefDecalage=NaN
			else
				print "'Erase regions' cancelled by user"
			EndIf
		break
	EndSwitch
end

//	Extrait les zones de déplacement à partir de la définition des zones de déplacement
Function DefinitZonesDeplacement()
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	SetDataFolder dfRatio
	Wave W_ZoneRefDeplacement
	NVAR V_Position0Deplacement, V_NumcoucheCourante, V_NbVoies, V_NbPlans, Dim
	SVAR S_VoieRefDeplacement
	variable vTest=0, i, vNumZone
	string sNomPileRef, sNomPile
	Make/N=(6)/FREE wZone
	//	Vérifie si les zones de référence et point de référence sont définis
	WaveStats/Q W_ZoneRefDeplacement		//	checking if there are NaN in zones
	if ( (V_numNans!=0) && (NumType(V_Position0Deplacement)!=0) && (V_Position0Deplacement<0) && (V_Position0Deplacement>=Dim))//	if zones and ref time-point are defined, extract the zone
		//	arrête l'exécution de la fonction appelante
		Abort	"Movement correction by zone must be initialized."
	endif

	for (i=0; (i<DimSize(W_ZoneRefDeplacement,1)); i+=1)
		sNomPileRef="P_RefDep"+num2str(i)
		Wave/Z pRef=$sNomPileRef
		if (!WaveExists(pRef))	//	une zone de référence est manquante
			vTest=1
		endif
	endfor

	if (vTest)	//	une zone est manquante
		//	On se met sur la position de référence
		V_NumcoucheCourante=V_Position0Deplacement
		//	lit les images brutes sur le cache
		GereImages(V_NumcoucheCourante, 1, 1)
		sNomPile="P_"+S_VoieRefDeplacement
		Wave pPile=$sNomPile
		//	Extrait les zones à partir de l'image brute
		for (vNumZone=0; (vNumZone<DimSize(W_ZoneRefDeplacement,1)); vNumZone+=1)
			wZone=W_ZoneRefDeplacement[p][vNumZone]
			//	Extrait la pile de référence
			Wave pRef=ExtraitZone(wZone, pPile)
			sNomPileRef="P_RefDep"+num2str(vNumZone)
			Killwaves/Z $sNomPileRef
			Rename pRef, $sNomPileRef
		endfor
		//	arrête l'exécution de la fonction appelante
		Abort "Reference zones are missing and will be extracted. Then, try again."
	endif
End

//	Réalise le déplacement d'un plan
//ThreadSafe Function DeplacePlan(NomPile, dx, dy, dz)
//	String NomPile
//	Variable dx, dy, dz
//	Wave Pile=$NomPile
//	if ((dx==0) && (dy==0) && (dz==0))	//	si le déplacement est nul, on termine
//		return(0)
//	endif
//	if ((NumType(dx)!=0) || (NumType(dy)!=0) || (NumType(dz)!=0))
//		print "erreur: on veut déplacer un plan d'une valeur non définie"
//		return(0)
//	endif
//	Variable taille=DimSize(Pile,2)
//	ImageTransform/IOFF={dx, dy,0} OffsetImage Pile	// opère la translation seulement dans le plan, en xy. 0 pour la valeur qui remplace les "vides" créés
//	Wave M_OffsetImage
//	FastOp Pile=M_OffsetImage
//	if ((taille>1) && (dz!=0))	//	on travaille en 3D ET il existe une valeur non-nulle de déplacement vertical
//		if (dz>0)
//			redimension/N=(-1,-1,taille-dz) Pile	//enlève dz plans à la fin de la pile
//			imagetransform flipPlanes Pile	// retourne la pile
//			redimension/N=(-1,-1,taille) Pile	// restaure la taille initiale en remplissant les plans vides par des zéros
//			imagetransform flipPlanes Pile	// remet la pile à l'endroit
//		else
//			imagetransform flipPlanes Pile	// retourne la pile
//			redimension/N=(-1,-1,taille+dz) Pile	//enlève dz plans à la fin de la pile
//			imagetransform flipPlanes Pile	// remet la pile à l'endroit
//			redimension/N=(-1,-1,taille) Pile	// restaure la taille initiale en remplissant les plans vides par des zéros
//		endif
//	endif
//end

//	Déplace les piles P_ACorr, P_BCorr et plus s'il y a
Function DeplaceLesPiles(dx, dy, dz, fond)
	Variable dx, dy, dz, fond
	
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	NVAR/SDFR=dfRatio V_NbVoies

	Variable vVoie
	String sNomWave

	For (vVoie=1; (vVoie<=V_NbVoies); vVoie+=1)	
		sNomWave="P_"+StringFromList(vVoie-1, ksABC)+"Corr"
		Wave wPile=$sNomWave
		tsDeplacePile(wPile, dx, dy, dz, fond)
	EndFor
End

Function tsDeplacePile(Pile, dx, dy, dz, fond)
	Wave Pile
	Variable dx, dy, dz, fond

	if ((dx==0) && (dy==0) && (dz==0))	//	si le déplacement est nul, on termine
		return(0)
	endif
	if ((NumType(dx)!=0) || (NumType(dy)!=0) || (NumType(dz)!=0))
//		print "erreur: on veut déplacer un plan d'une valeur non définie"
		return(0)
	endif
	Variable epaisseur=DimSize(Pile,2)
	ImageTransform/IOFF={dx, dy,fond} OffsetImage Pile	// opère la translation seulement dans le plan, en xy. 0 pour la valeur qui remplace les "vides" créés
	Wave M_OffsetImage
	FastOp Pile=M_OffsetImage
	if ((epaisseur>1) && (dz!=0))	//	on travaille en 3D ET il existe une valeur non-nulle de déplacement vertical
		if (dz>0)
			redimension/N=(-1,-1,epaisseur-dz) Pile	//enlève dz plans à la fin de la pile
			imagetransform flipPlanes Pile	// retourne la pile
			redimension/N=(-1,-1,epaisseur) Pile	// restaure la taille initiale en remplissant les plans vides par des zéros
			imagetransform flipPlanes Pile	// remet la pile à l'endroit
		else
			imagetransform flipPlanes Pile	// retourne la pile
			redimension/N=(-1,-1,epaisseur+dz) Pile	//enlève dz plans à la fin de la pile
			imagetransform flipPlanes Pile	// remet la pile à l'endroit
			redimension/N=(-1,-1,epaisseur) Pile	// restaure la taille initiale en remplissant les plans vides par des zéros
		endif
	endif
end

//	Réalise le recalage d'un plan sans toucher à l'autre
//	pour l'instant, uniquement sur wave 2D: en biphoton, inutile de faire un recalage.
//	dx et dy sont indiqués en sous-pixels; res est la résolution du sur-échantillonnage
ThreadSafe Function DeplacePlanSubPix(NomPile, dx, dy, res)
	String NomPile
	Variable dx, dy, res
	Wave Pile=$NomPile
	variable dz=0	//	on se réserve éventuellement la possibilité de faire ça en 3D
	if ((dx==0) && (dy==0) && (dz==0))	//	si le décalage est nul, on termine
		return(0)
	endif
	if ((NumType(dx)!=0) || (NumType(dy)!=0)  || (NumType(dz)!=0))
		print "erreur: on veut décaler un plan d'une valeur non définie"
		return(0)
	endif
	Variable dxx=round(dx*res), dyy=round(dy*res), dzz=round(dz*res)
	ImageInterpolate/F={(res), (res)} Bilinear Pile
	Wave M_InterpolatedImage
	ImageTransform/IOFF={dxx,dyy,0} OffsetImage M_InterpolatedImage
	Wave M_OffsetImage
	ImageInterpolate/F={(1/res), (1/res)} Bilinear M_OffsetImage
	FastOp Pile=M_InterpolatedImage
end

Function BoutonDeplacement(ctrlName): ButtonControl
	String ctrlName

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	Wave/SDFR=dfRatio W_Deplacement
	NVAR/SDFR=dfRatio i=V_NumcoucheCourante, V_NbPlans

	// le déplacement n'est pas défini, -> on l'initialie à 0
	if ((NumType(W_Deplacement[i][0]) != 0) && (NumType(W_Deplacement[i][1]) != 0) && (NumType(W_Deplacement[i][2]) != 0))
		W_Deplacement[i]=0
	endif
	Variable dx=0, dy=0, dz=0
	GereImages(i, 1, 1)		//	lit les fichiers P_A et P_B correspondant à la position courante
	strswitch (ctrlName)
		case "C_Haut":
			dy=-1
			break
		case "C_Bas":
			dy=1
			break
		case "C_Gauche":
			dx=-1
			break
		case "C_Droite":
			dx=1
			break
		case "C_Haut1":
			dy=-5
			break
		case "C_Bas1":
			dy=5
			break
		case "C_Gauche1":
			dx=-5
			break
		case "C_Droite1":
			dx=5
			break
		case "C_HautZ":
			dz=1
			break
		case "C_BasZ":
			dz=-1
			break
	endswitch
	W_Deplacement[i][0]+=dx
	W_Deplacement[i][1]+=dy
	W_Deplacement[i][2]+=dz
	MaJ_CasesACocher("Tous", 0, "", "")
end

Function BoutonEffaceDepDec(ctrlname) : ButtonControl
	string ctrlname

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	Wave/SDFR=dfRatio W_Deplacement, W_Decalage
	NVAR/SDFR=dfRatio i=V_NumCoucheCourante, Dim
	Variable j, vDimTemp=DimSize(W_Deplacement, 0)

	StrSwitch(ctrlName)
		case "C_EffaceDepCettePosition":
			GereImages(i,3, 2)
			GereImages(i,3, 3)
			W_Deplacement[i][]=NaN
		break
		case "C_EffaceDecCettePosition":
			GereImages(i,3, 2)
			GereImages(i,3, 3)
			W_Decalage[i][]=NaN
		break
		case "C_EffaceDepSuite":
			for (j=i+1; (j<vDimTemp); j+=1)
				GereImages(j,3, 2)
				GereImages(j,3, 3)
				W_Deplacement[j][]=NaN
			endfor
		break
		case "C_EffaceDecSuite":
			for (j=i+1; (j<vDimTemp); j+=1)
				GereImages(j,3, 2)
				GereImages(j,3, 3)
				W_Decalage[j][]=NaN
			endfor
		break
		case "C_EffaceDecAvant":
			for (j=0; (j<i); j+=1)
				GereImages(i,3, 2)
				GereImages(i,3, 3)
				W_Decalage[j][]=NaN
			endfor
		break
		case "C_EffaceDepAvant":
			for (j=0; (j<i); j+=1)
				GereImages(j,3, 2)
				GereImages(j,3, 3)
				W_Deplacement[j][]=NaN
			endfor
		break
	EndSwitch
end

Function BoutonDecalage(ctrlName): ButtonControl
	String ctrlName

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	NVAR/SDFR=dfRatio i=V_NumcoucheCourante, V_Res
	Wave/SDFR=dfRatio W_Decalage

	if ((NumType(W_Decalage[i][0]) != 0) && (NumType(W_Decalage[i][1]) != 0))	// le décalage n'est pas défini
		W_Decalage[i]=0
	endif
	Variable dx=0, dy=0
	GereImages(i, 1, 1)		//	lit les fichiers P_A et P_B correspondant à la position courante
	strswitch (ctrlName)
		case "C_Haut2":
			dy=-1/V_Res
			break
		case "C_Bas2":
			dy=1/V_Res
			break
		case "C_Gauche2":
			dx=-1/V_Res
			break
		case "C_Droite2":
			dx=1/V_Res
			break
		case "C_Haut3":
			dy=-10/V_Res
			break
		case "C_Gauche3":
			dx=-10/V_Res
			break
		case "C_Bas3":
			dy=10/V_Res
			break
		case "C_Droite3":
			dx=10/V_Res
			break
	endswitch
	W_Decalage[i][0]+=dx
	W_Decalage[i][1]+=dy
	MaJ_CasesACocher("Tous", 0, "", "")
	BoutonAnalyse("C_TraiteCettePosition")
end

Function BoutonDecalagePartout(ctrlName) : ButtonControl
	String ctrlName

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	Wave/SDFR=dfRatio W_Decalage
	NVAR/SDFR=dfRatio Dim, V_NumCoucheCourante
	variable i

	DoAlert 1, "Apply the registration values to all data points, and clear all calculated images ?"
	if (V_Flag==1)
		W_Decalage[][]=W_Decalage[V_NumCoucheCourante][q]
		for (i=0; (i<Dim); i+=1)	//	supprime tous les fichiers temp
			GereImages(i,3,2)
			GereImages(i,3,3)
		endfor
	endif
End

//	Fin déplacement décalage
//*******************

Function CheckAffichePosition(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	Wave/SDFR=dfRatio	W_PosX, W_PosY

	if (checked)
		String stemp=TraceNameList("F_TracesTriees", ";", 1)
		if (WhichListItem("W_PosY", stemp, ";")==-1)	// vérifie que ce ne soit pas déjà affiché
			AppendToGraph/R/W=F_TracesTriees W_PosY vs W_PosX
			ModifyGraph/W=F_TracesTriees mode(W_PosY)=1, rgb(W_PosY)=(65000,35000,10000), lsize(W_PosY)=1.5
		endif
		ModifyGraph/W=F_TracesTriees hideTrace(W_PosY)=0
	else
		ModifyGraph/Z/W=F_TracesTriees hideTrace(W_PosY)=1
//		RemoveFromGraph/Z/W=F_TracesTriees W_PosY
	endif
End


Function PopMenuTypeAxeX(ctrlName,popNum,popStr) : PopupMenuControl
	String ctrlName
	Variable popNum
	String popStr

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	SetDataFolder dfRatio
	Wave/SDFR=dfRatio W_Temps_min, W_Temps
	NVAR/SDFR=dfRatio Dim

	Switch(popNum)
		case 1:	//	temps en minutes
			SetFormula W_Temps_min, "(W_Temps-V_T0)/60"
			SetFormula W_EvtTempsCorr2, "W_EvtTempsCorr-V_T0/60"
			W_Temps[(dim), ]=nan
			Label/W=F_Traces bottom "Time (min)"
			Label/W=F_TracesTriees bottom "Time (min)"
			ValDisplay/Z C_TempsCourantNet title="Temps corrigé (min)", win=P_Controle
//			SetAxis/W=F_TracesTriees/A bottom		
//			SetAxis/W=F_Traces/A bottom		
		break
		case 2:	//	temps en secondes
			SetFormula W_Temps_min, "W_Temps-V_T0"
			SetFormula W_EvtTempsCorr2, "W_EvtTempsCorr*60-V_T0"
			W_Temps[(dim),]=nan
			Label/W=F_Traces bottom "Time (s)"
			Label/W=F_TracesTriees bottom "Time (s)"	
			ValDisplay/Z C_TempsCourantNet title="Temps corrigé (sec)", win=P_Controle
//			SetAxis/W=F_TracesTriees/A bottom		
//			SetAxis/W=F_Traces/A bottom		
		break
		case 3:
			setformula W_Temps_min, ""
			W_Temps_min[0, (dim-1)]=p
			W_Temps_min[(dim),]=nan
			Label/W=F_Traces bottom "Image number"
			Label/W=F_TracesTriees bottom "Image number"			
			ValDisplay/Z C_TempsCourantNet title="n° image", win=P_Controle
//			SetAxis/W=F_TracesTriees/A bottom		
//			SetAxis/W=F_Traces/A bottom		
		break
	endswitch
	SetDataFolder root:
End

Function BoutonDep(ctrlName): ButtonControl
	String ctrlName

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	NVAR/SDFR=dfRatio dx=V_X, dy=V_Y, dz=V_Z

	strswitch (ctrlName)
		case "C_Haut":
			dy+=1
			break
		case "C_Bas":
			dy+=-1
			break
		case "C_Gauche":
			dx+=-1
			break
		case "C_Droite":
			dx+=1
			break
		case "C_Haut1":
			dy+=5
			break
		case "C_Bas1":
			dy+=-5
			break
		case "C_Gauche1":
			dx+=-5
			break
		case "C_Droite1":
			dx+=5
			break
		case "C_HautZ":
			dz+=1
			break
		case "C_BasZ":
			dz-=1
			break
	endswitch
end

Function SliderPlanPile(ctrlName,sliderValue,event) : SliderControl
	String ctrlName
	Variable sliderValue
	Variable event	// bit field: bit 0: value set, 1: mouse down, 2: mouse up, 3: mouse moved
	if(event %& 0x1)	// bit 0, value set
		ModifyImage P_Pile plane= sliderValue
	endif
End

Function SliderContrastePile(ctrlName,sliderValue,event) : SliderControl
	String ctrlName
	Variable sliderValue
	Variable event	// bit field: bit 0: value set, 1: mouse down, 2: mouse up, 3: mouse moved

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]

	if(event %& 0x1)	// bit 0, value set
		NVAR/SDFR=dfRatio V_IMinP, V_IMaxP
		StrSwitch(ctrlName)
			case "C_IMin":
				V_IMinP= sliderValue
			break
			case "C_IMax":
				V_IMaxP= sliderValue
			break
		EndSwitch
		ModifyImage P_Pile ctab= {V_IMinP, V_IMaxP,Grays,0}
	endif
end

Function BoutonMesureDistance(ctrlName) : ButtonControl
	String ctrlName

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	NVAR/SDFR=dfRatio V_Resolution, V_Distance
	variable xA, yA, xB, yB, d

	xA=pcsr(A)
	yA=qcsr(A)
	xB=pcsr(B)
	yB=qcsr(B)
	d=sqrt((xB-xA)^2+(yB-yA)^2)*V_Resolution
	print "Distance : ", d, " µm"
	V_Distance=d
End

//	Loads the external image stack
Function CheckActivePile(ctrlName, checked) : CheckBoxControl
	String ctrlName
	variable checked

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	Wave/Z/SDFR=dfRatio W_Deplacement, W_Exposition, P_Pile, D_UnifA, D_UnifB
	NVAR/SDFR=dfRatio Dim, V_OffsetCamera, V_Miroir, V_MoyUnifA, V_MoyUnifB
	String nomFichier, listeFichiers, sVoie="B", sNomTemp
	variable i, j, dx, dy, vExposition

	SetDataFolder dfRatio

	if (checked && !WaveExists(P_Pile))	//	if checked and image stack not yet loaded
		//	Suppose that exposure duration is that of the last image in channel B
		vExposition=W_Exposition[Dim-1][1]
		Prompt vExposition, "Exposure duration"

		//	Load the external image stack
		//	Look in the data folder for a tif file containing "pile" in its name
		listeFichiers=IndexedFile(data, -1, ".tif")
		nomFichier=ListMatch(listeFichiers, "*pile*")		//	Only keeps "pile" ou "Pile"
		if (ItemsInList(nomFichier)==0)
			DoAlert/T="CheckActivePile" 0, "No image stack was found"; return(0)
		endif
		Prompt nomFichier, "Select file:", popup, nomFichier
		Prompt sVoie, "Channel used to acquire external stack:", popup, "A;B"
		if (ItemsInList(nomFichier)>1)		//	more than one file => the user has to select which file to load
			DoPrompt "External stack", nomFichier, sVoie, vExposition
		else
			DoPrompt "External stack", sVoie, vExposition
		endif
		if (V_flag==1)
			print "cancelled by user"
			return(0)
		endif

		nomFichier=RemoveEnding(nomFichier, ";")
		print "Loading stack"
		ImageLoad/P=data/T=tiff/O/S=0/C=-1/N=P_Pile nomFichier
		Wave P_Pile	//	that's the image stack

		sNomTemp="V_Gain"+sVoie
		NVAR vGain=$sNomTemp
		sNomTemp="D_Unif"+sVoie
		Wave dUnif=$sNomTemp
		
		//	Perform instrument corrections on the raw stack
		Redimension/S P_Pile
		ProcessRawDataCalculations(P_Pile, V_OffsetCamera, V_Miroir, vExposition, vGain, V_MoyUnifA, dUnif)
		
		//	Déplace la pile par la valeur de déplacement en fin de manip
		dx=W_Deplacement[Dim-1][0]; dy=W_Deplacement[Dim-1][1]
		tsDeplacePile(P_Pile, dx, dy, 0, 0)
		SignalSonore(5)		//	put a sound when loaded
	endif
	CheckActivePanneau("C_Active_F_Pile", checked)	//	déclenche l'affichage ou le masquage de la fenêtre
	MaJApparenceToutesROI(1)		//	To plot the ROIs on P_Pile
	SetDataFolder Root:
End


Function CheckProjection(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked

	if (checked)
		StrSwitch(ctrlName)
			case "C_PMI":
				CheckBox C_PMoy win=P_Controle,value= 0
			break
			case "C_PMoy":
				CheckBox C_PMI win=P_Controle,value= 0
			break
		EndSwitch
	endif
	MaJ_CasesACocher(ctrlName, 0, "", "")
End


//________________________________________________________________________
//*******************		masquage de zones dans une pile		*************

Function BoutonMasquage(ctrlName) : ButtonControl
	String ctrlName
	
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	NVAR/SDFR=dfRatio V_NumPlan, V_NbPlans
	Wave/SDFR=dfRatio P_Masque
	variable i, vLisse=80, vMin, vMax
	
	StrSwitch(ctrlName)
		case "C_DessineRegMasque":
			DoWindow/F F_RM
			ShowTools/A rect
			SetDrawLayer/K ProgFront	// efface la couche avant d'y tracer la roi
			SetDrawEnv linefgc= (65000,40000,65000),fillpat= 0,save
		break
		case "C_ValideMasque":
			DoWindow/F F_RM
			GraphNormal
			HideTools/A
			SetDrawLayer ProgFront			//	on sort du mode graphique, on génère le masque dessiné sur le plan courant et on ajuste les variables
			ImageGenerateROIMask/E=1/I=0 ''#0	// crée le masque de recalage sur l'image affichée dans F_RM
			Wave M_ROIMask
			Redimension/S M_ROIMask
			ControlInfo/W=P_Masquage C_PlanMin
			vMin=V_Value
			ControlInfo/W=P_Masquage C_PlanMax
			vMax=V_Value
			for (i=vMin; (i<=vMax);i+=1)
				ImageTransform/P=(i)/D=M_ROIMask setplane P_Masque
			endfor
			ImageTransform/P=(V_NumPlan) getplane P_Masque
			Wave M_ImagePlane
			M_ROIMask=M_ImagePlane * M_ROIMask
			ImageTransform/P=(V_NumPlan)/D=M_ROIMask setplane P_Masque
			KillWaves M_ROIMask
			SetDrawLayer/K ProgFront
		break
		case "C_EffacePlan":
			ImageTransform/P=(V_NumPlan) getplane P_Masque
			Wave M_ImagePlane
			M_ImagePlane=1
			ImageTransform/P=(V_NumPlan)/D=M_ImagePlane setplane P_Masque			
		break
		case "C_Lisse":
			for (i=0;(i<V_NbPlans);i+=1)
				ImageTransform/P=(i) getplane P_Masque
				Wave M_ImagePlane
				Smooth vLisse, M_ImagePlane
				Smooth/DIM=1 vLisse, M_ImagePlane
				ImageTransform/P=(i)/D=M_ImagePlane setplane P_Masque			
			endfor
		break
	EndSwitch
End

//________________________________________________________________________
//*******************		Zones		*************
//	Permet de dessiner de manière interactive une zone dans une image
//	Reçoit le nom d'une fenêtre affichant l'image; un commentaire à afficher dans le dialogue
//	Reçoit la référence d'une wave qui doit contenir le résultat.
//	La zone retournée est 3D: x0, y0, z0, x1, y1, z1. En attendant mieux, z0=z1=NaN
//	Ouvre une fenêtre modale; attend qu'on y dessine une sélection.
//	Panneau de contrôle positionné à côté.
//	Vérifie à la fin que la zone ne sorte pas du cadre de l'image.
Function SelectionneZone(sNomFenetre, sCommentaire, wZone)
	String sNomFenetre, sCommentaire
	Wave wZone
	
	String sImage
	DFREF dfSav=GetDataFolderDFR
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]

	DoWindow/F $sNomFenetre
	HideTools
	if (V_Flag == 0)
		DoAlert 0, "SelectionneZone: found no window named "+sNomFenetre
		return 0
	endif
	NewPanel /K=2 /W=(20,20,220,90) as "Define a region"
	DoWindow/C F_SelectionneZone
	AutoPositionWindow/M=0 /R=$sNomFenetre
	DrawText 10, 20, sCommentaire
	Button C_BoutonValide,pos={15,30},size={170,20},proc=BoutonDefinitZone,title="Validate selected region"
//	Button C_BoutonEsc,pos={165,30},size={60,20},proc=BoutonDefinitZone,title="Cancel"
	PauseForUser F_SelectionneZone, $sNomFenetre
	GetMarquee/K left, top

	//	Si nécessaire, modifie les bords pour qu'ils ne dépassent pas du cadre de l'image
	sImage=StringFromList(0, ImageNameList(sNomFenetre,";"))
	Wave/SDFR=dfRatio wImage=$sImage	//	récupère une référence à l'image pour obtenir sa taille
	V_left=trunc(max(0, V_Left))
	V_top=trunc(max(0, V_top))
	V_right=trunc(min(DimSize(wImage,0)-1, V_right))
	V_bottom=trunc(min(DimSize(wImage,1)-1, V_bottom))
	wZone=nan	//	wZone peut être une zone 3D; on commence par mettre toutes les valeurs à NaN, puis on spécifie les valeurs donnés par l'utilisateur
	wZone[0]=V_left; wZone[1]=V_top; wZone[2]=NaN; wZone[3]=V_right; wZone[4]=V_bottom; wZone[5]=NaN
	SetDrawLayer/W=$sNomFenetre ProgFront
	SetDrawEnv/W=$sNomFenetre xcoord=top, ycoord=left, linefgc=(65500,0,0),fillpat=0
	DrawRect/W=$sNomFenetre V_left, V_top, V_right, V_bottom
	SetDrawEnv/W=$sNomFenetre xcoord=top, ycoord=left,textrgb=(65535,0,0),fsize=20,textyjust=2
	DrawText V_left+10,V_top+10,sCommentaire
	SetDataFolder dfSav
	Return 1
End

//	La fonction ne fait que fermer la fenêtre modale.
Function BoutonDefinitZone(ctrlName) : ButtonControl
	String ctrlName
	DoWindow/K F_SelectionneZone
End

//	Extrait une zone dans wImage définie par les coordonnées indiquées dans wZone, réduit de manière à rester dans le cadre de l'image
//	wZone indique en 0 les points inclus, en 1 les points exclus
Function/WAVE ExtraitZone(wZone, wImage)
	Wave wZone, wImage
	String sNomWaveOut=NameOfWave(wImage)+"_Zone"
	Variable x0, y0, z0, x1, y1, z1
	Variable a, b, c, d, e, f
	x0=wZone[0]; y0=wZone[1]; z0=wZone[2]
	x1=wZone[3]; y1=wZone[4]; z1=wZone[5]
	Make/O/N=(x1-x0,y1-y0,z1-z0) $sNomWaveOut
	Wave wResultat=$sNomWaveOut
	a=max(0, x0)-x0
	b=min(x1, DimSize(wImage,0))-x0-1
	c=max(0, y0)-y0
	d=min(y1, DimSize(wImage,1))-y0-1
	e=max(0, z0)-z0
	f=min(z1, DimSize(wImage,2))-z0-1
	wResultat=NaN
// *** \\\ dirty programming here ***
	//	should precisely test whether the extracted part is within wImage
	if ( (b>0) && (d>0) )		//	to avoid a potential error
		wResultat[a,b][c,d][e,f]=wImage[x0+p][y0+q][z0+r]
	endif
	return wResultat
End

//________________________________________________________________________
//*******************		Utilitaires		*************
Function Son(wData)
	Wave wData
	Wave W_Son, W_Frq
	W_Son=NaN
	variable i, frq
	variable vIndex			//	index dans W_Frq
	variable vEch=0.0002	//	fréquence d'échantillonnage du son
	variable b=0.2			//	durée des périodes en seconde
	variable coeff=2
	SetScale/P x 0,vEch,"s", W_Son,W_Frq
	Redimension/n=(b*DimSize(wData,0)/vEch) W_Son, W_Frq
//	trunc(DimSize(W_Son,0)/DimSize(W_Data,0))
	for (i=0; (i<DimSize(wData,0)-1);i+=1)
		vIndex=i*b/vEch
		frq=((wData[i]-1)*coeff+1)*440
		W_Frq[vIndex, ]=frq
	endfor
	W_Son[]=sin(W_Frq[p]*2*pi*x)*32768
	smooth 1, W_Son
	PlaySound W_Son
End

function Test_Utilisation()
	string test
	variable stop=0
	variable temp
	Variable vDateSec0=date2secs(2024,05,14)	//	date for alerting
	Variable vDateSec1=vDateSec0+date2secs(1904,2,1)	//	after 1 more month, the version expires

	if (IgorVersion() < 7)
		Abort "Igor Pro version 7 or more recent is required"
	endif

	test=IgorInfo(5)

	if (datetime>vDateSec1)
		DoAlert 0, "This version is probably obsolete and should be replaced by the current one"
		DoAlert 0, "Yes, really, get an updated version!"
	endif
	if (datetime>vDateSec0)
		DoAlert 0, "An update should be installed within less than a month; if this message shows up, stop procrastinating, do it NOW !"
	endif
end

//	Recognizes the user name and returns 1 for me
//	To disable some features that are too dangerous for everyday's work
Function Autorisation()
	if (WhichListitem("pierre", SpecialDirPath("Documents", 0, 0, 0), ":")>=0)
		return(1)
	else
		return(0)
	endif
End

Function MesurerTemps()
	Variable timerRefNum
	Variable milliSeconds
	timerRefNum=StartMSTimer
	
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	Wave/SDFR=dfRatio P_ACorr, P_BCorr, P_Ratio
	NVAR/SDFR=dfRatio V_NumCoucheCourante, V_R0

	FastOp P_Ratio=(1/V_R0)*P_ACorr/P_BCorr
//	MatrixOp/O P_Ratio=(P_ACorr/P_BCorr)/V_R0
//	Redimension/S P_Ratio
	milliSeconds = stopMSTimer(timerRefNum)/1000
	Print milliSeconds, "milliseconds"
end

Function SignalSonore(i)
	variable i
	variable j
	switch(i)
		case 1:
			beep
		break
		case 5:
			PlaySound/A=2 root:z_Initialisation:f_Global:DingDong
		break
	EndSwitch
end

//	Returns the max time-point number of an experiment
Function FDim()
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	Wave/SDFR=dfRatio W_NumTrame
	variable j
	for (j=0; ( (j<DimSize(W_NumTrame,0)) && (NumType(W_NumTrame[j]) == 0) ); j+=1)
	endfor
	return(j)
end

//	Case à cocher pour mettre en route ou arrêter l'ajout automatique de nouveaux points de données en tâche de fond
//	Appelé par d'autres fonctions pour désactiver la tâche de fond
Function CheckAjoutePointsAuto(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	if (checked)
		CtrlNamedBackground AjoutePointsAuto, period=(60*5), proc=ProcAjoutePointsAuto
		CtrlNamedBackground AjoutePointsAuto, start
	else
		CtrlNamedBackground AjoutePointsAuto, stop
	endif
	CheckBox/Z C_AjoutePointsAuto win=P_Controle, value=checked	//	met à jour la case à cocher sur le panneau de contrôle
End

//	Définition de la fonction qui tourne en tâche de fond
Function ProcAjoutePointsAuto(s)
	STRUCT WMBackgroundStruct &s
	BoutonAnalyse("AjoutePointsAuto")
	Return(0)	//	nécessaire de retourner un code d'exécution correct
End

//_____________________________________________________________________________________________
//_____________________________________________________________________________________________
//*************************** fonctions de zoom  **********************************************

Function SetVarZoom(ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName
	Variable varNum
	String varStr
	String varName

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	NVAR/SDFR=dfRatio V_Taille_X, V_Taille_Y, V_Zoom, V_Zoom_Precedent, V_PosZoomX1, V_PosZoomX2, V_PosZoomY1, V_PosZoomY2, V_PosZoomDeltaX, V_PosZoomDeltaY, V_NbChiffres, V_PixParPix
	NVAR/SDFR=dfROI V_NumROI
	
	StrSwitch(ctrlName)
		case "C_Magnification":	//	changes the zoom for display
//			ModifyGraph/W=F_RM expand=V_Zoom
		break
		case "C_AutoZoomROI":
			Switch(varNum)
				case 0:
					V_Zoom=V_Zoom_Precedent
					DoWindow/F F_RM
					SetAxis/A top
					SetAxis/A/R left	//	image affichée par défaut avec un axe vertical inversé; on garde cette convention
				break
				case 1:
					V_Zoom=3					//	règle la valeur de zoom
					String NomROIx, NomROIy
					Variable xA, xB, yA, yB	//	xA=extrême gauche; xB: extrême droite
					Variable L, R, T, B, W, H, G=2	//	left, right, top, bottom, width, height, scale
					NomROIx=AjouteNumero("ROIx", V_NumROI, V_NbChiffres)
					Wave/SDFR=dfROI wROIx=$NomROIx
					NomROIy=AjouteNumero("ROIy", V_NumROI, V_NbChiffres)
					Wave/SDFR=dfROI wROIy=$NomROIy
					WaveStats/Q/M=1 wRoix
					xA=V_min; xB=V_max
					WaveStats/Q/M=1 wRoiy
					yA=V_min; yB=V_max
					W=xB-xA; H=yB-yA
					L=xA-W/G; R=xB+W/G
					T=yA-H/G; B=yB+H/G
					SetAxis/W=F_RM/R left B,T
					SetAxis/W=F_RM top L,R
					SetVarZoom("",NaN,"","")
				break
			EndSwitch
		break
		case "C_SauveRecadrage":		//	to get the position of the marquee drawn on the image
			GetMarquee/K/Z/W=F_RM left, top
			if (V_Flag==1)
				print V_Left, V_right, V_top, V_bottom
				V_PosZoomX1=V_left; V_PosZoomX2=V_right
				V_PosZoomY1=V_top; V_PosZoomY2=V_bottom
				V_PosZoomX1=abs(trunc(V_left)); V_PosZoomX2=min(trunc(V_right), V_Taille_X)
				V_PosZoomY1=abs(trunc(V_top)); V_PosZoomY2=min(trunc(V_bottom), V_Taille_Y)
				V_PosZoomDeltaX=V_PosZoomX2-V_PosZoomX1
				V_PosZoomDeltaY=V_PosZoomY2-V_PosZoomY1
//				SetMarquee/W=F_RM V_PosZoomX1, V_PosZoomY1, V_PosZoomX2, V_PosZoomY2
			else
				V_PosZoomX1=NaN; V_PosZoomX2=NaN; V_PosZoomY1=NaN; V_PosZoomY2=NaN; V_PosZoomDeltaX=NaN; V_PosZoomDeltaY=NaN
			endif
//			GetAxis/W=F_RM top
//			V_PosZoomX1=V_min; V_PosZoomX2=V_max
//			GetAxis/W=F_RM left
//			V_PosZoomY1=V_min; V_PosZoomY2=V_max
		break
		case "C_AppliqueRecadrage":
			SetAxis/W=F_RM top, V_PosZoomX1, V_PosZoomX2
			SetAxis/W=F_RM left, V_PosZoomY2, V_PosZoomY1
			Print "________________________"
			Print "Copy the following line in the notebook to save this zoom definition:"
			Print "SetDataFolder root:ratio;","V_PosZoomX1=", V_PosZoomX1, "; V_PosZoomX2=", V_PosZoomX2, "; V_PosZoomY1=", V_PosZoomY1, "; V_PosZoomY2=", V_PosZoomY2
		break
		case "C_ImageEntiere":
			SetAxis/W=F_RM/A top
			SetAxis/W=F_RM/A/R left
		break
		case "C_PosCadreX1":
		break	
		case "C_PosCadreDeltaX":
			V_PosZoomX2=V_PosZoomX1+V_PosZoomDeltaX
		break
		case "C_PosCadreDeltaY":
			V_PosZoomY2=V_PosZoomY1+V_PosZoomDeltaY
		break
		default:
		break
	EndSwitch

	//	At the end, makes sure that the image has square pixels and applies the zoom value
	DoWindow F_RM
	if (V_Flag)	//	Do it only if the window exists
		ModifyGraph/W=F_RM expand=V_Zoom
		DoUpdate/W=F_RM
		variable dd, ee, ff, gg
		GetAxis/W=F_RM/Q left
		dd=V_min; ee=V_max
		GetAxis/W=F_RM/Q top
		ff=V_min; gg=V_max
		ModifyGraph/W=F_RM width=abs(gg-ff)*V_PixParPix, height=abs(ee-dd)*V_PixParPix
	endif
	DoUpdate

	//	Updates width and height of displayed region
	V_PosZoomDeltaX=V_PosZoomX2-V_PosZoomX1
	V_PosZoomDeltaY=V_PosZoomY2-V_PosZoomY1

	//	Position the P_ROI panel to the right of F_RM
	DoWindow P_ROI
	if (V_flag)
		AutoPositionWindow/R=F_RM/M=0 P_ROI
	endif
End

//	Relay function for buttons
Function BoutonZoom(ctrlName) : ButtonControl
	String ctrlName
	SetVarZoom(ctrlName, 0, "", "")
End

//	Relay function for checkboxes
Function CheckZoom(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	SetVarZoom(ctrlName, checked, "", "")
End

//*************************** fonctions de zoom  **********************************************
//_____________________________________________________________________________________________
//_____________________________________________________________________________________________



//_____________________________________________________________________________________________
// *** Generate an average image from one channel, to be used as a reference for ratio calculation ***
//	Done in a quick and dirty way just to test the concept
//	Main problem : all images in the "B" channel are clones of the average image

Function ZebraAverage()
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfRatio=W_DFREF[3]
	NVAR/SDFR=dfRatio DIM
	Wave/SDFR=dfRatio P_A, P_B

	variable i, vResult

	SetDataFolder root:

	//	create the average
	Duplicate/O P_A, P_Avg	//	make an image the same size as standard images
	FastOp P_Avg=0
	for (i=0; (i<Dim); i+=1)	//	for each time-point
		vResult=GereImages(i, 1, 1)	//	read P_A and P_B from cache
		FastOp P_Avg=P_Avg+P_A		//	add P_A to the average
	endfor
	P_Avg/=Dim	//	divide by the number of images

	//	report the average to all datapoints in channel B
	for (i=0; (i<Dim); i+=1)
		vResult=GereImages(i, 1, 1)	//	read P_A and P_B from cache, again, for each time-point
		FastOp P_B=P_Avg
		vResult=GereImages(i, 2, 1)	//	save P_A, identical, but save P_B equal to P_Avg
	endfor
End