﻿#pragma TextEncoding = "UTF-8"
#pragma rtGlobals=3		// Use modern global access method and strict wave access.

// *** Copyright ***
// © 2023 CNRS Shared by Pierre VINCENT, under the MIT License.
//	Contact: pierre.vincent@cnrs.fr; IGF, 141 rue de la Cardonille, 34094 Montpellier cedex 5, France.


//**********************************    bouton qui fait le travail   *****************
//	gère les tables, graphes...
//	teste chaque trace, et chaque moyenne
//	passe la wave correspondante à StatsTrace, qui retourne le résultat des mesures
//	remplit les tableaux avec ces mesures
Function BoutonStatsRatio(ctrlName) : ButtonControl
	string ctrlName

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfExport=W_DFREF[1]; DFREF dfRatio=W_DFREF[3]
	
	Wave/SDFR=dfRatio W_Temps_min, W_Temps, W_NombreTag
	Wave/T/SDFR=dfRatio W_NombreTagStr, W_EvtStrCorr
	NVAR/SDFR=dfRatio V_NbChiffres, vDim=dim, V_T0, vR0=V_R0
	SVAR/SDFR=dfRatio S_Nom_Manip

	Wave/SDFR=dfROI W_Tag, W_CouleurTagTraces, W_CouleurTrace, W_EpaisseurTrace, W_Tag, W_IntensiteOK, W_Intensite
	NVAR/SDFR=dfROI V_NbROI, V_EpaisseurMoy

	SetDataFolder dfExport

	//	Nettoyage avant de commencer
	//	Il faut vider les df contenus dans dfExport car des traces peuvent rester d'un export précédent
	//	Pour cela, ces traces ne doivent pas être affichées
	KillWindow/Z F_MesuresRatio
	KillWindow/Z F_MesuresRatioTag
	KillDataFolder/Z Normalized	//	/Z car pourrait ne pas exister, la première fois
	KillDataFolder/Z Norm_Baseline
	KillDataFolder/Z Raw

	Wave/SDFR=dfExport W_Curseurs
	NVAR/SDFR=dfExport V_ModeCurseurs

	//	Copie la variable V_R0 dans le dfExport
	Variable/G V_R0=vR0
	
	Variable i, j, vTag, vA, vB, vNumTrace, vDimRes, vXIncrement
//	Variable/C vIntensite		//	pour récupérer la valeur d'intensité mesurée sur A et B
	String NomTrace, nomTableau, nomTraceMoy, nomTracePic_X, nomTracePic_Y, sNomExport, sNomFenetre, sNomTrace
	DFREF dfAffichage
	
	//	Determines the name of the currently selected export df
	sNomExport=GetDataFolder(0, dfExport)[(StrLen(ksNomExport))+1, inf]
	
	//	Arrête si le mode curseur est 0
	if (V_ModeCurseurs==0)
		DoAlert 0, "Measurement cursors must be activated"
		return(0)
	endif
	//	On continue seulement si mode curseur 1, 2, 3 ou 4
	if ((V_ModeCurseurs < 1) || (V_ModeCurseurs > 4))
		DoAlert 0, "Bug: unknown cursor mode."
		Return(0)
	endif

	SauveCurseurs(0)	//	garantit que les curseurs soient sur les traces R0000
	CheckAfficheMoy("BoutonStatsRatio", 1)	//	Calculates the average of traces in a group
	CheckAfficheMoy("BoutonStatsRatio", 0)	//	Hide the average traces

	//	définit les waves pour l'affichage des résultats
	Make/O/N=(V_NbROI+1, 20) dfExport:$(S_Nom_Manip+"_W_Ratio")
	Make/O/N=(4, 20) dfExport:$(S_Nom_Manip+"_W_RatioTag")
	Make/O/T/N=(V_NbROI+1, 2) dfExport:$(S_Nom_Manip+"_W_RatioStr")
	Make/O/T/N=(4, 2) dfExport:$(S_Nom_Manip+"_W_RatioTagStr")
	Wave/SDFR=dfExport wRatio=$(S_Nom_Manip+"_W_Ratio"), wRatioTag=$(S_Nom_Manip+"_W_RatioTag")
	Wave/T/SDFR=dfExport wRatioStr=$(S_Nom_Manip+"_W_RatioStr"), wRatioTagStr=$(S_Nom_Manip+"_W_RatioTagStr")
	wRatio=nan; 	wRatioTag=NaN; wRatioStr=""; wRatioTagStr=""

	//	Renseigne les waves texte
	for (j=1; (j<=V_NbROI);j+=1)	
		if (testegroupe(W_Tag[j]))
			wRatioStr[j][0]=S_Nom_Manip
			wRatioStr[j][1]=W_NombreTagStr[W_Tag[j]]
		endif
	EndFor
	for (i=0; (i<=3); i+=1)
		if (TesteGroupe(i))
			wRatioTagStr[i][0]=S_Nom_Manip
			wRatioTagStr[i][1]=W_NombreTagStr[i]
		endif
	endfor

	//	Affiche les tableaux contenant les résultats
	Edit/K=1/N=F_MesuresRatioTag/W=(1,44,1220,206) wRatioTagStr, wRatioTag as "Measurements for each group"
	ModifyTable format(Point)=1,width(wRatioTag)=86
	Edit/K=1/N=F_MesuresRatio/W=(1,231,1220,600) wRatioStr, wRatio as "Mesurements for individual ROIs"
	ModifyTable format(Point)=1,width(wRatio)=86
	
	//	Make 1 output graph per selected group
	for (i=0; (i<=3); i+=1)
		if (TesteGroupe(i))
			sNomFenetre="F_Curs_"+num2str(i)
			Dowindow/F $sNomFenetre
			if (V_flag==0)	//	If not yet present, create the output graphs
				Display/K=1/N=$sNomFenetre/W=(20,460,420,860)	as "Stats for "+W_NombreTagStr[i]
			endif
		endif
	endfor
//	Display/K=1/N=F_Pic/W=(20,20,420,420) as "Normalized traces for individual ROIs"//	créée la fenêtre pour afficher les pics
//	Display/K=1/N=F_PicMoy/W=(20,460,420,860)	as "Normalized averages"//	créée la fenêtre pour afficher les pics
//	Legend/C/N=text0/F=0/A=MC
		
	PopMenuTypeAxeX("",2,"")	//	pour forcer le temps en secondes
	PopUpMenu/Z C_ChangeTemps win=P_Controle, mode=2		//	pour mettre à jour le popup menu

	//	Positionne le temps 0 au curseur B
	V_T0=W_Temps[pcsr(B, "F_TracesTriees")]
	DoUpdate	//	pour mettre à jour W_Temp_min

	//	Vérifie si le temps est une fonction monotone
	if (!MonotonicCheck(W_Temps_min, vXIncrement))
		DoAlert 0, "Time is not a monotonic function, or contains duplicated values; execution cancelled"
		Return(0)
	endif
	//	génère la wave de temps et la met dans dfExport:
	nomTracePic_X=S_Nom_Manip+"_temps"
	Make/O/N=(vDim) dfExport:$nomTracePic_X
	Wave/SDFR=dfExport wTracePic_X=$nomTracePic_X
	wTracePic_X[]=W_Temps_min[p]
	
	//	pour chaque ROI, calcul du ratio sur les curseurs définis
	for (j=1; (j<=V_NbROI);j+=1)	
		if (testegroupe(W_Tag[j]))
			NomTrace=AjouteNumero("TraceR", j, V_NbChiffres)
			wave/SDFR=dfROI wTrace=$NomTrace
			//	Nom des traces normalisées
			nomTracePic_Y=S_Nom_Manip+"_"+AjouteNumero("",j, V_NbChiffres)
			//	Calcule les stats de la trace, et exporte dans les df
			Wave wResultatRatio=StatsTrace(wTrace, nomTracePic_Y, dfExport, vDim)
			vDimRes=DimSize(wResultatRatio,0)
			Redimension/N=(-1, vDimRes+2) wRatio		//	ajoute 2 colonnes pour y mettre les intensités
			wRatio[j][0, vDimRes-1]=wResultatRatio[q]
			KillWaves wResultatRatio
			wRatio[j][0]=j
			wRatio[j][1]=W_Tag[j]
			//	Transfer intensity value in the table
//			vIntensite=MesureIntensite(j)
//			wRatio[j][vDimRes]=real(vIntensite)
//			wRatio[j][vDimRes+1]=imag(vIntensite)
			if (W_IntensiteOK[j]==1)
				wRatio[j][vDimRes]=W_Intensite[j][0]
				wRatio[j][vDimRes+1]=W_Intensite[j][1]
			else
				wRatio[j][vDimRes]=NaN
				wRatio[j][vDimRes+1]=NaN
			endif

			//	Définit le type d'export à afficher
			ControlInfo/W=P_Export C_ChoixAffichage
			Switch(V_Value)
				case 1:
					dfAffichage=dfExport:Normalized
				break
				case 2:
					dfAffichage=dfExport:Norm_Baseline
				break
				case 3:
					dfAffichage=dfExport:Raw
				break
			endswitch
			//	Append the trace on the right window
			sNomFenetre="F_Curs_"+num2str(W_Tag[j])
			Wave/SDFR=dfAffichage wTracePic_Y=$nomTracePic_Y
			sNomTrace=nomTracePic_Y+"_"+sNomExport
			AppendToGraph/W=$sNomFenetre wTracePic_Y/TN=$sNomTrace vs wTracePic_X
			ModifyGraph/Z/W=$sNomFenetre rgb($sNomTrace)=(W_CouleurTrace[j][0],W_CouleurTrace[j][1],W_CouleurTrace[j][2])
			ModifyGraph/Z/W=$sNomFenetre lsize($sNomTrace)=W_EpaisseurTrace[j]
		endif
	endfor

	// pour chacune des valeurs de tag
	for (i=0; (i<=3); i+=1)
		if (TesteGroupe(i))
			NomTrace="TraceMoy"+Num2Str(i)
			Wave wTrace=dfROI:$NomTrace
			nomTracePic_Y=S_Nom_Manip+"_Moy_"+Num2Str(i)
			Wave wResultatRatio=StatsTrace(wTrace, nomTracePic_Y, dfExport, vDim)
			vDimRes=DimSize(wResultatRatio,0)
			Redimension/N=(-1, vDimRes) wRatioTag
			wRatioTag[i][]=wResultatRatio[q]
			wRatioTag[i][0]=W_NombreTag[i]
			wRatioTag[i][1]=i
			KillWaves wResultatRatio
			//	Append the trace on the right window
			sNomFenetre="F_Curs_"+num2str(i)
			Wave/SDFR=dfAffichage wTracePic_Y=$nomTracePic_Y					
			sNomTrace=nomTracePic_Y+"_"+sNomExport
			AppendToGraph/W=$sNomFenetre wTracePic_Y/TN=$sNomTrace vs wTracePic_X
			ModifyGraph/Z/W=$sNomFenetre rgb($sNomTrace)=(0, 0, 0), lsize($sNomTrace)=V_EpaisseurMoy
		endif
	endfor

	DoWindow/F P_Export
	SetDataFolder root:
End

//*********************   Mesure sur les zones définies par les curseurs  ***********************************
//	Mesure le niveau de ratio sur les zones définies par les curseurs
//	Effectue les calculs d'amplitude de réponse; retourne ces mesures dans wave résultat
//	Calcule les traces normalisées, selon le mode curseur;
//	Range dans des sous-datafolders différents les différents types de normalisation
Function/WAVE StatsTrace(wTrace, sNomTrace, dfExport, vDim)
	Wave wTrace
	String sNomTrace
	DFREF dfExport
	Variable vDim
	
	DFREF dfTemp
	Variable vAB, vCD, vEF, vGH, vIJ	//	Mesure de la moyenne pour chaque curseur
	Variable vBase=NaN, vRmin=NaN, vRmax=NaN
	String nomWaveOut=NameOfWave(wTrace)+"_Stats"	//	nom sous lequel sera exporté le résultat
	
	SetDataFolder dfExport
	NVAR V_ModeCurseurs

	DoWindow/F F_TracesTriees	//	Met F_TraceTriees au premier plan pour que les appels aux curseurs fonctionnent

	//	Wave qui va contenir les mesures
	Make/O dfExport:$nomWaveOut
	Wave/SDFR=dfExport wStats=$nomWaveOut
	wStats=NaN

	Switch(V_ModeCurseurs)
		case 1:
			Redimension/N=8 wStats
			vAB=mean(wTrace, xcsr(A, "F_TracesTriees"),xcsr(B, "F_TracesTriees"))
			vCD=mean(wTrace, xcsr(C, "F_TracesTriees"),xcsr(D, "F_TracesTriees"))
			vEF=mean(wTrace, xcsr(E, "F_TracesTriees"),xcsr(F, "F_TracesTriees"))
			vGH=mean(wTrace, xcsr(G, "F_TracesTriees"),xcsr(H, "F_TracesTriees"))
			wStats[2]=vAB; 	wStats[3]=vCD; wStats[4]=vEF; wStats[5]=vGH
			wStats[6]=(vGH-vEF)/vEF	//	% ∆R/R max
//			wStats[7]=(vCD-vAB)/(vGH-vEF)*100	//	% de la réponse par-rapport au max
			wStats[7]=(vCD-vAB)/(vGH-vEF)	//	fraction de la réponse par-rapport au changement max
			vBase=vAB
			vRmin=vEF
			vRmax=vGH
		break
		case 2:	//	
	//	CD=réponse D2; EF=réponse D1
			Redimension/N=6 wStats
			vAB=mean(wTrace, xcsr(A, "F_TracesTriees"),xcsr(B, "F_TracesTriees"))
			vCD=mean(wTrace, xcsr(C, "F_TracesTriees"),xcsr(D, "F_TracesTriees"))
			vEF=mean(wTrace, xcsr(E, "F_TracesTriees"),xcsr(F, "F_TracesTriees"))
			wStats[2]=vAB; wStats[3]=vCD; wStats[4]=vEF
			if (xcsr(C, "F_TracesTriees") < xcsr(E, "F_TracesTriees") )	//	réponse D2 en premier
				wStats[5]=(vCD-vAB)/(vEF-vAB)
			else		//	réponse D1 en premier
				wStats[5]=(vCD-vEF)/(vCD-vAB)
			endif
		break
		case 3:
			Redimension/N=5 wStats
			wStats[2]=mean(wTrace, xcsr(A, "F_TracesTriees"),xcsr(B, "F_TracesTriees"))
			wStats[3]=mean(wTrace, xcsr(C, "F_TracesTriees"),xcsr(D, "F_TracesTriees"))
			wStats[4]=mean(wTrace, xcsr(E, "F_TracesTriees"),xcsr(F, "F_TracesTriees"))
		break
		case 4:
			Redimension/N=10 wStats
			//	Mesure ratio brut
			vAB=mean(wTrace, xcsr(A, "F_TracesTriees"),xcsr(B, "F_TracesTriees"))
			vCD=mean(wTrace, xcsr(C, "F_TracesTriees"),xcsr(D, "F_TracesTriees"))
			vEF=mean(wTrace, xcsr(E, "F_TracesTriees"),xcsr(F, "F_TracesTriees"))
			vGH=mean(wTrace, xcsr(G, "F_TracesTriees"),xcsr(H, "F_TracesTriees"))
			vIJ=mean(wTrace, xcsr(I, "F_TracesTriees"),xcsr(J, "F_TracesTriees"))
			wStats[2]=vAB; 	wStats[3]=vCD; wStats[4]=vEF; wStats[5]=vGH; wStats[6]=vIJ
			wStats[7]=(vIJ-vGH)/vGH
			wStats[8]=(vAB-vGH)/(vIJ-vGH)
			wStats[9]=(vCD-vAB)/(vEF-vAB)
			vBase=vAB
			vRmin=vGH
			vRmax=vIJ
		break
	EndSwitch

	//	Normalized: trace normalisée sans soustraction de ligne de base; normalisé entre Rmin et Rmax
	dfTemp=dfExport:Normalized
	if (DataFolderRefStatus(dfTempNorm)==0)
		NewDataFolder/O Normalized
		dfTemp=dfExport:Normalized
	endif
	Make/O/N=(vDim) dfTemp:$sNomTrace
	Wave/SDFR=dfTemp wTracePic_Y=$sNomTrace
	wTracePic_Y[]=wTrace[p]
	wTracePic_Y[]-=vRmin	//	on soustrait Rmin
	wTracePic_Y[]/=vRmax-vRmin

	//	Norm_Baseline: trace normalisée avec soustraction de ligne de base mesurée sur AB
	dfTemp=dfExport:Norm_Baseline
	if (DataFolderRefStatus(dfTemp)==0)
		NewDataFolder/O Norm_Baseline
		dfTemp=dfExport:Norm_Baseline
	endif
	Make/O/N=(vDim) dfTemp:$sNomTrace
	Wave/SDFR=dfTemp wTracePic_Y=$sNomTrace
	wTracePic_Y[]=wTrace[p]
	wTracePic_Y[]-=vBase	//	on soustrait la ligne de base
	wTracePic_Y[]/=vRmax-vRmin	//	on divise par la réponse max

	//	Copie les traces brutes
	dfTemp=dfExport:Raw
	if (DataFolderRefStatus(dfTemp)==0)
		NewDataFolder/O Raw
		dfTemp=dfExport:Raw
	endif
	Make/O/N=(vDim) dfTemp:$sNomTrace
	Wave/SDFR=dfTemp wTracePic_Y=$sNomTrace
	wTracePic_Y[]=wTrace[p]
	
	Return wStats
end

//**********************************    réglage de la position des curseurs  ***********************************

Function DesactiveCurseurs()
	DoWindow/F F_TracesTriees
	//	Désactive tous les curseurs
	Cursor/A=0/M A
	Cursor/A=0/M B
	Cursor/A=0/M C
	Cursor/A=0/M D
	Cursor/A=0/M E
	Cursor/A=0/M F
	Cursor/A=0/M G
	Cursor/A=0/M H
	Cursor/A=0/M I
	Cursor/A=0/M J
End

Function BoutonZoomeSurB(ctrlName) : ButtonControl
	String ctrlName

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfExport=W_DFREF[1]; DFREF dfRatio=W_DFREF[3]
	Wave/SDFR=dfRatio W_Temps_min
	NVAR/SDFR=dfRatio Dim
	variable vMin, vMax, vPosB

	DoWindow/F F_TracesTriees
	DesactiveCurseurs()
	StrSwitch(CtrlName)
		case "C_ZoomeSurB":
			Cursor/A=1/M B	//	active le curseur B
			vPosB=pcsr(B)
			vMin=Max((vPosB-13), 0)
			vMin=W_Temps_min[vMin]
			vMax=Min((vPosB+20), Dim-1)
			vMax=W_Temps_min[vMax]
			SetAxis bottom vMin, vMax
			CheckROIMarqueurs("C_ROI_Points", 1)	//	affiche les trace sous forme de points
		break
		case "C_DeZoomePic":
			SetAxis/A
			SetVarEspaceEvt("",0,"","")
			CheckROIMarqueurs("C_ROI_Points", 0)	//	remet les traces en trait continu
			Cursor/A=0/M B
			HideInfo
		break
	EndSwitch
End

Function SetVarZoomCurseur(ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName
	Variable varNum
	String varStr
	String varName
	
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfExport=W_DFREF[1]; DFREF dfRatio=W_DFREF[3]
	NVAR/SDFR=dfRatio Dim
	Wave/SDFR=dfRatio wTemps=W_Temps
	variable vPosB, i

	DoWindow/F F_TracesTriees
	varNum=Max(varNum, 0)	//	évite de tomber sous 0
	varNum=Min(varNum, Dim)	//	évite de dépasser la limite de manip
	vPosB=pcsr(B)
	StrSwitch(ctrlName)
		case "C_PosCsrB":
			Cursor/A=1/H=2/L=0/S=1/W=F_TracesTriees B TraceR0000 (varNum)
			BoutonZoomeSurB("C_ZoomeSurB")
		break
		case "C_PosCsrB_Temps":
			//	cherche le point le plus proche dans le temps
			do
				i+=1
			while (varNum >= wTemps[i]/60)
			Cursor/A=1/H=2/L=0/S=1/W=F_TracesTriees B TraceR0000 (wTemps[i]/60)
			BoutonZoomeSurB("C_ZoomeSurB")			
		break
		case "C_PosCsrA":
			Cursor/A=1/H=2/L=0/S=1/W=F_TracesTriees A TraceR0000 (vPosB-varNum)
		break
		case "C_PosCsrC":
			Cursor/A=1/H=2/L=0/S=1/W=F_TracesTriees C TraceR0000 (vPosB+varNum)
		break
		case "C_PosCsrD":
			Cursor/A=1/H=2/L=0/S=1/W=F_TracesTriees D TraceR0000 (vPosB+varNum)
		break
	EndSwitch
End


//************************************************************
//____________________________________________________________
//___________________	Gestion des curseurs	______________________

Function MajCurseurs(sMode)
	string sMode	//	sMode="T0" ; "Ratio"

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfExport=W_DFREF[1]; DFREF dfRatio=W_DFREF[3]
	NVAR/SDFR=dfRatio V_ModeCurseurs

	DoWindow/F P_Export
	StrSwitch(sMode)
		case "T0":
			CheckModeCurseur("C_CheckCurs0", 0)
			CheckBox C_CheckCurs0, win=P_Controle, value=0, disable=2
			CheckBox C_CheckCurs1, win=P_Controle, value=0, disable=2
			CheckBox C_CheckCurs2, win=P_Controle, value=0, disable=2
			CheckBox C_CheckCurs3, win=P_Controle, value=0, disable=2
			CheckBox C_CheckCurs4, win=P_Controle, value=0, disable=2
		break
		case "Ratio":
			Switch(V_ModeCurseurs)
				case 0:
					CheckModeCurseur("C_CheckCurs0",1)
				break
				case 1:
					CheckModeCurseur("C_CheckCurs1",1)
				break
				case 2:
					CheckModeCurseur("C_CheckCurs2",1)
				break
				case 3:
					CheckModeCurseur("C_CheckCurs3",1)
				break
				case 4:
					CheckModeCurseur("C_CheckCurs4",1)
				break
			EndSwitch
		break
	EndSwitch
End

//________	sauve la position des curseurs	______________________
Function SauveCurseurs(vEfface)	//	vEfface = 1 si les curseurs doivent être enlevés du graphe
	variable vEfface	

	Wave/DF W_DFREF=root:W_DFREF
	DFREF dfROI=W_DFREF[0]; DFREF dfExport=W_DFREF[1]; DFREF dfRatio=W_DFREF[3]
	Wave/SDFR=dfExport/Z W_Curseurs
	if (!WaveExists(W_Curseurs))
		return(0)
	endif
	NVAR/Z/SDFR=dfExport V_ModeCurseurs
	Variable i, j, k, m
	String sCurs
	
	Switch(V_ModeCurseurs)
		case 0:
			return(0)
		break
		case 1: 		//	curseurs ratio
			k=0; m=7
		break
		case 2:		//	curseurs D1D2
			k=8; m=5
		break
		case 3:		//	curseurs pics
			k=14; m=5
		break
		case 4:		//	curseurs pics
			k=20; m=9
		break
	EndSwitch
	for (i=0;(i<=m);i+=1)			//	8 curseurs au maximum sont utilisés dans l'analyse
		sCurs=Num2Char(i+65)	//	définit la lettre A pour i=0, et les autres ensuite
		if (StrLen(CsrInfo($sCurs, "F_TracesTriees"))!=0)	//	Curseur concerné est présent sur le graphe
			j=pcsr($sCurs, "F_TracesTriees")
			cursor/A=0/H=2/S=1/W=F_TracesTriees $sCurs TraceR0000 j
			W_Curseurs[i+k]=pcsr($sCurs, "F_TracesTriees")	//	sauve la position du curseur dans le tableau à la bonne case, selon le type de curseur
			if (vEfface)
				cursor/K/M/W=F_TracesTriees $sCurs		//	enlève le curseur du graphe
			endif
		endif
	endfor
End

//**********************************************************************
//______________________________________________________________________
//___________________	 Fonctions du panneau	Export	____________________

Function BoutonCopieValeursCurseurs(ctrlName) : ButtonControl
	String ctrlName
	String sListe, sNom
	Variable vChoix
	Wave/DF W_DFREF=root:W_DFREF
	DFREF dfROI=W_DFREF[0]; DFREF dfExport=W_DFREF[1]; DFREF dfRatio=W_DFREF[3]
	Wave/SDFR=dfExport W_Curseurs
	NVAR/SDFR=dfExport V_ModeCurseurs
	NVAR/SDFR=dfRatio Dim
	if ((V_ModeCurseurs==1) || (V_ModeCurseurs==3))
		StrSwitch(ctrlName)
			case "C_CopieAutreReglage":
				sListe=ListeDF("root:", "Export")
				prompt vChoix, "What export datafolder ?", popup, sListe
				DoPrompt "Get cursor settings from...", vChoix
				if (V_Flag)
					Return(0)
				endif
				vChoix-=1
				//	Désactive l'affichage des curseurs
				CheckModeCurseur("C_CheckCurs0",1)
				sNom="root:" + StringFromList(vChoix, sListe)
				DFREF dfSource=$sNom
				Wave/SDFR=dfSource wCurseursSource=W_Curseurs
				W_Curseurs=wCurseursSource
				
			break
			case "C_SuperposeCsrEF":
				DoWindow/F F_TracesTriees
				cursor/A=0/L=1 E TraceR0000 pcsr(A)
				cursor/A=0/L=1 F TraceR0000 pcsr(B)
			break
			case "C_SuperposeCsrGH":
				DoWindow/F F_TracesTriees
				cursor/A=0/L=0 G TraceR0000 Dim-1
				cursor/A=0/L=0 H TraceR0000 Dim-5
			break
		EndSwitch
	endif
	if ((V_ModeCurseurs==4) && (CmpStr(ctrlName, "C_Copie")==0))
		cursor/A=0 A TraceR0000 W_Curseurs[0]
		cursor/A=0 B TraceR0000 W_Curseurs[1]
		cursor/A=0 C TraceR0000 W_Curseurs[2]
		cursor/A=0 D TraceR0000 W_Curseurs[3]
		cursor/A=0 G TraceR0000 W_Curseurs[4]
		cursor/A=0 H TraceR0000 W_Curseurs[5]
		cursor/A=0 I TraceR0000 W_Curseurs[6]
		cursor/A=0 J TraceR0000 W_Curseurs[7]
	endif
End

//	gère les boutons radio du mode de curseurs, pour les différents types de mesure.
Function CheckModeCurseur(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked

	Wave/DF W_DFREF=root:W_DFREF
	DFREF dfROI=W_DFREF[0]; DFREF dfExport=W_DFREF[1]; DFREF dfRatio=W_DFREF[3]

	Wave/SDFR=dfExport W_Curseurs
	NVAR/SDFR=dfExport V_ModeCurseurs
	Wave/SDFR=dfROI wCoul=W_CouleurTagROI
	NVAR/SDFR=dfRatio Dim
	
	Variable i

	if (CmpStr(ctrlName, "Init")==0)
		//	condition déclenchée lorsqu'on change de dfExport:
		// il ne faut PAS sauver les curseurs, et passer ensuite le nom du contrôle qui correspond à V_ModeCurseurs
		ctrlName="C_CheckCurs"+Num2Str(V_ModeCurseurs)
	else
		SauveCurseurs(1)
	endif
	StrSwitch(ctrlName)
		case "C_CheckCurs0":
			HideInfo/W=F_TracesTriees
			CheckBox C_CheckCurs0, win=P_Export, value=1
			CheckBox C_CheckCurs1, win=P_Export, value=0
			CheckBox C_CheckCurs2, win=P_Export, value=0
			CheckBox C_CheckCurs3, win=P_Export, value=0
			CheckBox C_CheckCurs4, win=P_Export, value=0
			V_ModeCurseurs=0
		break
		case "C_CheckCurs1":
			ShowInfo/CP={0,1,2,3}/W=F_TracesTriees
			CheckBox C_CheckCurs0, win=P_Export, value=0
			CheckBox C_CheckCurs1, win=P_Export, value=1
			CheckBox C_CheckCurs2, win=P_Export, value=0
			CheckBox C_CheckCurs3, win=P_Export, value=0
			CheckBox C_CheckCurs4, win=P_Export, value=0
			V_ModeCurseurs=1
			if (NumType(W_Curseurs[0])!=0)	// suppose que les positions de curseurs ne sont pas définies
				for (i=0; (i<=7); i+=1)	//	on remplit le tableau curseur avec des valeurs par défaut
					W_Curseurs[i]=i*3
				endfor
			endif
			cursor/A=0/C=(0, 0, 65535)/H=2/L=0/S=1/W=F_TracesTriees A TraceR0000 W_Curseurs[0]
			cursor/A=0/C=(0, 0, 65535)/H=2/L=0/S=1/W=F_TracesTriees B TraceR0000 W_Curseurs[1]
			cursor/A=0/C=(0, 65535, 0)/H=2/L=0/S=1/W=F_TracesTriees C TraceR0000 W_Curseurs[2]
			cursor/A=0/C=(0, 65535, 0)/H=2/L=0/S=1/W=F_TracesTriees D TraceR0000 W_Curseurs[3]
			cursor/A=0/C=(65535,43690,0)/H=2/L=0/S=1/W=F_TracesTriees E TraceR0000 W_Curseurs[4]
			cursor/A=0/C=(65535,43690,0)/H=2/L=0/S=1/W=F_TracesTriees F TraceR0000 W_Curseurs[5]
			cursor/A=0/C=(65535, 0, 20000)/H=2/L=0/S=1/W=F_TracesTriees G TraceR0000 W_Curseurs[6]
			cursor/A=0/C=(65535, 0, 20000)/H=2/L=0/S=1/W=F_TracesTriees H TraceR0000 W_Curseurs[7]
			DesactiveCurseurs()
		break
		case "C_CheckCurs2":
			ShowInfo/W=F_TracesTriees
			CheckBox C_CheckCurs0, win=P_Export, value=0
			CheckBox C_CheckCurs1, win=P_Export, value=0
			CheckBox C_CheckCurs2, win=P_Export, value=1
			CheckBox C_CheckCurs3, win=P_Export, value=0
			CheckBox C_CheckCurs4, win=P_Export, value=0
			V_ModeCurseurs=2
			if (NumType(W_Curseurs[8])!=0)	// suppose que les positions de curseurs ne sont pas définies
				for (i=8; (i<=13); i+=1)	//	on remplit le tableau curseur avec des valeurs par défaut
					W_Curseurs[i]=(i-8)*3
				endfor
			endif
			cursor/A=0/C=(0, 0, 0)/H=2/L=0/S=1/W=F_TracesTriees A TraceR0000 W_Curseurs[8]
			cursor/A=0/C=(0, 0, 0)/H=2/L=0/S=1/W=F_TracesTriees B TraceR0000 W_Curseurs[9]
			cursor/A=0/C=(wCoul[2][0], wCoul[2][1], wCoul[2][2])/H=2/L=0/S=1/W=F_TracesTriees C TraceR0000 W_Curseurs[10]
			cursor/A=0/C=(wCoul[2][0], wCoul[2][1], wCoul[2][2])/H=2/L=0/S=1/W=F_TracesTriees D TraceR0000 W_Curseurs[11]
			cursor/A=0/C=(wCoul[3][0], wCoul[3][1], wCoul[3][2])/H=2/L=0/S=1/W=F_TracesTriees E TraceR0000 W_Curseurs[12]
			cursor/A=0/C=(wCoul[3][0], wCoul[3][1], wCoul[3][2])/H=2/L=0/S=1/W=F_TracesTriees F TraceR0000 W_Curseurs[13]
			//	pour enlever les groupes non concernés par cette mesure
			CheckBox C_Groupe0 win=P_Controle, value=0
			CheckBox C_Groupe1 win=P_Controle, value=0
			CheckAfficheGroupe("C_Groupe0",0)
			CheckAfficheGroupe("C_Groupe1",0)
		break
		case "C_CheckCurs3":
			ShowInfo/W=F_TracesTriees
			CheckBox C_CheckCurs0, win=P_Export, value=0
			CheckBox C_CheckCurs1, win=P_Export, value=0
			CheckBox C_CheckCurs2, win=P_Export, value=0
			CheckBox C_CheckCurs3, win=P_Export, value=1
			CheckBox C_CheckCurs4, win=P_Export, value=0
			V_ModeCurseurs=3
			if (NumType(W_Curseurs[14])!=0)	// suppose que les positions de curseurs ne sont pas définies
				for (i=14; (i<=19); i+=1)	//	on remplit le tableau curseur avec des valeurs par défaut
					W_Curseurs[i]=(i-14)*3
				endfor
			endif
			cursor/A=0/C=(60000, 40000, 20000)/H=2/L=0/S=1/W=F_TracesTriees A TraceR0000 W_Curseurs[14]
			cursor/A=0/C=(60000, 40000, 20000)/H=2/L=0/S=1/W=F_TracesTriees B TraceR0000 W_Curseurs[15]
			cursor/A=0/C=(30000, 0, 60000)/H=2/L=0/S=1/W=F_TracesTriees C TraceR0000 W_Curseurs[16]
			cursor/A=0/C=(30000, 0, 60000)/H=2/L=0/S=1/W=F_TracesTriees D TraceR0000 W_Curseurs[17]
			cursor/A=0/C=(60000, 0, 0)/H=2/L=0/S=1/W=F_TracesTriees E TraceR0000 W_Curseurs[18]
			cursor/A=0/C=(60000, 0, 0)/H=2/L=0/S=1/W=F_TracesTriees F TraceR0000 W_Curseurs[19]
			//	pour enlever les groupes non concernés par cette mesure
			DoWindow P_Controle	//	vérifie que le panneau de contrôle est ouvert
			if (V_flag)
				CheckBox C_Groupe0 win=P_Controle, value=0
				CheckBox C_Groupe1 win=P_Controle, value=0
				CheckAfficheGroupe("C_Groupe0",0)
				CheckAfficheGroupe("C_Groupe1",0)
			endif
		break
		case "C_CheckCurs4":
			ShowInfo/CP={0,1,2,3,4}/W=F_TracesTriees
			CheckBox C_CheckCurs0, win=P_Export, value=0
			CheckBox C_CheckCurs1, win=P_Export, value=0
			CheckBox C_CheckCurs2, win=P_Export, value=0
			CheckBox C_CheckCurs3, win=P_Export, value=0
			CheckBox C_CheckCurs4, win=P_Export, value=1
			V_ModeCurseurs=4
			if (NumType(W_Curseurs[20])!=0)	// suppose que les positions de curseurs ne sont pas définies
				for (i=20; (i<=29); i+=1)	//	on remplit le tableau curseur avec des valeurs par défaut
					W_Curseurs[i]=(i-20)*3
				endfor
				W_Curseurs[28]=Dim-7
				W_Curseurs[29]=Dim-1
			endif
			cursor/A=0/C=(0, 0, 65535)/H=2/L=0/S=1/W=F_TracesTriees A TraceR0000 W_Curseurs[20]
			cursor/A=0/C=(0, 0, 65535)/H=2/L=0/S=1/W=F_TracesTriees B TraceR0000 W_Curseurs[21]
			cursor/A=0/C=(0,40371,8951)/H=2/L=0/S=1/W=F_TracesTriees C TraceR0000 W_Curseurs[22]
			cursor/A=0/C=(0,40371,8951)/H=2/L=0/S=1/W=F_TracesTriees D TraceR0000 W_Curseurs[23]
			cursor/A=0/C=(64315,30561,0)/H=2/L=0/S=1/W=F_TracesTriees E TraceR0000 W_Curseurs[24]
			cursor/A=0/C=(64315,30561,0)/H=2/L=0/S=1/W=F_TracesTriees F TraceR0000 W_Curseurs[25]
			cursor/A=0/C=(65535,0,50000)/H=2/L=0/S=1/W=F_TracesTriees G TraceR0000 W_Curseurs[26]
			cursor/A=0/C=(65535,0,50000)/H=2/L=0/S=1/W=F_TracesTriees H TraceR0000 W_Curseurs[27]
			cursor/A=0/C=(64227,0,0)/H=2/L=0/S=1/W=F_TracesTriees I TraceR0000 W_Curseurs[28]
			cursor/A=0/C=(64227,0,0)/H=2/L=0/S=1/W=F_TracesTriees J TraceR0000 W_Curseurs[29]
			DesactiveCurseurs()
		break
	EndSwitch
End

Function PopMenuSelectExport(ctrlName,popNum,popStr) : PopupMenuControl
	String ctrlName
	Variable popNum
	String popStr
	variable i, vDFREF_OK
	String sBaseNom, sNomDF
	
	Wave/DF W_DFREF=root:W_DFREF
	Wave/T WS_DFREF=root:WS_DFREF
	DFREF dfROI=W_DFREF[0]; DFREF dfExport=W_DFREF[1]; DFREF dfRatio=W_DFREF[3]
	
	variable vRang
	String sNomExport, sListe

	//	Avant de faire quoi que ce soit pour changer de dfExport
	//	on sauve la position des curseurs. Si le dfExport existe et est correct.
	sBaseNom=StringFromList(1, ksNomDataFolders)+"*"	//	base du nom des datafolder export
	vDFREF_OK=((StringMatch(GetDataFolder(0, dfExport), sBaseNom)) && (DataFolderRefStatus(dfExport)==1))
	if (vDFREF_OK)
		SauveCurseurs(1)
	endif

	StrSwitch(popStr)
		//	Appelé à l'initialisation du panneau
		case "Vide":	//	si un dfExport n'existe pas, ou bien celui indiqué dans dfExport n'est pas bon, désactiver les boutons
			if (!vDFREF_OK)	//	dfExport n'existe pas ou n'a pas un nom correct
				DoWindow/F P_Export
				CheckBox C_CheckCurs0 value=1, disable=2
				CheckBox C_CheckCurs1 value=0, disable=2
				CheckBox C_CheckCurs2 value=0, disable=2
				CheckBox C_CheckCurs3 value=0, disable=2
				CheckBox C_CheckCurs4 value=0, disable=2
				return(0)	//	rien à afficher dans le popop
			else	//	on indique dans le popup le nom du dfExport qui est le bon
				popStr=GetDataFolder(0, dfExport)
			endif
		break
		case "New":	//	crée le datafolder
			sNomExport=""
			prompt sNomExport, "How to name the Export datafolder ?"
			DoPrompt "Statistics: ", sNomExport
			if (V_Flag)
				print "Cancelled by user"; Return(0)
			endif
			if (StrLen(sNomExport)>0)
				sNomExport=ksNomExport+"_"+sNomExport
			else
				sNomExport=ksNomExport
			endif
			if (DataFolderExists("root:"+sNomExport))
				DoAlert 0, "This "+sNomExport+" datafolder already exists; choose another name."
				Return(0)
			endif
			NewDataFolder/S root:$sNomExport
			dfExport=GetDataFolderDFR()
		break
		case "Duplicate":
			Do
				sNomExport=""
				prompt sNomExport, "Name of the datafolder for this new export"
				DoPrompt "", sNomExport
				if (V_Flag)
					return(0)
				endif
				sNomExport=ksNomExport+"_"+sNomExport
			while (DataFolderExists(sNomExport))
			SetDataFolder root:
			DuplicateDataFolder dfExport, $sNomExport
			dfExport = root:$sNomExport
		break
		default:
			dfExport=root:$popStr
		break
	endswitch

	SetDataFolder dfExport
	W_DFREF[1]=dfExport
	WS_DFREF[1]=GetDataFolder(0, dfExport)
	
	//	initialise le dfExport; récupère éventuellement les objets anciens restés à la racine
	InitDataFolder("Init raplouf.pxp", "f_Export", GetDataFolder(0), sNomDataFolderAncien="root:")
	Wave W_Curseurs
	NVAR V_ModeCurseurs

	//	un résidus tu temps où W_Curseur ne comportait qu'un set de curseurs
	If (DimSize(W_Curseurs,0)<30)
		Redimension/N=30 W_Curseurs
		//	copie la ligne de base, si elle avait été déjà indiquée
		W_Curseurs[8]=W_Curseurs[4];	W_Curseurs[9]=W_Curseurs[5]
		W_Curseurs[14]=W_Curseurs[4];	W_Curseurs[15]=W_Curseurs[5]
		W_Curseurs[20]=W_Curseurs[4];	W_Curseurs[21]=W_Curseurs[5]
		W_Curseurs[22]=W_Curseurs[21]+2
		W_Curseurs[23]=W_Curseurs[21]+4
		W_Curseurs[24]=W_Curseurs[21]+6
		W_Curseurs[25]=W_Curseurs[21]+8
		W_Curseurs[26]=W_Curseurs[21]+10
		W_Curseurs[27]=W_Curseurs[21]+12
		W_Curseurs[28]=W_Curseurs[21]+14
		W_Curseurs[29]=W_Curseurs[21]+16
	endif
	
	//	Active les boutons
	DoWindow/F P_Export
	CheckBox C_CheckCurs0 value=1, disable=0
	CheckBox C_CheckCurs1 value=0, disable=0
	CheckBox C_CheckCurs2 value=0, disable=0
	CheckBox C_CheckCurs3 value=0, disable=0
	CheckBox C_CheckCurs4 value=0, disable=0

	//	pour afficher le DF export sélectionné
	sNomExport=GetDataFolder(0, dfExport)
	sListe=ListeDFExport()
	vRang=WhichListItem(sNomExport, sListe)		//	compte depuis 0
	PopupMenu C_DF_Export win=P_Export,mode=(vRang+1)	//	met à jour le popup; "mode" compte depuis 1
	
	//	Coche le bouton radio correspondant au bon mode curseur
	
	CheckModeCurseur("Init",0)
End

//	Pour afficher dans le popup qui définit le dfExport
Function/S ListeDFExport()
	String sListe="New;Duplicate;", sNomObj		//	on ajoute ces deux options
	variable i
	Do
		sNomObj=GetIndexedObjName("root:",4, i)	
		if (strlen(sNomObj) == 0)
			break
		endif
		if (CmpStr(sNomObj[0,5],ksNomExport)==0)
			sListe+=sNomObj
			sListe+=";"
		endif
		i+=1
	while(1)
	return(sListe)
End

//	Pour proposer de fermer toutes les fenêtres appelées par l'analyse
Function BoutonTermineStats(ctrlName) : ButtonControl
	String ctrlName
	variable i, j
	String sListeFenetres, sNomUneFenetre, sTest, sNomTrace, sListe="P_Export;F_Pic*;F_MesuresRatioTag;F_MesuresRatio;F_Curs_*"
	for (j=0; (j<ItemsInList(sListe)); j+=1)
		sTest=StringFromList(j, sListe)
		sListeFenetres=WinList(sTest, ";", "WIN:67")
		for (i=0; (i<ItemsInList(sListeFenetres));i+=1)
			sNomUneFenetre=StringFromList(i, sListeFenetres)
			KillWindow/Z $sNomUneFenetre
		endfor
	endfor
	SauveCurseurs(1)
	HideInfo/W=F_TracesTriees
	DoWindow P_Controle
	if (V_Flag)
		Checkbox C_Active_P_Export win=P_Controle, value=0
	endif
end
